// SpaFEDTe, a Template based C++ library for creating 
// Discontinuous Finite Element Spaces,
// Copyright (C) 2012 Lorenzo Alessio Botti

/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */ 
/* License as published by the Free Software Foundation either */ 
/* version 3.0 of the License, or (at your option) any later version. */

/* This software is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this software; if not, a copy of the full */
/* GNU Lesser General Public License can be found at */
/* http://www.gnu.org/licenses/ */

#ifndef __equispaced_finite_element_nodes_h__
#define __equispaced_finite_element_nodes_h__

#include "FiniteElementQuadrature.hpp"
#include "CollapsedMapping.hpp"
#include "Tuple.hpp"
#include "EnumElemType.hpp"
#include "Quadrature.hpp"

class EquiSpacedFiniteElementNodes 
{
public:
  typedef Quadrature::PointVector PointVector;
  typedef Quadrature::RealVector RealVector;

  EquiSpacedFiniteElementNodes() 
    {
      ;
    }
  
  ~EquiSpacedFiniteElementNodes() {}
  
public:
  static void initQRule(Quadrature& quadrature);
  
  static unsigned int nPoints(const unsigned int dim, const unsigned int q_Order, const ElemType itemType);

private:
  static void tetQuadrature(unsigned int order, PointVector& q_Qpoint);
  
  static void prismQuadrature(unsigned int order, PointVector& q_Qpoint);
  
  static void pyramidQuadrature(unsigned int order, PointVector& q_Qpoint);

  static void hexQuadrature(unsigned int order, PointVector& q_Qpoint);

  static void triQuadrature(unsigned int order, PointVector& q_Qpoint);

  static void quadQuadrature(unsigned int order, PointVector& q_Qpoint);
  
  static void lineQuadrature(unsigned int order, PointVector& q_Qpoint);
  
  static void tensorProduct2D(const RealVector& qPoints, PointVector& q_Qpoint);
  
  static void tensorProduct2D(const RealVector& qPoints1, const RealVector& qPoints2, PointVector& q_Qpoint);

  static void tensorProduct3D(const RealVector& qPoints, PointVector& q_Qpoint);
  
  static void tensorProduct3D(const RealVector& qPoints1, const RealVector& qPoints2, const RealVector& qPoints3, PointVector& q_Qpoint);

  static void equispacedNodes(const unsigned int n, RealVector& qPoints);
};

#endif 
