// SpaFEDTe, a Template based C++ library for creating 
// Discontinuous Finite Element Spaces,
// Copyright (C) 2012 Lorenzo Alessio Botti

/* This library is free software; you can redistribute it and/or */
/* modify it under the terms of the GNU Lesser General Public */ 
/* License as published by the Free Software Foundation either */ 
/* version 3.0 of the License, or (at your option) any later version. */

/* This software is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU */
/* Lesser General Public License for more details. */

/* You should have received a copy of the GNU Lesser General Public */
/* License along with this software; if not, a copy of the full */
/* GNU Lesser General Public License can be found at */
/* http://www.gnu.org/licenses/. */

#include "VTKHelperSpaMesh.hpp"

void VTKHelperSpaMesh::getSubElementsNodesE(const ElemType& itemType, const unsigned int& order, PointVector& nodes)
{
  switch(itemType)
  {
    case EDGE2:
    case EDGE3:
    {
      unsigned int orderSquared = order * order;
      switch(orderSquared)
      {
        case 0:
        case 1:
        {
          nodes.resize(2,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          return;
        }
        case 2:
        {
          nodes.resize(3,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) =  0.;
          nodes(2,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(2,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          nodes(2,2) =  0.;
          return;
        }
        case 3:
        {
          nodes.resize(4,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = -1./3.;
          nodes(2,0) =  1./3.;
          nodes(3,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(2,1) =  0.;
          nodes(3,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          nodes(2,2) =  0.;
          nodes(3,2) =  0.;
          return;
        }
        case 4:
        {
          nodes.resize(5,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = -1./2.;
          nodes(2,0) =  0.;
          nodes(3,0) =  1./2.;
          nodes(4,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(2,1) =  0.;
          nodes(3,1) =  0.;
          nodes(4,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          nodes(2,2) =  0.;
          nodes(3,2) =  0.;
          nodes(4,2) =  0.;
          return;
        }
        case 5:
        {
          nodes.resize(6,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = -3./5.;
          nodes(2,0) = -1./5.;
          nodes(3,0) =  1./5.;
          nodes(4,0) =  3./5.;
          nodes(5,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(2,1) =  0.;
          nodes(3,1) =  0.;
          nodes(4,1) =  0.;
          nodes(5,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          nodes(2,2) =  0.;
          nodes(3,2) =  0.;
          nodes(4,2) =  0.;
          nodes(5,2) =  0.;
          return;
        }
        case 6:
        {
          nodes.resize(7,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = -2./3.;
          nodes(2,0) = -1./3.;
          nodes(3,0) =  0.;
          nodes(4,0) =  1./3.;
          nodes(5,0) =  2./3.;
          nodes(6,0) =  1.;
          nodes(0,1) =  0.;
          nodes(1,1) =  0.;
          nodes(2,1) =  0.;
          nodes(3,1) =  0.;
          nodes(4,1) =  0.;
          nodes(5,1) =  0.;
          nodes(6,1) =  0.;
          nodes(0,2) =  0.;
          nodes(1,2) =  0.;
          nodes(2,2) =  0.;
          nodes(3,2) =  0.;
          nodes(4,2) =  0.;
          nodes(5,2) =  0.;
          nodes(6,2) =  0.;
          return;
        }
	default:
	{
	  unsigned int nNodes = orderSquared+1;
	  nodes.resize(nNodes,Eigen::NoChange);
	  double step = 2./(double(orderSquared));
	  for (unsigned int i = 0; i<nNodes; i++)
	  {
	    nodes(i,0) = -1. + step * double(i); 
	    nodes(i,1) = 0.;
	    nodes(i,2) = 0.;
	  }
	  return;
	}
      }
    }
    case TRI3:
    case TRI6:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(3,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) =  1.;
          nodes( 2,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          return;
        }
        case 2:
        {
          nodes.resize(6,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) =  0.;
          nodes( 2,0) =  1.;
          nodes( 3,0) = -1.;
          nodes( 4,0) =  0.;
          nodes( 5,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) =  0.;
          nodes( 4,1) =  0.;
          nodes( 5,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          return;
        }
        case 3:
        {
          nodes.resize(10,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -1./3.;
          nodes( 2,0) =  1./3.;
          nodes( 3,0) =  1.;
          nodes( 4,0) = -1.;
          nodes( 5,0) = -1./3.;
          nodes( 6,0) =  1./3.;
          nodes( 7,0) = -1.;
          nodes( 8,0) = -1./3.;
          nodes( 9,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1./3.;
          nodes( 5,1) = -1./3.;
          nodes( 6,1) = -1./3.;
          nodes( 7,1) =  1./3.;
          nodes( 8,1) =  1./3.;
          nodes( 9,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          return;
        }
        case 4:
        {
          nodes.resize(15,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -0.5;
          nodes( 2,0) =  0.;
          nodes( 3,0) =  0.5;
          nodes( 4,0) =  1.;
          nodes( 5,0) = -1.;
          nodes( 6,0) = -0.5;
          nodes( 7,0) =  0.;
          nodes( 8,0) =  0.5;
          nodes( 9,0) = -1.;
          nodes(10,0) = -0.5;
          nodes(11,0) =  0.;
          nodes(12,0) = -1.;
          nodes(13,0) = -0.5;
          nodes(14,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1.;
          nodes( 5,1) = -0.5;
          nodes( 6,1) = -0.5;
          nodes( 7,1) = -0.5;
          nodes( 8,1) = -0.5;
          nodes( 9,1) =  0.;
          nodes(10,1) =  0.;
          nodes(11,1) =  0.;
          nodes(12,1) =  0.5;
          nodes(13,1) =  0.5;
          nodes(14,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          nodes(10,2) =  0.;
          nodes(11,2) =  0.;
          nodes(12,2) =  0.;
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          return;
        }
        case 5:
        {
          nodes.resize(21,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -3./5.;
          nodes( 2,0) = -1./5.;
          nodes( 3,0) =  1./5.;
          nodes( 4,0) =  3./5.;
          nodes( 5,0) =  1.;
          nodes( 6,0) = -1.;
          nodes( 7,0) = -3./5.;
          nodes( 8,0) = -1./5.;
          nodes( 9,0) =  1./5.;
          nodes(10,0) =  3./5.;
          nodes(11,0) = -1.;
          nodes(12,0) = -3./5.;
          nodes(13,0) = -1./5.;
          nodes(14,0) =  1./5.;
          nodes(15,0) = -1.;
          nodes(16,0) = -3./5.;
          nodes(17,0) = -1./5.;
          nodes(18,0) = -1.;
          nodes(19,0) = -3./5.;
          nodes(20,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1.;
          nodes( 5,1) = -1.;
          nodes( 6,1) = -3./5.;
          nodes( 7,1) = -3./5.;
          nodes( 8,1) = -3./5.;
          nodes( 9,1) = -3./5.;
          nodes(10,1) = -3./5.;
          nodes(11,1) = -1./5.;
          nodes(12,1) = -1./5.;
          nodes(13,1) = -1./5.;
          nodes(14,1) = -1./5.;
          nodes(15,1) =  1./5.;
          nodes(16,1) =  1./5.;
          nodes(17,1) =  1./5.;
          nodes(18,1) =  3./5.;
          nodes(19,1) =  3./5.;
          nodes(20,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          nodes(10,2) =  0.;
          nodes(11,2) =  0.;
          nodes(12,2) =  0.;
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          nodes(15,2) =  0.;
          nodes(16,2) =  0.;
          nodes(17,2) =  0.;
          nodes(18,2) =  0.;
          nodes(19,2) =  0.;
          nodes(20,2) =  0.;
          return;
        }
        default:
        {
          nodes.resize(28,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -2./3.;
          nodes( 2,0) = -1./3.;
          nodes( 3,0) =  0.;
          nodes( 4,0) =  1./3.;
          nodes( 5,0) =  2./3.;
          nodes( 6,0) =  1.;
          nodes( 7,0) = -1.;
          nodes( 8,0) = -2./3.;
          nodes( 9,0) = -1./3.;
          nodes(10,0) =  0.;
          nodes(11,0) =  1./3.;
          nodes(12,0) =  2./3.;
          nodes(13,0) = -1.;
          nodes(14,0) = -2./3.;
          nodes(15,0) = -1./3.;
          nodes(16,0) =  0.;
          nodes(17,0) =  1./3.;
          nodes(18,0) = -1.;
          nodes(19,0) = -2./3.;
          nodes(20,0) = -1./3.;
          nodes(21,0) =  0.;
          nodes(22,0) = -1.;
          nodes(23,0) = -2./3.;
          nodes(24,0) = -1./3.;
          nodes(25,0) = -1.;
          nodes(26,0) = -2./3.;
          nodes(27,0) = -1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1.;
          nodes( 5,1) = -1.;
          nodes( 6,1) = -1.;
          nodes( 7,1) = -2./3.;
          nodes( 8,1) = -2./3.;
          nodes( 9,1) = -2./3.;
          nodes(10,1) = -2./3.;
          nodes(11,1) = -2./3.;
          nodes(12,1) = -2./3.;
          nodes(13,1) = -1./3.;
          nodes(14,1) = -1./3.;
          nodes(15,1) = -1./3.;
          nodes(16,1) = -1./3.;
          nodes(17,1) = -1./3.;
          nodes(18,1) =  0.;
          nodes(19,1) =  0.;
          nodes(20,1) =  0.;
          nodes(21,1) =  0.;
          nodes(22,1) =  1./3.;
          nodes(23,1) =  1./3.;
          nodes(24,1) =  1./3.;
          nodes(25,1) =  2./3.;
          nodes(26,1) =  2./3.;
          nodes(27,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          nodes(10,2) =  0.;
          nodes(11,2) =  0.;
          nodes(12,2) =  0.;
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          nodes(15,2) =  0.;
          nodes(16,2) =  0.;
          nodes(17,2) =  0.;
          nodes(18,2) =  0.;
          nodes(19,2) =  0.;
          nodes(20,2) =  0.;
          nodes(21,2) =  0.;
          nodes(22,2) =  0.;
          nodes(23,2) =  0.;
          nodes(24,2) =  0.;
          nodes(25,2) =  0.;
          nodes(26,2) =  0.;
          nodes(27,2) =  0.;
          return;
        }
      }
    } 
    case QUAD4:
    case QUAD8:
    case QUAD9:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(4,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) =  1.;
          nodes( 2,0) = -1.;
          nodes( 3,0) =  1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) =  1.;
          nodes( 3,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          return;
        }
        case 2:
        {
          nodes.resize(9,Eigen::NoChange);
          nodes( 0,0) = -1.;     
          nodes( 1,0) =  0.;  
          nodes( 2,0) =  1.;     
          nodes( 3,0) = -1.;     
          nodes( 4,0) =  0.;  
          nodes( 5,0) =  1.;     
          nodes( 6,0) = -1.;     
          nodes( 7,0) =  0.;  
          nodes( 8,0) =  1.;     
          nodes( 0,1) = -1.;     
          nodes( 1,1) = -1.;     
          nodes( 2,1) = -1.;     
          nodes( 3,1) =  0.;  
          nodes( 4,1) =  0.;  
          nodes( 5,1) =  0.;  
          nodes( 6,1) =  1.; 
          nodes( 7,1) =  1.;
          nodes( 8,1) =  1.;
          nodes( 0,2) =  0.;     
          nodes( 1,2) =  0.;     
          nodes( 2,2) =  0.;     
          nodes( 3,2) =  0.;  
          nodes( 4,2) =  0.;  
          nodes( 5,2) =  0.;  
          nodes( 6,2) =  0.; 
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          return;
        }
        case 3:
        {
          nodes.resize(16,Eigen::NoChange);
          nodes( 0,0) = -1.;       
          nodes( 1,0) = -1./3.;  
          nodes( 2,0) =  1./3.;  
          nodes( 3,0) =  1.;       
          nodes( 4,0) = -1.;       
          nodes( 5,0) = -1./3.;  
          nodes( 6,0) =  1./3.;  
          nodes( 7,0) =  1.;       
          nodes( 8,0) = -1.;       
          nodes( 9,0) = -1./3.;  
          nodes(10,0) =  1./3.;  
          nodes(11,0) =  1.;       
          nodes(12,0) = -1.;       
          nodes(13,0) = -1./3.;  
          nodes(14,0) =  1./3.;  
          nodes(15,0) =  1.;       
          nodes( 0,1) = -1.;       
          nodes( 1,1) = -1.;       
          nodes( 2,1) = -1.;       
          nodes( 3,1) = -1.;       
          nodes( 4,1) = -1./3.;  
          nodes( 5,1) = -1./3.;  
          nodes( 6,1) = -1./3.;  
          nodes( 7,1) = -1./3.;  
          nodes( 8,1) =  1./3.;  
          nodes( 9,1) =  1./3.;  
          nodes(10,1) =  1./3.;  
          nodes(11,1) =  1./3.;  
          nodes(12,1) =  1.; 
          nodes(13,1) =  1.;
          nodes(14,1) =  1.;
          nodes(15,1) =  1.;
          nodes( 0,2) =  0.;       
          nodes( 1,2) =  0.;       
          nodes( 2,2) =  0.;       
          nodes( 3,2) =  0.;       
          nodes( 4,2) =  0.;  
          nodes( 5,2) =  0.;  
          nodes( 6,2) =  0.;  
          nodes( 7,2) =  0.;  
          nodes( 8,2) =  0.;  
          nodes( 9,2) =  0.;  
          nodes(10,2) =  0.;  
          nodes(11,2) =  0.;  
          nodes(12,2) =  0.; 
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          nodes(15,2) =  0.;
          return;
        }
        case 4:
        {
          nodes.resize(25,Eigen::NoChange);
          nodes( 0,0) = -1.;       
          nodes( 1,0) = -1./2.;  
          nodes( 2,0) =  0.;    
          nodes( 3,0) =  1./2.;  
          nodes( 4,0) =  1.;       
          nodes( 5,0) = -1.;       
          nodes( 6,0) = -1./2.;  
          nodes( 7,0) =  0.;    
          nodes( 8,0) =  1./2.;  
          nodes( 9,0) =  1.;       
          nodes(10,0) = -1.;       
          nodes(11,0) = -1./2.;  
          nodes(12,0) =  0.;    
          nodes(13,0) =  1./2.;  
          nodes(14,0) =  1.;       
          nodes(15,0) = -1.;       
          nodes(16,0) = -1./2.;  
          nodes(17,0) =  0.;    
          nodes(18,0) =  1./2.;  
          nodes(19,0) =  1.;       
          nodes(20,0) = -1.;       
          nodes(21,0) = -1./2.;  
          nodes(22,0) =  0.;    
          nodes(23,0) =  1./2.;  
          nodes(24,0) =  1.;       
          nodes( 0,1) = -1.;       
          nodes( 1,1) = -1.;       
          nodes( 2,1) = -1.;       
          nodes( 3,1) = -1.;       
          nodes( 4,1) = -1.;       
          nodes( 5,1) = -1./2.;  
          nodes( 6,1) = -1./2.;  
          nodes( 7,1) = -1./2.;  
          nodes( 8,1) = -1./2.;  
          nodes( 9,1) = -1./2.;  
          nodes(10,1) =  0.;    
          nodes(11,1) =  0.;    
          nodes(12,1) =  0.;    
          nodes(13,1) =  0.;    
          nodes(14,1) =  0.;    
          nodes(15,1) =  1./2.;  
          nodes(16,1) =  1./2.;  
          nodes(17,1) =  1./2.;  
          nodes(18,1) =  1./2.;  
          nodes(19,1) =  1./2.;  
          nodes(20,1) =  1.; 
          nodes(21,1) =  1.;
          nodes(22,1) =  1.;
          nodes(23,1) =  1.;
          nodes(24,1) =  1.;
          nodes( 0,2) =  0.;       
          nodes( 1,2) =  0.;       
          nodes( 2,2) =  0.;       
          nodes( 3,2) =  0.;       
          nodes( 4,2) =  0.;       
          nodes( 5,2) =  0.;  
          nodes( 6,2) =  0.;  
          nodes( 7,2) =  0.;  
          nodes( 8,2) =  0.;  
          nodes( 9,2) =  0.;  
          nodes(10,2) =  0.;    
          nodes(11,2) =  0.;    
          nodes(12,2) =  0.;    
          nodes(13,2) =  0.;    
          nodes(14,2) =  0.;    
          nodes(15,2) =  0.;  
          nodes(16,2) =  0.;  
          nodes(17,2) =  0.;  
          nodes(18,2) =  0.;  
          nodes(19,2) =  0.;  
          nodes(20,2) =  0.; 
          nodes(21,2) =  0.;
          nodes(22,2) =  0.;
          nodes(23,2) =  0.;
          nodes(24,2) =  0.;
          return;
        }
        case 5:
        {
          nodes.resize(36,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -3./5.;
          nodes( 2,0) = -1./5.;
          nodes( 3,0) =  1./5.;
          nodes( 4,0) =  3./5.;
          nodes( 5,0) =  1.;
          nodes( 6,0) = -1.;
          nodes( 7,0) = -3./5.;
          nodes( 8,0) = -1./5.;
          nodes( 9,0) =  1./5.;
          nodes(10,0) =  3./5.;
          nodes(11,0) =  1.;
          nodes(12,0) = -1.;
          nodes(13,0) = -3./5.;
          nodes(14,0) = -1./5.;
          nodes(15,0) =  1./5.;
          nodes(16,0) =  3./5.;
          nodes(17,0) =  1.;
          nodes(18,0) = -1.;
          nodes(19,0) = -3./5.;
          nodes(20,0) = -1./5.;
          nodes(21,0) =  1./5.;
          nodes(22,0) =  3./5.;
          nodes(23,0) =  1.;
          nodes(24,0) = -1.;
          nodes(25,0) = -3./5.;
          nodes(26,0) = -1./5.;
          nodes(27,0) =  1./5.;
          nodes(28,0) =  3./5.;
          nodes(29,0) =  1.;
          nodes(30,0) = -1.;
          nodes(31,0) = -3./5.;
          nodes(32,0) = -1./5.;
          nodes(33,0) =  1./5.;
          nodes(34,0) =  3./5.;
          nodes(35,0) =  1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1.;
          nodes( 5,1) = -1.;
          nodes( 6,1) = -3./5.;
          nodes( 7,1) = -3./5.;
          nodes( 8,1) = -3./5.;
          nodes( 9,1) = -3./5.;
          nodes(10,1) = -3./5.;
          nodes(11,1) = -3./5.;
          nodes(12,1) = -1./5.;
          nodes(13,1) = -1./5.;
          nodes(14,1) = -1./5.;
          nodes(15,1) = -1./5.;
          nodes(16,1) = -1./5.;
          nodes(17,1) = -1./5.;
          nodes(18,1) =  1./5.;
          nodes(19,1) =  1./5.;
          nodes(20,1) =  1./5.;
          nodes(21,1) =  1./5.;
          nodes(22,1) =  1./5.;
          nodes(23,1) =  1./5.;
          nodes(24,1) =  3./5.;
          nodes(25,1) =  3./5.;
          nodes(26,1) =  3./5.;
          nodes(27,1) =  3./5.;
          nodes(28,1) =  3./5.;
          nodes(29,1) =  3./5.;
          nodes(30,1) =  1.;
          nodes(31,1) =  1.;
          nodes(32,1) =  1.;
          nodes(33,1) =  1.;
          nodes(34,1) =  1.;
          nodes(35,1) =  1.;
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          nodes(10,2) =  0.;
          nodes(11,2) =  0.;
          nodes(12,2) =  0.;
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          nodes(15,2) =  0.;
          nodes(16,2) =  0.;
          nodes(17,2) =  0.;
          nodes(18,2) =  0.;
          nodes(19,2) =  0.;
          nodes(20,2) =  0.;
          nodes(21,2) =  0.;
          nodes(22,2) =  0.;
          nodes(23,2) =  0.;
          nodes(24,2) =  0.;
          nodes(25,2) =  0.;
          nodes(26,2) =  0.;
          nodes(27,2) =  0.;
          nodes(28,2) =  0.;
          nodes(29,2) =  0.;
          nodes(30,2) =  0.;
          nodes(31,2) =  0.;
          nodes(32,2) =  0.;
          nodes(33,2) =  0.;
          nodes(34,2) =  0.;
          nodes(35,2) =  0.;
          return;
        }
        default:
        {
          nodes.resize(49,Eigen::NoChange);
          nodes( 0,0) = -1.;
          nodes( 1,0) = -2./3.;
          nodes( 2,0) = -1./3.;
          nodes( 3,0) =  0.;
          nodes( 4,0) =  1./3.;
          nodes( 5,0) =  2./3.;
          nodes( 6,0) =  1.;
          nodes( 7,0) = -1.;
          nodes( 8,0) = -2./3.;
          nodes( 9,0) = -1./3.;
          nodes(10,0) =  0.;
          nodes(11,0) =  1./3.;
          nodes(12,0) =  2./3.;
          nodes(13,0) =  1.;
          nodes(14,0) = -1.;
          nodes(15,0) = -2./3.;
          nodes(16,0) = -1./3.;
          nodes(17,0) =  0.;
          nodes(18,0) =  1./3.;
          nodes(19,0) =  2./3.;
          nodes(20,0) =  1.;
          nodes(21,0) = -1.;
          nodes(22,0) = -2./3.;
          nodes(23,0) = -1./3.;
          nodes(24,0) =  0.;
          nodes(25,0) =  1./3.;
          nodes(26,0) =  2./3.;
          nodes(27,0) =  1.;
          nodes(28,0) = -1.;
          nodes(29,0) = -2./3.;
          nodes(30,0) = -1./3.;
          nodes(31,0) =  0.;
          nodes(32,0) =  1./3.;
          nodes(33,0) =  2./3.;
          nodes(34,0) =  1.;
          nodes(35,0) = -1.;
          nodes(36,0) = -2./3.;
          nodes(37,0) = -1./3.;
          nodes(38,0) =  0.;
          nodes(39,0) =  1./3.;
          nodes(40,0) =  2./3.;
          nodes(41,0) =  1.;
          nodes(42,0) = -1.;
          nodes(43,0) = -2./3.;
          nodes(44,0) = -1./3.;
          nodes(45,0) =  0.;
          nodes(46,0) =  1./3.;
          nodes(47,0) =  2./3.;
          nodes(48,0) =  1.;
          nodes( 0,1) = -1.;
          nodes( 1,1) = -1.;
          nodes( 2,1) = -1.;
          nodes( 3,1) = -1.;
          nodes( 4,1) = -1.;
          nodes( 5,1) = -1.;
          nodes( 6,1) = -1.;
          nodes( 7,1) = -2./3.;
          nodes( 8,1) = -2./3.;
          nodes( 9,1) = -2./3.;
          nodes(10,1) = -2./3.;
          nodes(11,1) = -2./3.;
          nodes(12,1) = -2./3.;
          nodes(13,1) = -2./3.;
          nodes(14,1) = -1./3.;
          nodes(15,1) = -1./3.;
          nodes(16,1) = -1./3.;
          nodes(17,1) = -1./3.;
          nodes(18,1) = -1./3.;
          nodes(19,1) = -1./3.;
          nodes(20,1) = -1./3.;
          nodes(21,1) =  0.;
          nodes(22,1) =  0.;
          nodes(23,1) =  0.;
          nodes(24,1) =  0.;
          nodes(25,1) =  0.;
          nodes(26,1) =  0.;
          nodes(27,1) =  0.;
          nodes(28,1) =  1./3.;
          nodes(29,1) =  1./3.;
          nodes(30,1) =  1./3.;
          nodes(31,1) =  1./3.;
          nodes(32,1) =  1./3.;
          nodes(33,1) =  1./3.;
          nodes(34,1) =  1./3.;
          nodes(35,1) =  2./3.;
          nodes(36,1) =  2./3.;
          nodes(37,1) =  2./3.;
          nodes(38,1) =  2./3.;
          nodes(39,1) =  2./3.;
          nodes(40,1) =  2./3.;
          nodes(41,1) =  2./3.;
          nodes(42,1) =  1.; 
          nodes(43,1) =  1.; 
          nodes(44,1) =  1.; 
          nodes(45,1) =  1.; 
          nodes(46,1) =  1.; 
          nodes(47,1) =  1.; 
          nodes(48,1) =  1.; 
          nodes( 0,2) =  0.;
          nodes( 1,2) =  0.;
          nodes( 2,2) =  0.;
          nodes( 3,2) =  0.;
          nodes( 4,2) =  0.;
          nodes( 5,2) =  0.;
          nodes( 6,2) =  0.;
          nodes( 7,2) =  0.;
          nodes( 8,2) =  0.;
          nodes( 9,2) =  0.;
          nodes(10,2) =  0.;
          nodes(11,2) =  0.;
          nodes(12,2) =  0.;
          nodes(13,2) =  0.;
          nodes(14,2) =  0.;
          nodes(15,2) =  0.;
          nodes(16,2) =  0.;
          nodes(17,2) =  0.;
          nodes(18,2) =  0.;
          nodes(19,2) =  0.;
          nodes(20,2) =  0.;
          nodes(21,2) =  0.;
          nodes(22,2) =  0.;
          nodes(23,2) =  0.;
          nodes(24,2) =  0.;
          nodes(25,2) =  0.;
          nodes(26,2) =  0.;
          nodes(27,2) =  0.;
          nodes(28,2) =  0.;
          nodes(29,2) =  0.;
          nodes(30,2) =  0.;
          nodes(31,2) =  0.;
          nodes(32,2) =  0.;
          nodes(33,2) =  0.;
          nodes(34,2) =  0.;
          nodes(35,2) =  0.;
          nodes(36,2) =  0.;
          nodes(37,2) =  0.;
          nodes(38,2) =  0.;
          nodes(39,2) =  0.;
          nodes(40,2) =  0.;
          nodes(41,2) =  0.;
          nodes(42,2) =  0.; 
          nodes(43,2) =  0.; 
          nodes(44,2) =  0.; 
          nodes(45,2) =  0.; 
          nodes(46,2) =  0.; 
          nodes(47,2) =  0.; 
          nodes(48,2) =  0.; 
          return;
        }
      }
    }
    case PRISM6:
    case PRISM15:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(6,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 1.;
          nodes(5,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          return;
        }
        case 2:
        {
          nodes.resize(18,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 1.;
          nodes(5,0) = -1.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = -1.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = 0.;
          nodes(12,0) = 1.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 0.;
          nodes(16,0) = -1.;
          nodes(17,0) = 0.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 1.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 1.;
          nodes(15,1) = -1.;
          nodes(16,1) = 0.;
          nodes(17,1) = 0.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = -1.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = 1.;
          nodes(16,2) = 1.;
          nodes(17,2) = 1.;
          return;
        }
        case 3:
        case 4:
        {
          nodes.resize(75,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 1.;
          nodes(5,0) = -1.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = -1.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = 0.;
          nodes(12,0) = 1.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 0.;
          nodes(16,0) = -1.;
          nodes(17,0) = 0.;
          nodes(18,0) = -0.5;
          nodes(19,0) = -1.;
          nodes(20,0) = -1.;
          nodes(21,0) = -0.5;
          nodes(22,0) = -1.;
          nodes(23,0) = -0.5;
          nodes(24,0) = 0.;
          nodes(25,0) = -0.5;
          nodes(26,0) = -1.;
          nodes(27,0) = -0.5;
          nodes(28,0) = -1.;
          nodes(29,0) = -0.5;
          nodes(30,0) = 0.5;
          nodes(31,0) = 0.;
          nodes(32,0) = 0.5;
          nodes(33,0) = 0.;
          nodes(34,0) = 0.5;
          nodes(35,0) = 1.;
          nodes(36,0) = 0.5;
          nodes(37,0) = 0.;
          nodes(38,0) = 0.5;
          nodes(39,0) = 0.;
          nodes(40,0) = 0.5;
          nodes(41,0) = -0.5;
          nodes(42,0) = -1.;
          nodes(43,0) = -0.5;
          nodes(44,0) = -1.;
          nodes(45,0) = -0.5;
          nodes(46,0) = -0.5;
          nodes(47,0) = -1.;
          nodes(48,0) = -0.5;
          nodes(49,0) = -1.;
          nodes(50,0) = -0.5;
          nodes(51,0) = -1.;
          nodes(52,0) = -0.5;
          nodes(53,0) = -1.;
          nodes(54,0) = 0.;
          nodes(55,0) = -0.5;
          nodes(56,0) = -1.;
          nodes(57,0) = -0.5;
          nodes(58,0) = -1.;
          nodes(59,0) = -0.5;
          nodes(60,0) = 0.5;
          nodes(61,0) = 0.;
          nodes(62,0) = 1.;
          nodes(63,0) = 0.5;
          nodes(64,0) = 0.;
          nodes(65,0) = 0.5;
          nodes(66,0) = 0.;
          nodes(67,0) = 0.5;
          nodes(68,0) = -0.5;
          nodes(69,0) = -1.;
          nodes(70,0) = -0.5;
          nodes(71,0) = -1.;
          nodes(72,0) = -0.5;
          nodes(73,0) = -1.;
          nodes(74,0) = -0.5;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 1.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 1.;
          nodes(15,1) = -1.;
          nodes(16,1) = 0.;
          nodes(17,1) = 0.;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -1.;
          nodes(22,1) = -0.5;
          nodes(23,1) = -0.5;
          nodes(24,1) = -1.;
          nodes(25,1) = -0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = -1.;
          nodes(28,1) = -0.5;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(35,1) = -1.;
          nodes(36,1) = -0.5;
          nodes(37,1) = 0.;
          nodes(38,1) = -1.;
          nodes(39,1) = -0.5;
          nodes(40,1) = -0.5;
          nodes(41,1) = 0.;
          nodes(42,1) = 0.5;
          nodes(43,1) = 0.;
          nodes(44,1) = 0.5;
          nodes(45,1) = 0.5;
          nodes(46,1) = 0.5;
          nodes(47,1) = 1.;
          nodes(48,1) = 0.;
          nodes(49,1) = 0.5;
          nodes(50,1) = 0.5;
          nodes(51,1) = -1.;
          nodes(52,1) = -1.;
          nodes(53,1) = -0.5;
          nodes(54,1) = -1.;
          nodes(55,1) = -0.5;
          nodes(56,1) = 0.;
          nodes(57,1) = -1.;
          nodes(58,1) = -0.5;
          nodes(59,1) = -0.5;
          nodes(60,1) = -1.;
          nodes(61,1) = -0.5;
          nodes(62,1) = -1.;
          nodes(63,1) = -0.5;
          nodes(64,1) = 0.;
          nodes(65,1) = -1.;
          nodes(66,1) = -0.5;
          nodes(67,1) = -0.5;
          nodes(68,1) = 0.;
          nodes(69,1) = 0.5;
          nodes(70,1) = 0.5;
          nodes(71,1) = 1.;
          nodes(72,1) = 0.;
          nodes(73,1) = 0.5;
          nodes(74,1) = 0.5;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = -1.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = 1.;
          nodes(16,2) = 1.;
          nodes(17,2) = 1.;
          nodes(18,2) = -1.;
          nodes(19,2) = -1.;
          nodes(20,2) = -0.5;
          nodes(21,2) = -0.5;
          nodes(22,2) = -0.5;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -0.5;
          nodes(26,2) = -0.5;
          nodes(27,2) = 0.;
          nodes(28,2) = 0.;
          nodes(29,2) = 0.;
          nodes(30,2) = -1.;
          nodes(31,2) = -1.;
          nodes(32,2) = -0.5;
          nodes(33,2) = -0.5;
          nodes(34,2) = -1.;
          nodes(35,2) = -0.5;
          nodes(36,2) = -0.5;
          nodes(37,2) = -0.5;
          nodes(38,2) = 0.;
          nodes(39,2) = 0.;
          nodes(40,2) = 0.;
          nodes(41,2) = -1.;
          nodes(42,2) = -1.;
          nodes(43,2) = -0.5;
          nodes(44,2) = -0.5;
          nodes(45,2) = -1.;
          nodes(46,2) = -0.5;
          nodes(47,2) = -0.5;
          nodes(48,2) = 0.;
          nodes(49,2) = 0.;
          nodes(50,2) = 0.;
          nodes(51,2) = 0.5;
          nodes(52,2) = 0.5;
          nodes(53,2) = 0.5;
          nodes(54,2) = 0.5;
          nodes(55,2) = 0.5;
          nodes(56,2) = 0.5;
          nodes(57,2) = 1.;
          nodes(58,2) = 1.;
          nodes(59,2) = 1.;
          nodes(60,2) = 0.5;
          nodes(61,2) = 0.5;
          nodes(62,2) = 0.5;
          nodes(63,2) = 0.5;
          nodes(64,2) = 0.5;
          nodes(65,2) = 1.;
          nodes(66,2) = 1.;
          nodes(67,2) = 1.;
          nodes(68,2) = 0.5;
          nodes(69,2) = 0.5;
          nodes(70,2) = 0.5;
          nodes(71,2) = 0.5;
          nodes(72,2) = 1.;
          nodes(73,2) = 1.;
          nodes(74,2) = 1.;
          return;
        }
        default:
        {
          nodes.resize(405,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 1.;
          nodes(5,0) = -1.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = -1.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = 0.;
          nodes(12,0) = 1.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 0.;
          nodes(16,0) = -1.;
          nodes(17,0) = 0.;
          nodes(18,0) = -0.5;
          nodes(19,0) = -1.;
          nodes(20,0) = -1.;
          nodes(21,0) = -0.5;
          nodes(22,0) = -1.;
          nodes(23,0) = -0.5;
          nodes(24,0) = 0.;
          nodes(25,0) = -0.5;
          nodes(26,0) = -1.;
          nodes(27,0) = -0.5;
          nodes(28,0) = -1.;
          nodes(29,0) = -0.5;
          nodes(30,0) = 0.5;
          nodes(31,0) = 0.;
          nodes(32,0) = 0.5;
          nodes(33,0) = 0.;
          nodes(34,0) = 0.5;
          nodes(35,0) = 1.;
          nodes(36,0) = 0.5;
          nodes(37,0) = 0.;
          nodes(38,0) = 0.5;
          nodes(39,0) = 0.;
          nodes(40,0) = 0.5;
          nodes(41,0) = -0.5;
          nodes(42,0) = -1.;
          nodes(43,0) = -0.5;
          nodes(44,0) = -1.;
          nodes(45,0) = -0.5;
          nodes(46,0) = -0.5;
          nodes(47,0) = -1.;
          nodes(48,0) = -0.5;
          nodes(49,0) = -1.;
          nodes(50,0) = -0.5;
          nodes(51,0) = -1.;
          nodes(52,0) = -0.5;
          nodes(53,0) = -1.;
          nodes(54,0) = 0.;
          nodes(55,0) = -0.5;
          nodes(56,0) = -1.;
          nodes(57,0) = -0.5;
          nodes(58,0) = -1.;
          nodes(59,0) = -0.5;
          nodes(60,0) = 0.5;
          nodes(61,0) = 0.;
          nodes(62,0) = 1.;
          nodes(63,0) = 0.5;
          nodes(64,0) = 0.;
          nodes(65,0) = 0.5;
          nodes(66,0) = 0.;
          nodes(67,0) = 0.5;
          nodes(68,0) = -0.5;
          nodes(69,0) = -1.;
          nodes(70,0) = -0.5;
          nodes(71,0) = -1.;
          nodes(72,0) = -0.5;
          nodes(73,0) = -1.;
          nodes(74,0) = -0.5;
          nodes(75,0) = -0.75;
          nodes(76,0) = -1.;
          nodes(77,0) = -1.;
          nodes(78,0) = -0.75;
          nodes(79,0) = -1.;
          nodes(80,0) = -0.75;
          nodes(81,0) = -0.5;
          nodes(82,0) = -0.75;
          nodes(83,0) = -1.;
          nodes(84,0) = -0.75;
          nodes(85,0) = -1.;
          nodes(86,0) = -0.75;
          nodes(87,0) = -0.25;
          nodes(88,0) = -0.5;
          nodes(89,0) = -0.25;
          nodes(90,0) = -0.5;
          nodes(91,0) = -0.25;
          nodes(92,0) = 0.;
          nodes(93,0) = -0.25;
          nodes(94,0) = -0.5;
          nodes(95,0) = -0.25;
          nodes(96,0) = -0.5;
          nodes(97,0) = -0.25;
          nodes(98,0) = -0.75;
          nodes(99,0) = -1.;
          nodes(100,0) = -0.75;
          nodes(101,0) = -1.;
          nodes(102,0) = -0.75;
          nodes(103,0) = -0.75;
          nodes(104,0) = -1.;
          nodes(105,0) = -0.75;
          nodes(106,0) = -1.;
          nodes(107,0) = -0.75;
          nodes(108,0) = -1.;
          nodes(109,0) = -0.75;
          nodes(110,0) = -1.;
          nodes(111,0) = -0.5;
          nodes(112,0) = -0.75;
          nodes(113,0) = -1.;
          nodes(114,0) = -0.75;
          nodes(115,0) = -1.;
          nodes(116,0) = -0.75;
          nodes(117,0) = -0.25;
          nodes(118,0) = -0.5;
          nodes(119,0) = 0.;
          nodes(120,0) = -0.25;
          nodes(121,0) = -0.5;
          nodes(122,0) = -0.25;
          nodes(123,0) = -0.5;
          nodes(124,0) = -0.25;
          nodes(125,0) = -0.75;
          nodes(126,0) = -1.;
          nodes(127,0) = -0.75;
          nodes(128,0) = -1.;
          nodes(129,0) = -0.75;
          nodes(130,0) = -1.;
          nodes(131,0) = -0.75;
          nodes(132,0) = 0.25;
          nodes(133,0) = 0.;
          nodes(134,0) = 0.25;
          nodes(135,0) = 0.;
          nodes(136,0) = 0.25;
          nodes(137,0) = 0.5;
          nodes(138,0) = 0.25;
          nodes(139,0) = 0.;
          nodes(140,0) = 0.25;
          nodes(141,0) = 0.;
          nodes(142,0) = 0.25;
          nodes(143,0) = 0.75;
          nodes(144,0) = 0.5;
          nodes(145,0) = 0.75;
          nodes(146,0) = 0.5;
          nodes(147,0) = 0.75;
          nodes(148,0) = 1.;
          nodes(149,0) = 0.75;
          nodes(150,0) = 0.5;
          nodes(151,0) = 0.75;
          nodes(152,0) = 0.5;
          nodes(153,0) = 0.75;
          nodes(154,0) = 0.25;
          nodes(155,0) = 0.;
          nodes(156,0) = 0.25;
          nodes(157,0) = 0.;
          nodes(158,0) = 0.25;
          nodes(159,0) = 0.25;
          nodes(160,0) = 0.;
          nodes(161,0) = 0.25;
          nodes(162,0) = 0.;
          nodes(163,0) = 0.25;
          nodes(164,0) = 0.25;
          nodes(165,0) = 0.;
          nodes(166,0) = 0.5;
          nodes(167,0) = 0.25;
          nodes(168,0) = 0.;
          nodes(169,0) = 0.25;
          nodes(170,0) = 0.;
          nodes(171,0) = 0.25;
          nodes(172,0) = 0.75;
          nodes(173,0) = 0.5;
          nodes(174,0) = 1.;
          nodes(175,0) = 0.75;
          nodes(176,0) = 0.5;
          nodes(177,0) = 0.75;
          nodes(178,0) = 0.5;
          nodes(179,0) = 0.75;
          nodes(180,0) = 0.25;
          nodes(181,0) = 0.;
          nodes(182,0) = 0.25;
          nodes(183,0) = 0.;
          nodes(184,0) = 0.25;
          nodes(185,0) = 0.;
          nodes(186,0) = 0.25;
          nodes(187,0) = -0.75;
          nodes(188,0) = -1.;
          nodes(189,0) = -0.75;
          nodes(190,0) = -1.;
          nodes(191,0) = -0.75;
          nodes(192,0) = -0.5;
          nodes(193,0) = -0.75;
          nodes(194,0) = -1.;
          nodes(195,0) = -0.75;
          nodes(196,0) = -1.;
          nodes(197,0) = -0.75;
          nodes(198,0) = -0.25;
          nodes(199,0) = -0.5;
          nodes(200,0) = -0.25;
          nodes(201,0) = -0.5;
          nodes(202,0) = -0.25;
          nodes(203,0) = -0.25;
          nodes(204,0) = -0.5;
          nodes(205,0) = -0.25;
          nodes(206,0) = -0.5;
          nodes(207,0) = -0.25;
          nodes(208,0) = -0.75;
          nodes(209,0) = -1.;
          nodes(210,0) = -0.75;
          nodes(211,0) = -1.;
          nodes(212,0) = -0.75;
          nodes(213,0) = -0.75;
          nodes(214,0) = -1.;
          nodes(215,0) = -0.75;
          nodes(216,0) = -1.;
          nodes(217,0) = -0.75;
          nodes(218,0) = -0.75;
          nodes(219,0) = -1.;
          nodes(220,0) = -0.5;
          nodes(221,0) = -0.75;
          nodes(222,0) = -1.;
          nodes(223,0) = -0.75;
          nodes(224,0) = -1.;
          nodes(225,0) = -0.75;
          nodes(226,0) = -0.25;
          nodes(227,0) = -0.5;
          nodes(228,0) = -0.25;
          nodes(229,0) = -0.5;
          nodes(230,0) = -0.25;
          nodes(231,0) = -0.5;
          nodes(232,0) = -0.25;
          nodes(233,0) = -0.75;
          nodes(234,0) = -1.;
          nodes(235,0) = -0.75;
          nodes(236,0) = -1.;
          nodes(237,0) = -0.75;
          nodes(238,0) = -1.;
          nodes(239,0) = -0.75;
          nodes(240,0) = -0.25;
          nodes(241,0) = -0.25;
          nodes(242,0) = -0.25;
          nodes(243,0) = -0.25;
          nodes(244,0) = -0.25;
          nodes(245,0) = -0.25;
          nodes(246,0) = -0.5;
          nodes(247,0) = -0.5;
          nodes(248,0) = -0.5;
          nodes(249,0) = -0.25;
          nodes(250,0) = -0.25;
          nodes(251,0) = -0.25;
          nodes(252,0) = -0.25;
          nodes(253,0) = -0.5;
          nodes(254,0) = -0.5;
          nodes(255,0) = -1.;
          nodes(256,0) = -0.75;
          nodes(257,0) = -1.;
          nodes(258,0) = -0.5;
          nodes(259,0) = -0.75;
          nodes(260,0) = -1.;
          nodes(261,0) = -0.75;
          nodes(262,0) = -1.;
          nodes(263,0) = -0.75;
          nodes(264,0) = -0.25;
          nodes(265,0) = -0.5;
          nodes(266,0) = 0.;
          nodes(267,0) = -0.25;
          nodes(268,0) = -0.5;
          nodes(269,0) = -0.25;
          nodes(270,0) = -0.5;
          nodes(271,0) = -0.25;
          nodes(272,0) = -0.75;
          nodes(273,0) = -1.;
          nodes(274,0) = -0.75;
          nodes(275,0) = -1.;
          nodes(276,0) = -0.75;
          nodes(277,0) = -1.;
          nodes(278,0) = -0.75;
          nodes(279,0) = -1.;
          nodes(280,0) = -0.75;
          nodes(281,0) = -1.;
          nodes(282,0) = -0.5;
          nodes(283,0) = -0.75;
          nodes(284,0) = -1.;
          nodes(285,0) = -0.75;
          nodes(286,0) = -1.;
          nodes(287,0) = -0.75;
          nodes(288,0) = -0.25;
          nodes(289,0) = -0.5;
          nodes(290,0) = 0.;
          nodes(291,0) = -0.25;
          nodes(292,0) = -0.5;
          nodes(293,0) = -0.25;
          nodes(294,0) = -0.5;
          nodes(295,0) = -0.25;
          nodes(296,0) = -0.75;
          nodes(297,0) = -1.;
          nodes(298,0) = -0.75;
          nodes(299,0) = -1.;
          nodes(300,0) = -0.75;
          nodes(301,0) = -1.;
          nodes(302,0) = -0.75;
          nodes(303,0) = 0.25;
          nodes(304,0) = 0.;
          nodes(305,0) = 0.5;
          nodes(306,0) = 0.25;
          nodes(307,0) = 0.;
          nodes(308,0) = 0.25;
          nodes(309,0) = 0.;
          nodes(310,0) = 0.25;
          nodes(311,0) = 0.75;
          nodes(312,0) = 0.5;
          nodes(313,0) = 1.;
          nodes(314,0) = 0.75;
          nodes(315,0) = 0.5;
          nodes(316,0) = 0.75;
          nodes(317,0) = 0.5;
          nodes(318,0) = 0.75;
          nodes(319,0) = 0.25;
          nodes(320,0) = 0.;
          nodes(321,0) = 0.25;
          nodes(322,0) = 0.;
          nodes(323,0) = 0.25;
          nodes(324,0) = 0.;
          nodes(325,0) = 0.25;
          nodes(326,0) = 0.25;
          nodes(327,0) = 0.;
          nodes(328,0) = 0.5;
          nodes(329,0) = 0.25;
          nodes(330,0) = 0.;
          nodes(331,0) = 0.25;
          nodes(332,0) = 0.;
          nodes(333,0) = 0.25;
          nodes(334,0) = 0.75;
          nodes(335,0) = 0.5;
          nodes(336,0) = 1.;
          nodes(337,0) = 0.75;
          nodes(338,0) = 0.5;
          nodes(339,0) = 0.75;
          nodes(340,0) = 0.5;
          nodes(341,0) = 0.75;
          nodes(342,0) = 0.25;
          nodes(343,0) = 0.;
          nodes(344,0) = 0.25;
          nodes(345,0) = 0.;
          nodes(346,0) = 0.25;
          nodes(347,0) = 0.;
          nodes(348,0) = 0.25;
          nodes(349,0) = -0.75;
          nodes(350,0) = -1.;
          nodes(351,0) = -0.5;
          nodes(352,0) = -0.75;
          nodes(353,0) = -1.;
          nodes(354,0) = -0.75;
          nodes(355,0) = -1.;
          nodes(356,0) = -0.75;
          nodes(357,0) = -0.25;
          nodes(358,0) = -0.5;
          nodes(359,0) = -0.25;
          nodes(360,0) = -0.5;
          nodes(361,0) = -0.25;
          nodes(362,0) = -0.5;
          nodes(363,0) = -0.25;
          nodes(364,0) = -0.75;
          nodes(365,0) = -1.;
          nodes(366,0) = -0.75;
          nodes(367,0) = -1.;
          nodes(368,0) = -0.75;
          nodes(369,0) = -1.;
          nodes(370,0) = -0.75;
          nodes(371,0) = -0.75;
          nodes(372,0) = -1.;
          nodes(373,0) = -0.5;
          nodes(374,0) = -0.75;
          nodes(375,0) = -1.;
          nodes(376,0) = -0.75;
          nodes(377,0) = -1.;
          nodes(378,0) = -0.75;
          nodes(379,0) = -0.25;
          nodes(380,0) = -0.5;
          nodes(381,0) = -0.25;
          nodes(382,0) = -0.5;
          nodes(383,0) = -0.25;
          nodes(384,0) = -0.5;
          nodes(385,0) = -0.25;
          nodes(386,0) = -0.75;
          nodes(387,0) = -1.;
          nodes(388,0) = -0.75;
          nodes(389,0) = -1.;
          nodes(390,0) = -0.75;
          nodes(391,0) = -1.;
          nodes(392,0) = -0.75;
          nodes(393,0) = -0.25;
          nodes(394,0) = -0.25;
          nodes(395,0) = -0.25;
          nodes(396,0) = -0.25;
          nodes(397,0) = -0.5;
          nodes(398,0) = -0.5;
          nodes(399,0) = -0.25;
          nodes(400,0) = -0.25;
          nodes(401,0) = -0.25;
          nodes(402,0) = -0.25;
          nodes(403,0) = -0.5;
          nodes(404,0) = -0.5;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 1.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 1.;
          nodes(15,1) = -1.;
          nodes(16,1) = 0.;
          nodes(17,1) = 0.;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -1.;
          nodes(22,1) = -0.5;
          nodes(23,1) = -0.5;
          nodes(24,1) = -1.;
          nodes(25,1) = -0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = -1.;
          nodes(28,1) = -0.5;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(35,1) = -1.;
          nodes(36,1) = -0.5;
          nodes(37,1) = 0.;
          nodes(38,1) = -1.;
          nodes(39,1) = -0.5;
          nodes(40,1) = -0.5;
          nodes(41,1) = 0.;
          nodes(42,1) = 0.5;
          nodes(43,1) = 0.;
          nodes(44,1) = 0.5;
          nodes(45,1) = 0.5;
          nodes(46,1) = 0.5;
          nodes(47,1) = 1.;
          nodes(48,1) = 0.;
          nodes(49,1) = 0.5;
          nodes(50,1) = 0.5;
          nodes(51,1) = -1.;
          nodes(52,1) = -1.;
          nodes(53,1) = -0.5;
          nodes(54,1) = -1.;
          nodes(55,1) = -0.5;
          nodes(56,1) = 0.;
          nodes(57,1) = -1.;
          nodes(58,1) = -0.5;
          nodes(59,1) = -0.5;
          nodes(60,1) = -1.;
          nodes(61,1) = -0.5;
          nodes(62,1) = -1.;
          nodes(63,1) = -0.5;
          nodes(64,1) = 0.;
          nodes(65,1) = -1.;
          nodes(66,1) = -0.5;
          nodes(67,1) = -0.5;
          nodes(68,1) = 0.;
          nodes(69,1) = 0.5;
          nodes(70,1) = 0.5;
          nodes(71,1) = 1.;
          nodes(72,1) = 0.;
          nodes(73,1) = 0.5;
          nodes(74,1) = 0.5;
          nodes(75,1) = -1.;
          nodes(76,1) = -0.75;
          nodes(77,1) = -1.;
          nodes(78,1) = -1.;
          nodes(79,1) = -0.75;
          nodes(80,1) = -0.75;
          nodes(81,1) = -1.;
          nodes(82,1) = -0.75;
          nodes(83,1) = -0.5;
          nodes(84,1) = -1.;
          nodes(85,1) = -0.75;
          nodes(86,1) = -0.75;
          nodes(87,1) = -1.;
          nodes(88,1) = -0.75;
          nodes(89,1) = -1.;
          nodes(90,1) = -0.75;
          nodes(91,1) = -0.75;
          nodes(92,1) = -1.;
          nodes(93,1) = -0.75;
          nodes(94,1) = -0.5;
          nodes(95,1) = -1.;
          nodes(96,1) = -0.75;
          nodes(97,1) = -0.75;
          nodes(98,1) = -0.5;
          nodes(99,1) = -0.25;
          nodes(100,1) = -0.5;
          nodes(101,1) = -0.25;
          nodes(102,1) = -0.25;
          nodes(103,1) = -0.25;
          nodes(104,1) = 0.;
          nodes(105,1) = -0.5;
          nodes(106,1) = -0.25;
          nodes(107,1) = -0.25;
          nodes(108,1) = -1.;
          nodes(109,1) = -1.;
          nodes(110,1) = -0.75;
          nodes(111,1) = -1.;
          nodes(112,1) = -0.75;
          nodes(113,1) = -0.5;
          nodes(114,1) = -1.;
          nodes(115,1) = -0.75;
          nodes(116,1) = -0.75;
          nodes(117,1) = -1.;
          nodes(118,1) = -0.75;
          nodes(119,1) = -1.;
          nodes(120,1) = -0.75;
          nodes(121,1) = -0.5;
          nodes(122,1) = -1.;
          nodes(123,1) = -0.75;
          nodes(124,1) = -0.75;
          nodes(125,1) = -0.5;
          nodes(126,1) = -0.25;
          nodes(127,1) = -0.25;
          nodes(128,1) = 0.;
          nodes(129,1) = -0.5;
          nodes(130,1) = -0.25;
          nodes(131,1) = -0.25;
          nodes(132,1) = -1.;
          nodes(133,1) = -0.75;
          nodes(134,1) = -1.;
          nodes(135,1) = -0.75;
          nodes(136,1) = -0.75;
          nodes(137,1) = -1.;
          nodes(138,1) = -0.75;
          nodes(139,1) = -0.5;
          nodes(140,1) = -1.;
          nodes(141,1) = -0.75;
          nodes(142,1) = -0.75;
          nodes(143,1) = -1.;
          nodes(144,1) = -0.75;
          nodes(145,1) = -1.;
          nodes(146,1) = -0.75;
          nodes(147,1) = -0.75;
          nodes(148,1) = -1.;
          nodes(149,1) = -0.75;
          nodes(150,1) = -0.5;
          nodes(151,1) = -1.;
          nodes(152,1) = -0.75;
          nodes(153,1) = -0.75;
          nodes(154,1) = -0.5;
          nodes(155,1) = -0.25;
          nodes(156,1) = -0.5;
          nodes(157,1) = -0.25;
          nodes(158,1) = -0.25;
          nodes(159,1) = -0.25;
          nodes(160,1) = 0.;
          nodes(161,1) = -0.5;
          nodes(162,1) = -0.25;
          nodes(163,1) = -0.25;
          nodes(164,1) = -1.;
          nodes(165,1) = -0.75;
          nodes(166,1) = -1.;
          nodes(167,1) = -0.75;
          nodes(168,1) = -0.5;
          nodes(169,1) = -1.;
          nodes(170,1) = -0.75;
          nodes(171,1) = -0.75;
          nodes(172,1) = -1.;
          nodes(173,1) = -0.75;
          nodes(174,1) = -1.;
          nodes(175,1) = -0.75;
          nodes(176,1) = -0.5;
          nodes(177,1) = -1.;
          nodes(178,1) = -0.75;
          nodes(179,1) = -0.75;
          nodes(180,1) = -0.5;
          nodes(181,1) = -0.25;
          nodes(182,1) = -0.25;
          nodes(183,1) = 0.;
          nodes(184,1) = -0.5;
          nodes(185,1) = -0.25;
          nodes(186,1) = -0.25;
          nodes(187,1) = 0.;
          nodes(188,1) = 0.25;
          nodes(189,1) = 0.;
          nodes(190,1) = 0.25;
          nodes(191,1) = 0.25;
          nodes(192,1) = 0.;
          nodes(193,1) = 0.25;
          nodes(194,1) = 0.5;
          nodes(195,1) = 0.;
          nodes(196,1) = 0.25;
          nodes(197,1) = 0.25;
          nodes(198,1) = 0.;
          nodes(199,1) = 0.25;
          nodes(200,1) = 0.;
          nodes(201,1) = 0.25;
          nodes(202,1) = 0.25;
          nodes(203,1) = 0.25;
          nodes(204,1) = 0.5;
          nodes(205,1) = 0.;
          nodes(206,1) = 0.25;
          nodes(207,1) = 0.25;
          nodes(208,1) = 0.5;
          nodes(209,1) = 0.75;
          nodes(210,1) = 0.5;
          nodes(211,1) = 0.75;
          nodes(212,1) = 0.75;
          nodes(213,1) = 0.75;
          nodes(214,1) = 1.;
          nodes(215,1) = 0.5;
          nodes(216,1) = 0.75;
          nodes(217,1) = 0.75;
          nodes(218,1) = 0.;
          nodes(219,1) = 0.25;
          nodes(220,1) = 0.;
          nodes(221,1) = 0.25;
          nodes(222,1) = 0.5;
          nodes(223,1) = 0.;
          nodes(224,1) = 0.25;
          nodes(225,1) = 0.25;
          nodes(226,1) = 0.;
          nodes(227,1) = 0.25;
          nodes(228,1) = 0.25;
          nodes(229,1) = 0.5;
          nodes(230,1) = 0.;
          nodes(231,1) = 0.25;
          nodes(232,1) = 0.25;
          nodes(233,1) = 0.5;
          nodes(234,1) = 0.75;
          nodes(235,1) = 0.75;
          nodes(236,1) = 1.;
          nodes(237,1) = 0.5;
          nodes(238,1) = 0.75;
          nodes(239,1) = 0.75;
          nodes(240,1) = -0.5;
          nodes(241,1) = -0.5;
          nodes(242,1) = -0.5;
          nodes(243,1) = -0.25;
          nodes(244,1) = -0.25;
          nodes(245,1) = -0.25;
          nodes(246,1) = -0.25;
          nodes(247,1) = -0.25;
          nodes(248,1) = -0.25;
          nodes(249,1) = -0.5;
          nodes(250,1) = -0.5;
          nodes(251,1) = -0.25;
          nodes(252,1) = -0.25;
          nodes(253,1) = -0.25;
          nodes(254,1) = -0.25;
          nodes(255,1) = -1.;
          nodes(256,1) = -1.;
          nodes(257,1) = -0.75;
          nodes(258,1) = -1.;
          nodes(259,1) = -0.75;
          nodes(260,1) = -0.5;
          nodes(261,1) = -1.;
          nodes(262,1) = -0.75;
          nodes(263,1) = -0.75;
          nodes(264,1) = -1.;
          nodes(265,1) = -0.75;
          nodes(266,1) = -1.;
          nodes(267,1) = -0.75;
          nodes(268,1) = -0.5;
          nodes(269,1) = -1.;
          nodes(270,1) = -0.75;
          nodes(271,1) = -0.75;
          nodes(272,1) = -0.5;
          nodes(273,1) = -0.25;
          nodes(274,1) = -0.25;
          nodes(275,1) = 0.;
          nodes(276,1) = -0.5;
          nodes(277,1) = -0.25;
          nodes(278,1) = -0.25;
          nodes(279,1) = -1.;
          nodes(280,1) = -1.;
          nodes(281,1) = -0.75;
          nodes(282,1) = -1.;
          nodes(283,1) = -0.75;
          nodes(284,1) = -0.5;
          nodes(285,1) = -1.;
          nodes(286,1) = -0.75;
          nodes(287,1) = -0.75;
          nodes(288,1) = -1.;
          nodes(289,1) = -0.75;
          nodes(290,1) = -1.;
          nodes(291,1) = -0.75;
          nodes(292,1) = -0.5;
          nodes(293,1) = -1.;
          nodes(294,1) = -0.75;
          nodes(295,1) = -0.75;
          nodes(296,1) = -0.5;
          nodes(297,1) = -0.25;
          nodes(298,1) = -0.25;
          nodes(299,1) = 0.;
          nodes(300,1) = -0.5;
          nodes(301,1) = -0.25;
          nodes(302,1) = -0.25;
          nodes(303,1) = -1.;
          nodes(304,1) = -0.75;
          nodes(305,1) = -1.;
          nodes(306,1) = -0.75;
          nodes(307,1) = -0.5;
          nodes(308,1) = -1.;
          nodes(309,1) = -0.75;
          nodes(310,1) = -0.75;
          nodes(311,1) = -1.;
          nodes(312,1) = -0.75;
          nodes(313,1) = -1.;
          nodes(314,1) = -0.75;
          nodes(315,1) = -0.5;
          nodes(316,1) = -1.;
          nodes(317,1) = -0.75;
          nodes(318,1) = -0.75;
          nodes(319,1) = -0.5;
          nodes(320,1) = -0.25;
          nodes(321,1) = -0.25;
          nodes(322,1) = 0.;
          nodes(323,1) = -0.5;
          nodes(324,1) = -0.25;
          nodes(325,1) = -0.25;
          nodes(326,1) = -1.;
          nodes(327,1) = -0.75;
          nodes(328,1) = -1.;
          nodes(329,1) = -0.75;
          nodes(330,1) = -0.5;
          nodes(331,1) = -1.;
          nodes(332,1) = -0.75;
          nodes(333,1) = -0.75;
          nodes(334,1) = -1.;
          nodes(335,1) = -0.75;
          nodes(336,1) = -1.;
          nodes(337,1) = -0.75;
          nodes(338,1) = -0.5;
          nodes(339,1) = -1.;
          nodes(340,1) = -0.75;
          nodes(341,1) = -0.75;
          nodes(342,1) = -0.5;
          nodes(343,1) = -0.25;
          nodes(344,1) = -0.25;
          nodes(345,1) = 0.;
          nodes(346,1) = -0.5;
          nodes(347,1) = -0.25;
          nodes(348,1) = -0.25;
          nodes(349,1) = 0.;
          nodes(350,1) = 0.25;
          nodes(351,1) = 0.;
          nodes(352,1) = 0.25;
          nodes(353,1) = 0.5;
          nodes(354,1) = 0.;
          nodes(355,1) = 0.25;
          nodes(356,1) = 0.25;
          nodes(357,1) = 0.;
          nodes(358,1) = 0.25;
          nodes(359,1) = 0.25;
          nodes(360,1) = 0.5;
          nodes(361,1) = 0.;
          nodes(362,1) = 0.25;
          nodes(363,1) = 0.25;
          nodes(364,1) = 0.5;
          nodes(365,1) = 0.75;
          nodes(366,1) = 0.75;
          nodes(367,1) = 1.;
          nodes(368,1) = 0.5;
          nodes(369,1) = 0.75;
          nodes(370,1) = 0.75;
          nodes(371,1) = 0.;
          nodes(372,1) = 0.25;
          nodes(373,1) = 0.;
          nodes(374,1) = 0.25;
          nodes(375,1) = 0.5;
          nodes(376,1) = 0.;
          nodes(377,1) = 0.25;
          nodes(378,1) = 0.25;
          nodes(379,1) = 0.;
          nodes(380,1) = 0.25;
          nodes(381,1) = 0.25;
          nodes(382,1) = 0.5;
          nodes(383,1) = 0.;
          nodes(384,1) = 0.25;
          nodes(385,1) = 0.25;
          nodes(386,1) = 0.5;
          nodes(387,1) = 0.75;
          nodes(388,1) = 0.75;
          nodes(389,1) = 1.;
          nodes(390,1) = 0.5;
          nodes(391,1) = 0.75;
          nodes(392,1) = 0.75;
          nodes(393,1) = -0.5;
          nodes(394,1) = -0.5;
          nodes(395,1) = -0.25;
          nodes(396,1) = -0.25;
          nodes(397,1) = -0.25;
          nodes(398,1) = -0.25;
          nodes(399,1) = -0.5;
          nodes(400,1) = -0.5;
          nodes(401,1) = -0.25;
          nodes(402,1) = -0.25;
          nodes(403,1) = -0.25;
          nodes(404,1) = -0.25;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = -1.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = 1.;
          nodes(16,2) = 1.;
          nodes(17,2) = 1.;
          nodes(18,2) = -1.;
          nodes(19,2) = -1.;
          nodes(20,2) = -0.5;
          nodes(21,2) = -0.5;
          nodes(22,2) = -0.5;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -0.5;
          nodes(26,2) = -0.5;
          nodes(27,2) = 0.;
          nodes(28,2) = 0.;
          nodes(29,2) = 0.;
          nodes(30,2) = -1.;
          nodes(31,2) = -1.;
          nodes(32,2) = -0.5;
          nodes(33,2) = -0.5;
          nodes(34,2) = -1.;
          nodes(35,2) = -0.5;
          nodes(36,2) = -0.5;
          nodes(37,2) = -0.5;
          nodes(38,2) = 0.;
          nodes(39,2) = 0.;
          nodes(40,2) = 0.;
          nodes(41,2) = -1.;
          nodes(42,2) = -1.;
          nodes(43,2) = -0.5;
          nodes(44,2) = -0.5;
          nodes(45,2) = -1.;
          nodes(46,2) = -0.5;
          nodes(47,2) = -0.5;
          nodes(48,2) = 0.;
          nodes(49,2) = 0.;
          nodes(50,2) = 0.;
          nodes(51,2) = 0.5;
          nodes(52,2) = 0.5;
          nodes(53,2) = 0.5;
          nodes(54,2) = 0.5;
          nodes(55,2) = 0.5;
          nodes(56,2) = 0.5;
          nodes(57,2) = 1.;
          nodes(58,2) = 1.;
          nodes(59,2) = 1.;
          nodes(60,2) = 0.5;
          nodes(61,2) = 0.5;
          nodes(62,2) = 0.5;
          nodes(63,2) = 0.5;
          nodes(64,2) = 0.5;
          nodes(65,2) = 1.;
          nodes(66,2) = 1.;
          nodes(67,2) = 1.;
          nodes(68,2) = 0.5;
          nodes(69,2) = 0.5;
          nodes(70,2) = 0.5;
          nodes(71,2) = 0.5;
          nodes(72,2) = 1.;
          nodes(73,2) = 1.;
          nodes(74,2) = 1.;
          nodes(75,2) = -1.;
          nodes(76,2) = -1.;
          nodes(77,2) = -0.75;
          nodes(78,2) = -0.75;
          nodes(79,2) = -0.75;
          nodes(80,2) = -1.;
          nodes(81,2) = -0.75;
          nodes(82,2) = -0.75;
          nodes(83,2) = -0.75;
          nodes(84,2) = -0.5;
          nodes(85,2) = -0.5;
          nodes(86,2) = -0.5;
          nodes(87,2) = -1.;
          nodes(88,2) = -1.;
          nodes(89,2) = -0.75;
          nodes(90,2) = -0.75;
          nodes(91,2) = -1.;
          nodes(92,2) = -0.75;
          nodes(93,2) = -0.75;
          nodes(94,2) = -0.75;
          nodes(95,2) = -0.5;
          nodes(96,2) = -0.5;
          nodes(97,2) = -0.5;
          nodes(98,2) = -1.;
          nodes(99,2) = -1.;
          nodes(100,2) = -0.75;
          nodes(101,2) = -0.75;
          nodes(102,2) = -1.;
          nodes(103,2) = -0.75;
          nodes(104,2) = -0.75;
          nodes(105,2) = -0.5;
          nodes(106,2) = -0.5;
          nodes(107,2) = -0.5;
          nodes(108,2) = -0.25;
          nodes(109,2) = -0.25;
          nodes(110,2) = -0.25;
          nodes(111,2) = -0.25;
          nodes(112,2) = -0.25;
          nodes(113,2) = -0.25;
          nodes(114,2) = 0.;
          nodes(115,2) = 0.;
          nodes(116,2) = 0.;
          nodes(117,2) = -0.25;
          nodes(118,2) = -0.25;
          nodes(119,2) = -0.25;
          nodes(120,2) = -0.25;
          nodes(121,2) = -0.25;
          nodes(122,2) = 0.;
          nodes(123,2) = 0.;
          nodes(124,2) = 0.;
          nodes(125,2) = -0.25;
          nodes(126,2) = -0.25;
          nodes(127,2) = -0.25;
          nodes(128,2) = -0.25;
          nodes(129,2) = 0.;
          nodes(130,2) = 0.;
          nodes(131,2) = 0.;
          nodes(132,2) = -1.;
          nodes(133,2) = -1.;
          nodes(134,2) = -0.75;
          nodes(135,2) = -0.75;
          nodes(136,2) = -1.;
          nodes(137,2) = -0.75;
          nodes(138,2) = -0.75;
          nodes(139,2) = -0.75;
          nodes(140,2) = -0.5;
          nodes(141,2) = -0.5;
          nodes(142,2) = -0.5;
          nodes(143,2) = -1.;
          nodes(144,2) = -1.;
          nodes(145,2) = -0.75;
          nodes(146,2) = -0.75;
          nodes(147,2) = -1.;
          nodes(148,2) = -0.75;
          nodes(149,2) = -0.75;
          nodes(150,2) = -0.75;
          nodes(151,2) = -0.5;
          nodes(152,2) = -0.5;
          nodes(153,2) = -0.5;
          nodes(154,2) = -1.;
          nodes(155,2) = -1.;
          nodes(156,2) = -0.75;
          nodes(157,2) = -0.75;
          nodes(158,2) = -1.;
          nodes(159,2) = -0.75;
          nodes(160,2) = -0.75;
          nodes(161,2) = -0.5;
          nodes(162,2) = -0.5;
          nodes(163,2) = -0.5;
          nodes(164,2) = -0.25;
          nodes(165,2) = -0.25;
          nodes(166,2) = -0.25;
          nodes(167,2) = -0.25;
          nodes(168,2) = -0.25;
          nodes(169,2) = 0.;
          nodes(170,2) = 0.;
          nodes(171,2) = 0.;
          nodes(172,2) = -0.25;
          nodes(173,2) = -0.25;
          nodes(174,2) = -0.25;
          nodes(175,2) = -0.25;
          nodes(176,2) = -0.25;
          nodes(177,2) = 0.;
          nodes(178,2) = 0.;
          nodes(179,2) = 0.;
          nodes(180,2) = -0.25;
          nodes(181,2) = -0.25;
          nodes(182,2) = -0.25;
          nodes(183,2) = -0.25;
          nodes(184,2) = 0.;
          nodes(185,2) = 0.;
          nodes(186,2) = 0.;
          nodes(187,2) = -1.;
          nodes(188,2) = -1.;
          nodes(189,2) = -0.75;
          nodes(190,2) = -0.75;
          nodes(191,2) = -1.;
          nodes(192,2) = -0.75;
          nodes(193,2) = -0.75;
          nodes(194,2) = -0.75;
          nodes(195,2) = -0.5;
          nodes(196,2) = -0.5;
          nodes(197,2) = -0.5;
          nodes(198,2) = -1.;
          nodes(199,2) = -1.;
          nodes(200,2) = -0.75;
          nodes(201,2) = -0.75;
          nodes(202,2) = -1.;
          nodes(203,2) = -0.75;
          nodes(204,2) = -0.75;
          nodes(205,2) = -0.5;
          nodes(206,2) = -0.5;
          nodes(207,2) = -0.5;
          nodes(208,2) = -1.;
          nodes(209,2) = -1.;
          nodes(210,2) = -0.75;
          nodes(211,2) = -0.75;
          nodes(212,2) = -1.;
          nodes(213,2) = -0.75;
          nodes(214,2) = -0.75;
          nodes(215,2) = -0.5;
          nodes(216,2) = -0.5;
          nodes(217,2) = -0.5;
          nodes(218,2) = -0.25;
          nodes(219,2) = -0.25;
          nodes(220,2) = -0.25;
          nodes(221,2) = -0.25;
          nodes(222,2) = -0.25;
          nodes(223,2) = 0.;
          nodes(224,2) = 0.;
          nodes(225,2) = 0.;
          nodes(226,2) = -0.25;
          nodes(227,2) = -0.25;
          nodes(228,2) = -0.25;
          nodes(229,2) = -0.25;
          nodes(230,2) = 0.;
          nodes(231,2) = 0.;
          nodes(232,2) = 0.;
          nodes(233,2) = -0.25;
          nodes(234,2) = -0.25;
          nodes(235,2) = -0.25;
          nodes(236,2) = -0.25;
          nodes(237,2) = 0.;
          nodes(238,2) = 0.;
          nodes(239,2) = 0.;
          nodes(240,2) = -1.;
          nodes(241,2) = -0.75;
          nodes(242,2) = -0.5;
          nodes(243,2) = -1.;
          nodes(244,2) = -0.75;
          nodes(245,2) = -0.5;
          nodes(246,2) = -1.;
          nodes(247,2) = -0.75;
          nodes(248,2) = -0.5;
          nodes(249,2) = -0.25;
          nodes(250,2) = 0.;
          nodes(251,2) = -0.25;
          nodes(252,2) = 0.;
          nodes(253,2) = -0.25;
          nodes(254,2) = 0.;
          nodes(255,2) = 0.25;
          nodes(256,2) = 0.25;
          nodes(257,2) = 0.25;
          nodes(258,2) = 0.25;
          nodes(259,2) = 0.25;
          nodes(260,2) = 0.25;
          nodes(261,2) = 0.5;
          nodes(262,2) = 0.5;
          nodes(263,2) = 0.5;
          nodes(264,2) = 0.25;
          nodes(265,2) = 0.25;
          nodes(266,2) = 0.25;
          nodes(267,2) = 0.25;
          nodes(268,2) = 0.25;
          nodes(269,2) = 0.5;
          nodes(270,2) = 0.5;
          nodes(271,2) = 0.5;
          nodes(272,2) = 0.25;
          nodes(273,2) = 0.25;
          nodes(274,2) = 0.25;
          nodes(275,2) = 0.25;
          nodes(276,2) = 0.5;
          nodes(277,2) = 0.5;
          nodes(278,2) = 0.5;
          nodes(279,2) = 0.75;
          nodes(280,2) = 0.75;
          nodes(281,2) = 0.75;
          nodes(282,2) = 0.75;
          nodes(283,2) = 0.75;
          nodes(284,2) = 0.75;
          nodes(285,2) = 1.;
          nodes(286,2) = 1.;
          nodes(287,2) = 1.;
          nodes(288,2) = 0.75;
          nodes(289,2) = 0.75;
          nodes(290,2) = 0.75;
          nodes(291,2) = 0.75;
          nodes(292,2) = 0.75;
          nodes(293,2) = 1.;
          nodes(294,2) = 1.;
          nodes(295,2) = 1.;
          nodes(296,2) = 0.75;
          nodes(297,2) = 0.75;
          nodes(298,2) = 0.75;
          nodes(299,2) = 0.75;
          nodes(300,2) = 1.;
          nodes(301,2) = 1.;
          nodes(302,2) = 1.;
          nodes(303,2) = 0.25;
          nodes(304,2) = 0.25;
          nodes(305,2) = 0.25;
          nodes(306,2) = 0.25;
          nodes(307,2) = 0.25;
          nodes(308,2) = 0.5;
          nodes(309,2) = 0.5;
          nodes(310,2) = 0.5;
          nodes(311,2) = 0.25;
          nodes(312,2) = 0.25;
          nodes(313,2) = 0.25;
          nodes(314,2) = 0.25;
          nodes(315,2) = 0.25;
          nodes(316,2) = 0.5;
          nodes(317,2) = 0.5;
          nodes(318,2) = 0.5;
          nodes(319,2) = 0.25;
          nodes(320,2) = 0.25;
          nodes(321,2) = 0.25;
          nodes(322,2) = 0.25;
          nodes(323,2) = 0.5;
          nodes(324,2) = 0.5;
          nodes(325,2) = 0.5;
          nodes(326,2) = 0.75;
          nodes(327,2) = 0.75;
          nodes(328,2) = 0.75;
          nodes(329,2) = 0.75;
          nodes(330,2) = 0.75;
          nodes(331,2) = 1.;
          nodes(332,2) = 1.;
          nodes(333,2) = 1.;
          nodes(334,2) = 0.75;
          nodes(335,2) = 0.75;
          nodes(336,2) = 0.75;
          nodes(337,2) = 0.75;
          nodes(338,2) = 0.75;
          nodes(339,2) = 1.;
          nodes(340,2) = 1.;
          nodes(341,2) = 1.;
          nodes(342,2) = 0.75;
          nodes(343,2) = 0.75;
          nodes(344,2) = 0.75;
          nodes(345,2) = 0.75;
          nodes(346,2) = 1.;
          nodes(347,2) = 1.;
          nodes(348,2) = 1.;
          nodes(349,2) = 0.25;
          nodes(350,2) = 0.25;
          nodes(351,2) = 0.25;
          nodes(352,2) = 0.25;
          nodes(353,2) = 0.25;
          nodes(354,2) = 0.5;
          nodes(355,2) = 0.5;
          nodes(356,2) = 0.5;
          nodes(357,2) = 0.25;
          nodes(358,2) = 0.25;
          nodes(359,2) = 0.25;
          nodes(360,2) = 0.25;
          nodes(361,2) = 0.5;
          nodes(362,2) = 0.5;
          nodes(363,2) = 0.5;
          nodes(364,2) = 0.25;
          nodes(365,2) = 0.25;
          nodes(366,2) = 0.25;
          nodes(367,2) = 0.25;
          nodes(368,2) = 0.5;
          nodes(369,2) = 0.5;
          nodes(370,2) = 0.5;
          nodes(371,2) = 0.75;
          nodes(372,2) = 0.75;
          nodes(373,2) = 0.75;
          nodes(374,2) = 0.75;
          nodes(375,2) = 0.75;
          nodes(376,2) = 1.;
          nodes(377,2) = 1.;
          nodes(378,2) = 1.;
          nodes(379,2) = 0.75;
          nodes(380,2) = 0.75;
          nodes(381,2) = 0.75;
          nodes(382,2) = 0.75;
          nodes(383,2) = 1.;
          nodes(384,2) = 1.;
          nodes(385,2) = 1.;
          nodes(386,2) = 0.75;
          nodes(387,2) = 0.75;
          nodes(388,2) = 0.75;
          nodes(389,2) = 0.75;
          nodes(390,2) = 1.;
          nodes(391,2) = 1.;
          nodes(392,2) = 1.;
          nodes(393,2) = 0.25;
          nodes(394,2) = 0.5;
          nodes(395,2) = 0.25;
          nodes(396,2) = 0.5;
          nodes(397,2) = 0.25;
          nodes(398,2) = 0.5;
          nodes(399,2) = 0.75;
          nodes(400,2) = 1.;
          nodes(401,2) = 0.75;
          nodes(402,2) = 1.;
          nodes(403,2) = 0.75;
          nodes(404,2) = 1.;
          return;
        }
      }
    } 
    case PYRAMID5:
    case PYRAMID13:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(5,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          return;
        }
        case 2:
        {
          nodes.resize(14,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 0.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = 1.;
          nodes(9,0) = 0.;
          nodes(10,0) = 0.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = 1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 0.;
          nodes(7,1) = -1.;
          nodes(8,1) = 0.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = 1.;
          nodes(13,1) = 0.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = -1.;
          nodes(7,2) = 0.;
          nodes(8,2) = -1.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = -1.;
          nodes(13,2) = 0.;
          return;
        }
        case 3:
        case 4:
        {
          nodes.resize(55,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 0.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = 1.;
          nodes(9,0) = 0.;
          nodes(10,0) = 0.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = -1.;
          nodes(14,0) = -0.5;
          nodes(15,0) = -0.5;
          nodes(16,0) = -1.;
          nodes(17,0) = 0.;
          nodes(18,0) = -0.5;
          nodes(19,0) = -0.5;
          nodes(20,0) = 0.5;
          nodes(21,0) = 0.5;
          nodes(22,0) = 0.;
          nodes(23,0) = 1.;
          nodes(24,0) = 0.5;
          nodes(25,0) = 0.5;
          nodes(26,0) = 0.5;
          nodes(27,0) = 0.5;
          nodes(28,0) = 0.;
          nodes(29,0) = 1.;
          nodes(30,0) = 0.5;
          nodes(31,0) = 0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -0.5;
          nodes(34,0) = -1.;
          nodes(35,0) = 0.;
          nodes(36,0) = -0.5;
          nodes(37,0) = -0.5;
          nodes(38,0) = 0.;
          nodes(39,0) = -1.;
          nodes(40,0) = -0.5;
          nodes(41,0) = -1.;
          nodes(42,0) = 0.;
          nodes(43,0) = 0.5;
          nodes(44,0) = 0.;
          nodes(45,0) = -0.5;
          nodes(46,0) = -1.;
          nodes(47,0) = -1.;
          nodes(48,0) = -0.5;
          nodes(49,0) = -0.5;
          nodes(50,0) = -1.;
          nodes(51,0) = -1.;
          nodes(52,0) = -0.5;
          nodes(53,0) = -1.;
          nodes(54,0) = -0.5;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = 1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 0.;
          nodes(7,1) = -1.;
          nodes(8,1) = 0.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = 1.;
          nodes(13,1) = 0.;
          nodes(14,1) = -1.;
          nodes(15,1) = -0.5;
          nodes(16,1) = -1.;
          nodes(17,1) = -0.5;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -0.5;
          nodes(22,1) = -1.;
          nodes(23,1) = -0.5;
          nodes(24,1) = -1.;
          nodes(25,1) = -0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = 0.5;
          nodes(28,1) = 0.;
          nodes(29,1) = 0.5;
          nodes(30,1) = 0.;
          nodes(31,1) = 0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -1.;
          nodes(35,1) = -0.5;
          nodes(36,1) = -1.;
          nodes(37,1) = -0.5;
          nodes(38,1) = -0.5;
          nodes(39,1) = -0.5;
          nodes(40,1) = 0.;
          nodes(41,1) = -0.5;
          nodes(42,1) = 0.5;
          nodes(43,1) = 1.;
          nodes(44,1) = 0.5;
          nodes(45,1) = 0.5;
          nodes(46,1) = 0.5;
          nodes(47,1) = 0.;
          nodes(48,1) = 1.;
          nodes(49,1) = 0.5;
          nodes(50,1) = 0.5;
          nodes(51,1) = -0.5;
          nodes(52,1) = 0.;
          nodes(53,1) = -0.5;
          nodes(54,1) = 0.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = -1.;
          nodes(7,2) = 0.;
          nodes(8,2) = -1.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = -1.;
          nodes(13,2) = 0.;
          nodes(14,2) = -1.;
          nodes(15,2) = -1.;
          nodes(16,2) = -0.5;
          nodes(17,2) = -1.;
          nodes(18,2) = -0.5;
          nodes(19,2) = -0.5;
          nodes(20,2) = -1.;
          nodes(21,2) = -1.;
          nodes(22,2) = -0.5;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -0.5;
          nodes(26,2) = -1.;
          nodes(27,2) = -1.;
          nodes(28,2) = -0.5;
          nodes(29,2) = -1.;
          nodes(30,2) = -0.5;
          nodes(31,2) = -0.5;
          nodes(32,2) = 0.;
          nodes(33,2) = 0.;
          nodes(34,2) = 0.5;
          nodes(35,2) = 0.;
          nodes(36,2) = 0.5;
          nodes(37,2) = 0.5;
          nodes(38,2) = -0.5;
          nodes(39,2) = -1.;
          nodes(40,2) = -1.;
          nodes(41,2) = -0.5;
          nodes(42,2) = -1.;
          nodes(43,2) = -1.;
          nodes(44,2) = -0.5;
          nodes(45,2) = -1.;
          nodes(46,2) = -1.;
          nodes(47,2) = -0.5;
          nodes(48,2) = -1.;
          nodes(49,2) = -0.5;
          nodes(50,2) = -0.5;
          nodes(51,2) = 0.;
          nodes(52,2) = 0.;
          nodes(53,2) = 0.5;
          nodes(54,2) = -0.5;
          return;
        }
        default:
        {
          nodes.resize(285,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 0.;
          nodes(6,0) = 0.;
          nodes(7,0) = -1.;
          nodes(8,0) = 1.;
          nodes(9,0) = 0.;
          nodes(10,0) = 0.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = -1.;
          nodes(14,0) = -0.5;
          nodes(15,0) = -0.5;
          nodes(16,0) = -1.;
          nodes(17,0) = 0.;
          nodes(18,0) = -0.5;
          nodes(19,0) = -0.5;
          nodes(20,0) = 0.5;
          nodes(21,0) = 0.5;
          nodes(22,0) = 0.;
          nodes(23,0) = 1.;
          nodes(24,0) = 0.5;
          nodes(25,0) = 0.5;
          nodes(26,0) = 0.5;
          nodes(27,0) = 0.5;
          nodes(28,0) = 0.;
          nodes(29,0) = 1.;
          nodes(30,0) = 0.5;
          nodes(31,0) = 0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -0.5;
          nodes(34,0) = -1.;
          nodes(35,0) = 0.;
          nodes(36,0) = -0.5;
          nodes(37,0) = -0.5;
          nodes(38,0) = 0.;
          nodes(39,0) = -1.;
          nodes(40,0) = -0.5;
          nodes(41,0) = -1.;
          nodes(42,0) = 0.;
          nodes(43,0) = 0.5;
          nodes(44,0) = 0.;
          nodes(45,0) = -0.5;
          nodes(46,0) = -1.;
          nodes(47,0) = -1.;
          nodes(48,0) = -0.5;
          nodes(49,0) = -0.5;
          nodes(50,0) = -1.;
          nodes(51,0) = -1.;
          nodes(52,0) = -0.5;
          nodes(53,0) = -1.;
          nodes(54,0) = -0.5;
          nodes(55,0) = -0.75;
          nodes(56,0) = -0.75;
          nodes(57,0) = -1.;
          nodes(58,0) = -0.5;
          nodes(59,0) = -0.75;
          nodes(60,0) = -0.75;
          nodes(61,0) = -0.25;
          nodes(62,0) = -0.25;
          nodes(63,0) = -0.5;
          nodes(64,0) = 0.;
          nodes(65,0) = -0.25;
          nodes(66,0) = -0.25;
          nodes(67,0) = -0.25;
          nodes(68,0) = -0.25;
          nodes(69,0) = -0.5;
          nodes(70,0) = 0.;
          nodes(71,0) = -0.25;
          nodes(72,0) = -0.25;
          nodes(73,0) = -0.75;
          nodes(74,0) = -0.75;
          nodes(75,0) = -1.;
          nodes(76,0) = -0.5;
          nodes(77,0) = -0.75;
          nodes(78,0) = -0.75;
          nodes(79,0) = -0.5;
          nodes(80,0) = 0.25;
          nodes(81,0) = 0.25;
          nodes(82,0) = 0.;
          nodes(83,0) = 0.5;
          nodes(84,0) = 0.25;
          nodes(85,0) = 0.25;
          nodes(86,0) = 0.75;
          nodes(87,0) = 0.75;
          nodes(88,0) = 0.5;
          nodes(89,0) = 1.;
          nodes(90,0) = 0.75;
          nodes(91,0) = 0.75;
          nodes(92,0) = 0.75;
          nodes(93,0) = 0.75;
          nodes(94,0) = 0.5;
          nodes(95,0) = 1.;
          nodes(96,0) = 0.75;
          nodes(97,0) = 0.75;
          nodes(98,0) = 0.25;
          nodes(99,0) = 0.25;
          nodes(100,0) = 0.;
          nodes(101,0) = 0.5;
          nodes(102,0) = 0.25;
          nodes(103,0) = 0.25;
          nodes(104,0) = 0.5;
          nodes(105,0) = 0.25;
          nodes(106,0) = 0.25;
          nodes(107,0) = 0.;
          nodes(108,0) = 0.5;
          nodes(109,0) = 0.25;
          nodes(110,0) = 0.25;
          nodes(111,0) = 0.75;
          nodes(112,0) = 0.75;
          nodes(113,0) = 0.5;
          nodes(114,0) = 1.;
          nodes(115,0) = 0.75;
          nodes(116,0) = 0.75;
          nodes(117,0) = 0.75;
          nodes(118,0) = 0.75;
          nodes(119,0) = 0.5;
          nodes(120,0) = 1.;
          nodes(121,0) = 0.75;
          nodes(122,0) = 0.75;
          nodes(123,0) = 0.25;
          nodes(124,0) = 0.25;
          nodes(125,0) = 0.;
          nodes(126,0) = 0.5;
          nodes(127,0) = 0.25;
          nodes(128,0) = 0.25;
          nodes(129,0) = 0.5;
          nodes(130,0) = -0.75;
          nodes(131,0) = -0.75;
          nodes(132,0) = -1.;
          nodes(133,0) = -0.5;
          nodes(134,0) = -0.75;
          nodes(135,0) = -0.75;
          nodes(136,0) = -0.25;
          nodes(137,0) = -0.25;
          nodes(138,0) = -0.5;
          nodes(139,0) = 0.;
          nodes(140,0) = -0.25;
          nodes(141,0) = -0.25;
          nodes(142,0) = -0.25;
          nodes(143,0) = -0.25;
          nodes(144,0) = -0.5;
          nodes(145,0) = 0.;
          nodes(146,0) = -0.25;
          nodes(147,0) = -0.25;
          nodes(148,0) = -0.75;
          nodes(149,0) = -0.75;
          nodes(150,0) = -1.;
          nodes(151,0) = -0.5;
          nodes(152,0) = -0.75;
          nodes(153,0) = -0.75;
          nodes(154,0) = -0.5;
          nodes(155,0) = 0.;
          nodes(156,0) = -0.25;
          nodes(157,0) = 0.;
          nodes(158,0) = -0.25;
          nodes(159,0) = 0.;
          nodes(160,0) = -0.25;
          nodes(161,0) = 0.;
          nodes(162,0) = 0.;
          nodes(163,0) = -0.25;
          nodes(164,0) = -0.5;
          nodes(165,0) = -0.5;
          nodes(166,0) = -0.25;
          nodes(167,0) = 0.25;
          nodes(168,0) = 0.5;
          nodes(169,0) = 0.5;
          nodes(170,0) = 0.25;
          nodes(171,0) = 0.25;
          nodes(172,0) = 0.25;
          nodes(173,0) = 0.25;
          nodes(174,0) = 0.;
          nodes(175,0) = 0.5;
          nodes(176,0) = 0.;
          nodes(177,0) = 0.25;
          nodes(178,0) = 0.;
          nodes(179,0) = 0.25;
          nodes(180,0) = 0.25;
          nodes(181,0) = -0.25;
          nodes(182,0) = -0.25;
          nodes(183,0) = -0.5;
          nodes(184,0) = -0.25;
          nodes(185,0) = -1.;
          nodes(186,0) = -0.75;
          nodes(187,0) = -1.;
          nodes(188,0) = -0.5;
          nodes(189,0) = -0.25;
          nodes(190,0) = -0.5;
          nodes(191,0) = -0.75;
          nodes(192,0) = -1.;
          nodes(193,0) = -1.;
          nodes(194,0) = -0.75;
          nodes(195,0) = -0.75;
          nodes(196,0) = -1.;
          nodes(197,0) = -1.;
          nodes(198,0) = -0.75;
          nodes(199,0) = -1.;
          nodes(200,0) = -0.75;
          nodes(201,0) = 0.;
          nodes(202,0) = 0.25;
          nodes(203,0) = 0.;
          nodes(204,0) = 0.5;
          nodes(205,0) = 0.75;
          nodes(206,0) = 0.5;
          nodes(207,0) = 0.25;
          nodes(208,0) = 0.;
          nodes(209,0) = 0.;
          nodes(210,0) = 0.25;
          nodes(211,0) = 0.25;
          nodes(212,0) = 0.;
          nodes(213,0) = 0.;
          nodes(214,0) = 0.25;
          nodes(215,0) = 0.;
          nodes(216,0) = 0.25;
          nodes(217,0) = -0.75;
          nodes(218,0) = -1.;
          nodes(219,0) = -1.;
          nodes(220,0) = -0.75;
          nodes(221,0) = -0.75;
          nodes(222,0) = -1.;
          nodes(223,0) = -0.25;
          nodes(224,0) = -0.5;
          nodes(225,0) = -0.5;
          nodes(226,0) = -0.25;
          nodes(227,0) = -0.25;
          nodes(228,0) = -0.5;
          nodes(229,0) = -0.75;
          nodes(230,0) = -1.;
          nodes(231,0) = -1.;
          nodes(232,0) = -0.75;
          nodes(233,0) = -0.75;
          nodes(234,0) = -1.;
          nodes(235,0) = -0.75;
          nodes(236,0) = -1.;
          nodes(237,0) = -1.;
          nodes(238,0) = -0.75;
          nodes(239,0) = -0.75;
          nodes(240,0) = -1.;
          nodes(241,0) = -0.75;
          nodes(242,0) = -1.;
          nodes(243,0) = -0.75;
          nodes(244,0) = -1.;
          nodes(245,0) = -0.5;
          nodes(246,0) = -0.25;
          nodes(247,0) = -0.5;
          nodes(248,0) = -0.75;
          nodes(249,0) = -1.;
          nodes(250,0) = -1.;
          nodes(251,0) = -0.75;
          nodes(252,0) = -0.75;
          nodes(253,0) = -1.;
          nodes(254,0) = -1.;
          nodes(255,0) = -0.75;
          nodes(256,0) = -1.;
          nodes(257,0) = -0.75;
          nodes(258,0) = -0.25;
          nodes(259,0) = -0.25;
          nodes(260,0) = -0.25;
          nodes(261,0) = -0.5;
          nodes(262,0) = -0.5;
          nodes(263,0) = -0.75;
          nodes(264,0) = -0.75;
          nodes(265,0) = -0.75;
          nodes(266,0) = -0.5;
          nodes(267,0) = -0.25;
          nodes(268,0) = -0.25;
          nodes(269,0) = -0.25;
          nodes(270,0) = -0.5;
          nodes(271,0) = -0.5;
          nodes(272,0) = -0.25;
          nodes(273,0) = -0.25;
          nodes(274,0) = -0.75;
          nodes(275,0) = -0.25;
          nodes(276,0) = -0.75;
          nodes(277,0) = -0.5;
          nodes(278,0) = -0.5;
          nodes(279,0) = -0.25;
          nodes(280,0) = -0.5;
          nodes(281,0) = -1.;
          nodes(282,0) = -1.;
          nodes(283,0) = -1.;
          nodes(284,0) = -0.75;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = 1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 0.;
          nodes(7,1) = -1.;
          nodes(8,1) = 0.;
          nodes(9,1) = -1.;
          nodes(10,1) = 0.;
          nodes(11,1) = 0.;
          nodes(12,1) = 1.;
          nodes(13,1) = 0.;
          nodes(14,1) = -1.;
          nodes(15,1) = -0.5;
          nodes(16,1) = -1.;
          nodes(17,1) = -0.5;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -0.5;
          nodes(22,1) = -1.;
          nodes(23,1) = -0.5;
          nodes(24,1) = -1.;
          nodes(25,1) = -0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = 0.5;
          nodes(28,1) = 0.;
          nodes(29,1) = 0.5;
          nodes(30,1) = 0.;
          nodes(31,1) = 0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -1.;
          nodes(35,1) = -0.5;
          nodes(36,1) = -1.;
          nodes(37,1) = -0.5;
          nodes(38,1) = -0.5;
          nodes(39,1) = -0.5;
          nodes(40,1) = 0.;
          nodes(41,1) = -0.5;
          nodes(42,1) = 0.5;
          nodes(43,1) = 1.;
          nodes(44,1) = 0.5;
          nodes(45,1) = 0.5;
          nodes(46,1) = 0.5;
          nodes(47,1) = 0.;
          nodes(48,1) = 1.;
          nodes(49,1) = 0.5;
          nodes(50,1) = 0.5;
          nodes(51,1) = -0.5;
          nodes(52,1) = 0.;
          nodes(53,1) = -0.5;
          nodes(54,1) = 0.;
          nodes(55,1) = -1.;
          nodes(56,1) = -0.75;
          nodes(57,1) = -1.;
          nodes(58,1) = -0.75;
          nodes(59,1) = -1.;
          nodes(60,1) = -0.75;
          nodes(61,1) = -1.;
          nodes(62,1) = -0.75;
          nodes(63,1) = -1.;
          nodes(64,1) = -0.75;
          nodes(65,1) = -1.;
          nodes(66,1) = -0.75;
          nodes(67,1) = -0.5;
          nodes(68,1) = -0.25;
          nodes(69,1) = -0.5;
          nodes(70,1) = -0.25;
          nodes(71,1) = -0.5;
          nodes(72,1) = -0.25;
          nodes(73,1) = -1.;
          nodes(74,1) = -0.75;
          nodes(75,1) = -1.;
          nodes(76,1) = -0.75;
          nodes(77,1) = -1.;
          nodes(78,1) = -0.75;
          nodes(79,1) = -0.75;
          nodes(80,1) = -1.;
          nodes(81,1) = -0.75;
          nodes(82,1) = -1.;
          nodes(83,1) = -0.75;
          nodes(84,1) = -1.;
          nodes(85,1) = -0.75;
          nodes(86,1) = -1.;
          nodes(87,1) = -0.75;
          nodes(88,1) = -1.;
          nodes(89,1) = -0.75;
          nodes(90,1) = -1.;
          nodes(91,1) = -0.75;
          nodes(92,1) = -0.5;
          nodes(93,1) = -0.25;
          nodes(94,1) = -0.5;
          nodes(95,1) = -0.25;
          nodes(96,1) = -0.5;
          nodes(97,1) = -0.25;
          nodes(98,1) = -1.;
          nodes(99,1) = -0.75;
          nodes(100,1) = -1.;
          nodes(101,1) = -0.75;
          nodes(102,1) = -1.;
          nodes(103,1) = -0.75;
          nodes(104,1) = -0.75;
          nodes(105,1) = 0.;
          nodes(106,1) = 0.25;
          nodes(107,1) = 0.;
          nodes(108,1) = 0.25;
          nodes(109,1) = 0.;
          nodes(110,1) = 0.25;
          nodes(111,1) = 0.;
          nodes(112,1) = 0.25;
          nodes(113,1) = 0.;
          nodes(114,1) = 0.25;
          nodes(115,1) = 0.;
          nodes(116,1) = 0.25;
          nodes(117,1) = 0.5;
          nodes(118,1) = 0.75;
          nodes(119,1) = 0.5;
          nodes(120,1) = 0.75;
          nodes(121,1) = 0.5;
          nodes(122,1) = 0.75;
          nodes(123,1) = 0.;
          nodes(124,1) = 0.25;
          nodes(125,1) = 0.;
          nodes(126,1) = 0.25;
          nodes(127,1) = 0.;
          nodes(128,1) = 0.25;
          nodes(129,1) = 0.25;
          nodes(130,1) = -1.;
          nodes(131,1) = -0.75;
          nodes(132,1) = -1.;
          nodes(133,1) = -0.75;
          nodes(134,1) = -1.;
          nodes(135,1) = -0.75;
          nodes(136,1) = -1.;
          nodes(137,1) = -0.75;
          nodes(138,1) = -1.;
          nodes(139,1) = -0.75;
          nodes(140,1) = -1.;
          nodes(141,1) = -0.75;
          nodes(142,1) = -0.5;
          nodes(143,1) = -0.25;
          nodes(144,1) = -0.5;
          nodes(145,1) = -0.25;
          nodes(146,1) = -0.5;
          nodes(147,1) = -0.25;
          nodes(148,1) = -1.;
          nodes(149,1) = -0.75;
          nodes(150,1) = -1.;
          nodes(151,1) = -0.75;
          nodes(152,1) = -1.;
          nodes(153,1) = -0.75;
          nodes(154,1) = -0.75;
          nodes(155,1) = -0.75;
          nodes(156,1) = -1.;
          nodes(157,1) = -0.75;
          nodes(158,1) = -1.;
          nodes(159,1) = -0.75;
          nodes(160,1) = -0.75;
          nodes(161,1) = -0.5;
          nodes(162,1) = -0.25;
          nodes(163,1) = -0.5;
          nodes(164,1) = -1.;
          nodes(165,1) = -0.75;
          nodes(166,1) = -0.75;
          nodes(167,1) = -0.5;
          nodes(168,1) = -0.25;
          nodes(169,1) = -0.25;
          nodes(170,1) = -0.25;
          nodes(171,1) = -0.25;
          nodes(172,1) = -0.5;
          nodes(173,1) = -0.5;
          nodes(174,1) = -0.25;
          nodes(175,1) = -0.25;
          nodes(176,1) = -0.25;
          nodes(177,1) = -0.25;
          nodes(178,1) = -0.5;
          nodes(179,1) = -0.5;
          nodes(180,1) = -0.25;
          nodes(181,1) = -0.25;
          nodes(182,1) = -0.25;
          nodes(183,1) = -0.5;
          nodes(184,1) = -0.5;
          nodes(185,1) = -0.75;
          nodes(186,1) = -0.5;
          nodes(187,1) = -0.75;
          nodes(188,1) = -0.25;
          nodes(189,1) = 0.;
          nodes(190,1) = -0.25;
          nodes(191,1) = -0.25;
          nodes(192,1) = -0.25;
          nodes(193,1) = -0.5;
          nodes(194,1) = 0.;
          nodes(195,1) = -0.25;
          nodes(196,1) = -0.25;
          nodes(197,1) = -0.75;
          nodes(198,1) = -0.5;
          nodes(199,1) = -0.75;
          nodes(200,1) = -0.5;
          nodes(201,1) = 0.25;
          nodes(202,1) = 0.5;
          nodes(203,1) = 0.25;
          nodes(204,1) = 0.75;
          nodes(205,1) = 1.;
          nodes(206,1) = 0.75;
          nodes(207,1) = 0.75;
          nodes(208,1) = 0.75;
          nodes(209,1) = 0.5;
          nodes(210,1) = 1.;
          nodes(211,1) = 0.75;
          nodes(212,1) = 0.75;
          nodes(213,1) = 0.25;
          nodes(214,1) = 0.5;
          nodes(215,1) = 0.25;
          nodes(216,1) = 0.5;
          nodes(217,1) = 0.25;
          nodes(218,1) = 0.25;
          nodes(219,1) = 0.;
          nodes(220,1) = 0.5;
          nodes(221,1) = 0.25;
          nodes(222,1) = 0.25;
          nodes(223,1) = 0.75;
          nodes(224,1) = 0.75;
          nodes(225,1) = 0.5;
          nodes(226,1) = 1.;
          nodes(227,1) = 0.75;
          nodes(228,1) = 0.75;
          nodes(229,1) = 0.75;
          nodes(230,1) = 0.75;
          nodes(231,1) = 0.5;
          nodes(232,1) = 1.;
          nodes(233,1) = 0.75;
          nodes(234,1) = 0.75;
          nodes(235,1) = 0.25;
          nodes(236,1) = 0.25;
          nodes(237,1) = 0.;
          nodes(238,1) = 0.5;
          nodes(239,1) = 0.25;
          nodes(240,1) = 0.25;
          nodes(241,1) = 0.5;
          nodes(242,1) = -0.75;
          nodes(243,1) = -0.5;
          nodes(244,1) = -0.75;
          nodes(245,1) = -0.25;
          nodes(246,1) = 0.;
          nodes(247,1) = -0.25;
          nodes(248,1) = -0.25;
          nodes(249,1) = -0.25;
          nodes(250,1) = -0.5;
          nodes(251,1) = 0.;
          nodes(252,1) = -0.25;
          nodes(253,1) = -0.25;
          nodes(254,1) = -0.75;
          nodes(255,1) = -0.5;
          nodes(256,1) = -0.75;
          nodes(257,1) = -0.5;
          nodes(258,1) = 0.;
          nodes(259,1) = 0.;
          nodes(260,1) = 0.;
          nodes(261,1) = -0.25;
          nodes(262,1) = 0.;
          nodes(263,1) = 0.;
          nodes(264,1) = -0.25;
          nodes(265,1) = -0.5;
          nodes(266,1) = -0.25;
          nodes(267,1) = 0.25;
          nodes(268,1) = 0.5;
          nodes(269,1) = 0.5;
          nodes(270,1) = 0.25;
          nodes(271,1) = 0.25;
          nodes(272,1) = 0.25;
          nodes(273,1) = 0.25;
          nodes(274,1) = 0.;
          nodes(275,1) = 0.5;
          nodes(276,1) = 0.;
          nodes(277,1) = 0.25;
          nodes(278,1) = 0.;
          nodes(279,1) = 0.25;
          nodes(280,1) = 0.25;
          nodes(281,1) = -0.25;
          nodes(282,1) = -0.25;
          nodes(283,1) = -0.5;
          nodes(284,1) = -0.25;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = -1.;
          nodes(7,2) = 0.;
          nodes(8,2) = -1.;
          nodes(9,2) = 0.;
          nodes(10,2) = 0.;
          nodes(11,2) = -1.;
          nodes(12,2) = -1.;
          nodes(13,2) = 0.;
          nodes(14,2) = -1.;
          nodes(15,2) = -1.;
          nodes(16,2) = -0.5;
          nodes(17,2) = -1.;
          nodes(18,2) = -0.5;
          nodes(19,2) = -0.5;
          nodes(20,2) = -1.;
          nodes(21,2) = -1.;
          nodes(22,2) = -0.5;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -0.5;
          nodes(26,2) = -1.;
          nodes(27,2) = -1.;
          nodes(28,2) = -0.5;
          nodes(29,2) = -1.;
          nodes(30,2) = -0.5;
          nodes(31,2) = -0.5;
          nodes(32,2) = 0.;
          nodes(33,2) = 0.;
          nodes(34,2) = 0.5;
          nodes(35,2) = 0.;
          nodes(36,2) = 0.5;
          nodes(37,2) = 0.5;
          nodes(38,2) = -0.5;
          nodes(39,2) = -1.;
          nodes(40,2) = -1.;
          nodes(41,2) = -0.5;
          nodes(42,2) = -1.;
          nodes(43,2) = -1.;
          nodes(44,2) = -0.5;
          nodes(45,2) = -1.;
          nodes(46,2) = -1.;
          nodes(47,2) = -0.5;
          nodes(48,2) = -1.;
          nodes(49,2) = -0.5;
          nodes(50,2) = -0.5;
          nodes(51,2) = 0.;
          nodes(52,2) = 0.;
          nodes(53,2) = 0.5;
          nodes(54,2) = -0.5;
          nodes(55,2) = -1.;
          nodes(56,2) = -1.;
          nodes(57,2) = -0.75;
          nodes(58,2) = -1.;
          nodes(59,2) = -0.75;
          nodes(60,2) = -0.75;
          nodes(61,2) = -1.;
          nodes(62,2) = -1.;
          nodes(63,2) = -0.75;
          nodes(64,2) = -1.;
          nodes(65,2) = -0.75;
          nodes(66,2) = -0.75;
          nodes(67,2) = -1.;
          nodes(68,2) = -1.;
          nodes(69,2) = -0.75;
          nodes(70,2) = -1.;
          nodes(71,2) = -0.75;
          nodes(72,2) = -0.75;
          nodes(73,2) = -0.5;
          nodes(74,2) = -0.5;
          nodes(75,2) = -0.25;
          nodes(76,2) = -0.5;
          nodes(77,2) = -0.25;
          nodes(78,2) = -0.25;
          nodes(79,2) = -0.75;
          nodes(80,2) = -1.;
          nodes(81,2) = -1.;
          nodes(82,2) = -0.75;
          nodes(83,2) = -1.;
          nodes(84,2) = -0.75;
          nodes(85,2) = -0.75;
          nodes(86,2) = -1.;
          nodes(87,2) = -1.;
          nodes(88,2) = -0.75;
          nodes(89,2) = -1.;
          nodes(90,2) = -0.75;
          nodes(91,2) = -0.75;
          nodes(92,2) = -1.;
          nodes(93,2) = -1.;
          nodes(94,2) = -0.75;
          nodes(95,2) = -1.;
          nodes(96,2) = -0.75;
          nodes(97,2) = -0.75;
          nodes(98,2) = -0.5;
          nodes(99,2) = -0.5;
          nodes(100,2) = -0.25;
          nodes(101,2) = -0.5;
          nodes(102,2) = -0.25;
          nodes(103,2) = -0.25;
          nodes(104,2) = -0.75;
          nodes(105,2) = -1.;
          nodes(106,2) = -1.;
          nodes(107,2) = -0.75;
          nodes(108,2) = -1.;
          nodes(109,2) = -0.75;
          nodes(110,2) = -0.75;
          nodes(111,2) = -1.;
          nodes(112,2) = -1.;
          nodes(113,2) = -0.75;
          nodes(114,2) = -1.;
          nodes(115,2) = -0.75;
          nodes(116,2) = -0.75;
          nodes(117,2) = -1.;
          nodes(118,2) = -1.;
          nodes(119,2) = -0.75;
          nodes(120,2) = -1.;
          nodes(121,2) = -0.75;
          nodes(122,2) = -0.75;
          nodes(123,2) = -0.5;
          nodes(124,2) = -0.5;
          nodes(125,2) = -0.25;
          nodes(126,2) = -0.5;
          nodes(127,2) = -0.25;
          nodes(128,2) = -0.25;
          nodes(129,2) = -0.75;
          nodes(130,2) = 0.;
          nodes(131,2) = 0.;
          nodes(132,2) = 0.25;
          nodes(133,2) = 0.;
          nodes(134,2) = 0.25;
          nodes(135,2) = 0.25;
          nodes(136,2) = 0.;
          nodes(137,2) = 0.;
          nodes(138,2) = 0.25;
          nodes(139,2) = 0.;
          nodes(140,2) = 0.25;
          nodes(141,2) = 0.25;
          nodes(142,2) = 0.;
          nodes(143,2) = 0.;
          nodes(144,2) = 0.25;
          nodes(145,2) = 0.;
          nodes(146,2) = 0.25;
          nodes(147,2) = 0.25;
          nodes(148,2) = 0.5;
          nodes(149,2) = 0.5;
          nodes(150,2) = 0.75;
          nodes(151,2) = 0.5;
          nodes(152,2) = 0.75;
          nodes(153,2) = 0.75;
          nodes(154,2) = 0.25;
          nodes(155,2) = -0.75;
          nodes(156,2) = -0.5;
          nodes(157,2) = -0.5;
          nodes(158,2) = -0.25;
          nodes(159,2) = -0.25;
          nodes(160,2) = -0.25;
          nodes(161,2) = -0.75;
          nodes(162,2) = -0.75;
          nodes(163,2) = -0.5;
          nodes(164,2) = -0.25;
          nodes(165,2) = -0.25;
          nodes(166,2) = -0.5;
          nodes(167,2) = -1.;
          nodes(168,2) = -1.;
          nodes(169,2) = -0.75;
          nodes(170,2) = -1.;
          nodes(171,2) = -0.75;
          nodes(172,2) = -0.5;
          nodes(173,2) = -0.75;
          nodes(174,2) = -0.5;
          nodes(175,2) = -0.5;
          nodes(176,2) = -0.25;
          nodes(177,2) = -0.25;
          nodes(178,2) = -0.25;
          nodes(179,2) = -0.25;
          nodes(180,2) = -0.5;
          nodes(181,2) = -0.5;
          nodes(182,2) = -0.25;
          nodes(183,2) = -0.25;
          nodes(184,2) = -0.25;
          nodes(185,2) = -1.;
          nodes(186,2) = -1.;
          nodes(187,2) = -0.75;
          nodes(188,2) = -1.;
          nodes(189,2) = -1.;
          nodes(190,2) = -0.75;
          nodes(191,2) = -1.;
          nodes(192,2) = -1.;
          nodes(193,2) = -0.75;
          nodes(194,2) = -1.;
          nodes(195,2) = -0.75;
          nodes(196,2) = -0.75;
          nodes(197,2) = -0.5;
          nodes(198,2) = -0.5;
          nodes(199,2) = -0.25;
          nodes(200,2) = -0.75;
          nodes(201,2) = -1.;
          nodes(202,2) = -1.;
          nodes(203,2) = -0.75;
          nodes(204,2) = -1.;
          nodes(205,2) = -1.;
          nodes(206,2) = -0.75;
          nodes(207,2) = -1.;
          nodes(208,2) = -1.;
          nodes(209,2) = -0.75;
          nodes(210,2) = -1.;
          nodes(211,2) = -0.75;
          nodes(212,2) = -0.75;
          nodes(213,2) = -0.5;
          nodes(214,2) = -0.5;
          nodes(215,2) = -0.25;
          nodes(216,2) = -0.75;
          nodes(217,2) = -1.;
          nodes(218,2) = -1.;
          nodes(219,2) = -0.75;
          nodes(220,2) = -1.;
          nodes(221,2) = -0.75;
          nodes(222,2) = -0.75;
          nodes(223,2) = -1.;
          nodes(224,2) = -1.;
          nodes(225,2) = -0.75;
          nodes(226,2) = -1.;
          nodes(227,2) = -0.75;
          nodes(228,2) = -0.75;
          nodes(229,2) = -1.;
          nodes(230,2) = -1.;
          nodes(231,2) = -0.75;
          nodes(232,2) = -1.;
          nodes(233,2) = -0.75;
          nodes(234,2) = -0.75;
          nodes(235,2) = -0.5;
          nodes(236,2) = -0.5;
          nodes(237,2) = -0.25;
          nodes(238,2) = -0.5;
          nodes(239,2) = -0.25;
          nodes(240,2) = -0.25;
          nodes(241,2) = -0.75;
          nodes(242,2) = 0.;
          nodes(243,2) = 0.;
          nodes(244,2) = 0.25;
          nodes(245,2) = 0.;
          nodes(246,2) = 0.;
          nodes(247,2) = 0.25;
          nodes(248,2) = 0.;
          nodes(249,2) = 0.;
          nodes(250,2) = 0.25;
          nodes(251,2) = 0.;
          nodes(252,2) = 0.25;
          nodes(253,2) = 0.25;
          nodes(254,2) = 0.5;
          nodes(255,2) = 0.5;
          nodes(256,2) = 0.75;
          nodes(257,2) = 0.25;
          nodes(258,2) = -0.75;
          nodes(259,2) = -0.5;
          nodes(260,2) = -0.25;
          nodes(261,2) = -0.25;
          nodes(262,2) = -0.75;
          nodes(263,2) = -0.75;
          nodes(264,2) = -0.5;
          nodes(265,2) = -0.25;
          nodes(266,2) = -0.5;
          nodes(267,2) = -1.;
          nodes(268,2) = -1.;
          nodes(269,2) = -0.75;
          nodes(270,2) = -1.;
          nodes(271,2) = -0.75;
          nodes(272,2) = -0.5;
          nodes(273,2) = -0.75;
          nodes(274,2) = -0.5;
          nodes(275,2) = -0.5;
          nodes(276,2) = -0.25;
          nodes(277,2) = -0.25;
          nodes(278,2) = -0.25;
          nodes(279,2) = -0.25;
          nodes(280,2) = -0.5;
          nodes(281,2) = -0.5;
          nodes(282,2) = -0.25;
          nodes(283,2) = -0.25;
          nodes(284,2) = -0.25;
          return;
        }
      }
    } 
    case TET4:
    case TET10:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(4,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          return;
        }
        case 2:
        {
          nodes.resize(10,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 0.;
          nodes(5,0) = -1.;
          nodes(6,0) = -1.;
          nodes(7,0) = 0.;
          nodes(8,0) = 0.;
          nodes(9,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 0.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = 0.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          return;
        }
        case 3:
        case 4:
        {
          nodes.resize(35,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 0.;
          nodes(5,0) = -1.;
          nodes(6,0) = -1.;
          nodes(7,0) = 0.;
          nodes(8,0) = 0.;
          nodes(9,0) = -1.;
          nodes(10,0) = -0.5;
          nodes(11,0) = -1.;
          nodes(12,0) = -1.;
          nodes(13,0) = -0.5;
          nodes(14,0) = -0.5;
          nodes(15,0) = -1.;
          nodes(16,0) = 0.5;
          nodes(17,0) = 0.;
          nodes(18,0) = 0.;
          nodes(19,0) = 0.5;
          nodes(20,0) = 0.5;
          nodes(21,0) = 0.;
          nodes(22,0) = -0.5;
          nodes(23,0) = -1.;
          nodes(24,0) = -1.;
          nodes(25,0) = -0.5;
          nodes(26,0) = -0.5;
          nodes(27,0) = -1.;
          nodes(28,0) = -0.5;
          nodes(29,0) = -1.;
          nodes(30,0) = -1.;
          nodes(31,0) = -0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -1.;
          nodes(34,0) = -0.5;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 0.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(10,1) = -1.;
          nodes(11,1) = -0.5;
          nodes(12,1) = -1.;
          nodes(13,1) = -0.5;
          nodes(14,1) = -1.;
          nodes(15,1) = -0.5;
          nodes(16,1) = -1.;
          nodes(17,1) = -0.5;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -0.5;
          nodes(22,1) = 0.;
          nodes(23,1) = 0.5;
          nodes(24,1) = 0.;
          nodes(25,1) = 0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = 0.5;
          nodes(28,1) = -1.;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = 0.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          nodes(10,2) = -1.;
          nodes(11,2) = -1.;
          nodes(12,2) = -0.5;
          nodes(13,2) = -1.;
          nodes(14,2) = -0.5;
          nodes(15,2) = -0.5;
          nodes(16,2) = -1.;
          nodes(17,2) = -1.;
          nodes(18,2) = -0.5;
          nodes(19,2) = -1.;
          nodes(20,2) = -0.5;
          nodes(21,2) = -0.5;
          nodes(22,2) = -1.;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -1.;
          nodes(26,2) = -0.5;
          nodes(27,2) = -0.5;
          nodes(28,2) = 0.;
          nodes(29,2) = 0.;
          nodes(30,2) = 0.5;
          nodes(31,2) = 0.;
          nodes(32,2) = 0.5;
          nodes(33,2) = 0.5;
          nodes(34,2) = -0.5;
          return;
        }
        default:
        {
          nodes.resize(165,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = -1.;
          nodes(3,0) = -1.;
          nodes(4,0) = 0.;
          nodes(5,0) = -1.;
          nodes(6,0) = -1.;
          nodes(7,0) = 0.;
          nodes(8,0) = 0.;
          nodes(9,0) = -1.;
          nodes(10,0) = -0.5;
          nodes(11,0) = -1.;
          nodes(12,0) = -1.;
          nodes(13,0) = -0.5;
          nodes(14,0) = -0.5;
          nodes(15,0) = -1.;
          nodes(16,0) = 0.5;
          nodes(17,0) = 0.;
          nodes(18,0) = 0.;
          nodes(19,0) = 0.5;
          nodes(20,0) = 0.5;
          nodes(21,0) = 0.;
          nodes(22,0) = -0.5;
          nodes(23,0) = -1.;
          nodes(24,0) = -1.;
          nodes(25,0) = -0.5;
          nodes(26,0) = -0.5;
          nodes(27,0) = -1.;
          nodes(28,0) = -0.5;
          nodes(29,0) = -1.;
          nodes(30,0) = -1.;
          nodes(31,0) = -0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -1.;
          nodes(34,0) = -0.5;
          nodes(35,0) = -0.75;
          nodes(36,0) = -1.;
          nodes(37,0) = -1.;
          nodes(38,0) = -0.75;
          nodes(39,0) = -0.75;
          nodes(40,0) = -1.;
          nodes(41,0) = -0.25;
          nodes(42,0) = -0.5;
          nodes(43,0) = -0.5;
          nodes(44,0) = -0.25;
          nodes(45,0) = -0.25;
          nodes(46,0) = -0.5;
          nodes(47,0) = -0.75;
          nodes(48,0) = -1.;
          nodes(49,0) = -1.;
          nodes(50,0) = -0.75;
          nodes(51,0) = -0.75;
          nodes(52,0) = -1.;
          nodes(53,0) = -0.75;
          nodes(54,0) = -1.;
          nodes(55,0) = -1.;
          nodes(56,0) = -0.75;
          nodes(57,0) = -0.75;
          nodes(58,0) = -1.;
          nodes(59,0) = -0.75;
          nodes(60,0) = 0.25;
          nodes(61,0) = 0.;
          nodes(62,0) = 0.;
          nodes(63,0) = 0.25;
          nodes(64,0) = 0.25;
          nodes(65,0) = 0.;
          nodes(66,0) = 0.75;
          nodes(67,0) = 0.5;
          nodes(68,0) = 0.5;
          nodes(69,0) = 0.75;
          nodes(70,0) = 0.75;
          nodes(71,0) = 0.5;
          nodes(72,0) = 0.25;
          nodes(73,0) = 0.;
          nodes(74,0) = 0.;
          nodes(75,0) = 0.25;
          nodes(76,0) = 0.25;
          nodes(77,0) = 0.;
          nodes(78,0) = 0.25;
          nodes(79,0) = 0.;
          nodes(80,0) = 0.;
          nodes(81,0) = 0.25;
          nodes(82,0) = 0.25;
          nodes(83,0) = 0.;
          nodes(84,0) = 0.25;
          nodes(85,0) = -0.75;
          nodes(86,0) = -1.;
          nodes(87,0) = -1.;
          nodes(88,0) = -0.75;
          nodes(89,0) = -0.75;
          nodes(90,0) = -1.;
          nodes(91,0) = -0.25;
          nodes(92,0) = -0.5;
          nodes(93,0) = -0.5;
          nodes(94,0) = -0.25;
          nodes(95,0) = -0.25;
          nodes(96,0) = -0.5;
          nodes(97,0) = -0.75;
          nodes(98,0) = -1.;
          nodes(99,0) = -1.;
          nodes(100,0) = -0.75;
          nodes(101,0) = -0.75;
          nodes(102,0) = -1.;
          nodes(103,0) = -0.75;
          nodes(104,0) = -1.;
          nodes(105,0) = -1.;
          nodes(106,0) = -0.75;
          nodes(107,0) = -0.75;
          nodes(108,0) = -1.;
          nodes(109,0) = -0.75;
          nodes(110,0) = -0.75;
          nodes(111,0) = -1.;
          nodes(112,0) = -1.;
          nodes(113,0) = -0.75;
          nodes(114,0) = -0.75;
          nodes(115,0) = -1.;
          nodes(116,0) = -0.25;
          nodes(117,0) = -0.5;
          nodes(118,0) = -0.5;
          nodes(119,0) = -0.25;
          nodes(120,0) = -0.25;
          nodes(121,0) = -0.5;
          nodes(122,0) = -0.75;
          nodes(123,0) = -1.;
          nodes(124,0) = -1.;
          nodes(125,0) = -0.75;
          nodes(126,0) = -0.75;
          nodes(127,0) = -1.;
          nodes(128,0) = -0.75;
          nodes(129,0) = -1.;
          nodes(130,0) = -1.;
          nodes(131,0) = -0.75;
          nodes(132,0) = -0.75;
          nodes(133,0) = -1.;
          nodes(134,0) = -0.75;
          nodes(135,0) = -0.25;
          nodes(136,0) = -0.25;
          nodes(137,0) = -0.25;
          nodes(138,0) = -0.25;
          nodes(139,0) = -0.25;
          nodes(140,0) = -0.5;
          nodes(141,0) = -0.5;
          nodes(142,0) = -0.75;
          nodes(143,0) = -0.75;
          nodes(144,0) = -0.5;
          nodes(145,0) = -0.75;
          nodes(146,0) = -0.5;
          nodes(147,0) = -0.25;
          nodes(148,0) = -0.25;
          nodes(149,0) = -0.25;
          nodes(150,0) = -0.5;
          nodes(151,0) = -0.5;
          nodes(152,0) = -0.25;
          nodes(153,0) = -0.25;
          nodes(154,0) = -0.75;
          nodes(155,0) = -0.25;
          nodes(156,0) = -0.75;
          nodes(157,0) = -0.5;
          nodes(158,0) = -0.5;
          nodes(159,0) = -0.25;
          nodes(160,0) = -0.5;
          nodes(161,0) = -1.;
          nodes(162,0) = -1.;
          nodes(163,0) = -1.;
          nodes(164,0) = -0.75;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = -1.;
          nodes(4,1) = -1.;
          nodes(5,1) = 0.;
          nodes(6,1) = -1.;
          nodes(7,1) = 0.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(10,1) = -1.;
          nodes(11,1) = -0.5;
          nodes(12,1) = -1.;
          nodes(13,1) = -0.5;
          nodes(14,1) = -1.;
          nodes(15,1) = -0.5;
          nodes(16,1) = -1.;
          nodes(17,1) = -0.5;
          nodes(18,1) = -1.;
          nodes(19,1) = -0.5;
          nodes(20,1) = -1.;
          nodes(21,1) = -0.5;
          nodes(22,1) = 0.;
          nodes(23,1) = 0.5;
          nodes(24,1) = 0.;
          nodes(25,1) = 0.5;
          nodes(26,1) = 0.;
          nodes(27,1) = 0.5;
          nodes(28,1) = -1.;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -0.5;
          nodes(32,1) = -1.;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(35,1) = -1.;
          nodes(36,1) = -0.75;
          nodes(37,1) = -1.;
          nodes(38,1) = -0.75;
          nodes(39,1) = -1.;
          nodes(40,1) = -0.75;
          nodes(41,1) = -1.;
          nodes(42,1) = -0.75;
          nodes(43,1) = -1.;
          nodes(44,1) = -0.75;
          nodes(45,1) = -1.;
          nodes(46,1) = -0.75;
          nodes(47,1) = -0.5;
          nodes(48,1) = -0.25;
          nodes(49,1) = -0.5;
          nodes(50,1) = -0.25;
          nodes(51,1) = -0.5;
          nodes(52,1) = -0.25;
          nodes(53,1) = -1.;
          nodes(54,1) = -0.75;
          nodes(55,1) = -1.;
          nodes(56,1) = -0.75;
          nodes(57,1) = -1.;
          nodes(58,1) = -0.75;
          nodes(59,1) = -0.75;
          nodes(60,1) = -1.;
          nodes(61,1) = -0.75;
          nodes(62,1) = -1.;
          nodes(63,1) = -0.75;
          nodes(64,1) = -1.;
          nodes(65,1) = -0.75;
          nodes(66,1) = -1.;
          nodes(67,1) = -0.75;
          nodes(68,1) = -1.;
          nodes(69,1) = -0.75;
          nodes(70,1) = -1.;
          nodes(71,1) = -0.75;
          nodes(72,1) = -0.5;
          nodes(73,1) = -0.25;
          nodes(74,1) = -0.5;
          nodes(75,1) = -0.25;
          nodes(76,1) = -0.5;
          nodes(77,1) = -0.25;
          nodes(78,1) = -1.;
          nodes(79,1) = -0.75;
          nodes(80,1) = -1.;
          nodes(81,1) = -0.75;
          nodes(82,1) = -1.;
          nodes(83,1) = -0.75;
          nodes(84,1) = -0.75;
          nodes(85,1) = 0.;
          nodes(86,1) = 0.25;
          nodes(87,1) = 0.;
          nodes(88,1) = 0.25;
          nodes(89,1) = 0.;
          nodes(90,1) = 0.25;
          nodes(91,1) = 0.;
          nodes(92,1) = 0.25;
          nodes(93,1) = 0.;
          nodes(94,1) = 0.25;
          nodes(95,1) = 0.;
          nodes(96,1) = 0.25;
          nodes(97,1) = 0.5;
          nodes(98,1) = 0.75;
          nodes(99,1) = 0.5;
          nodes(100,1) = 0.75;
          nodes(101,1) = 0.5;
          nodes(102,1) = 0.75;
          nodes(103,1) = 0.;
          nodes(104,1) = 0.25;
          nodes(105,1) = 0.;
          nodes(106,1) = 0.25;
          nodes(107,1) = 0.;
          nodes(108,1) = 0.25;
          nodes(109,1) = 0.25;
          nodes(110,1) = -1.;
          nodes(111,1) = -0.75;
          nodes(112,1) = -1.;
          nodes(113,1) = -0.75;
          nodes(114,1) = -1.;
          nodes(115,1) = -0.75;
          nodes(116,1) = -1.;
          nodes(117,1) = -0.75;
          nodes(118,1) = -1.;
          nodes(119,1) = -0.75;
          nodes(120,1) = -1.;
          nodes(121,1) = -0.75;
          nodes(122,1) = -0.5;
          nodes(123,1) = -0.25;
          nodes(124,1) = -0.5;
          nodes(125,1) = -0.25;
          nodes(126,1) = -0.5;
          nodes(127,1) = -0.25;
          nodes(128,1) = -1.;
          nodes(129,1) = -0.75;
          nodes(130,1) = -1.;
          nodes(131,1) = -0.75;
          nodes(132,1) = -1.;
          nodes(133,1) = -0.75;
          nodes(134,1) = -0.75;
          nodes(135,1) = -0.75;
          nodes(136,1) = -1.;
          nodes(137,1) = -0.75;
          nodes(138,1) = -1.;
          nodes(139,1) = -0.75;
          nodes(140,1) = -0.75;
          nodes(141,1) = -0.5;
          nodes(142,1) = -0.25;
          nodes(143,1) = -0.5;
          nodes(144,1) = -1.;
          nodes(145,1) = -0.75;
          nodes(146,1) = -0.75;
          nodes(147,1) = -0.5;
          nodes(148,1) = -0.25;
          nodes(149,1) = -0.25;
          nodes(150,1) = -0.25;
          nodes(151,1) = -0.25;
          nodes(152,1) = -0.5;
          nodes(153,1) = -0.5;
          nodes(154,1) = -0.25;
          nodes(155,1) = -0.25;
          nodes(156,1) = -0.25;
          nodes(157,1) = -0.25;
          nodes(158,1) = -0.5;
          nodes(159,1) = -0.5;
          nodes(160,1) = -0.25;
          nodes(161,1) = -0.25;
          nodes(162,1) = -0.25;
          nodes(163,1) = -0.5;
          nodes(164,1) = -0.5;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = 1.;
          nodes(4,2) = -1.;
          nodes(5,2) = -1.;
          nodes(6,2) = 0.;
          nodes(7,2) = -1.;
          nodes(8,2) = 0.;
          nodes(9,2) = 0.;
          nodes(10,2) = -1.;
          nodes(11,2) = -1.;
          nodes(12,2) = -0.5;
          nodes(13,2) = -1.;
          nodes(14,2) = -0.5;
          nodes(15,2) = -0.5;
          nodes(16,2) = -1.;
          nodes(17,2) = -1.;
          nodes(18,2) = -0.5;
          nodes(19,2) = -1.;
          nodes(20,2) = -0.5;
          nodes(21,2) = -0.5;
          nodes(22,2) = -1.;
          nodes(23,2) = -1.;
          nodes(24,2) = -0.5;
          nodes(25,2) = -1.;
          nodes(26,2) = -0.5;
          nodes(27,2) = -0.5;
          nodes(28,2) = 0.;
          nodes(29,2) = 0.;
          nodes(30,2) = 0.5;
          nodes(31,2) = 0.;
          nodes(32,2) = 0.5;
          nodes(33,2) = 0.5;
          nodes(34,2) = -0.5;
          nodes(35,2) = -1.;
          nodes(36,2) = -1.;
          nodes(37,2) = -0.75;
          nodes(38,2) = -1.;
          nodes(39,2) = -0.75;
          nodes(40,2) = -0.75;
          nodes(41,2) = -1.;
          nodes(42,2) = -1.;
          nodes(43,2) = -0.75;
          nodes(44,2) = -1.;
          nodes(45,2) = -0.75;
          nodes(46,2) = -0.75;
          nodes(47,2) = -1.;
          nodes(48,2) = -1.;
          nodes(49,2) = -0.75;
          nodes(50,2) = -1.;
          nodes(51,2) = -0.75;
          nodes(52,2) = -0.75;
          nodes(53,2) = -0.5;
          nodes(54,2) = -0.5;
          nodes(55,2) = -0.25;
          nodes(56,2) = -0.5;
          nodes(57,2) = -0.25;
          nodes(58,2) = -0.25;
          nodes(59,2) = -0.75;
          nodes(60,2) = -1.;
          nodes(61,2) = -1.;
          nodes(62,2) = -0.75;
          nodes(63,2) = -1.;
          nodes(64,2) = -0.75;
          nodes(65,2) = -0.75;
          nodes(66,2) = -1.;
          nodes(67,2) = -1.;
          nodes(68,2) = -0.75;
          nodes(69,2) = -1.;
          nodes(70,2) = -0.75;
          nodes(71,2) = -0.75;
          nodes(72,2) = -1.;
          nodes(73,2) = -1.;
          nodes(74,2) = -0.75;
          nodes(75,2) = -1.;
          nodes(76,2) = -0.75;
          nodes(77,2) = -0.75;
          nodes(78,2) = -0.5;
          nodes(79,2) = -0.5;
          nodes(80,2) = -0.25;
          nodes(81,2) = -0.5;
          nodes(82,2) = -0.25;
          nodes(83,2) = -0.25;
          nodes(84,2) = -0.75;
          nodes(85,2) = -1.;
          nodes(86,2) = -1.;
          nodes(87,2) = -0.75;
          nodes(88,2) = -1.;
          nodes(89,2) = -0.75;
          nodes(90,2) = -0.75;
          nodes(91,2) = -1.;
          nodes(92,2) = -1.;
          nodes(93,2) = -0.75;
          nodes(94,2) = -1.;
          nodes(95,2) = -0.75;
          nodes(96,2) = -0.75;
          nodes(97,2) = -1.;
          nodes(98,2) = -1.;
          nodes(99,2) = -0.75;
          nodes(100,2) = -1.;
          nodes(101,2) = -0.75;
          nodes(102,2) = -0.75;
          nodes(103,2) = -0.5;
          nodes(104,2) = -0.5;
          nodes(105,2) = -0.25;
          nodes(106,2) = -0.5;
          nodes(107,2) = -0.25;
          nodes(108,2) = -0.25;
          nodes(109,2) = -0.75;
          nodes(110,2) = 0.;
          nodes(111,2) = 0.;
          nodes(112,2) = 0.25;
          nodes(113,2) = 0.;
          nodes(114,2) = 0.25;
          nodes(115,2) = 0.25;
          nodes(116,2) = 0.;
          nodes(117,2) = 0.;
          nodes(118,2) = 0.25;
          nodes(119,2) = 0.;
          nodes(120,2) = 0.25;
          nodes(121,2) = 0.25;
          nodes(122,2) = 0.;
          nodes(123,2) = 0.;
          nodes(124,2) = 0.25;
          nodes(125,2) = 0.;
          nodes(126,2) = 0.25;
          nodes(127,2) = 0.25;
          nodes(128,2) = 0.5;
          nodes(129,2) = 0.5;
          nodes(130,2) = 0.75;
          nodes(131,2) = 0.5;
          nodes(132,2) = 0.75;
          nodes(133,2) = 0.75;
          nodes(134,2) = 0.25;
          nodes(135,2) = -0.75;
          nodes(136,2) = -0.5;
          nodes(137,2) = -0.5;
          nodes(138,2) = -0.25;
          nodes(139,2) = -0.25;
          nodes(140,2) = -0.25;
          nodes(141,2) = -0.75;
          nodes(142,2) = -0.75;
          nodes(143,2) = -0.5;
          nodes(144,2) = -0.25;
          nodes(145,2) = -0.25;
          nodes(146,2) = -0.5;
          nodes(147,2) = -1.;
          nodes(148,2) = -1.;
          nodes(149,2) = -0.75;
          nodes(150,2) = -1.;
          nodes(151,2) = -0.75;
          nodes(152,2) = -0.5;
          nodes(153,2) = -0.75;
          nodes(154,2) = -0.5;
          nodes(155,2) = -0.5;
          nodes(156,2) = -0.25;
          nodes(157,2) = -0.25;
          nodes(158,2) = -0.25;
          nodes(159,2) = -0.25;
          nodes(160,2) = -0.5;
          nodes(161,2) = -0.5;
          nodes(162,2) = -0.25;
          nodes(163,2) = -0.25;
          nodes(164,2) = -0.25;
          return;
        }
      }
    } 
    case HEX8:
    case HEX20:
    case HEX27:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          nodes.resize(8,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 1.;
          nodes(6,0) = 1.;
          nodes(7,0) = -1.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = 1.;
          nodes(4,1) = -1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 1.;
          nodes(7,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = -1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = 1.;
          nodes(7,2) = 1.;
          return;
        }
        case 2:
        {
          nodes.resize(27,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 1.;
          nodes(6,0) = 1.;
          nodes(7,0) = -1.;
          nodes(8,0) = 0.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 1.;
          nodes(16,0) = 1.;
          nodes(17,0) = 1.;
          nodes(18,0) = 0.;
          nodes(19,0) = 0.;
          nodes(20,0) = -1.;
          nodes(21,0) = 1.;
          nodes(22,0) = 0.;
          nodes(23,0) = 0.;
          nodes(24,0) = -1.;
          nodes(25,0) = 1.;
          nodes(26,0) = 0.;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = 1.;
          nodes(4,1) = -1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 1.;
          nodes(7,1) = 1.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(10,1) = 0.;
          nodes(11,1) = -1.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 0.;
          nodes(15,1) = 0.;
          nodes(16,1) = -1.;
          nodes(17,1) = 0.;
          nodes(18,1) = 1.;
          nodes(19,1) = 1.;
          nodes(20,1) = 1.;
          nodes(21,1) = 1.;
          nodes(22,1) = -1.;
          nodes(23,1) = 0.;
          nodes(24,1) = 0.;
          nodes(25,1) = 0.;
          nodes(26,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = -1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = 1.;
          nodes(7,2) = 1.;
          nodes(8,2) = -1.;
          nodes(9,2) = -1.;
          nodes(10,2) = -1.;
          nodes(11,2) = 0.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = -1.;
          nodes(16,2) = 0.;
          nodes(17,2) = 0.;
          nodes(18,2) = -1.;
          nodes(19,2) = 0.;
          nodes(20,2) = 0.;
          nodes(21,2) = 0.;
          nodes(22,2) = 1.;
          nodes(23,2) = 1.;
          nodes(24,2) = 1.;
          nodes(25,2) = 1.;
          nodes(26,2) = 1.;
          return;
        }
        case 3:
        case 4:
        {
          nodes.resize(125,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 1.;
          nodes(6,0) = 1.;
          nodes(7,0) = -1.;
          nodes(8,0) = 0.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 1.;
          nodes(16,0) = 1.;
          nodes(17,0) = 1.;
          nodes(18,0) = 0.;
          nodes(19,0) = 0.;
          nodes(20,0) = -1.;
          nodes(21,0) = 1.;
          nodes(22,0) = 0.;
          nodes(23,0) = 0.;
          nodes(24,0) = -1.;
          nodes(25,0) = 1.;
          nodes(26,0) = 0.;
          nodes(27,0) = -0.5;
          nodes(28,0) = -0.5;
          nodes(29,0) = -1.;
          nodes(30,0) = -1.;
          nodes(31,0) = -0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -1.;
          nodes(34,0) = 0.;
          nodes(35,0) = 0.;
          nodes(36,0) = 0.;
          nodes(37,0) = -0.5;
          nodes(38,0) = -0.5;
          nodes(39,0) = -1.;
          nodes(40,0) = 0.;
          nodes(41,0) = -0.5;
          nodes(42,0) = -0.5;
          nodes(43,0) = -1.;
          nodes(44,0) = 0.;
          nodes(45,0) = -0.5;
          nodes(46,0) = 0.5;
          nodes(47,0) = 0.5;
          nodes(48,0) = 0.5;
          nodes(49,0) = 0.5;
          nodes(50,0) = 1.;
          nodes(51,0) = 1.;
          nodes(52,0) = 1.;
          nodes(53,0) = 0.5;
          nodes(54,0) = 0.5;
          nodes(55,0) = 1.;
          nodes(56,0) = 0.5;
          nodes(57,0) = 0.5;
          nodes(58,0) = 1.;
          nodes(59,0) = 0.5;
          nodes(60,0) = -0.5;
          nodes(61,0) = -1.;
          nodes(62,0) = -0.5;
          nodes(63,0) = -1.;
          nodes(64,0) = 0.;
          nodes(65,0) = 0.;
          nodes(66,0) = -0.5;
          nodes(67,0) = -0.5;
          nodes(68,0) = -1.;
          nodes(69,0) = 0.;
          nodes(70,0) = -0.5;
          nodes(71,0) = -1.;
          nodes(72,0) = 0.;
          nodes(73,0) = -0.5;
          nodes(74,0) = 0.5;
          nodes(75,0) = 0.5;
          nodes(76,0) = 1.;
          nodes(77,0) = 1.;
          nodes(78,0) = 0.5;
          nodes(79,0) = 0.5;
          nodes(80,0) = 1.;
          nodes(81,0) = 0.5;
          nodes(82,0) = 1.;
          nodes(83,0) = 0.5;
          nodes(84,0) = -1.;
          nodes(85,0) = -0.5;
          nodes(86,0) = -0.5;
          nodes(87,0) = -1.;
          nodes(88,0) = 0.;
          nodes(89,0) = 0.;
          nodes(90,0) = -0.5;
          nodes(91,0) = -1.;
          nodes(92,0) = 0.;
          nodes(93,0) = -0.5;
          nodes(94,0) = -0.5;
          nodes(95,0) = -1.;
          nodes(96,0) = 0.;
          nodes(97,0) = -0.5;
          nodes(98,0) = 0.5;
          nodes(99,0) = 0.5;
          nodes(100,0) = 1.;
          nodes(101,0) = 1.;
          nodes(102,0) = 0.5;
          nodes(103,0) = 1.;
          nodes(104,0) = 0.5;
          nodes(105,0) = 0.5;
          nodes(106,0) = 1.;
          nodes(107,0) = 0.5;
          nodes(108,0) = -0.5;
          nodes(109,0) = -1.;
          nodes(110,0) = 0.;
          nodes(111,0) = -0.5;
          nodes(112,0) = -1.;
          nodes(113,0) = 0.;
          nodes(114,0) = -0.5;
          nodes(115,0) = -1.;
          nodes(116,0) = 0.;
          nodes(117,0) = -0.5;
          nodes(118,0) = 0.5;
          nodes(119,0) = 1.;
          nodes(120,0) = 0.5;
          nodes(121,0) = 1.;
          nodes(122,0) = 0.5;
          nodes(123,0) = 1.;
          nodes(124,0) = 0.5;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = 1.;
          nodes(4,1) = -1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 1.;
          nodes(7,1) = 1.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(10,1) = 0.;
          nodes(11,1) = -1.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 0.;
          nodes(15,1) = 0.;
          nodes(16,1) = -1.;
          nodes(17,1) = 0.;
          nodes(18,1) = 1.;
          nodes(19,1) = 1.;
          nodes(20,1) = 1.;
          nodes(21,1) = 1.;
          nodes(22,1) = -1.;
          nodes(23,1) = 0.;
          nodes(24,1) = 0.;
          nodes(25,1) = 0.;
          nodes(26,1) = 1.;
          nodes(27,1) = -1.;
          nodes(28,1) = -0.5;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -1.;
          nodes(32,1) = -0.5;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(35,1) = -1.;
          nodes(36,1) = -0.5;
          nodes(37,1) = 0.;
          nodes(38,1) = 0.;
          nodes(39,1) = 0.;
          nodes(40,1) = 0.;
          nodes(41,1) = -1.;
          nodes(42,1) = -0.5;
          nodes(43,1) = -0.5;
          nodes(44,1) = -0.5;
          nodes(45,1) = 0.;
          nodes(46,1) = -1.;
          nodes(47,1) = -0.5;
          nodes(48,1) = -1.;
          nodes(49,1) = -0.5;
          nodes(50,1) = -0.5;
          nodes(51,1) = -1.;
          nodes(52,1) = -0.5;
          nodes(53,1) = 0.;
          nodes(54,1) = 0.;
          nodes(55,1) = 0.;
          nodes(56,1) = -1.;
          nodes(57,1) = -0.5;
          nodes(58,1) = -0.5;
          nodes(59,1) = 0.;
          nodes(60,1) = 0.5;
          nodes(61,1) = 0.5;
          nodes(62,1) = 0.5;
          nodes(63,1) = 0.5;
          nodes(64,1) = 0.5;
          nodes(65,1) = 0.5;
          nodes(66,1) = 1.;
          nodes(67,1) = 1.;
          nodes(68,1) = 1.;
          nodes(69,1) = 1.;
          nodes(70,1) = 0.5;
          nodes(71,1) = 0.5;
          nodes(72,1) = 0.5;
          nodes(73,1) = 1.;
          nodes(74,1) = 0.5;
          nodes(75,1) = 0.5;
          nodes(76,1) = 0.5;
          nodes(77,1) = 0.5;
          nodes(78,1) = 1.;
          nodes(79,1) = 1.;
          nodes(80,1) = 1.;
          nodes(81,1) = 0.5;
          nodes(82,1) = 0.5;
          nodes(83,1) = 1.;
          nodes(84,1) = -1.;
          nodes(85,1) = -1.;
          nodes(86,1) = -0.5;
          nodes(87,1) = -0.5;
          nodes(88,1) = -1.;
          nodes(89,1) = -0.5;
          nodes(90,1) = 0.;
          nodes(91,1) = 0.;
          nodes(92,1) = 0.;
          nodes(93,1) = -1.;
          nodes(94,1) = -0.5;
          nodes(95,1) = -0.5;
          nodes(96,1) = -0.5;
          nodes(97,1) = 0.;
          nodes(98,1) = -1.;
          nodes(99,1) = -0.5;
          nodes(100,1) = -1.;
          nodes(101,1) = -0.5;
          nodes(102,1) = 0.;
          nodes(103,1) = 0.;
          nodes(104,1) = -1.;
          nodes(105,1) = -0.5;
          nodes(106,1) = -0.5;
          nodes(107,1) = 0.;
          nodes(108,1) = 0.5;
          nodes(109,1) = 0.5;
          nodes(110,1) = 0.5;
          nodes(111,1) = 1.;
          nodes(112,1) = 1.;
          nodes(113,1) = 1.;
          nodes(114,1) = 0.5;
          nodes(115,1) = 0.5;
          nodes(116,1) = 0.5;
          nodes(117,1) = 1.;
          nodes(118,1) = 0.5;
          nodes(119,1) = 0.5;
          nodes(120,1) = 1.;
          nodes(121,1) = 1.;
          nodes(122,1) = 0.5;
          nodes(123,1) = 0.5;
          nodes(124,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = -1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = 1.;
          nodes(7,2) = 1.;
          nodes(8,2) = -1.;
          nodes(9,2) = -1.;
          nodes(10,2) = -1.;
          nodes(11,2) = 0.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = -1.;
          nodes(16,2) = 0.;
          nodes(17,2) = 0.;
          nodes(18,2) = -1.;
          nodes(19,2) = 0.;
          nodes(20,2) = 0.;
          nodes(21,2) = 0.;
          nodes(22,2) = 1.;
          nodes(23,2) = 1.;
          nodes(24,2) = 1.;
          nodes(25,2) = 1.;
          nodes(26,2) = 1.;
          nodes(27,2) = -1.;
          nodes(28,2) = -1.;
          nodes(29,2) = -1.;
          nodes(30,2) = -0.5;
          nodes(31,2) = -0.5;
          nodes(32,2) = -0.5;
          nodes(33,2) = -0.5;
          nodes(34,2) = -1.;
          nodes(35,2) = -0.5;
          nodes(36,2) = -0.5;
          nodes(37,2) = -1.;
          nodes(38,2) = -0.5;
          nodes(39,2) = -0.5;
          nodes(40,2) = -0.5;
          nodes(41,2) = 0.;
          nodes(42,2) = 0.;
          nodes(43,2) = 0.;
          nodes(44,2) = 0.;
          nodes(45,2) = 0.;
          nodes(46,2) = -1.;
          nodes(47,2) = -1.;
          nodes(48,2) = -0.5;
          nodes(49,2) = -0.5;
          nodes(50,2) = -1.;
          nodes(51,2) = -0.5;
          nodes(52,2) = -0.5;
          nodes(53,2) = -1.;
          nodes(54,2) = -0.5;
          nodes(55,2) = -0.5;
          nodes(56,2) = 0.;
          nodes(57,2) = 0.;
          nodes(58,2) = 0.;
          nodes(59,2) = 0.;
          nodes(60,2) = -1.;
          nodes(61,2) = -1.;
          nodes(62,2) = -0.5;
          nodes(63,2) = -0.5;
          nodes(64,2) = -1.;
          nodes(65,2) = -0.5;
          nodes(66,2) = -1.;
          nodes(67,2) = -0.5;
          nodes(68,2) = -0.5;
          nodes(69,2) = -0.5;
          nodes(70,2) = 0.;
          nodes(71,2) = 0.;
          nodes(72,2) = 0.;
          nodes(73,2) = 0.;
          nodes(74,2) = -1.;
          nodes(75,2) = -0.5;
          nodes(76,2) = -1.;
          nodes(77,2) = -0.5;
          nodes(78,2) = -1.;
          nodes(79,2) = -0.5;
          nodes(80,2) = -0.5;
          nodes(81,2) = 0.;
          nodes(82,2) = 0.;
          nodes(83,2) = 0.;
          nodes(84,2) = 0.5;
          nodes(85,2) = 0.5;
          nodes(86,2) = 0.5;
          nodes(87,2) = 0.5;
          nodes(88,2) = 0.5;
          nodes(89,2) = 0.5;
          nodes(90,2) = 0.5;
          nodes(91,2) = 0.5;
          nodes(92,2) = 0.5;
          nodes(93,2) = 1.;
          nodes(94,2) = 1.;
          nodes(95,2) = 1.;
          nodes(96,2) = 1.;
          nodes(97,2) = 1.;
          nodes(98,2) = 0.5;
          nodes(99,2) = 0.5;
          nodes(100,2) = 0.5;
          nodes(101,2) = 0.5;
          nodes(102,2) = 0.5;
          nodes(103,2) = 0.5;
          nodes(104,2) = 1.;
          nodes(105,2) = 1.;
          nodes(106,2) = 1.;
          nodes(107,2) = 1.;
          nodes(108,2) = 0.5;
          nodes(109,2) = 0.5;
          nodes(110,2) = 0.5;
          nodes(111,2) = 0.5;
          nodes(112,2) = 0.5;
          nodes(113,2) = 0.5;
          nodes(114,2) = 1.;
          nodes(115,2) = 1.;
          nodes(116,2) = 1.;
          nodes(117,2) = 1.;
          nodes(118,2) = 0.5;
          nodes(119,2) = 0.5;
          nodes(120,2) = 0.5;
          nodes(121,2) = 0.5;
          nodes(122,2) = 1.;
          nodes(123,2) = 1.;
          nodes(124,2) = 1.;
          return;
        }
        default:
        {
          nodes.resize(729,Eigen::NoChange);
          nodes(0,0) = -1.;
          nodes(1,0) = 1.;
          nodes(2,0) = 1.;
          nodes(3,0) = -1.;
          nodes(4,0) = -1.;
          nodes(5,0) = 1.;
          nodes(6,0) = 1.;
          nodes(7,0) = -1.;
          nodes(8,0) = 0.;
          nodes(9,0) = 0.;
          nodes(10,0) = -1.;
          nodes(11,0) = -1.;
          nodes(12,0) = 0.;
          nodes(13,0) = 0.;
          nodes(14,0) = -1.;
          nodes(15,0) = 1.;
          nodes(16,0) = 1.;
          nodes(17,0) = 1.;
          nodes(18,0) = 0.;
          nodes(19,0) = 0.;
          nodes(20,0) = -1.;
          nodes(21,0) = 1.;
          nodes(22,0) = 0.;
          nodes(23,0) = 0.;
          nodes(24,0) = -1.;
          nodes(25,0) = 1.;
          nodes(26,0) = 0.;
          nodes(27,0) = -0.5;
          nodes(28,0) = -0.5;
          nodes(29,0) = -1.;
          nodes(30,0) = -1.;
          nodes(31,0) = -0.5;
          nodes(32,0) = -0.5;
          nodes(33,0) = -1.;
          nodes(34,0) = 0.;
          nodes(35,0) = 0.;
          nodes(36,0) = 0.;
          nodes(37,0) = -0.5;
          nodes(38,0) = -0.5;
          nodes(39,0) = -1.;
          nodes(40,0) = 0.;
          nodes(41,0) = -0.5;
          nodes(42,0) = -0.5;
          nodes(43,0) = -1.;
          nodes(44,0) = 0.;
          nodes(45,0) = -0.5;
          nodes(46,0) = 0.5;
          nodes(47,0) = 0.5;
          nodes(48,0) = 0.5;
          nodes(49,0) = 0.5;
          nodes(50,0) = 1.;
          nodes(51,0) = 1.;
          nodes(52,0) = 1.;
          nodes(53,0) = 0.5;
          nodes(54,0) = 0.5;
          nodes(55,0) = 1.;
          nodes(56,0) = 0.5;
          nodes(57,0) = 0.5;
          nodes(58,0) = 1.;
          nodes(59,0) = 0.5;
          nodes(60,0) = -0.5;
          nodes(61,0) = -1.;
          nodes(62,0) = -0.5;
          nodes(63,0) = -1.;
          nodes(64,0) = 0.;
          nodes(65,0) = 0.;
          nodes(66,0) = -0.5;
          nodes(67,0) = -0.5;
          nodes(68,0) = -1.;
          nodes(69,0) = 0.;
          nodes(70,0) = -0.5;
          nodes(71,0) = -1.;
          nodes(72,0) = 0.;
          nodes(73,0) = -0.5;
          nodes(74,0) = 0.5;
          nodes(75,0) = 0.5;
          nodes(76,0) = 1.;
          nodes(77,0) = 1.;
          nodes(78,0) = 0.5;
          nodes(79,0) = 0.5;
          nodes(80,0) = 1.;
          nodes(81,0) = 0.5;
          nodes(82,0) = 1.;
          nodes(83,0) = 0.5;
          nodes(84,0) = -1.;
          nodes(85,0) = -0.5;
          nodes(86,0) = -0.5;
          nodes(87,0) = -1.;
          nodes(88,0) = 0.;
          nodes(89,0) = 0.;
          nodes(90,0) = -0.5;
          nodes(91,0) = -1.;
          nodes(92,0) = 0.;
          nodes(93,0) = -0.5;
          nodes(94,0) = -0.5;
          nodes(95,0) = -1.;
          nodes(96,0) = 0.;
          nodes(97,0) = -0.5;
          nodes(98,0) = 0.5;
          nodes(99,0) = 0.5;
          nodes(100,0) = 1.;
          nodes(101,0) = 1.;
          nodes(102,0) = 0.5;
          nodes(103,0) = 1.;
          nodes(104,0) = 0.5;
          nodes(105,0) = 0.5;
          nodes(106,0) = 1.;
          nodes(107,0) = 0.5;
          nodes(108,0) = -0.5;
          nodes(109,0) = -1.;
          nodes(110,0) = 0.;
          nodes(111,0) = -0.5;
          nodes(112,0) = -1.;
          nodes(113,0) = 0.;
          nodes(114,0) = -0.5;
          nodes(115,0) = -1.;
          nodes(116,0) = 0.;
          nodes(117,0) = -0.5;
          nodes(118,0) = 0.5;
          nodes(119,0) = 1.;
          nodes(120,0) = 0.5;
          nodes(121,0) = 1.;
          nodes(122,0) = 0.5;
          nodes(123,0) = 1.;
          nodes(124,0) = 0.5;
          nodes(125,0) = -0.75;
          nodes(126,0) = -0.75;
          nodes(127,0) = -1.;
          nodes(128,0) = -1.;
          nodes(129,0) = -0.75;
          nodes(130,0) = -0.75;
          nodes(131,0) = -1.;
          nodes(132,0) = -0.5;
          nodes(133,0) = -0.5;
          nodes(134,0) = -0.5;
          nodes(135,0) = -0.75;
          nodes(136,0) = -0.75;
          nodes(137,0) = -1.;
          nodes(138,0) = -0.5;
          nodes(139,0) = -0.75;
          nodes(140,0) = -0.75;
          nodes(141,0) = -1.;
          nodes(142,0) = -0.5;
          nodes(143,0) = -0.75;
          nodes(144,0) = -0.25;
          nodes(145,0) = -0.25;
          nodes(146,0) = -0.25;
          nodes(147,0) = -0.25;
          nodes(148,0) = 0.;
          nodes(149,0) = 0.;
          nodes(150,0) = 0.;
          nodes(151,0) = -0.25;
          nodes(152,0) = -0.25;
          nodes(153,0) = 0.;
          nodes(154,0) = -0.25;
          nodes(155,0) = -0.25;
          nodes(156,0) = 0.;
          nodes(157,0) = -0.25;
          nodes(158,0) = -0.75;
          nodes(159,0) = -1.;
          nodes(160,0) = -0.75;
          nodes(161,0) = -1.;
          nodes(162,0) = -0.5;
          nodes(163,0) = -0.5;
          nodes(164,0) = -0.75;
          nodes(165,0) = -0.75;
          nodes(166,0) = -1.;
          nodes(167,0) = -0.5;
          nodes(168,0) = -0.75;
          nodes(169,0) = -1.;
          nodes(170,0) = -0.5;
          nodes(171,0) = -0.75;
          nodes(172,0) = -0.25;
          nodes(173,0) = -0.25;
          nodes(174,0) = 0.;
          nodes(175,0) = 0.;
          nodes(176,0) = -0.25;
          nodes(177,0) = -0.25;
          nodes(178,0) = 0.;
          nodes(179,0) = -0.25;
          nodes(180,0) = 0.;
          nodes(181,0) = -0.25;
          nodes(182,0) = -1.;
          nodes(183,0) = -0.75;
          nodes(184,0) = -0.75;
          nodes(185,0) = -1.;
          nodes(186,0) = -0.5;
          nodes(187,0) = -0.5;
          nodes(188,0) = -0.75;
          nodes(189,0) = -1.;
          nodes(190,0) = -0.5;
          nodes(191,0) = -0.75;
          nodes(192,0) = -0.75;
          nodes(193,0) = -1.;
          nodes(194,0) = -0.5;
          nodes(195,0) = -0.75;
          nodes(196,0) = -0.25;
          nodes(197,0) = -0.25;
          nodes(198,0) = 0.;
          nodes(199,0) = 0.;
          nodes(200,0) = -0.25;
          nodes(201,0) = 0.;
          nodes(202,0) = -0.25;
          nodes(203,0) = -0.25;
          nodes(204,0) = 0.;
          nodes(205,0) = -0.25;
          nodes(206,0) = -0.75;
          nodes(207,0) = -1.;
          nodes(208,0) = -0.5;
          nodes(209,0) = -0.75;
          nodes(210,0) = -1.;
          nodes(211,0) = -0.5;
          nodes(212,0) = -0.75;
          nodes(213,0) = -1.;
          nodes(214,0) = -0.5;
          nodes(215,0) = -0.75;
          nodes(216,0) = -0.25;
          nodes(217,0) = 0.;
          nodes(218,0) = -0.25;
          nodes(219,0) = 0.;
          nodes(220,0) = -0.25;
          nodes(221,0) = 0.;
          nodes(222,0) = -0.25;
          nodes(223,0) = 0.25;
          nodes(224,0) = 0.25;
          nodes(225,0) = 0.25;
          nodes(226,0) = 0.25;
          nodes(227,0) = 0.5;
          nodes(228,0) = 0.5;
          nodes(229,0) = 0.5;
          nodes(230,0) = 0.25;
          nodes(231,0) = 0.25;
          nodes(232,0) = 0.5;
          nodes(233,0) = 0.25;
          nodes(234,0) = 0.25;
          nodes(235,0) = 0.5;
          nodes(236,0) = 0.25;
          nodes(237,0) = 0.75;
          nodes(238,0) = 0.75;
          nodes(239,0) = 0.75;
          nodes(240,0) = 0.75;
          nodes(241,0) = 1.;
          nodes(242,0) = 1.;
          nodes(243,0) = 1.;
          nodes(244,0) = 0.75;
          nodes(245,0) = 0.75;
          nodes(246,0) = 1.;
          nodes(247,0) = 0.75;
          nodes(248,0) = 0.75;
          nodes(249,0) = 1.;
          nodes(250,0) = 0.75;
          nodes(251,0) = 0.25;
          nodes(252,0) = 0.25;
          nodes(253,0) = 0.5;
          nodes(254,0) = 0.5;
          nodes(255,0) = 0.25;
          nodes(256,0) = 0.25;
          nodes(257,0) = 0.5;
          nodes(258,0) = 0.25;
          nodes(259,0) = 0.5;
          nodes(260,0) = 0.25;
          nodes(261,0) = 0.75;
          nodes(262,0) = 0.75;
          nodes(263,0) = 1.;
          nodes(264,0) = 1.;
          nodes(265,0) = 0.75;
          nodes(266,0) = 0.75;
          nodes(267,0) = 1.;
          nodes(268,0) = 0.75;
          nodes(269,0) = 1.;
          nodes(270,0) = 0.75;
          nodes(271,0) = 0.25;
          nodes(272,0) = 0.25;
          nodes(273,0) = 0.5;
          nodes(274,0) = 0.5;
          nodes(275,0) = 0.25;
          nodes(276,0) = 0.5;
          nodes(277,0) = 0.25;
          nodes(278,0) = 0.25;
          nodes(279,0) = 0.5;
          nodes(280,0) = 0.25;
          nodes(281,0) = 0.75;
          nodes(282,0) = 0.75;
          nodes(283,0) = 1.;
          nodes(284,0) = 1.;
          nodes(285,0) = 0.75;
          nodes(286,0) = 1.;
          nodes(287,0) = 0.75;
          nodes(288,0) = 0.75;
          nodes(289,0) = 1.;
          nodes(290,0) = 0.75;
          nodes(291,0) = 0.25;
          nodes(292,0) = 0.5;
          nodes(293,0) = 0.25;
          nodes(294,0) = 0.5;
          nodes(295,0) = 0.25;
          nodes(296,0) = 0.5;
          nodes(297,0) = 0.25;
          nodes(298,0) = 0.75;
          nodes(299,0) = 1.;
          nodes(300,0) = 0.75;
          nodes(301,0) = 1.;
          nodes(302,0) = 0.75;
          nodes(303,0) = 1.;
          nodes(304,0) = 0.75;
          nodes(305,0) = -0.75;
          nodes(306,0) = -1.;
          nodes(307,0) = -0.75;
          nodes(308,0) = -1.;
          nodes(309,0) = -0.5;
          nodes(310,0) = -0.5;
          nodes(311,0) = -0.75;
          nodes(312,0) = -0.75;
          nodes(313,0) = -1.;
          nodes(314,0) = -0.5;
          nodes(315,0) = -0.75;
          nodes(316,0) = -1.;
          nodes(317,0) = -0.5;
          nodes(318,0) = -0.75;
          nodes(319,0) = -0.25;
          nodes(320,0) = -0.25;
          nodes(321,0) = 0.;
          nodes(322,0) = 0.;
          nodes(323,0) = -0.25;
          nodes(324,0) = -0.25;
          nodes(325,0) = 0.;
          nodes(326,0) = -0.25;
          nodes(327,0) = 0.;
          nodes(328,0) = -0.25;
          nodes(329,0) = -0.75;
          nodes(330,0) = -1.;
          nodes(331,0) = -0.75;
          nodes(332,0) = -1.;
          nodes(333,0) = -0.5;
          nodes(334,0) = -0.5;
          nodes(335,0) = -0.75;
          nodes(336,0) = -0.75;
          nodes(337,0) = -1.;
          nodes(338,0) = -0.5;
          nodes(339,0) = -0.75;
          nodes(340,0) = -1.;
          nodes(341,0) = -0.5;
          nodes(342,0) = -0.75;
          nodes(343,0) = -0.25;
          nodes(344,0) = -0.25;
          nodes(345,0) = 0.;
          nodes(346,0) = 0.;
          nodes(347,0) = -0.25;
          nodes(348,0) = -0.25;
          nodes(349,0) = 0.;
          nodes(350,0) = -0.25;
          nodes(351,0) = 0.;
          nodes(352,0) = -0.25;
          nodes(353,0) = -0.75;
          nodes(354,0) = -1.;
          nodes(355,0) = -0.5;
          nodes(356,0) = -0.75;
          nodes(357,0) = -1.;
          nodes(358,0) = -0.5;
          nodes(359,0) = -0.75;
          nodes(360,0) = -1.;
          nodes(361,0) = -0.5;
          nodes(362,0) = -0.75;
          nodes(363,0) = -0.25;
          nodes(364,0) = 0.;
          nodes(365,0) = -0.25;
          nodes(366,0) = 0.;
          nodes(367,0) = -0.25;
          nodes(368,0) = 0.;
          nodes(369,0) = -0.25;
          nodes(370,0) = -0.75;
          nodes(371,0) = -1.;
          nodes(372,0) = -0.5;
          nodes(373,0) = -0.75;
          nodes(374,0) = -1.;
          nodes(375,0) = -0.5;
          nodes(376,0) = -0.75;
          nodes(377,0) = -1.;
          nodes(378,0) = -0.5;
          nodes(379,0) = -0.75;
          nodes(380,0) = -0.25;
          nodes(381,0) = 0.;
          nodes(382,0) = -0.25;
          nodes(383,0) = 0.;
          nodes(384,0) = -0.25;
          nodes(385,0) = 0.;
          nodes(386,0) = -0.25;
          nodes(387,0) = 0.25;
          nodes(388,0) = 0.25;
          nodes(389,0) = 0.5;
          nodes(390,0) = 0.5;
          nodes(391,0) = 0.25;
          nodes(392,0) = 0.25;
          nodes(393,0) = 0.5;
          nodes(394,0) = 0.25;
          nodes(395,0) = 0.5;
          nodes(396,0) = 0.25;
          nodes(397,0) = 0.75;
          nodes(398,0) = 0.75;
          nodes(399,0) = 1.;
          nodes(400,0) = 1.;
          nodes(401,0) = 0.75;
          nodes(402,0) = 0.75;
          nodes(403,0) = 1.;
          nodes(404,0) = 0.75;
          nodes(405,0) = 1.;
          nodes(406,0) = 0.75;
          nodes(407,0) = 0.25;
          nodes(408,0) = 0.25;
          nodes(409,0) = 0.5;
          nodes(410,0) = 0.5;
          nodes(411,0) = 0.25;
          nodes(412,0) = 0.25;
          nodes(413,0) = 0.5;
          nodes(414,0) = 0.25;
          nodes(415,0) = 0.5;
          nodes(416,0) = 0.25;
          nodes(417,0) = 0.75;
          nodes(418,0) = 0.75;
          nodes(419,0) = 1.;
          nodes(420,0) = 1.;
          nodes(421,0) = 0.75;
          nodes(422,0) = 0.75;
          nodes(423,0) = 1.;
          nodes(424,0) = 0.75;
          nodes(425,0) = 1.;
          nodes(426,0) = 0.75;
          nodes(427,0) = 0.25;
          nodes(428,0) = 0.5;
          nodes(429,0) = 0.25;
          nodes(430,0) = 0.5;
          nodes(431,0) = 0.25;
          nodes(432,0) = 0.5;
          nodes(433,0) = 0.25;
          nodes(434,0) = 0.75;
          nodes(435,0) = 1.;
          nodes(436,0) = 0.75;
          nodes(437,0) = 1.;
          nodes(438,0) = 0.75;
          nodes(439,0) = 1.;
          nodes(440,0) = 0.75;
          nodes(441,0) = 0.25;
          nodes(442,0) = 0.5;
          nodes(443,0) = 0.25;
          nodes(444,0) = 0.5;
          nodes(445,0) = 0.25;
          nodes(446,0) = 0.5;
          nodes(447,0) = 0.25;
          nodes(448,0) = 0.75;
          nodes(449,0) = 1.;
          nodes(450,0) = 0.75;
          nodes(451,0) = 1.;
          nodes(452,0) = 0.75;
          nodes(453,0) = 1.;
          nodes(454,0) = 0.75;
          nodes(455,0) = -1.;
          nodes(456,0) = -0.75;
          nodes(457,0) = -0.75;
          nodes(458,0) = -1.;
          nodes(459,0) = -0.5;
          nodes(460,0) = -0.5;
          nodes(461,0) = -0.75;
          nodes(462,0) = -1.;
          nodes(463,0) = -0.5;
          nodes(464,0) = -0.75;
          nodes(465,0) = -0.75;
          nodes(466,0) = -1.;
          nodes(467,0) = -0.5;
          nodes(468,0) = -0.75;
          nodes(469,0) = -0.25;
          nodes(470,0) = -0.25;
          nodes(471,0) = 0.;
          nodes(472,0) = 0.;
          nodes(473,0) = -0.25;
          nodes(474,0) = 0.;
          nodes(475,0) = -0.25;
          nodes(476,0) = -0.25;
          nodes(477,0) = 0.;
          nodes(478,0) = -0.25;
          nodes(479,0) = -0.75;
          nodes(480,0) = -1.;
          nodes(481,0) = -0.5;
          nodes(482,0) = -0.75;
          nodes(483,0) = -1.;
          nodes(484,0) = -0.5;
          nodes(485,0) = -0.75;
          nodes(486,0) = -1.;
          nodes(487,0) = -0.5;
          nodes(488,0) = -0.75;
          nodes(489,0) = -0.25;
          nodes(490,0) = 0.;
          nodes(491,0) = -0.25;
          nodes(492,0) = 0.;
          nodes(493,0) = -0.25;
          nodes(494,0) = 0.;
          nodes(495,0) = -0.25;
          nodes(496,0) = -1.;
          nodes(497,0) = -0.75;
          nodes(498,0) = -0.75;
          nodes(499,0) = -1.;
          nodes(500,0) = -0.5;
          nodes(501,0) = -0.5;
          nodes(502,0) = -0.75;
          nodes(503,0) = -1.;
          nodes(504,0) = -0.5;
          nodes(505,0) = -0.75;
          nodes(506,0) = -0.75;
          nodes(507,0) = -1.;
          nodes(508,0) = -0.5;
          nodes(509,0) = -0.75;
          nodes(510,0) = -0.25;
          nodes(511,0) = -0.25;
          nodes(512,0) = 0.;
          nodes(513,0) = 0.;
          nodes(514,0) = -0.25;
          nodes(515,0) = 0.;
          nodes(516,0) = -0.25;
          nodes(517,0) = -0.25;
          nodes(518,0) = 0.;
          nodes(519,0) = -0.25;
          nodes(520,0) = -0.75;
          nodes(521,0) = -1.;
          nodes(522,0) = -0.5;
          nodes(523,0) = -0.75;
          nodes(524,0) = -1.;
          nodes(525,0) = -0.5;
          nodes(526,0) = -0.75;
          nodes(527,0) = -1.;
          nodes(528,0) = -0.5;
          nodes(529,0) = -0.75;
          nodes(530,0) = -0.25;
          nodes(531,0) = 0.;
          nodes(532,0) = -0.25;
          nodes(533,0) = 0.;
          nodes(534,0) = -0.25;
          nodes(535,0) = 0.;
          nodes(536,0) = -0.25;
          nodes(537,0) = 0.25;
          nodes(538,0) = 0.25;
          nodes(539,0) = 0.5;
          nodes(540,0) = 0.5;
          nodes(541,0) = 0.25;
          nodes(542,0) = 0.5;
          nodes(543,0) = 0.25;
          nodes(544,0) = 0.25;
          nodes(545,0) = 0.5;
          nodes(546,0) = 0.25;
          nodes(547,0) = 0.75;
          nodes(548,0) = 0.75;
          nodes(549,0) = 1.;
          nodes(550,0) = 1.;
          nodes(551,0) = 0.75;
          nodes(552,0) = 1.;
          nodes(553,0) = 0.75;
          nodes(554,0) = 0.75;
          nodes(555,0) = 1.;
          nodes(556,0) = 0.75;
          nodes(557,0) = 0.25;
          nodes(558,0) = 0.5;
          nodes(559,0) = 0.25;
          nodes(560,0) = 0.5;
          nodes(561,0) = 0.25;
          nodes(562,0) = 0.5;
          nodes(563,0) = 0.25;
          nodes(564,0) = 0.75;
          nodes(565,0) = 1.;
          nodes(566,0) = 0.75;
          nodes(567,0) = 1.;
          nodes(568,0) = 0.75;
          nodes(569,0) = 1.;
          nodes(570,0) = 0.75;
          nodes(571,0) = 0.25;
          nodes(572,0) = 0.25;
          nodes(573,0) = 0.5;
          nodes(574,0) = 0.5;
          nodes(575,0) = 0.25;
          nodes(576,0) = 0.5;
          nodes(577,0) = 0.25;
          nodes(578,0) = 0.25;
          nodes(579,0) = 0.5;
          nodes(580,0) = 0.25;
          nodes(581,0) = 0.75;
          nodes(582,0) = 0.75;
          nodes(583,0) = 1.;
          nodes(584,0) = 1.;
          nodes(585,0) = 0.75;
          nodes(586,0) = 1.;
          nodes(587,0) = 0.75;
          nodes(588,0) = 0.75;
          nodes(589,0) = 1.;
          nodes(590,0) = 0.75;
          nodes(591,0) = 0.25;
          nodes(592,0) = 0.5;
          nodes(593,0) = 0.25;
          nodes(594,0) = 0.5;
          nodes(595,0) = 0.25;
          nodes(596,0) = 0.5;
          nodes(597,0) = 0.25;
          nodes(598,0) = 0.75;
          nodes(599,0) = 1.;
          nodes(600,0) = 0.75;
          nodes(601,0) = 1.;
          nodes(602,0) = 0.75;
          nodes(603,0) = 1.;
          nodes(604,0) = 0.75;
          nodes(605,0) = -0.75;
          nodes(606,0) = -1.;
          nodes(607,0) = -0.5;
          nodes(608,0) = -0.75;
          nodes(609,0) = -1.;
          nodes(610,0) = -0.5;
          nodes(611,0) = -0.75;
          nodes(612,0) = -1.;
          nodes(613,0) = -0.5;
          nodes(614,0) = -0.75;
          nodes(615,0) = -0.25;
          nodes(616,0) = 0.;
          nodes(617,0) = -0.25;
          nodes(618,0) = 0.;
          nodes(619,0) = -0.25;
          nodes(620,0) = 0.;
          nodes(621,0) = -0.25;
          nodes(622,0) = -0.75;
          nodes(623,0) = -1.;
          nodes(624,0) = -0.5;
          nodes(625,0) = -0.75;
          nodes(626,0) = -1.;
          nodes(627,0) = -0.5;
          nodes(628,0) = -0.75;
          nodes(629,0) = -1.;
          nodes(630,0) = -0.5;
          nodes(631,0) = -0.75;
          nodes(632,0) = -0.25;
          nodes(633,0) = 0.;
          nodes(634,0) = -0.25;
          nodes(635,0) = 0.;
          nodes(636,0) = -0.25;
          nodes(637,0) = 0.;
          nodes(638,0) = -0.25;
          nodes(639,0) = -0.75;
          nodes(640,0) = -1.;
          nodes(641,0) = -0.5;
          nodes(642,0) = -0.75;
          nodes(643,0) = -1.;
          nodes(644,0) = -0.5;
          nodes(645,0) = -0.75;
          nodes(646,0) = -1.;
          nodes(647,0) = -0.5;
          nodes(648,0) = -0.75;
          nodes(649,0) = -0.25;
          nodes(650,0) = 0.;
          nodes(651,0) = -0.25;
          nodes(652,0) = 0.;
          nodes(653,0) = -0.25;
          nodes(654,0) = 0.;
          nodes(655,0) = -0.25;
          nodes(656,0) = -0.75;
          nodes(657,0) = -1.;
          nodes(658,0) = -0.5;
          nodes(659,0) = -0.75;
          nodes(660,0) = -1.;
          nodes(661,0) = -0.5;
          nodes(662,0) = -0.75;
          nodes(663,0) = -1.;
          nodes(664,0) = -0.5;
          nodes(665,0) = -0.75;
          nodes(666,0) = -0.25;
          nodes(667,0) = 0.;
          nodes(668,0) = -0.25;
          nodes(669,0) = 0.;
          nodes(670,0) = -0.25;
          nodes(671,0) = 0.;
          nodes(672,0) = -0.25;
          nodes(673,0) = 0.25;
          nodes(674,0) = 0.5;
          nodes(675,0) = 0.25;
          nodes(676,0) = 0.5;
          nodes(677,0) = 0.25;
          nodes(678,0) = 0.5;
          nodes(679,0) = 0.25;
          nodes(680,0) = 0.75;
          nodes(681,0) = 1.;
          nodes(682,0) = 0.75;
          nodes(683,0) = 1.;
          nodes(684,0) = 0.75;
          nodes(685,0) = 1.;
          nodes(686,0) = 0.75;
          nodes(687,0) = 0.25;
          nodes(688,0) = 0.5;
          nodes(689,0) = 0.25;
          nodes(690,0) = 0.5;
          nodes(691,0) = 0.25;
          nodes(692,0) = 0.5;
          nodes(693,0) = 0.25;
          nodes(694,0) = 0.75;
          nodes(695,0) = 1.;
          nodes(696,0) = 0.75;
          nodes(697,0) = 1.;
          nodes(698,0) = 0.75;
          nodes(699,0) = 1.;
          nodes(700,0) = 0.75;
          nodes(701,0) = 0.25;
          nodes(702,0) = 0.5;
          nodes(703,0) = 0.25;
          nodes(704,0) = 0.5;
          nodes(705,0) = 0.25;
          nodes(706,0) = 0.5;
          nodes(707,0) = 0.25;
          nodes(708,0) = 0.75;
          nodes(709,0) = 1.;
          nodes(710,0) = 0.75;
          nodes(711,0) = 1.;
          nodes(712,0) = 0.75;
          nodes(713,0) = 1.;
          nodes(714,0) = 0.75;
          nodes(715,0) = 0.25;
          nodes(716,0) = 0.5;
          nodes(717,0) = 0.25;
          nodes(718,0) = 0.5;
          nodes(719,0) = 0.25;
          nodes(720,0) = 0.5;
          nodes(721,0) = 0.25;
          nodes(722,0) = 0.75;
          nodes(723,0) = 1.;
          nodes(724,0) = 0.75;
          nodes(725,0) = 1.;
          nodes(726,0) = 0.75;
          nodes(727,0) = 1.;
          nodes(728,0) = 0.75;
          nodes(0,1) = -1.;
          nodes(1,1) = -1.;
          nodes(2,1) = 1.;
          nodes(3,1) = 1.;
          nodes(4,1) = -1.;
          nodes(5,1) = -1.;
          nodes(6,1) = 1.;
          nodes(7,1) = 1.;
          nodes(8,1) = -1.;
          nodes(9,1) = 0.;
          nodes(10,1) = 0.;
          nodes(11,1) = -1.;
          nodes(12,1) = -1.;
          nodes(13,1) = 0.;
          nodes(14,1) = 0.;
          nodes(15,1) = 0.;
          nodes(16,1) = -1.;
          nodes(17,1) = 0.;
          nodes(18,1) = 1.;
          nodes(19,1) = 1.;
          nodes(20,1) = 1.;
          nodes(21,1) = 1.;
          nodes(22,1) = -1.;
          nodes(23,1) = 0.;
          nodes(24,1) = 0.;
          nodes(25,1) = 0.;
          nodes(26,1) = 1.;
          nodes(27,1) = -1.;
          nodes(28,1) = -0.5;
          nodes(29,1) = -0.5;
          nodes(30,1) = -1.;
          nodes(31,1) = -1.;
          nodes(32,1) = -0.5;
          nodes(33,1) = -0.5;
          nodes(34,1) = -0.5;
          nodes(35,1) = -1.;
          nodes(36,1) = -0.5;
          nodes(37,1) = 0.;
          nodes(38,1) = 0.;
          nodes(39,1) = 0.;
          nodes(40,1) = 0.;
          nodes(41,1) = -1.;
          nodes(42,1) = -0.5;
          nodes(43,1) = -0.5;
          nodes(44,1) = -0.5;
          nodes(45,1) = 0.;
          nodes(46,1) = -1.;
          nodes(47,1) = -0.5;
          nodes(48,1) = -1.;
          nodes(49,1) = -0.5;
          nodes(50,1) = -0.5;
          nodes(51,1) = -1.;
          nodes(52,1) = -0.5;
          nodes(53,1) = 0.;
          nodes(54,1) = 0.;
          nodes(55,1) = 0.;
          nodes(56,1) = -1.;
          nodes(57,1) = -0.5;
          nodes(58,1) = -0.5;
          nodes(59,1) = 0.;
          nodes(60,1) = 0.5;
          nodes(61,1) = 0.5;
          nodes(62,1) = 0.5;
          nodes(63,1) = 0.5;
          nodes(64,1) = 0.5;
          nodes(65,1) = 0.5;
          nodes(66,1) = 1.;
          nodes(67,1) = 1.;
          nodes(68,1) = 1.;
          nodes(69,1) = 1.;
          nodes(70,1) = 0.5;
          nodes(71,1) = 0.5;
          nodes(72,1) = 0.5;
          nodes(73,1) = 1.;
          nodes(74,1) = 0.5;
          nodes(75,1) = 0.5;
          nodes(76,1) = 0.5;
          nodes(77,1) = 0.5;
          nodes(78,1) = 1.;
          nodes(79,1) = 1.;
          nodes(80,1) = 1.;
          nodes(81,1) = 0.5;
          nodes(82,1) = 0.5;
          nodes(83,1) = 1.;
          nodes(84,1) = -1.;
          nodes(85,1) = -1.;
          nodes(86,1) = -0.5;
          nodes(87,1) = -0.5;
          nodes(88,1) = -1.;
          nodes(89,1) = -0.5;
          nodes(90,1) = 0.;
          nodes(91,1) = 0.;
          nodes(92,1) = 0.;
          nodes(93,1) = -1.;
          nodes(94,1) = -0.5;
          nodes(95,1) = -0.5;
          nodes(96,1) = -0.5;
          nodes(97,1) = 0.;
          nodes(98,1) = -1.;
          nodes(99,1) = -0.5;
          nodes(100,1) = -1.;
          nodes(101,1) = -0.5;
          nodes(102,1) = 0.;
          nodes(103,1) = 0.;
          nodes(104,1) = -1.;
          nodes(105,1) = -0.5;
          nodes(106,1) = -0.5;
          nodes(107,1) = 0.;
          nodes(108,1) = 0.5;
          nodes(109,1) = 0.5;
          nodes(110,1) = 0.5;
          nodes(111,1) = 1.;
          nodes(112,1) = 1.;
          nodes(113,1) = 1.;
          nodes(114,1) = 0.5;
          nodes(115,1) = 0.5;
          nodes(116,1) = 0.5;
          nodes(117,1) = 1.;
          nodes(118,1) = 0.5;
          nodes(119,1) = 0.5;
          nodes(120,1) = 1.;
          nodes(121,1) = 1.;
          nodes(122,1) = 0.5;
          nodes(123,1) = 0.5;
          nodes(124,1) = 1.;
          nodes(125,1) = -1.;
          nodes(126,1) = -0.75;
          nodes(127,1) = -0.75;
          nodes(128,1) = -1.;
          nodes(129,1) = -1.;
          nodes(130,1) = -0.75;
          nodes(131,1) = -0.75;
          nodes(132,1) = -0.75;
          nodes(133,1) = -1.;
          nodes(134,1) = -0.75;
          nodes(135,1) = -0.5;
          nodes(136,1) = -0.5;
          nodes(137,1) = -0.5;
          nodes(138,1) = -0.5;
          nodes(139,1) = -1.;
          nodes(140,1) = -0.75;
          nodes(141,1) = -0.75;
          nodes(142,1) = -0.75;
          nodes(143,1) = -0.5;
          nodes(144,1) = -1.;
          nodes(145,1) = -0.75;
          nodes(146,1) = -1.;
          nodes(147,1) = -0.75;
          nodes(148,1) = -0.75;
          nodes(149,1) = -1.;
          nodes(150,1) = -0.75;
          nodes(151,1) = -0.5;
          nodes(152,1) = -0.5;
          nodes(153,1) = -0.5;
          nodes(154,1) = -1.;
          nodes(155,1) = -0.75;
          nodes(156,1) = -0.75;
          nodes(157,1) = -0.5;
          nodes(158,1) = -0.25;
          nodes(159,1) = -0.25;
          nodes(160,1) = -0.25;
          nodes(161,1) = -0.25;
          nodes(162,1) = -0.25;
          nodes(163,1) = -0.25;
          nodes(164,1) = 0.;
          nodes(165,1) = 0.;
          nodes(166,1) = 0.;
          nodes(167,1) = 0.;
          nodes(168,1) = -0.25;
          nodes(169,1) = -0.25;
          nodes(170,1) = -0.25;
          nodes(171,1) = 0.;
          nodes(172,1) = -0.25;
          nodes(173,1) = -0.25;
          nodes(174,1) = -0.25;
          nodes(175,1) = -0.25;
          nodes(176,1) = 0.;
          nodes(177,1) = 0.;
          nodes(178,1) = 0.;
          nodes(179,1) = -0.25;
          nodes(180,1) = -0.25;
          nodes(181,1) = 0.;
          nodes(182,1) = -1.;
          nodes(183,1) = -1.;
          nodes(184,1) = -0.75;
          nodes(185,1) = -0.75;
          nodes(186,1) = -1.;
          nodes(187,1) = -0.75;
          nodes(188,1) = -0.5;
          nodes(189,1) = -0.5;
          nodes(190,1) = -0.5;
          nodes(191,1) = -1.;
          nodes(192,1) = -0.75;
          nodes(193,1) = -0.75;
          nodes(194,1) = -0.75;
          nodes(195,1) = -0.5;
          nodes(196,1) = -1.;
          nodes(197,1) = -0.75;
          nodes(198,1) = -1.;
          nodes(199,1) = -0.75;
          nodes(200,1) = -0.5;
          nodes(201,1) = -0.5;
          nodes(202,1) = -1.;
          nodes(203,1) = -0.75;
          nodes(204,1) = -0.75;
          nodes(205,1) = -0.5;
          nodes(206,1) = -0.25;
          nodes(207,1) = -0.25;
          nodes(208,1) = -0.25;
          nodes(209,1) = 0.;
          nodes(210,1) = 0.;
          nodes(211,1) = 0.;
          nodes(212,1) = -0.25;
          nodes(213,1) = -0.25;
          nodes(214,1) = -0.25;
          nodes(215,1) = 0.;
          nodes(216,1) = -0.25;
          nodes(217,1) = -0.25;
          nodes(218,1) = 0.;
          nodes(219,1) = 0.;
          nodes(220,1) = -0.25;
          nodes(221,1) = -0.25;
          nodes(222,1) = 0.;
          nodes(223,1) = -1.;
          nodes(224,1) = -0.75;
          nodes(225,1) = -1.;
          nodes(226,1) = -0.75;
          nodes(227,1) = -0.75;
          nodes(228,1) = -1.;
          nodes(229,1) = -0.75;
          nodes(230,1) = -0.5;
          nodes(231,1) = -0.5;
          nodes(232,1) = -0.5;
          nodes(233,1) = -1.;
          nodes(234,1) = -0.75;
          nodes(235,1) = -0.75;
          nodes(236,1) = -0.5;
          nodes(237,1) = -1.;
          nodes(238,1) = -0.75;
          nodes(239,1) = -1.;
          nodes(240,1) = -0.75;
          nodes(241,1) = -0.75;
          nodes(242,1) = -1.;
          nodes(243,1) = -0.75;
          nodes(244,1) = -0.5;
          nodes(245,1) = -0.5;
          nodes(246,1) = -0.5;
          nodes(247,1) = -1.;
          nodes(248,1) = -0.75;
          nodes(249,1) = -0.75;
          nodes(250,1) = -0.5;
          nodes(251,1) = -0.25;
          nodes(252,1) = -0.25;
          nodes(253,1) = -0.25;
          nodes(254,1) = -0.25;
          nodes(255,1) = 0.;
          nodes(256,1) = 0.;
          nodes(257,1) = 0.;
          nodes(258,1) = -0.25;
          nodes(259,1) = -0.25;
          nodes(260,1) = 0.;
          nodes(261,1) = -0.25;
          nodes(262,1) = -0.25;
          nodes(263,1) = -0.25;
          nodes(264,1) = -0.25;
          nodes(265,1) = 0.;
          nodes(266,1) = 0.;
          nodes(267,1) = 0.;
          nodes(268,1) = -0.25;
          nodes(269,1) = -0.25;
          nodes(270,1) = 0.;
          nodes(271,1) = -1.;
          nodes(272,1) = -0.75;
          nodes(273,1) = -1.;
          nodes(274,1) = -0.75;
          nodes(275,1) = -0.5;
          nodes(276,1) = -0.5;
          nodes(277,1) = -1.;
          nodes(278,1) = -0.75;
          nodes(279,1) = -0.75;
          nodes(280,1) = -0.5;
          nodes(281,1) = -1.;
          nodes(282,1) = -0.75;
          nodes(283,1) = -1.;
          nodes(284,1) = -0.75;
          nodes(285,1) = -0.5;
          nodes(286,1) = -0.5;
          nodes(287,1) = -1.;
          nodes(288,1) = -0.75;
          nodes(289,1) = -0.75;
          nodes(290,1) = -0.5;
          nodes(291,1) = -0.25;
          nodes(292,1) = -0.25;
          nodes(293,1) = 0.;
          nodes(294,1) = 0.;
          nodes(295,1) = -0.25;
          nodes(296,1) = -0.25;
          nodes(297,1) = 0.;
          nodes(298,1) = -0.25;
          nodes(299,1) = -0.25;
          nodes(300,1) = 0.;
          nodes(301,1) = 0.;
          nodes(302,1) = -0.25;
          nodes(303,1) = -0.25;
          nodes(304,1) = 0.;
          nodes(305,1) = 0.25;
          nodes(306,1) = 0.25;
          nodes(307,1) = 0.25;
          nodes(308,1) = 0.25;
          nodes(309,1) = 0.25;
          nodes(310,1) = 0.25;
          nodes(311,1) = 0.5;
          nodes(312,1) = 0.5;
          nodes(313,1) = 0.5;
          nodes(314,1) = 0.5;
          nodes(315,1) = 0.25;
          nodes(316,1) = 0.25;
          nodes(317,1) = 0.25;
          nodes(318,1) = 0.5;
          nodes(319,1) = 0.25;
          nodes(320,1) = 0.25;
          nodes(321,1) = 0.25;
          nodes(322,1) = 0.25;
          nodes(323,1) = 0.5;
          nodes(324,1) = 0.5;
          nodes(325,1) = 0.5;
          nodes(326,1) = 0.25;
          nodes(327,1) = 0.25;
          nodes(328,1) = 0.5;
          nodes(329,1) = 0.75;
          nodes(330,1) = 0.75;
          nodes(331,1) = 0.75;
          nodes(332,1) = 0.75;
          nodes(333,1) = 0.75;
          nodes(334,1) = 0.75;
          nodes(335,1) = 1.;
          nodes(336,1) = 1.;
          nodes(337,1) = 1.;
          nodes(338,1) = 1.;
          nodes(339,1) = 0.75;
          nodes(340,1) = 0.75;
          nodes(341,1) = 0.75;
          nodes(342,1) = 1.;
          nodes(343,1) = 0.75;
          nodes(344,1) = 0.75;
          nodes(345,1) = 0.75;
          nodes(346,1) = 0.75;
          nodes(347,1) = 1.;
          nodes(348,1) = 1.;
          nodes(349,1) = 1.;
          nodes(350,1) = 0.75;
          nodes(351,1) = 0.75;
          nodes(352,1) = 1.;
          nodes(353,1) = 0.25;
          nodes(354,1) = 0.25;
          nodes(355,1) = 0.25;
          nodes(356,1) = 0.5;
          nodes(357,1) = 0.5;
          nodes(358,1) = 0.5;
          nodes(359,1) = 0.25;
          nodes(360,1) = 0.25;
          nodes(361,1) = 0.25;
          nodes(362,1) = 0.5;
          nodes(363,1) = 0.25;
          nodes(364,1) = 0.25;
          nodes(365,1) = 0.5;
          nodes(366,1) = 0.5;
          nodes(367,1) = 0.25;
          nodes(368,1) = 0.25;
          nodes(369,1) = 0.5;
          nodes(370,1) = 0.75;
          nodes(371,1) = 0.75;
          nodes(372,1) = 0.75;
          nodes(373,1) = 1.;
          nodes(374,1) = 1.;
          nodes(375,1) = 1.;
          nodes(376,1) = 0.75;
          nodes(377,1) = 0.75;
          nodes(378,1) = 0.75;
          nodes(379,1) = 1.;
          nodes(380,1) = 0.75;
          nodes(381,1) = 0.75;
          nodes(382,1) = 1.;
          nodes(383,1) = 1.;
          nodes(384,1) = 0.75;
          nodes(385,1) = 0.75;
          nodes(386,1) = 1.;
          nodes(387,1) = 0.25;
          nodes(388,1) = 0.25;
          nodes(389,1) = 0.25;
          nodes(390,1) = 0.25;
          nodes(391,1) = 0.5;
          nodes(392,1) = 0.5;
          nodes(393,1) = 0.5;
          nodes(394,1) = 0.25;
          nodes(395,1) = 0.25;
          nodes(396,1) = 0.5;
          nodes(397,1) = 0.25;
          nodes(398,1) = 0.25;
          nodes(399,1) = 0.25;
          nodes(400,1) = 0.25;
          nodes(401,1) = 0.5;
          nodes(402,1) = 0.5;
          nodes(403,1) = 0.5;
          nodes(404,1) = 0.25;
          nodes(405,1) = 0.25;
          nodes(406,1) = 0.5;
          nodes(407,1) = 0.75;
          nodes(408,1) = 0.75;
          nodes(409,1) = 0.75;
          nodes(410,1) = 0.75;
          nodes(411,1) = 1.;
          nodes(412,1) = 1.;
          nodes(413,1) = 1.;
          nodes(414,1) = 0.75;
          nodes(415,1) = 0.75;
          nodes(416,1) = 1.;
          nodes(417,1) = 0.75;
          nodes(418,1) = 0.75;
          nodes(419,1) = 0.75;
          nodes(420,1) = 0.75;
          nodes(421,1) = 1.;
          nodes(422,1) = 1.;
          nodes(423,1) = 1.;
          nodes(424,1) = 0.75;
          nodes(425,1) = 0.75;
          nodes(426,1) = 1.;
          nodes(427,1) = 0.25;
          nodes(428,1) = 0.25;
          nodes(429,1) = 0.5;
          nodes(430,1) = 0.5;
          nodes(431,1) = 0.25;
          nodes(432,1) = 0.25;
          nodes(433,1) = 0.5;
          nodes(434,1) = 0.25;
          nodes(435,1) = 0.25;
          nodes(436,1) = 0.5;
          nodes(437,1) = 0.5;
          nodes(438,1) = 0.25;
          nodes(439,1) = 0.25;
          nodes(440,1) = 0.5;
          nodes(441,1) = 0.75;
          nodes(442,1) = 0.75;
          nodes(443,1) = 1.;
          nodes(444,1) = 1.;
          nodes(445,1) = 0.75;
          nodes(446,1) = 0.75;
          nodes(447,1) = 1.;
          nodes(448,1) = 0.75;
          nodes(449,1) = 0.75;
          nodes(450,1) = 1.;
          nodes(451,1) = 1.;
          nodes(452,1) = 0.75;
          nodes(453,1) = 0.75;
          nodes(454,1) = 1.;
          nodes(455,1) = -1.;
          nodes(456,1) = -1.;
          nodes(457,1) = -0.75;
          nodes(458,1) = -0.75;
          nodes(459,1) = -1.;
          nodes(460,1) = -0.75;
          nodes(461,1) = -0.5;
          nodes(462,1) = -0.5;
          nodes(463,1) = -0.5;
          nodes(464,1) = -1.;
          nodes(465,1) = -0.75;
          nodes(466,1) = -0.75;
          nodes(467,1) = -0.75;
          nodes(468,1) = -0.5;
          nodes(469,1) = -1.;
          nodes(470,1) = -0.75;
          nodes(471,1) = -1.;
          nodes(472,1) = -0.75;
          nodes(473,1) = -0.5;
          nodes(474,1) = -0.5;
          nodes(475,1) = -1.;
          nodes(476,1) = -0.75;
          nodes(477,1) = -0.75;
          nodes(478,1) = -0.5;
          nodes(479,1) = -0.25;
          nodes(480,1) = -0.25;
          nodes(481,1) = -0.25;
          nodes(482,1) = 0.;
          nodes(483,1) = 0.;
          nodes(484,1) = 0.;
          nodes(485,1) = -0.25;
          nodes(486,1) = -0.25;
          nodes(487,1) = -0.25;
          nodes(488,1) = 0.;
          nodes(489,1) = -0.25;
          nodes(490,1) = -0.25;
          nodes(491,1) = 0.;
          nodes(492,1) = 0.;
          nodes(493,1) = -0.25;
          nodes(494,1) = -0.25;
          nodes(495,1) = 0.;
          nodes(496,1) = -1.;
          nodes(497,1) = -1.;
          nodes(498,1) = -0.75;
          nodes(499,1) = -0.75;
          nodes(500,1) = -1.;
          nodes(501,1) = -0.75;
          nodes(502,1) = -0.5;
          nodes(503,1) = -0.5;
          nodes(504,1) = -0.5;
          nodes(505,1) = -1.;
          nodes(506,1) = -0.75;
          nodes(507,1) = -0.75;
          nodes(508,1) = -0.75;
          nodes(509,1) = -0.5;
          nodes(510,1) = -1.;
          nodes(511,1) = -0.75;
          nodes(512,1) = -1.;
          nodes(513,1) = -0.75;
          nodes(514,1) = -0.5;
          nodes(515,1) = -0.5;
          nodes(516,1) = -1.;
          nodes(517,1) = -0.75;
          nodes(518,1) = -0.75;
          nodes(519,1) = -0.5;
          nodes(520,1) = -0.25;
          nodes(521,1) = -0.25;
          nodes(522,1) = -0.25;
          nodes(523,1) = 0.;
          nodes(524,1) = 0.;
          nodes(525,1) = 0.;
          nodes(526,1) = -0.25;
          nodes(527,1) = -0.25;
          nodes(528,1) = -0.25;
          nodes(529,1) = 0.;
          nodes(530,1) = -0.25;
          nodes(531,1) = -0.25;
          nodes(532,1) = 0.;
          nodes(533,1) = 0.;
          nodes(534,1) = -0.25;
          nodes(535,1) = -0.25;
          nodes(536,1) = 0.;
          nodes(537,1) = -1.;
          nodes(538,1) = -0.75;
          nodes(539,1) = -1.;
          nodes(540,1) = -0.75;
          nodes(541,1) = -0.5;
          nodes(542,1) = -0.5;
          nodes(543,1) = -1.;
          nodes(544,1) = -0.75;
          nodes(545,1) = -0.75;
          nodes(546,1) = -0.5;
          nodes(547,1) = -1.;
          nodes(548,1) = -0.75;
          nodes(549,1) = -1.;
          nodes(550,1) = -0.75;
          nodes(551,1) = -0.5;
          nodes(552,1) = -0.5;
          nodes(553,1) = -1.;
          nodes(554,1) = -0.75;
          nodes(555,1) = -0.75;
          nodes(556,1) = -0.5;
          nodes(557,1) = -0.25;
          nodes(558,1) = -0.25;
          nodes(559,1) = 0.;
          nodes(560,1) = 0.;
          nodes(561,1) = -0.25;
          nodes(562,1) = -0.25;
          nodes(563,1) = 0.;
          nodes(564,1) = -0.25;
          nodes(565,1) = -0.25;
          nodes(566,1) = 0.;
          nodes(567,1) = 0.;
          nodes(568,1) = -0.25;
          nodes(569,1) = -0.25;
          nodes(570,1) = 0.;
          nodes(571,1) = -1.;
          nodes(572,1) = -0.75;
          nodes(573,1) = -1.;
          nodes(574,1) = -0.75;
          nodes(575,1) = -0.5;
          nodes(576,1) = -0.5;
          nodes(577,1) = -1.;
          nodes(578,1) = -0.75;
          nodes(579,1) = -0.75;
          nodes(580,1) = -0.5;
          nodes(581,1) = -1.;
          nodes(582,1) = -0.75;
          nodes(583,1) = -1.;
          nodes(584,1) = -0.75;
          nodes(585,1) = -0.5;
          nodes(586,1) = -0.5;
          nodes(587,1) = -1.;
          nodes(588,1) = -0.75;
          nodes(589,1) = -0.75;
          nodes(590,1) = -0.5;
          nodes(591,1) = -0.25;
          nodes(592,1) = -0.25;
          nodes(593,1) = 0.;
          nodes(594,1) = 0.;
          nodes(595,1) = -0.25;
          nodes(596,1) = -0.25;
          nodes(597,1) = 0.;
          nodes(598,1) = -0.25;
          nodes(599,1) = -0.25;
          nodes(600,1) = 0.;
          nodes(601,1) = 0.;
          nodes(602,1) = -0.25;
          nodes(603,1) = -0.25;
          nodes(604,1) = 0.;
          nodes(605,1) = 0.25;
          nodes(606,1) = 0.25;
          nodes(607,1) = 0.25;
          nodes(608,1) = 0.5;
          nodes(609,1) = 0.5;
          nodes(610,1) = 0.5;
          nodes(611,1) = 0.25;
          nodes(612,1) = 0.25;
          nodes(613,1) = 0.25;
          nodes(614,1) = 0.5;
          nodes(615,1) = 0.25;
          nodes(616,1) = 0.25;
          nodes(617,1) = 0.5;
          nodes(618,1) = 0.5;
          nodes(619,1) = 0.25;
          nodes(620,1) = 0.25;
          nodes(621,1) = 0.5;
          nodes(622,1) = 0.75;
          nodes(623,1) = 0.75;
          nodes(624,1) = 0.75;
          nodes(625,1) = 1.;
          nodes(626,1) = 1.;
          nodes(627,1) = 1.;
          nodes(628,1) = 0.75;
          nodes(629,1) = 0.75;
          nodes(630,1) = 0.75;
          nodes(631,1) = 1.;
          nodes(632,1) = 0.75;
          nodes(633,1) = 0.75;
          nodes(634,1) = 1.;
          nodes(635,1) = 1.;
          nodes(636,1) = 0.75;
          nodes(637,1) = 0.75;
          nodes(638,1) = 1.;
          nodes(639,1) = 0.25;
          nodes(640,1) = 0.25;
          nodes(641,1) = 0.25;
          nodes(642,1) = 0.5;
          nodes(643,1) = 0.5;
          nodes(644,1) = 0.5;
          nodes(645,1) = 0.25;
          nodes(646,1) = 0.25;
          nodes(647,1) = 0.25;
          nodes(648,1) = 0.5;
          nodes(649,1) = 0.25;
          nodes(650,1) = 0.25;
          nodes(651,1) = 0.5;
          nodes(652,1) = 0.5;
          nodes(653,1) = 0.25;
          nodes(654,1) = 0.25;
          nodes(655,1) = 0.5;
          nodes(656,1) = 0.75;
          nodes(657,1) = 0.75;
          nodes(658,1) = 0.75;
          nodes(659,1) = 1.;
          nodes(660,1) = 1.;
          nodes(661,1) = 1.;
          nodes(662,1) = 0.75;
          nodes(663,1) = 0.75;
          nodes(664,1) = 0.75;
          nodes(665,1) = 1.;
          nodes(666,1) = 0.75;
          nodes(667,1) = 0.75;
          nodes(668,1) = 1.;
          nodes(669,1) = 1.;
          nodes(670,1) = 0.75;
          nodes(671,1) = 0.75;
          nodes(672,1) = 1.;
          nodes(673,1) = 0.25;
          nodes(674,1) = 0.25;
          nodes(675,1) = 0.5;
          nodes(676,1) = 0.5;
          nodes(677,1) = 0.25;
          nodes(678,1) = 0.25;
          nodes(679,1) = 0.5;
          nodes(680,1) = 0.25;
          nodes(681,1) = 0.25;
          nodes(682,1) = 0.5;
          nodes(683,1) = 0.5;
          nodes(684,1) = 0.25;
          nodes(685,1) = 0.25;
          nodes(686,1) = 0.5;
          nodes(687,1) = 0.75;
          nodes(688,1) = 0.75;
          nodes(689,1) = 1.;
          nodes(690,1) = 1.;
          nodes(691,1) = 0.75;
          nodes(692,1) = 0.75;
          nodes(693,1) = 1.;
          nodes(694,1) = 0.75;
          nodes(695,1) = 0.75;
          nodes(696,1) = 1.;
          nodes(697,1) = 1.;
          nodes(698,1) = 0.75;
          nodes(699,1) = 0.75;
          nodes(700,1) = 1.;
          nodes(701,1) = 0.25;
          nodes(702,1) = 0.25;
          nodes(703,1) = 0.5;
          nodes(704,1) = 0.5;
          nodes(705,1) = 0.25;
          nodes(706,1) = 0.25;
          nodes(707,1) = 0.5;
          nodes(708,1) = 0.25;
          nodes(709,1) = 0.25;
          nodes(710,1) = 0.5;
          nodes(711,1) = 0.5;
          nodes(712,1) = 0.25;
          nodes(713,1) = 0.25;
          nodes(714,1) = 0.5;
          nodes(715,1) = 0.75;
          nodes(716,1) = 0.75;
          nodes(717,1) = 1.;
          nodes(718,1) = 1.;
          nodes(719,1) = 0.75;
          nodes(720,1) = 0.75;
          nodes(721,1) = 1.;
          nodes(722,1) = 0.75;
          nodes(723,1) = 0.75;
          nodes(724,1) = 1.;
          nodes(725,1) = 1.;
          nodes(726,1) = 0.75;
          nodes(727,1) = 0.75;
          nodes(728,1) = 1.;
          nodes(0,2) = -1.;
          nodes(1,2) = -1.;
          nodes(2,2) = -1.;
          nodes(3,2) = -1.;
          nodes(4,2) = 1.;
          nodes(5,2) = 1.;
          nodes(6,2) = 1.;
          nodes(7,2) = 1.;
          nodes(8,2) = -1.;
          nodes(9,2) = -1.;
          nodes(10,2) = -1.;
          nodes(11,2) = 0.;
          nodes(12,2) = 0.;
          nodes(13,2) = 0.;
          nodes(14,2) = 0.;
          nodes(15,2) = -1.;
          nodes(16,2) = 0.;
          nodes(17,2) = 0.;
          nodes(18,2) = -1.;
          nodes(19,2) = 0.;
          nodes(20,2) = 0.;
          nodes(21,2) = 0.;
          nodes(22,2) = 1.;
          nodes(23,2) = 1.;
          nodes(24,2) = 1.;
          nodes(25,2) = 1.;
          nodes(26,2) = 1.;
          nodes(27,2) = -1.;
          nodes(28,2) = -1.;
          nodes(29,2) = -1.;
          nodes(30,2) = -0.5;
          nodes(31,2) = -0.5;
          nodes(32,2) = -0.5;
          nodes(33,2) = -0.5;
          nodes(34,2) = -1.;
          nodes(35,2) = -0.5;
          nodes(36,2) = -0.5;
          nodes(37,2) = -1.;
          nodes(38,2) = -0.5;
          nodes(39,2) = -0.5;
          nodes(40,2) = -0.5;
          nodes(41,2) = 0.;
          nodes(42,2) = 0.;
          nodes(43,2) = 0.;
          nodes(44,2) = 0.;
          nodes(45,2) = 0.;
          nodes(46,2) = -1.;
          nodes(47,2) = -1.;
          nodes(48,2) = -0.5;
          nodes(49,2) = -0.5;
          nodes(50,2) = -1.;
          nodes(51,2) = -0.5;
          nodes(52,2) = -0.5;
          nodes(53,2) = -1.;
          nodes(54,2) = -0.5;
          nodes(55,2) = -0.5;
          nodes(56,2) = 0.;
          nodes(57,2) = 0.;
          nodes(58,2) = 0.;
          nodes(59,2) = 0.;
          nodes(60,2) = -1.;
          nodes(61,2) = -1.;
          nodes(62,2) = -0.5;
          nodes(63,2) = -0.5;
          nodes(64,2) = -1.;
          nodes(65,2) = -0.5;
          nodes(66,2) = -1.;
          nodes(67,2) = -0.5;
          nodes(68,2) = -0.5;
          nodes(69,2) = -0.5;
          nodes(70,2) = 0.;
          nodes(71,2) = 0.;
          nodes(72,2) = 0.;
          nodes(73,2) = 0.;
          nodes(74,2) = -1.;
          nodes(75,2) = -0.5;
          nodes(76,2) = -1.;
          nodes(77,2) = -0.5;
          nodes(78,2) = -1.;
          nodes(79,2) = -0.5;
          nodes(80,2) = -0.5;
          nodes(81,2) = 0.;
          nodes(82,2) = 0.;
          nodes(83,2) = 0.;
          nodes(84,2) = 0.5;
          nodes(85,2) = 0.5;
          nodes(86,2) = 0.5;
          nodes(87,2) = 0.5;
          nodes(88,2) = 0.5;
          nodes(89,2) = 0.5;
          nodes(90,2) = 0.5;
          nodes(91,2) = 0.5;
          nodes(92,2) = 0.5;
          nodes(93,2) = 1.;
          nodes(94,2) = 1.;
          nodes(95,2) = 1.;
          nodes(96,2) = 1.;
          nodes(97,2) = 1.;
          nodes(98,2) = 0.5;
          nodes(99,2) = 0.5;
          nodes(100,2) = 0.5;
          nodes(101,2) = 0.5;
          nodes(102,2) = 0.5;
          nodes(103,2) = 0.5;
          nodes(104,2) = 1.;
          nodes(105,2) = 1.;
          nodes(106,2) = 1.;
          nodes(107,2) = 1.;
          nodes(108,2) = 0.5;
          nodes(109,2) = 0.5;
          nodes(110,2) = 0.5;
          nodes(111,2) = 0.5;
          nodes(112,2) = 0.5;
          nodes(113,2) = 0.5;
          nodes(114,2) = 1.;
          nodes(115,2) = 1.;
          nodes(116,2) = 1.;
          nodes(117,2) = 1.;
          nodes(118,2) = 0.5;
          nodes(119,2) = 0.5;
          nodes(120,2) = 0.5;
          nodes(121,2) = 0.5;
          nodes(122,2) = 1.;
          nodes(123,2) = 1.;
          nodes(124,2) = 1.;
          nodes(125,2) = -1.;
          nodes(126,2) = -1.;
          nodes(127,2) = -1.;
          nodes(128,2) = -0.75;
          nodes(129,2) = -0.75;
          nodes(130,2) = -0.75;
          nodes(131,2) = -0.75;
          nodes(132,2) = -1.;
          nodes(133,2) = -0.75;
          nodes(134,2) = -0.75;
          nodes(135,2) = -1.;
          nodes(136,2) = -0.75;
          nodes(137,2) = -0.75;
          nodes(138,2) = -0.75;
          nodes(139,2) = -0.5;
          nodes(140,2) = -0.5;
          nodes(141,2) = -0.5;
          nodes(142,2) = -0.5;
          nodes(143,2) = -0.5;
          nodes(144,2) = -1.;
          nodes(145,2) = -1.;
          nodes(146,2) = -0.75;
          nodes(147,2) = -0.75;
          nodes(148,2) = -1.;
          nodes(149,2) = -0.75;
          nodes(150,2) = -0.75;
          nodes(151,2) = -1.;
          nodes(152,2) = -0.75;
          nodes(153,2) = -0.75;
          nodes(154,2) = -0.5;
          nodes(155,2) = -0.5;
          nodes(156,2) = -0.5;
          nodes(157,2) = -0.5;
          nodes(158,2) = -1.;
          nodes(159,2) = -1.;
          nodes(160,2) = -0.75;
          nodes(161,2) = -0.75;
          nodes(162,2) = -1.;
          nodes(163,2) = -0.75;
          nodes(164,2) = -1.;
          nodes(165,2) = -0.75;
          nodes(166,2) = -0.75;
          nodes(167,2) = -0.75;
          nodes(168,2) = -0.5;
          nodes(169,2) = -0.5;
          nodes(170,2) = -0.5;
          nodes(171,2) = -0.5;
          nodes(172,2) = -1.;
          nodes(173,2) = -0.75;
          nodes(174,2) = -1.;
          nodes(175,2) = -0.75;
          nodes(176,2) = -1.;
          nodes(177,2) = -0.75;
          nodes(178,2) = -0.75;
          nodes(179,2) = -0.5;
          nodes(180,2) = -0.5;
          nodes(181,2) = -0.5;
          nodes(182,2) = -0.25;
          nodes(183,2) = -0.25;
          nodes(184,2) = -0.25;
          nodes(185,2) = -0.25;
          nodes(186,2) = -0.25;
          nodes(187,2) = -0.25;
          nodes(188,2) = -0.25;
          nodes(189,2) = -0.25;
          nodes(190,2) = -0.25;
          nodes(191,2) = 0.;
          nodes(192,2) = 0.;
          nodes(193,2) = 0.;
          nodes(194,2) = 0.;
          nodes(195,2) = 0.;
          nodes(196,2) = -0.25;
          nodes(197,2) = -0.25;
          nodes(198,2) = -0.25;
          nodes(199,2) = -0.25;
          nodes(200,2) = -0.25;
          nodes(201,2) = -0.25;
          nodes(202,2) = 0.;
          nodes(203,2) = 0.;
          nodes(204,2) = 0.;
          nodes(205,2) = 0.;
          nodes(206,2) = -0.25;
          nodes(207,2) = -0.25;
          nodes(208,2) = -0.25;
          nodes(209,2) = -0.25;
          nodes(210,2) = -0.25;
          nodes(211,2) = -0.25;
          nodes(212,2) = 0.;
          nodes(213,2) = 0.;
          nodes(214,2) = 0.;
          nodes(215,2) = 0.;
          nodes(216,2) = -0.25;
          nodes(217,2) = -0.25;
          nodes(218,2) = -0.25;
          nodes(219,2) = -0.25;
          nodes(220,2) = 0.;
          nodes(221,2) = 0.;
          nodes(222,2) = 0.;
          nodes(223,2) = -1.;
          nodes(224,2) = -1.;
          nodes(225,2) = -0.75;
          nodes(226,2) = -0.75;
          nodes(227,2) = -1.;
          nodes(228,2) = -0.75;
          nodes(229,2) = -0.75;
          nodes(230,2) = -1.;
          nodes(231,2) = -0.75;
          nodes(232,2) = -0.75;
          nodes(233,2) = -0.5;
          nodes(234,2) = -0.5;
          nodes(235,2) = -0.5;
          nodes(236,2) = -0.5;
          nodes(237,2) = -1.;
          nodes(238,2) = -1.;
          nodes(239,2) = -0.75;
          nodes(240,2) = -0.75;
          nodes(241,2) = -1.;
          nodes(242,2) = -0.75;
          nodes(243,2) = -0.75;
          nodes(244,2) = -1.;
          nodes(245,2) = -0.75;
          nodes(246,2) = -0.75;
          nodes(247,2) = -0.5;
          nodes(248,2) = -0.5;
          nodes(249,2) = -0.5;
          nodes(250,2) = -0.5;
          nodes(251,2) = -1.;
          nodes(252,2) = -0.75;
          nodes(253,2) = -1.;
          nodes(254,2) = -0.75;
          nodes(255,2) = -1.;
          nodes(256,2) = -0.75;
          nodes(257,2) = -0.75;
          nodes(258,2) = -0.5;
          nodes(259,2) = -0.5;
          nodes(260,2) = -0.5;
          nodes(261,2) = -1.;
          nodes(262,2) = -0.75;
          nodes(263,2) = -1.;
          nodes(264,2) = -0.75;
          nodes(265,2) = -1.;
          nodes(266,2) = -0.75;
          nodes(267,2) = -0.75;
          nodes(268,2) = -0.5;
          nodes(269,2) = -0.5;
          nodes(270,2) = -0.5;
          nodes(271,2) = -0.25;
          nodes(272,2) = -0.25;
          nodes(273,2) = -0.25;
          nodes(274,2) = -0.25;
          nodes(275,2) = -0.25;
          nodes(276,2) = -0.25;
          nodes(277,2) = 0.;
          nodes(278,2) = 0.;
          nodes(279,2) = 0.;
          nodes(280,2) = 0.;
          nodes(281,2) = -0.25;
          nodes(282,2) = -0.25;
          nodes(283,2) = -0.25;
          nodes(284,2) = -0.25;
          nodes(285,2) = -0.25;
          nodes(286,2) = -0.25;
          nodes(287,2) = 0.;
          nodes(288,2) = 0.;
          nodes(289,2) = 0.;
          nodes(290,2) = 0.;
          nodes(291,2) = -0.25;
          nodes(292,2) = -0.25;
          nodes(293,2) = -0.25;
          nodes(294,2) = -0.25;
          nodes(295,2) = 0.;
          nodes(296,2) = 0.;
          nodes(297,2) = 0.;
          nodes(298,2) = -0.25;
          nodes(299,2) = -0.25;
          nodes(300,2) = -0.25;
          nodes(301,2) = -0.25;
          nodes(302,2) = 0.;
          nodes(303,2) = 0.;
          nodes(304,2) = 0.;
          nodes(305,2) = -1.;
          nodes(306,2) = -1.;
          nodes(307,2) = -0.75;
          nodes(308,2) = -0.75;
          nodes(309,2) = -1.;
          nodes(310,2) = -0.75;
          nodes(311,2) = -1.;
          nodes(312,2) = -0.75;
          nodes(313,2) = -0.75;
          nodes(314,2) = -0.75;
          nodes(315,2) = -0.5;
          nodes(316,2) = -0.5;
          nodes(317,2) = -0.5;
          nodes(318,2) = -0.5;
          nodes(319,2) = -1.;
          nodes(320,2) = -0.75;
          nodes(321,2) = -1.;
          nodes(322,2) = -0.75;
          nodes(323,2) = -1.;
          nodes(324,2) = -0.75;
          nodes(325,2) = -0.75;
          nodes(326,2) = -0.5;
          nodes(327,2) = -0.5;
          nodes(328,2) = -0.5;
          nodes(329,2) = -1.;
          nodes(330,2) = -1.;
          nodes(331,2) = -0.75;
          nodes(332,2) = -0.75;
          nodes(333,2) = -1.;
          nodes(334,2) = -0.75;
          nodes(335,2) = -1.;
          nodes(336,2) = -0.75;
          nodes(337,2) = -0.75;
          nodes(338,2) = -0.75;
          nodes(339,2) = -0.5;
          nodes(340,2) = -0.5;
          nodes(341,2) = -0.5;
          nodes(342,2) = -0.5;
          nodes(343,2) = -1.;
          nodes(344,2) = -0.75;
          nodes(345,2) = -1.;
          nodes(346,2) = -0.75;
          nodes(347,2) = -1.;
          nodes(348,2) = -0.75;
          nodes(349,2) = -0.75;
          nodes(350,2) = -0.5;
          nodes(351,2) = -0.5;
          nodes(352,2) = -0.5;
          nodes(353,2) = -0.25;
          nodes(354,2) = -0.25;
          nodes(355,2) = -0.25;
          nodes(356,2) = -0.25;
          nodes(357,2) = -0.25;
          nodes(358,2) = -0.25;
          nodes(359,2) = 0.;
          nodes(360,2) = 0.;
          nodes(361,2) = 0.;
          nodes(362,2) = 0.;
          nodes(363,2) = -0.25;
          nodes(364,2) = -0.25;
          nodes(365,2) = -0.25;
          nodes(366,2) = -0.25;
          nodes(367,2) = 0.;
          nodes(368,2) = 0.;
          nodes(369,2) = 0.;
          nodes(370,2) = -0.25;
          nodes(371,2) = -0.25;
          nodes(372,2) = -0.25;
          nodes(373,2) = -0.25;
          nodes(374,2) = -0.25;
          nodes(375,2) = -0.25;
          nodes(376,2) = 0.;
          nodes(377,2) = 0.;
          nodes(378,2) = 0.;
          nodes(379,2) = 0.;
          nodes(380,2) = -0.25;
          nodes(381,2) = -0.25;
          nodes(382,2) = -0.25;
          nodes(383,2) = -0.25;
          nodes(384,2) = 0.;
          nodes(385,2) = 0.;
          nodes(386,2) = 0.;
          nodes(387,2) = -1.;
          nodes(388,2) = -0.75;
          nodes(389,2) = -1.;
          nodes(390,2) = -0.75;
          nodes(391,2) = -1.;
          nodes(392,2) = -0.75;
          nodes(393,2) = -0.75;
          nodes(394,2) = -0.5;
          nodes(395,2) = -0.5;
          nodes(396,2) = -0.5;
          nodes(397,2) = -1.;
          nodes(398,2) = -0.75;
          nodes(399,2) = -1.;
          nodes(400,2) = -0.75;
          nodes(401,2) = -1.;
          nodes(402,2) = -0.75;
          nodes(403,2) = -0.75;
          nodes(404,2) = -0.5;
          nodes(405,2) = -0.5;
          nodes(406,2) = -0.5;
          nodes(407,2) = -1.;
          nodes(408,2) = -0.75;
          nodes(409,2) = -1.;
          nodes(410,2) = -0.75;
          nodes(411,2) = -1.;
          nodes(412,2) = -0.75;
          nodes(413,2) = -0.75;
          nodes(414,2) = -0.5;
          nodes(415,2) = -0.5;
          nodes(416,2) = -0.5;
          nodes(417,2) = -1.;
          nodes(418,2) = -0.75;
          nodes(419,2) = -1.;
          nodes(420,2) = -0.75;
          nodes(421,2) = -1.;
          nodes(422,2) = -0.75;
          nodes(423,2) = -0.75;
          nodes(424,2) = -0.5;
          nodes(425,2) = -0.5;
          nodes(426,2) = -0.5;
          nodes(427,2) = -0.25;
          nodes(428,2) = -0.25;
          nodes(429,2) = -0.25;
          nodes(430,2) = -0.25;
          nodes(431,2) = 0.;
          nodes(432,2) = 0.;
          nodes(433,2) = 0.;
          nodes(434,2) = -0.25;
          nodes(435,2) = -0.25;
          nodes(436,2) = -0.25;
          nodes(437,2) = -0.25;
          nodes(438,2) = 0.;
          nodes(439,2) = 0.;
          nodes(440,2) = 0.;
          nodes(441,2) = -0.25;
          nodes(442,2) = -0.25;
          nodes(443,2) = -0.25;
          nodes(444,2) = -0.25;
          nodes(445,2) = 0.;
          nodes(446,2) = 0.;
          nodes(447,2) = 0.;
          nodes(448,2) = -0.25;
          nodes(449,2) = -0.25;
          nodes(450,2) = -0.25;
          nodes(451,2) = -0.25;
          nodes(452,2) = 0.;
          nodes(453,2) = 0.;
          nodes(454,2) = 0.;
          nodes(455,2) = 0.25;
          nodes(456,2) = 0.25;
          nodes(457,2) = 0.25;
          nodes(458,2) = 0.25;
          nodes(459,2) = 0.25;
          nodes(460,2) = 0.25;
          nodes(461,2) = 0.25;
          nodes(462,2) = 0.25;
          nodes(463,2) = 0.25;
          nodes(464,2) = 0.5;
          nodes(465,2) = 0.5;
          nodes(466,2) = 0.5;
          nodes(467,2) = 0.5;
          nodes(468,2) = 0.5;
          nodes(469,2) = 0.25;
          nodes(470,2) = 0.25;
          nodes(471,2) = 0.25;
          nodes(472,2) = 0.25;
          nodes(473,2) = 0.25;
          nodes(474,2) = 0.25;
          nodes(475,2) = 0.5;
          nodes(476,2) = 0.5;
          nodes(477,2) = 0.5;
          nodes(478,2) = 0.5;
          nodes(479,2) = 0.25;
          nodes(480,2) = 0.25;
          nodes(481,2) = 0.25;
          nodes(482,2) = 0.25;
          nodes(483,2) = 0.25;
          nodes(484,2) = 0.25;
          nodes(485,2) = 0.5;
          nodes(486,2) = 0.5;
          nodes(487,2) = 0.5;
          nodes(488,2) = 0.5;
          nodes(489,2) = 0.25;
          nodes(490,2) = 0.25;
          nodes(491,2) = 0.25;
          nodes(492,2) = 0.25;
          nodes(493,2) = 0.5;
          nodes(494,2) = 0.5;
          nodes(495,2) = 0.5;
          nodes(496,2) = 0.75;
          nodes(497,2) = 0.75;
          nodes(498,2) = 0.75;
          nodes(499,2) = 0.75;
          nodes(500,2) = 0.75;
          nodes(501,2) = 0.75;
          nodes(502,2) = 0.75;
          nodes(503,2) = 0.75;
          nodes(504,2) = 0.75;
          nodes(505,2) = 1.;
          nodes(506,2) = 1.;
          nodes(507,2) = 1.;
          nodes(508,2) = 1.;
          nodes(509,2) = 1.;
          nodes(510,2) = 0.75;
          nodes(511,2) = 0.75;
          nodes(512,2) = 0.75;
          nodes(513,2) = 0.75;
          nodes(514,2) = 0.75;
          nodes(515,2) = 0.75;
          nodes(516,2) = 1.;
          nodes(517,2) = 1.;
          nodes(518,2) = 1.;
          nodes(519,2) = 1.;
          nodes(520,2) = 0.75;
          nodes(521,2) = 0.75;
          nodes(522,2) = 0.75;
          nodes(523,2) = 0.75;
          nodes(524,2) = 0.75;
          nodes(525,2) = 0.75;
          nodes(526,2) = 1.;
          nodes(527,2) = 1.;
          nodes(528,2) = 1.;
          nodes(529,2) = 1.;
          nodes(530,2) = 0.75;
          nodes(531,2) = 0.75;
          nodes(532,2) = 0.75;
          nodes(533,2) = 0.75;
          nodes(534,2) = 1.;
          nodes(535,2) = 1.;
          nodes(536,2) = 1.;
          nodes(537,2) = 0.25;
          nodes(538,2) = 0.25;
          nodes(539,2) = 0.25;
          nodes(540,2) = 0.25;
          nodes(541,2) = 0.25;
          nodes(542,2) = 0.25;
          nodes(543,2) = 0.5;
          nodes(544,2) = 0.5;
          nodes(545,2) = 0.5;
          nodes(546,2) = 0.5;
          nodes(547,2) = 0.25;
          nodes(548,2) = 0.25;
          nodes(549,2) = 0.25;
          nodes(550,2) = 0.25;
          nodes(551,2) = 0.25;
          nodes(552,2) = 0.25;
          nodes(553,2) = 0.5;
          nodes(554,2) = 0.5;
          nodes(555,2) = 0.5;
          nodes(556,2) = 0.5;
          nodes(557,2) = 0.25;
          nodes(558,2) = 0.25;
          nodes(559,2) = 0.25;
          nodes(560,2) = 0.25;
          nodes(561,2) = 0.5;
          nodes(562,2) = 0.5;
          nodes(563,2) = 0.5;
          nodes(564,2) = 0.25;
          nodes(565,2) = 0.25;
          nodes(566,2) = 0.25;
          nodes(567,2) = 0.25;
          nodes(568,2) = 0.5;
          nodes(569,2) = 0.5;
          nodes(570,2) = 0.5;
          nodes(571,2) = 0.75;
          nodes(572,2) = 0.75;
          nodes(573,2) = 0.75;
          nodes(574,2) = 0.75;
          nodes(575,2) = 0.75;
          nodes(576,2) = 0.75;
          nodes(577,2) = 1.;
          nodes(578,2) = 1.;
          nodes(579,2) = 1.;
          nodes(580,2) = 1.;
          nodes(581,2) = 0.75;
          nodes(582,2) = 0.75;
          nodes(583,2) = 0.75;
          nodes(584,2) = 0.75;
          nodes(585,2) = 0.75;
          nodes(586,2) = 0.75;
          nodes(587,2) = 1.;
          nodes(588,2) = 1.;
          nodes(589,2) = 1.;
          nodes(590,2) = 1.;
          nodes(591,2) = 0.75;
          nodes(592,2) = 0.75;
          nodes(593,2) = 0.75;
          nodes(594,2) = 0.75;
          nodes(595,2) = 1.;
          nodes(596,2) = 1.;
          nodes(597,2) = 1.;
          nodes(598,2) = 0.75;
          nodes(599,2) = 0.75;
          nodes(600,2) = 0.75;
          nodes(601,2) = 0.75;
          nodes(602,2) = 1.;
          nodes(603,2) = 1.;
          nodes(604,2) = 1.;
          nodes(605,2) = 0.25;
          nodes(606,2) = 0.25;
          nodes(607,2) = 0.25;
          nodes(608,2) = 0.25;
          nodes(609,2) = 0.25;
          nodes(610,2) = 0.25;
          nodes(611,2) = 0.5;
          nodes(612,2) = 0.5;
          nodes(613,2) = 0.5;
          nodes(614,2) = 0.5;
          nodes(615,2) = 0.25;
          nodes(616,2) = 0.25;
          nodes(617,2) = 0.25;
          nodes(618,2) = 0.25;
          nodes(619,2) = 0.5;
          nodes(620,2) = 0.5;
          nodes(621,2) = 0.5;
          nodes(622,2) = 0.25;
          nodes(623,2) = 0.25;
          nodes(624,2) = 0.25;
          nodes(625,2) = 0.25;
          nodes(626,2) = 0.25;
          nodes(627,2) = 0.25;
          nodes(628,2) = 0.5;
          nodes(629,2) = 0.5;
          nodes(630,2) = 0.5;
          nodes(631,2) = 0.5;
          nodes(632,2) = 0.25;
          nodes(633,2) = 0.25;
          nodes(634,2) = 0.25;
          nodes(635,2) = 0.25;
          nodes(636,2) = 0.5;
          nodes(637,2) = 0.5;
          nodes(638,2) = 0.5;
          nodes(639,2) = 0.75;
          nodes(640,2) = 0.75;
          nodes(641,2) = 0.75;
          nodes(642,2) = 0.75;
          nodes(643,2) = 0.75;
          nodes(644,2) = 0.75;
          nodes(645,2) = 1.;
          nodes(646,2) = 1.;
          nodes(647,2) = 1.;
          nodes(648,2) = 1.;
          nodes(649,2) = 0.75;
          nodes(650,2) = 0.75;
          nodes(651,2) = 0.75;
          nodes(652,2) = 0.75;
          nodes(653,2) = 1.;
          nodes(654,2) = 1.;
          nodes(655,2) = 1.;
          nodes(656,2) = 0.75;
          nodes(657,2) = 0.75;
          nodes(658,2) = 0.75;
          nodes(659,2) = 0.75;
          nodes(660,2) = 0.75;
          nodes(661,2) = 0.75;
          nodes(662,2) = 1.;
          nodes(663,2) = 1.;
          nodes(664,2) = 1.;
          nodes(665,2) = 1.;
          nodes(666,2) = 0.75;
          nodes(667,2) = 0.75;
          nodes(668,2) = 0.75;
          nodes(669,2) = 0.75;
          nodes(670,2) = 1.;
          nodes(671,2) = 1.;
          nodes(672,2) = 1.;
          nodes(673,2) = 0.25;
          nodes(674,2) = 0.25;
          nodes(675,2) = 0.25;
          nodes(676,2) = 0.25;
          nodes(677,2) = 0.5;
          nodes(678,2) = 0.5;
          nodes(679,2) = 0.5;
          nodes(680,2) = 0.25;
          nodes(681,2) = 0.25;
          nodes(682,2) = 0.25;
          nodes(683,2) = 0.25;
          nodes(684,2) = 0.5;
          nodes(685,2) = 0.5;
          nodes(686,2) = 0.5;
          nodes(687,2) = 0.25;
          nodes(688,2) = 0.25;
          nodes(689,2) = 0.25;
          nodes(690,2) = 0.25;
          nodes(691,2) = 0.5;
          nodes(692,2) = 0.5;
          nodes(693,2) = 0.5;
          nodes(694,2) = 0.25;
          nodes(695,2) = 0.25;
          nodes(696,2) = 0.25;
          nodes(697,2) = 0.25;
          nodes(698,2) = 0.5;
          nodes(699,2) = 0.5;
          nodes(700,2) = 0.5;
          nodes(701,2) = 0.75;
          nodes(702,2) = 0.75;
          nodes(703,2) = 0.75;
          nodes(704,2) = 0.75;
          nodes(705,2) = 1.;
          nodes(706,2) = 1.;
          nodes(707,2) = 1.;
          nodes(708,2) = 0.75;
          nodes(709,2) = 0.75;
          nodes(710,2) = 0.75;
          nodes(711,2) = 0.75;
          nodes(712,2) = 1.;
          nodes(713,2) = 1.;
          nodes(714,2) = 1.;
          nodes(715,2) = 0.75;
          nodes(716,2) = 0.75;
          nodes(717,2) = 0.75;
          nodes(718,2) = 0.75;
          nodes(719,2) = 1.;
          nodes(720,2) = 1.;
          nodes(721,2) = 1.;
          nodes(722,2) = 0.75;
          nodes(723,2) = 0.75;
          nodes(724,2) = 0.75;
          nodes(725,2) = 0.75;
          nodes(726,2) = 1.;
          nodes(727,2) = 1.;
          nodes(728,2) = 1.;
          return;
        }
      }
    }
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return;
    }
  }
}

unsigned int VTKHelperSpaMesh::getNumberOfSubElementsUnlimited(const ElemType& elemType, const unsigned int& order)
{
  switch(elemType)
  {
    case EDGE2:
    case EDGE3:
    {
      return order*order;
    }
    case TRI3:
    case TRI6:
    case QUAD4:
    case QUAD8:
    case QUAD9:
    {
      return order*order;
    } 
    case PRISM6:
    case PRISM15:
    case TET4:
    case TET10:
    case HEX8:
    case HEX20:
    case HEX27:
    case PYRAMID5:
    case PYRAMID13:
    {
      return order*order*order;
    } 
    default:
    {
      std::cerr<<"Error in VTKWriter: itemtype "<<elemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}

unsigned int VTKHelperSpaMesh::getNumberOfSubElementsUnlimited(const ElemTypesMesh& itemType, const unsigned int& order)
{
  switch(itemType)
  {
    case SpaMesh::line:
    {
      return order*order;
    }
    case SpaMesh::tri:
    case SpaMesh::quad:
    {
      return order*order;
    } 
    case SpaMesh::prism:
    case SpaMesh::tet:
    case SpaMesh::hex:
    case SpaMesh::pyram:
    {
      return order*order*order; 
    } 
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}

unsigned int VTKHelperSpaMesh::getNumberOfSubElements(const ElemType& elemType, const unsigned int& order)
{
  switch(elemType)
  {
    case EDGE2:
    case EDGE3:
    {
      unsigned int orderSquared = order*order;
      switch(orderSquared)
      {
        case 0:
        case 1:
        {
          return 1;
        }
	default:
	{
	  return orderSquared;
	}
      }
    }
    case TRI3:
    case TRI6:
    case QUAD4:
    case QUAD8:
    case QUAD9:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 1;
        }
        case 2:
        {
          return 4;
        }
        case 3:
        {
          return 9;
        }
        case 4:
        {
          return 16;
        }
        case 5:
        {
          return 25;
        }
        default:
        {
          return 36;
        }
      }
    } 
    case PRISM6:
    case PRISM15:
    case TET4:
    case TET10:
    case HEX8:
    case HEX20:
    case HEX27:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 1;
        }
        case 2:
        {
          return 8;
        }
        case 3:
        case 4:
        {
          return 64;
        }
        default: 
        {
          return 512;
        }
      }
    } 
    case PYRAMID5:
    case PYRAMID13:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 2;
        }
        case 2:
        {
          return 16;
        }
        case 3:
        case 4:
        {
          return 128;
        }
        default: 
        {
          return 1024;
        }
      }
    } 
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<elemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}

unsigned int VTKHelperSpaMesh::getNumberOfSubElements(const ElemTypesMesh& itemType, const unsigned int& order)
{
  switch(itemType)
  {
    case SpaMesh::line:
    {
      unsigned int orderSquared = order*order;
      switch(orderSquared)
      {
        case 0:
        case 1:
        {
          return 1;
        }
	default:
	{
	  return orderSquared;
	}
      }
    }
    case SpaMesh::tri:
    case SpaMesh::quad:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 1;
        }
        case 2:
        {
          return 4;
        }
        case 3:
        {
          return 9;
        }
        case 4:
        {
          return 16;
        }
        case 5:
        {
          return 25;
        }
        default:
        {
          return 36;
        }
      }
    } 
    case SpaMesh::prism:
    case SpaMesh::tet:
    case SpaMesh::hex:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 1;
        }
        case 2:
        {
          return 8;
        }
        case 3:
        case 4:
        {
          return 64;
        }
        default: 
        {
          return 512;
        }
      }
    } 
    case SpaMesh::pyram:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          return 2;
        }
        case 2:
        {
          return 16;
        }
        case 3:
        case 4:
        {
          return 128;
        }
        default: 
        {
          return 1024;
        }
      }
    } 
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}

unsigned int VTKHelperSpaMesh::getSubElementsTotalNodesNumber(const ElemType& elemType, const unsigned int& order, unsigned int& subElementNodesNumber)
{
  switch(elemType)
  {
    case EDGE2:
    case EDGE3:
    {
      subElementNodesNumber = 2;
      unsigned int orderSquared = order*order;
      switch(orderSquared)
      {
        case 0: 
        case 1:
        {
          return 2;
        }
        default:
        {
          return orderSquared+1;
        }
      }
    }
    case TRI3:
    case TRI6:
    {
      subElementNodesNumber = 3;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 3;
        }
        case 2:
        {
          return 6;
        }
        case 3:
        {
          return 10;
        }
        case 4:
        {
          return 15;
        }
        case 5:
        {
          return 21;
        }
        default:
        {
          return 28;
        }
      }
    } 
    case QUAD4:
    case QUAD8:
    case QUAD9:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 4;
        }
        case 2:
        {
          return 9;
        }
        case 3:
        {
          return 16;
        }
        case 4:
        {
          return 25;
        }
        case 5:
        {
          return 36;
        }
        default:
        {
          return 49;
        }
      }
    }
    case PRISM6:
    case PRISM15:
    {
      subElementNodesNumber = 6;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 6;
        }
        case 2:
        {
          return 18;
        }
        case 3:
        case 4:
        {
          return 75;
        }
        default:
        {
          return 405;
        }
      }
    } 
    case PYRAMID5:
    case PYRAMID13:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 5;
        }
        case 2:
        {
          return 14;
        }
        case 3:
        case 4:
        {
          return 55;
        }
        default: 
        {
          return 285;
        }
      }
    } 
    case TET4:
    case TET10:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 4;
        }
        case 2:
        {
          return 10;
        }
        case 3:
        case 4:
        {
          return 35;
        }
        default: 
        {
          return 165;
        }
      }
    } 
    case HEX8:
    case HEX20:
    case HEX27:
    {
      subElementNodesNumber = 8;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 8;
        }
        case 2:
        {
          return 27;    
        }
        case 3:
        case 4:
        {
          return 125;
        }
        default: 
        {
          return 729;
        }
      }
    }
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: elemtype "<<elemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}

unsigned int VTKHelperSpaMesh::getSubElementsTotalNodesNumber(const ElemTypesMesh& itemType, const unsigned int& order, unsigned int& subElementNodesNumber)
{
  switch(itemType)
  {
    case SpaMesh::line:
    {
      subElementNodesNumber = 2;
      unsigned int orderSquared = order*order;
      switch(orderSquared)
      {
        case 0: 
        case 1:
        {
          return 2;
        }
        default:
        {
          return orderSquared+1;
        }
      }
    }
    case SpaMesh::tri:
    {
      subElementNodesNumber = 3;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 3;
        }
        case 2:
        {
          return 6;
        }
        case 3:
        {
          return 10;
        }
        case 4:
        {
          return 15;
        }
        case 5:
        {
          return 21;
        }
        default:
        {
          return 28;
        }
      }
    } 
    case SpaMesh::quad:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 4;
        }
        case 2:
        {
          return 9;
        }
        case 3:
        {
          return 16;
        }
        case 4:
        {
          return 25;
        }
        case 5:
        {
          return 36;
        }
        default:
        {
          return 49;
        }
      }
    }
    case SpaMesh::prism:
    {
      subElementNodesNumber = 6;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 6;
        }
        case 2:
        {
          return 18;
        }
        case 3:
        case 4:
        {
          return 75;
        }
        default:
        {
          return 405;
        }
      }
    } 
    case SpaMesh::pyram:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 5;
        }
        case 2:
        {
          return 14;
        }
        case 3:
        case 4:
        {
          return 55;
        }
        default: 
        {
          return 285;
        }
      }
    } 
    case SpaMesh::tet:
    {
      subElementNodesNumber = 4;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 4;
        }
        case 2:
        {
          return 10;
        }
        case 3:
        case 4:
        {
          return 35;
        }
        default: 
        {
          return 165;
        }
      }
    } 
    case SpaMesh::hex:
    {
      subElementNodesNumber = 8;
      switch(order)
      {
        case 0:
        case 1:
        {
          return 8;
        }
        case 2:
        {
          return 27;    
        }
        case 3:
        case 4:
        {
          return 125;
        }
        default: 
        {
          return 729;
        }
      }
    }
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return 0;
    }
  }
}


void VTKHelperSpaMesh::getSubElementsNodesNumbering(const ElemTypesMesh itemType, const unsigned int order, 
                                                    Eigen::Matrix<unsigned int,Eigen::Dynamic,Eigen::Dynamic,Eigen::RowMajor>& numbering)
{
  switch(itemType)
  {
    case SpaMesh::line:
    {
      unsigned int orderSquared = order*order;
      switch(orderSquared)
      {
        case 0:
        {
          numbering.resize(1,2);
          numbering(0,0) = 0;
          numbering(0,1) = 1;
          return;
        }
        default:
        {
          numbering.resize(orderSquared,2);
          for (unsigned int i = 0; i < orderSquared; i++)
          {
            numbering(i,0) = i;
            numbering(i,1) = i+1;
          }
          return;
        }
      }
    }
    case SpaMesh::tri:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(1,3);
          numbering(0,0) = 0;
          numbering(0,1) = 1;
          numbering(0,2) = 2;
          return;
        }
        case 2:
        {
          numbering.resize(4,3);
          numbering(0,0) = 0; 
          numbering(0,1) = 1;
          numbering(0,2) = 3;
          numbering(1,0) = 1;
          numbering(1,1) = 2;
          numbering(1,2) = 4; 
          numbering(2,0) = 1; 
          numbering(2,1) = 4;
          numbering(2,2) = 3;
          numbering(3,0) = 3; 
          numbering(3,1) = 4;
          numbering(3,2) = 5;
          return;
        }
        case 3:
        {
          numbering.resize(9,3);
          numbering(0,0) = 0; 
          numbering(0,1) = 1;
          numbering(0,2) = 4;
          numbering(1,0) = 1;
          numbering(1,1) = 2;
          numbering(1,2) = 5;
          numbering(2,0) = 2;
          numbering(2,1) = 3;
          numbering(2,2) = 6;
          numbering(3,0) = 1; 
          numbering(3,1) = 5;
          numbering(3,2) = 4;
          numbering(4,0) = 2; 
          numbering(4,1) = 6;
          numbering(4,2) = 5;
          numbering(5,0) = 4; 
          numbering(5,1) = 5;
          numbering(5,2) = 7;
          numbering(6,0) = 5;
          numbering(6,1) = 6;
          numbering(6,2) = 8;
          numbering(7,0) = 5;
          numbering(7,1) = 8;
          numbering(7,2) = 7;
          numbering(8,0) = 7;
          numbering(8,1) = 8;
          numbering(8,2) = 9;
          return;
        }
        case 4:
        {
          numbering.resize(16,3);
          numbering( 0,0) = 0; 
          numbering( 0,1) = 1;
          numbering( 0,2) = 5;
          numbering( 1,0) = 1;
          numbering( 1,1) = 2;
          numbering( 1,2) = 6;
          numbering( 2,0) = 2;
          numbering( 2,1) = 3;
          numbering( 2,2) = 7;
          numbering( 3,0) = 3; 
          numbering( 3,1) = 4;
          numbering( 3,2) = 8;
          numbering( 4,0) = 1; 
          numbering( 4,1) = 6;
          numbering( 4,2) = 5;
          numbering( 5,0) = 2; 
          numbering( 5,1) = 7;
          numbering( 5,2) = 6;
          numbering( 6,0) = 3;
          numbering( 6,1) = 8;
          numbering( 6,2) = 7;
          numbering( 7,0) = 5;
          numbering( 7,1) = 6;
          numbering( 7,2) = 9; 
          numbering( 8,0) = 6;
          numbering( 8,1) = 7;
          numbering( 8,2) = 10;
          numbering( 9,0) = 7;
          numbering( 9,1) = 8;
          numbering( 9,2) = 11;
          numbering(10,0) = 6;
          numbering(10,1) = 10;
          numbering(10,2) = 9; 
          numbering(11,0) = 7;
          numbering(11,1) = 11;
          numbering(11,2) = 10;
          numbering(12,0) = 9;
          numbering(12,1) = 10; 
          numbering(12,2) = 12;
          numbering(13,0) = 10;
          numbering(13,1) = 11;
          numbering(13,2) = 13;
          numbering(14,0) = 10;
          numbering(14,1) = 13;
          numbering(14,2) = 12;
          numbering(15,0) = 12;
          numbering(15,1) = 13;
          numbering(15,2) = 14;
          return;
        }
        case 5:
        {
          numbering.resize(25,3);
          numbering( 0,0) = 0; 
          numbering( 0,1) = 1;
          numbering( 0,2) = 6;
          numbering( 1,0) = 1;   
          numbering( 1,1) = 2;   
          numbering( 1,2) = 7;   
          numbering( 2,0) = 2;   
          numbering( 2,1) = 3;   
          numbering( 2,2) = 8;   
          numbering( 3,0) = 3;   
          numbering( 3,1) = 4;   
          numbering( 3,2) = 9;  
          numbering( 4,0) = 4;   
          numbering( 4,1) = 5;   
          numbering( 4,2) = 10; 
          numbering( 5,0) = 1;  
          numbering( 5,1) = 7;  
          numbering( 5,2) = 6;  
          numbering( 6,0) = 2;  
          numbering( 6,1) = 8;  
          numbering( 6,2) = 7;  
          numbering( 7,0) = 3;  
          numbering( 7,1) = 9; 
          numbering( 7,2) = 8;  
          numbering( 8,0) = 4;  
          numbering( 8,1) = 10;  
          numbering( 8,2) = 9;  
          numbering( 9,0) = 6;  
          numbering( 9,1) = 7;   
          numbering( 9,2) = 11;  
          numbering(10,0) = 7;  
          numbering(10,1) = 8;   
          numbering(10,2) = 12;  
          numbering(11,0) = 8;  
          numbering(11,1) = 9;  
          numbering(11,2) = 13;  
          numbering(12,0) = 9;  
          numbering(12,1) = 10;  
          numbering(12,2) = 14; 
          numbering(13,0) = 7;  
          numbering(13,1) = 12;  
          numbering(13,2) = 11;  
          numbering(14,0) = 8;  
          numbering(14,1) = 13;  
          numbering(14,2) = 12;  
          numbering(15,0) = 9;  
          numbering(15,1) = 14;  
          numbering(15,2) = 13; 
          numbering(16,0) = 11;  
          numbering(16,1) = 12;  
          numbering(16,2) = 15;  
          numbering(17,0) = 12;  
          numbering(17,1) = 13;  
          numbering(17,2) = 16;  
          numbering(18,0) = 13;  
          numbering(18,1) = 14;  
          numbering(18,2) = 17;  
          numbering(19,0) = 12;  
          numbering(19,1) = 16;  
          numbering(19,2) = 15;  
          numbering(20,0) = 13;  
          numbering(20,1) = 17;  
          numbering(20,2) = 16;  
          numbering(21,0) = 15;  
          numbering(21,1) = 16;  
          numbering(21,2) = 18;  
          numbering(22,0) = 16;  
          numbering(22,1) = 17;  
          numbering(22,2) = 19;  
          numbering(23,0) = 16;  
          numbering(23,1) = 19;  
          numbering(23,2) = 18;  
          numbering(24,0) = 18;  
          numbering(24,1) = 19;  
          numbering(24,2) = 20;  
          return;
        }
        default:
        {
          numbering.resize(36,3);
          numbering( 0,0) = 0;  
          numbering( 0,1) = 1;
          numbering( 0,2) = 7;
          numbering( 1,0) = 1;
          numbering( 1,1) = 2;
          numbering( 1,2) = 8;
          numbering( 2,0) = 2;
          numbering( 2,1) = 3;
          numbering( 2,2) = 9; 
          numbering( 3,0) = 3; 
          numbering( 3,1) = 4;
          numbering( 3,2) = 10;
          numbering( 4,0) = 4; 
          numbering( 4,1) = 5;
          numbering( 4,2) = 11;
          numbering( 5,0) = 5;
          numbering( 5,1) = 6;
          numbering( 5,2) = 12;
          numbering( 6,0) = 1;
          numbering( 6,1) = 8;
          numbering( 6,2) = 7;
          numbering( 7,0) = 2;
          numbering( 7,1) = 9;
          numbering( 7,2) = 8;
          numbering( 8,0) = 3;
          numbering( 8,1) = 10;
          numbering( 8,2) = 9; 
          numbering( 9,0) = 4;
          numbering( 9,1) = 11;
          numbering( 9,2) = 10;
          numbering(10,0) = 5;
          numbering(10,1) = 12;
          numbering(10,2) = 11;
          numbering(11,0) = 7;
          numbering(11,1) = 8;
          numbering(11,2) = 13;
          numbering(12,0) = 8;
          numbering(12,1) = 9;
          numbering(12,2) = 14;
          numbering(13,0) = 9;
          numbering(13,1) = 10;
          numbering(13,2) = 15;
          numbering(14,0) = 10;
          numbering(14,1) = 11;
          numbering(14,2) = 16;
          numbering(15,0) = 11;
          numbering(15,1) = 12;
          numbering(15,2) = 17;
          numbering(16,0) = 8;
          numbering(16,1) = 14;
          numbering(16,2) = 13;
          numbering(17,0) = 9;
          numbering(17,1) = 15; 
          numbering(17,2) = 14;
          numbering(18,0) = 10;
          numbering(18,1) = 16;
          numbering(18,2) = 15;
          numbering(19,0) = 11;
          numbering(19,1) = 17;
          numbering(19,2) = 16;
          numbering(20,0) = 13;
          numbering(20,1) = 14;
          numbering(20,2) = 18;
          numbering(21,0) = 14;
          numbering(21,1) = 15;
          numbering(21,2) = 19; 
          numbering(22,0) = 15;
          numbering(22,1) = 16;
          numbering(22,2) = 20;
          numbering(23,0) = 16;
          numbering(23,1) = 17;
          numbering(23,2) = 21;
          numbering(24,0) = 14;
          numbering(24,1) = 19;
          numbering(24,2) = 18;
          numbering(25,0) = 15;
          numbering(25,1) = 20;
          numbering(25,2) = 19;
          numbering(26,0) = 16;
          numbering(26,1) = 21;
          numbering(26,2) = 20;
          numbering(27,0) = 18;
          numbering(27,1) = 19;
          numbering(27,2) = 22;
          numbering(28,0) = 19; 
          numbering(28,1) = 20;
          numbering(28,2) = 23;
          numbering(29,0) = 20;
          numbering(29,1) = 21;
          numbering(29,2) = 24;
          numbering(30,0) = 19;
          numbering(30,1) = 23;
          numbering(30,2) = 22;
          numbering(31,0) = 20;
          numbering(31,1) = 24;
          numbering(31,2) = 23;
          numbering(32,0) = 22;
          numbering(32,1) = 23;
          numbering(32,2) = 25;
          numbering(33,0) = 23;
          numbering(33,1) = 24;
          numbering(33,2) = 26;
          numbering(34,0) = 23;
          numbering(34,1) = 26;
          numbering(34,2) = 25;
          numbering(35,0) = 25;
          numbering(35,1) = 26;
          numbering(35,2) = 27;
          return;
        }
      }
    } 
    case SpaMesh::quad:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(1,4);
          numbering(0,0) = 0; 
          numbering(0,1) = 1;
          numbering(0,2) = 3;
          numbering(0,3) = 2;
          return;
        }
        case 2:
        {
          numbering.resize(4,4);
          numbering(0,0) = 0; 
          numbering(0,1) = 1;
          numbering(0,2) = 4;
          numbering(0,3) = 3;
          numbering(1,0) = 1;
          numbering(1,1) = 2;
          numbering(1,2) = 5; 
          numbering(1,3) = 4; 
          numbering(2,0) = 3; 
          numbering(2,1) = 4;
          numbering(2,2) = 7;
          numbering(2,3) = 6;
          numbering(3,0) = 4; 
          numbering(3,1) = 5;
          numbering(3,2) = 8;
          numbering(3,3) = 7;
          return;
        }
        case 3:
        {
          numbering.resize(9,4);
          numbering(0,0) = 0; 
          numbering(0,1) = 1;
          numbering(0,2) = 5;
          numbering(0,3) = 4;
          numbering(1,0) = 1;
          numbering(1,1) = 2;
          numbering(1,2) = 6;
          numbering(1,3) = 5;
          numbering(2,0) = 2;
          numbering(2,1) = 3;
          numbering(2,2) = 7;
          numbering(2,3) = 6;
          numbering(3,0) = 4; 
          numbering(3,1) = 5;
          numbering(3,2) = 9; 
          numbering(3,3) = 8;
          numbering(4,0) = 5; 
          numbering(4,1) = 6;
          numbering(4,2) = 10;
          numbering(4,3) = 9;
          numbering(5,0) = 6; 
          numbering(5,1) = 7;
          numbering(5,2) = 11;
          numbering(5,3) = 10;
          numbering(6,0) = 8;
          numbering(6,1) = 9;
          numbering(6,2) = 13;
          numbering(6,3) = 12;
          numbering(7,0) = 9; 
          numbering(7,1) = 10; 
          numbering(7,2) = 14;
          numbering(7,3) = 13;
          numbering(8,0) = 10;
          numbering(8,1) = 11;
          numbering(8,2) = 15;
          numbering(8,3) = 14;
          return;
        }
        case 4:
        {
          numbering.resize(16,4);
          numbering( 0,0) = 0; 
          numbering( 0,1) = 1;
          numbering( 0,2) = 6;
          numbering( 0,3) = 5;
          numbering( 1,0) = 1;   
          numbering( 1,1) = 2;   
          numbering( 1,2) = 7;   
          numbering( 1,3) = 6;   
          numbering( 2,0) = 2;   
          numbering( 2,1) = 3;   
          numbering( 2,2) = 8;   
          numbering( 2,3) = 7;   
          numbering( 3,0) = 3;   
          numbering( 3,1) = 4;   
          numbering( 3,2) = 9;  
          numbering( 3,3) = 8;   
          numbering( 4,0) = 5;   
          numbering( 4,1) = 6;   
          numbering( 4,2) = 11;  
          numbering( 4,3) = 10;  
          numbering( 5,0) = 6;   
          numbering( 5,1) = 7;   
          numbering( 5,2) = 12;  
          numbering( 5,3) = 11;  
          numbering( 6,0) = 7;   
          numbering( 6,1) = 8;   
          numbering( 6,2) = 13;  
          numbering( 6,3) = 12; 
          numbering( 7,0) = 8;   
          numbering( 7,1) = 9;  
          numbering( 7,2) = 14;  
          numbering( 7,3) = 13;  
          numbering( 8,0) = 10;  
          numbering( 8,1) = 11;  
          numbering( 8,2) = 16;  
          numbering( 8,3) = 15;  
          numbering( 9,0) = 11;  
          numbering( 9,1) = 12;  
          numbering( 9,2) = 17;  
          numbering( 9,3) = 16;  
          numbering(10,0) = 12;  
          numbering(10,1) = 13;  
          numbering(10,2) = 18;  
          numbering(10,3) = 17;  
          numbering(11,0) = 13;  
          numbering(11,1) = 14;  
          numbering(11,2) = 19;  
          numbering(11,3) = 18;  
          numbering(12,0) = 15;  
          numbering(12,1) = 16;  
          numbering(12,2) = 21;  
          numbering(12,3) = 20;  
          numbering(13,0) = 16;  
          numbering(13,1) = 17;  
          numbering(13,2) = 22;  
          numbering(13,3) = 21;  
          numbering(14,0) = 17;  
          numbering(14,1) = 18;  
          numbering(14,2) = 23;  
          numbering(14,3) = 22;  
          numbering(15,0) = 18;  
          numbering(15,1) = 19; 
          numbering(15,2) = 24;  
          numbering(15,3) = 23;  
          return;
        }
        case 5:
        {
          numbering.resize(25,4);
          numbering( 0,0) = 0;  
          numbering( 0,1) = 1;
          numbering( 0,2) = 7;
          numbering( 0,3) = 6;
          numbering( 1,0) = 1;  
          numbering( 1,1) = 2;  
          numbering( 1,2) = 8;  
          numbering( 1,3) = 7;  
          numbering( 2,0) = 2;  
          numbering( 2,1) = 3;  
          numbering( 2,2) = 9;
          numbering( 2,3) = 8;  
          numbering( 3,0) = 3;  
          numbering( 3,1) = 4;  
          numbering( 3,2) = 10; 
          numbering( 3,3) = 9; 
          numbering( 4,0) = 4;  
          numbering( 4,1) = 5;  
          numbering( 4,2) = 11; 
          numbering( 4,3) = 10; 
          numbering( 5,0) = 6;  
          numbering( 5,1) = 7;  
          numbering( 5,2) = 13; 
          numbering( 5,3) = 12; 
          numbering( 6,0) = 7;  
          numbering( 6,1) = 8;  
          numbering( 6,2) = 14; 
          numbering( 6,3) = 13; 
          numbering( 7,0) = 8;  
          numbering( 7,1) = 9;  
          numbering( 7,2) = 15; 
          numbering( 7,3) = 14; 
          numbering( 8,0) = 9; 
          numbering( 8,1) = 10; 
          numbering( 8,2) = 16; 
          numbering( 8,3) = 15; 
          numbering( 9,0) = 10; 
          numbering( 9,1) = 11; 
          numbering( 9,2) = 17; 
          numbering( 9,3) = 16; 
          numbering(10,0) = 12; 
          numbering(10,1) = 13; 
          numbering(10,2) = 19; 
          numbering(10,3) = 18; 
          numbering(11,0) = 13; 
          numbering(11,1) = 14; 
          numbering(11,2) = 20; 
          numbering(11,3) = 19; 
          numbering(12,0) = 14; 
          numbering(12,1) = 15; 
          numbering(12,2) = 21; 
          numbering(12,3) = 20; 
          numbering(13,0) = 15; 
          numbering(13,1) = 16; 
          numbering(13,2) = 22; 
          numbering(13,3) = 21; 
          numbering(14,0) = 16; 
          numbering(14,1) = 17; 
          numbering(14,2) = 23; 
          numbering(14,3) = 22; 
          numbering(15,0) = 18; 
          numbering(15,1) = 19; 
          numbering(15,2) = 25; 
          numbering(15,3) = 24; 
          numbering(16,0) = 19; 
          numbering(16,1) = 20; 
          numbering(16,2) = 26; 
          numbering(16,3) = 25; 
          numbering(17,0) = 20; 
          numbering(17,1) = 21; 
          numbering(17,2) = 27; 
          numbering(17,3) = 26; 
          numbering(18,0) = 21; 
          numbering(18,1) = 22; 
          numbering(18,2) = 28; 
          numbering(18,3) = 27; 
          numbering(19,0) = 22; 
          numbering(19,1) = 23; 
          numbering(19,2) = 29; 
          numbering(19,3) = 28; 
          numbering(20,0) = 24; 
          numbering(20,1) = 25; 
          numbering(20,2) = 31; 
          numbering(20,3) = 30; 
          numbering(21,0) = 25; 
          numbering(21,1) = 26; 
          numbering(21,2) = 32; 
          numbering(21,3) = 31; 
          numbering(22,0) = 26; 
          numbering(22,1) = 27; 
          numbering(22,2) = 33; 
          numbering(22,3) = 32; 
          numbering(23,0) = 27; 
          numbering(23,1) = 28; 
          numbering(23,2) = 34; 
          numbering(23,3) = 33; 
          numbering(24,0) = 28; 
          numbering(24,1) = 29; 
          numbering(24,2) = 35; 
          numbering(24,3) = 34; 
          return;
        }
        default:
        {
          numbering.resize(36,4);
          numbering( 0,0) = 0;
          numbering( 0,1) = 1;
          numbering( 0,2) = 8;
          numbering( 0,3) = 7;
          numbering( 1,0) = 1; 
          numbering( 1,1) = 2;  
          numbering( 1,2) = 9; 
          numbering( 1,3) = 8;  
          numbering( 2,0) = 2;  
          numbering( 2,1) = 3;  
          numbering( 2,2) = 10; 
          numbering( 2,3) = 9; 
          numbering( 3,0) = 3;  
          numbering( 3,1) = 4;  
          numbering( 3,2) = 11; 
          numbering( 3,3) = 10; 
          numbering( 4,0) = 4;  
          numbering( 4,1) = 5;  
          numbering( 4,2) = 12; 
          numbering( 4,3) = 11; 
          numbering( 5,0) = 5;  
          numbering( 5,1) = 6;  
          numbering( 5,2) = 13; 
          numbering( 5,3) = 12; 
          numbering( 6,0) = 7;  
          numbering( 6,1) = 8;  
          numbering( 6,2) = 15; 
          numbering( 6,3) = 14; 
          numbering( 7,0) = 8;  
          numbering( 7,1) = 9; 
          numbering( 7,2) = 16; 
          numbering( 7,3) = 15; 
          numbering( 8,0) = 9; 
          numbering( 8,1) = 10; 
          numbering( 8,2) = 17; 
          numbering( 8,3) = 16; 
          numbering( 9,0) = 10; 
          numbering( 9,1) = 11; 
          numbering( 9,2) = 18; 
          numbering( 9,3) = 17; 
          numbering(10,0) = 11; 
          numbering(10,1) = 12; 
          numbering(10,2) = 19; 
          numbering(10,3) = 18; 
          numbering(11,0) = 12; 
          numbering(11,1) = 13; 
          numbering(11,2) = 20; 
          numbering(11,3) = 19; 
          numbering(12,0) = 14; 
          numbering(12,1) = 15; 
          numbering(12,2) = 22; 
          numbering(12,3) = 21; 
          numbering(13,0) = 15; 
          numbering(13,1) = 16; 
          numbering(13,2) = 23; 
          numbering(13,3) = 22; 
          numbering(14,0) = 16; 
          numbering(14,1) = 17; 
          numbering(14,2) = 24; 
          numbering(14,3) = 23; 
          numbering(15,0) = 17; 
          numbering(15,1) = 18; 
          numbering(15,2) = 25; 
          numbering(15,3) = 24; 
          numbering(16,0) = 18; 
          numbering(16,1) = 19; 
          numbering(16,2) = 26; 
          numbering(16,3) = 25; 
          numbering(17,0) = 19; 
          numbering(17,1) = 20; 
          numbering(17,2) = 27; 
          numbering(17,3) = 26; 
          numbering(18,0) = 21; 
          numbering(18,1) = 22; 
          numbering(18,2) = 29; 
          numbering(18,3) = 28; 
          numbering(19,0) = 22; 
          numbering(19,1) = 23; 
          numbering(19,2) = 30; 
          numbering(19,3) = 29; 
          numbering(20,0) = 23; 
          numbering(20,1) = 24; 
          numbering(20,2) = 31; 
          numbering(20,3) = 30; 
          numbering(21,0) = 24; 
          numbering(21,1) = 25; 
          numbering(21,2) = 32; 
          numbering(21,3) = 31; 
          numbering(22,0) = 25; 
          numbering(22,1) = 26; 
          numbering(22,2) = 33; 
          numbering(22,3) = 32; 
          numbering(23,0) = 26; 
          numbering(23,1) = 27; 
          numbering(23,2) = 34; 
          numbering(23,3) = 33; 
          numbering(24,0) = 28; 
          numbering(24,1) = 29; 
          numbering(24,2) = 36; 
          numbering(24,3) = 35; 
          numbering(25,0) = 29; 
          numbering(25,1) = 30; 
          numbering(25,2) = 37; 
          numbering(25,3) = 36; 
          numbering(26,0) = 30; 
          numbering(26,1) = 31; 
          numbering(26,2) = 38; 
          numbering(26,3) = 37; 
          numbering(27,0) = 31; 
          numbering(27,1) = 32; 
          numbering(27,2) = 39; 
          numbering(27,3) = 38; 
          numbering(28,0) = 32; 
          numbering(28,1) = 33; 
          numbering(28,2) = 40; 
          numbering(28,3) = 39; 
          numbering(29,0) = 33; 
          numbering(29,1) = 34; 
          numbering(29,2) = 41; 
          numbering(29,3) = 40; 
          numbering(30,0) = 35; 
          numbering(30,1) = 36; 
          numbering(30,2) = 43; 
          numbering(30,3) = 42; 
          numbering(31,0) = 36; 
          numbering(31,1) = 37; 
          numbering(31,2) = 44; 
          numbering(31,3) = 43; 
          numbering(32,0) = 37; 
          numbering(32,1) = 38; 
          numbering(32,2) = 45; 
          numbering(32,3) = 44; 
          numbering(33,0) = 38; 
          numbering(33,1) = 39; 
          numbering(33,2) = 46; 
          numbering(33,3) = 45; 
          numbering(34,0) = 39; 
          numbering(34,1) = 40; 
          numbering(34,2) = 47; 
          numbering(34,3) = 46; 
          numbering(35,0) = 40; 
          numbering(35,1) = 41; 
          numbering(35,2) = 48; 
          numbering(35,3) = 47; 
          return;
        }
      }
    }
    case SpaMesh::prism:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(1,6);
          numbering(0,0) = 0;
          numbering(0,1) = 2;
          numbering(0,2) = 1;
          numbering(0,3) = 3;
          numbering(0,4) = 5;
          numbering(0,5) = 4;
          return;
        }
        case 2:
        {
          numbering.resize(8,6);
          numbering(0,0) = 0;
          numbering(0,1) = 7;
          numbering(0,2) = 6;
          numbering(0,3) = 8;
          numbering(0,4) = 10;
          numbering(0,5) = 9;
          numbering(1,0) = 6;
          numbering(1,1) = 11;
          numbering(1,2) = 1;
          numbering(1,3) = 9;
          numbering(1,4) = 13;
          numbering(1,5) = 12;
          numbering(2,0) = 7;
          numbering(2,1) = 2;
          numbering(2,2) = 11;
          numbering(2,3) = 10;
          numbering(2,4) = 14;
          numbering(2,5) = 13;
          numbering(3,0) = 6;
          numbering(3,1) = 7;
          numbering(3,2) = 11;
          numbering(3,3) = 9;
          numbering(3,4) = 10;
          numbering(3,5) = 13;
          numbering(4,0) = 8;
          numbering(4,1) = 10;
          numbering(4,2) = 9;
          numbering(4,3) = 3;
          numbering(4,4) = 16;
          numbering(4,5) = 15;
          numbering(5,0) = 9;
          numbering(5,1) = 13;
          numbering(5,2) = 12;
          numbering(5,3) = 15;
          numbering(5,4) = 17;
          numbering(5,5) = 4;
          numbering(6,0) = 10;
          numbering(6,1) = 14;
          numbering(6,2) = 13;
          numbering(6,3) = 16;
          numbering(6,4) = 5;
          numbering(6,5) = 17;
          numbering(7,0) = 9;
          numbering(7,1) = 10;
          numbering(7,2) = 13;
          numbering(7,3) = 15;
          numbering(7,4) = 16;
          numbering(7,5) = 17;
          return;
        }
        case 3:
        case 4:
        {
          numbering.resize(64,6);
          numbering(0,0) = 0;
          numbering(0,1) = 19;
          numbering(0,2) = 18;
          numbering(0,3) = 20;
          numbering(0,4) = 22;
          numbering(0,5) = 21;
          numbering(1,0) = 18;
          numbering(1,1) = 23;
          numbering(1,2) = 6;
          numbering(1,3) = 21;
          numbering(1,4) = 25;
          numbering(1,5) = 24;
          numbering(2,0) = 19;
          numbering(2,1) = 7;
          numbering(2,2) = 23;
          numbering(2,3) = 22;
          numbering(2,4) = 26;
          numbering(2,5) = 25;
          numbering(3,0) = 18;
          numbering(3,1) = 19;
          numbering(3,2) = 23;
          numbering(3,3) = 21;
          numbering(3,4) = 22;
          numbering(3,5) = 25;
          numbering(4,0) = 20;
          numbering(4,1) = 22;
          numbering(4,2) = 21;
          numbering(4,3) = 8;
          numbering(4,4) = 28;
          numbering(4,5) = 27;
          numbering(5,0) = 21;
          numbering(5,1) = 25;
          numbering(5,2) = 24;
          numbering(5,3) = 27;
          numbering(5,4) = 29;
          numbering(5,5) = 9;
          numbering(6,0) = 22;
          numbering(6,1) = 26;
          numbering(6,2) = 25;
          numbering(6,3) = 28;
          numbering(6,4) = 10;
          numbering(6,5) = 29;
          numbering(7,0) = 21;
          numbering(7,1) = 22;
          numbering(7,2) = 25;
          numbering(7,3) = 27;
          numbering(7,4) = 28;
          numbering(7,5) = 29;
          numbering(8,0) = 6;
          numbering(8,1) = 31;
          numbering(8,2) = 30;
          numbering(8,3) = 24;
          numbering(8,4) = 33;
          numbering(8,5) = 32;
          numbering(9,0) = 30;
          numbering(9,1) = 34;
          numbering(9,2) = 1;
          numbering(9,3) = 32;
          numbering(9,4) = 36;
          numbering(9,5) = 35;
          numbering(10,0) = 31;
          numbering(10,1) = 11;
          numbering(10,2) = 34;
          numbering(10,3) = 33;
          numbering(10,4) = 37;
          numbering(10,5) = 36;
          numbering(11,0) = 30;
          numbering(11,1) = 31;
          numbering(11,2) = 34;
          numbering(11,3) = 32;
          numbering(11,4) = 33;
          numbering(11,5) = 36;
          numbering(12,0) = 24;
          numbering(12,1) = 33;
          numbering(12,2) = 32;
          numbering(12,3) = 9;
          numbering(12,4) = 39;
          numbering(12,5) = 38;
          numbering(13,0) = 32;
          numbering(13,1) = 36;
          numbering(13,2) = 35;
          numbering(13,3) = 38;
          numbering(13,4) = 40;
          numbering(13,5) = 12;
          numbering(14,0) = 33;
          numbering(14,1) = 37;
          numbering(14,2) = 36;
          numbering(14,3) = 39;
          numbering(14,4) = 13;
          numbering(14,5) = 40;
          numbering(15,0) = 32;
          numbering(15,1) = 33;
          numbering(15,2) = 36;
          numbering(15,3) = 38;
          numbering(15,4) = 39;
          numbering(15,5) = 40;
          numbering(16,0) = 7;
          numbering(16,1) = 42;
          numbering(16,2) = 41;
          numbering(16,3) = 26;
          numbering(16,4) = 44;
          numbering(16,5) = 43;
          numbering(17,0) = 41;
          numbering(17,1) = 45;
          numbering(17,2) = 11;
          numbering(17,3) = 43;
          numbering(17,4) = 46;
          numbering(17,5) = 37;
          numbering(18,0) = 42;
          numbering(18,1) = 2;
          numbering(18,2) = 45;
          numbering(18,3) = 44;
          numbering(18,4) = 47;
          numbering(18,5) = 46;
          numbering(19,0) = 41;
          numbering(19,1) = 42;
          numbering(19,2) = 45;
          numbering(19,3) = 43;
          numbering(19,4) = 44;
          numbering(19,5) = 46;
          numbering(20,0) = 26;
          numbering(20,1) = 44;
          numbering(20,2) = 43;
          numbering(20,3) = 10;
          numbering(20,4) = 49;
          numbering(20,5) = 48;
          numbering(21,0) = 43;
          numbering(21,1) = 46;
          numbering(21,2) = 37;
          numbering(21,3) = 48;
          numbering(21,4) = 50;
          numbering(21,5) = 13;
          numbering(22,0) = 44;
          numbering(22,1) = 47;
          numbering(22,2) = 46;
          numbering(22,3) = 49;
          numbering(22,4) = 14;
          numbering(22,5) = 50;
          numbering(23,0) = 43;
          numbering(23,1) = 44;
          numbering(23,2) = 46;
          numbering(23,3) = 48;
          numbering(23,4) = 49;
          numbering(23,5) = 50;
          numbering(24,0) = 6;
          numbering(24,1) = 23;
          numbering(24,2) = 31;
          numbering(24,3) = 24;
          numbering(24,4) = 25;
          numbering(24,5) = 33;
          numbering(25,0) = 31;
          numbering(25,1) = 41;
          numbering(25,2) = 11;
          numbering(25,3) = 33;
          numbering(25,4) = 43;
          numbering(25,5) = 37;
          numbering(26,0) = 23;
          numbering(26,1) = 7;
          numbering(26,2) = 41;
          numbering(26,3) = 25;
          numbering(26,4) = 26;
          numbering(26,5) = 43;
          numbering(27,0) = 31;
          numbering(27,1) = 23;
          numbering(27,2) = 41;
          numbering(27,3) = 33;
          numbering(27,4) = 25;
          numbering(27,5) = 43;
          numbering(28,0) = 24;
          numbering(28,1) = 25;
          numbering(28,2) = 33;
          numbering(28,3) = 9;
          numbering(28,4) = 29;
          numbering(28,5) = 39;
          numbering(29,0) = 33;
          numbering(29,1) = 43;
          numbering(29,2) = 37;
          numbering(29,3) = 39;
          numbering(29,4) = 48;
          numbering(29,5) = 13;
          numbering(30,0) = 25;
          numbering(30,1) = 26;
          numbering(30,2) = 43;
          numbering(30,3) = 29;
          numbering(30,4) = 10;
          numbering(30,5) = 48;
          numbering(31,0) = 33;
          numbering(31,1) = 25;
          numbering(31,2) = 43;
          numbering(31,3) = 39;
          numbering(31,4) = 29;
          numbering(31,5) = 48;
          numbering(32,0) = 8;
          numbering(32,1) = 28;
          numbering(32,2) = 27;
          numbering(32,3) = 51;
          numbering(32,4) = 53;
          numbering(32,5) = 52;
          numbering(33,0) = 27;
          numbering(33,1) = 29;
          numbering(33,2) = 9;
          numbering(33,3) = 52;
          numbering(33,4) = 55;
          numbering(33,5) = 54;
          numbering(34,0) = 28;
          numbering(34,1) = 10;
          numbering(34,2) = 29;
          numbering(34,3) = 53;
          numbering(34,4) = 56;
          numbering(34,5) = 55;
          numbering(35,0) = 27;
          numbering(35,1) = 28;
          numbering(35,2) = 29;
          numbering(35,3) = 52;
          numbering(35,4) = 53;
          numbering(35,5) = 55;
          numbering(36,0) = 51;
          numbering(36,1) = 53;
          numbering(36,2) = 52;
          numbering(36,3) = 3;
          numbering(36,4) = 58;
          numbering(36,5) = 57;
          numbering(37,0) = 52;
          numbering(37,1) = 55;
          numbering(37,2) = 54;
          numbering(37,3) = 57;
          numbering(37,4) = 59;
          numbering(37,5) = 15;
          numbering(38,0) = 53;
          numbering(38,1) = 56;
          numbering(38,2) = 55;
          numbering(38,3) = 58;
          numbering(38,4) = 16;
          numbering(38,5) = 59;
          numbering(39,0) = 52;
          numbering(39,1) = 53;
          numbering(39,2) = 55;
          numbering(39,3) = 57;
          numbering(39,4) = 58;
          numbering(39,5) = 59;
          numbering(40,0) = 9;
          numbering(40,1) = 39;
          numbering(40,2) = 38;
          numbering(40,3) = 54;
          numbering(40,4) = 61;
          numbering(40,5) = 60;
          numbering(41,0) = 38;
          numbering(41,1) = 40;
          numbering(41,2) = 12;
          numbering(41,3) = 60;
          numbering(41,4) = 63;
          numbering(41,5) = 62;
          numbering(42,0) = 39;
          numbering(42,1) = 13;
          numbering(42,2) = 40;
          numbering(42,3) = 61;
          numbering(42,4) = 64;
          numbering(42,5) = 63;
          numbering(43,0) = 38;
          numbering(43,1) = 39;
          numbering(43,2) = 40;
          numbering(43,3) = 60;
          numbering(43,4) = 61;
          numbering(43,5) = 63;
          numbering(44,0) = 54;
          numbering(44,1) = 61;
          numbering(44,2) = 60;
          numbering(44,3) = 15;
          numbering(44,4) = 66;
          numbering(44,5) = 65;
          numbering(45,0) = 60;
          numbering(45,1) = 63;
          numbering(45,2) = 62;
          numbering(45,3) = 65;
          numbering(45,4) = 67;
          numbering(45,5) = 4;
          numbering(46,0) = 61;
          numbering(46,1) = 64;
          numbering(46,2) = 63;
          numbering(46,3) = 66;
          numbering(46,4) = 17;
          numbering(46,5) = 67;
          numbering(47,0) = 60;
          numbering(47,1) = 61;
          numbering(47,2) = 63;
          numbering(47,3) = 65;
          numbering(47,4) = 66;
          numbering(47,5) = 67;
          numbering(48,0) = 10;
          numbering(48,1) = 49;
          numbering(48,2) = 48;
          numbering(48,3) = 56;
          numbering(48,4) = 69;
          numbering(48,5) = 68;
          numbering(49,0) = 48;
          numbering(49,1) = 50;
          numbering(49,2) = 13;
          numbering(49,3) = 68;
          numbering(49,4) = 70;
          numbering(49,5) = 64;
          numbering(50,0) = 49;
          numbering(50,1) = 14;
          numbering(50,2) = 50;
          numbering(50,3) = 69;
          numbering(50,4) = 71;
          numbering(50,5) = 70;
          numbering(51,0) = 48;
          numbering(51,1) = 49;
          numbering(51,2) = 50;
          numbering(51,3) = 68;
          numbering(51,4) = 69;
          numbering(51,5) = 70;
          numbering(52,0) = 56;
          numbering(52,1) = 69;
          numbering(52,2) = 68;
          numbering(52,3) = 16;
          numbering(52,4) = 73;
          numbering(52,5) = 72;
          numbering(53,0) = 68;
          numbering(53,1) = 70;
          numbering(53,2) = 64;
          numbering(53,3) = 72;
          numbering(53,4) = 74;
          numbering(53,5) = 17;
          numbering(54,0) = 69;
          numbering(54,1) = 71;
          numbering(54,2) = 70;
          numbering(54,3) = 73;
          numbering(54,4) = 5;
          numbering(54,5) = 74;
          numbering(55,0) = 68;
          numbering(55,1) = 69;
          numbering(55,2) = 70;
          numbering(55,3) = 72;
          numbering(55,4) = 73;
          numbering(55,5) = 74;
          numbering(56,0) = 9;
          numbering(56,1) = 29;
          numbering(56,2) = 39;
          numbering(56,3) = 54;
          numbering(56,4) = 55;
          numbering(56,5) = 61;
          numbering(57,0) = 39;
          numbering(57,1) = 48;
          numbering(57,2) = 13;
          numbering(57,3) = 61;
          numbering(57,4) = 68;
          numbering(57,5) = 64;
          numbering(58,0) = 29;
          numbering(58,1) = 10;
          numbering(58,2) = 48;
          numbering(58,3) = 55;
          numbering(58,4) = 56;
          numbering(58,5) = 68;
          numbering(59,0) = 39;
          numbering(59,1) = 29;
          numbering(59,2) = 48;
          numbering(59,3) = 61;
          numbering(59,4) = 55;
          numbering(59,5) = 68;
          numbering(60,0) = 54;
          numbering(60,1) = 55;
          numbering(60,2) = 61;
          numbering(60,3) = 15;
          numbering(60,4) = 59;
          numbering(60,5) = 66;
          numbering(61,0) = 61;
          numbering(61,1) = 68;
          numbering(61,2) = 64;
          numbering(61,3) = 66;
          numbering(61,4) = 72;
          numbering(61,5) = 17;
          numbering(62,0) = 55;
          numbering(62,1) = 56;
          numbering(62,2) = 68;
          numbering(62,3) = 59;
          numbering(62,4) = 16;
          numbering(62,5) = 72;
          numbering(63,0) = 61;
          numbering(63,1) = 55;
          numbering(63,2) = 68;
          numbering(63,3) = 66;
          numbering(63,4) = 59;
          numbering(63,5) = 72;
          return;
        }
        default:
        {
          numbering.resize(512,6);
          numbering(0,0) = 0;
          numbering(0,1) = 76;
          numbering(0,2) = 75;
          numbering(0,3) = 77;
          numbering(0,4) = 79;
          numbering(0,5) = 78;
          numbering(1,0) = 75;
          numbering(1,1) = 80;
          numbering(1,2) = 18;
          numbering(1,3) = 78;
          numbering(1,4) = 82;
          numbering(1,5) = 81;
          numbering(2,0) = 76;
          numbering(2,1) = 19;
          numbering(2,2) = 80;
          numbering(2,3) = 79;
          numbering(2,4) = 83;
          numbering(2,5) = 82;
          numbering(3,0) = 75;
          numbering(3,1) = 76;
          numbering(3,2) = 80;
          numbering(3,3) = 78;
          numbering(3,4) = 79;
          numbering(3,5) = 82;
          numbering(4,0) = 77;
          numbering(4,1) = 79;
          numbering(4,2) = 78;
          numbering(4,3) = 20;
          numbering(4,4) = 85;
          numbering(4,5) = 84;
          numbering(5,0) = 78;
          numbering(5,1) = 82;
          numbering(5,2) = 81;
          numbering(5,3) = 84;
          numbering(5,4) = 86;
          numbering(5,5) = 21;
          numbering(6,0) = 79;
          numbering(6,1) = 83;
          numbering(6,2) = 82;
          numbering(6,3) = 85;
          numbering(6,4) = 22;
          numbering(6,5) = 86;
          numbering(7,0) = 78;
          numbering(7,1) = 79;
          numbering(7,2) = 82;
          numbering(7,3) = 84;
          numbering(7,4) = 85;
          numbering(7,5) = 86;
          numbering(8,0) = 18;
          numbering(8,1) = 88;
          numbering(8,2) = 87;
          numbering(8,3) = 81;
          numbering(8,4) = 90;
          numbering(8,5) = 89;
          numbering(9,0) = 87;
          numbering(9,1) = 91;
          numbering(9,2) = 6;
          numbering(9,3) = 89;
          numbering(9,4) = 93;
          numbering(9,5) = 92;
          numbering(10,0) = 88;
          numbering(10,1) = 23;
          numbering(10,2) = 91;
          numbering(10,3) = 90;
          numbering(10,4) = 94;
          numbering(10,5) = 93;
          numbering(11,0) = 87;
          numbering(11,1) = 88;
          numbering(11,2) = 91;
          numbering(11,3) = 89;
          numbering(11,4) = 90;
          numbering(11,5) = 93;
          numbering(12,0) = 81;
          numbering(12,1) = 90;
          numbering(12,2) = 89;
          numbering(12,3) = 21;
          numbering(12,4) = 96;
          numbering(12,5) = 95;
          numbering(13,0) = 89;
          numbering(13,1) = 93;
          numbering(13,2) = 92;
          numbering(13,3) = 95;
          numbering(13,4) = 97;
          numbering(13,5) = 24;
          numbering(14,0) = 90;
          numbering(14,1) = 94;
          numbering(14,2) = 93;
          numbering(14,3) = 96;
          numbering(14,4) = 25;
          numbering(14,5) = 97;
          numbering(15,0) = 89;
          numbering(15,1) = 90;
          numbering(15,2) = 93;
          numbering(15,3) = 95;
          numbering(15,4) = 96;
          numbering(15,5) = 97;
          numbering(16,0) = 19;
          numbering(16,1) = 99;
          numbering(16,2) = 98;
          numbering(16,3) = 83;
          numbering(16,4) = 101;
          numbering(16,5) = 100;
          numbering(17,0) = 98;
          numbering(17,1) = 102;
          numbering(17,2) = 23;
          numbering(17,3) = 100;
          numbering(17,4) = 103;
          numbering(17,5) = 94;
          numbering(18,0) = 99;
          numbering(18,1) = 7;
          numbering(18,2) = 102;
          numbering(18,3) = 101;
          numbering(18,4) = 104;
          numbering(18,5) = 103;
          numbering(19,0) = 98;
          numbering(19,1) = 99;
          numbering(19,2) = 102;
          numbering(19,3) = 100;
          numbering(19,4) = 101;
          numbering(19,5) = 103;
          numbering(20,0) = 83;
          numbering(20,1) = 101;
          numbering(20,2) = 100;
          numbering(20,3) = 22;
          numbering(20,4) = 106;
          numbering(20,5) = 105;
          numbering(21,0) = 100;
          numbering(21,1) = 103;
          numbering(21,2) = 94;
          numbering(21,3) = 105;
          numbering(21,4) = 107;
          numbering(21,5) = 25;
          numbering(22,0) = 101;
          numbering(22,1) = 104;
          numbering(22,2) = 103;
          numbering(22,3) = 106;
          numbering(22,4) = 26;
          numbering(22,5) = 107;
          numbering(23,0) = 100;
          numbering(23,1) = 101;
          numbering(23,2) = 103;
          numbering(23,3) = 105;
          numbering(23,4) = 106;
          numbering(23,5) = 107;
          numbering(24,0) = 18;
          numbering(24,1) = 80;
          numbering(24,2) = 88;
          numbering(24,3) = 81;
          numbering(24,4) = 82;
          numbering(24,5) = 90;
          numbering(25,0) = 88;
          numbering(25,1) = 98;
          numbering(25,2) = 23;
          numbering(25,3) = 90;
          numbering(25,4) = 100;
          numbering(25,5) = 94;
          numbering(26,0) = 80;
          numbering(26,1) = 19;
          numbering(26,2) = 98;
          numbering(26,3) = 82;
          numbering(26,4) = 83;
          numbering(26,5) = 100;
          numbering(27,0) = 88;
          numbering(27,1) = 80;
          numbering(27,2) = 98;
          numbering(27,3) = 90;
          numbering(27,4) = 82;
          numbering(27,5) = 100;
          numbering(28,0) = 81;
          numbering(28,1) = 82;
          numbering(28,2) = 90;
          numbering(28,3) = 21;
          numbering(28,4) = 86;
          numbering(28,5) = 96;
          numbering(29,0) = 90;
          numbering(29,1) = 100;
          numbering(29,2) = 94;
          numbering(29,3) = 96;
          numbering(29,4) = 105;
          numbering(29,5) = 25;
          numbering(30,0) = 82;
          numbering(30,1) = 83;
          numbering(30,2) = 100;
          numbering(30,3) = 86;
          numbering(30,4) = 22;
          numbering(30,5) = 105;
          numbering(31,0) = 90;
          numbering(31,1) = 82;
          numbering(31,2) = 100;
          numbering(31,3) = 96;
          numbering(31,4) = 86;
          numbering(31,5) = 105;
          numbering(32,0) = 20;
          numbering(32,1) = 85;
          numbering(32,2) = 84;
          numbering(32,3) = 108;
          numbering(32,4) = 110;
          numbering(32,5) = 109;
          numbering(33,0) = 84;
          numbering(33,1) = 86;
          numbering(33,2) = 21;
          numbering(33,3) = 109;
          numbering(33,4) = 112;
          numbering(33,5) = 111;
          numbering(34,0) = 85;
          numbering(34,1) = 22;
          numbering(34,2) = 86;
          numbering(34,3) = 110;
          numbering(34,4) = 113;
          numbering(34,5) = 112;
          numbering(35,0) = 84;
          numbering(35,1) = 85;
          numbering(35,2) = 86;
          numbering(35,3) = 109;
          numbering(35,4) = 110;
          numbering(35,5) = 112;
          numbering(36,0) = 108;
          numbering(36,1) = 110;
          numbering(36,2) = 109;
          numbering(36,3) = 8;
          numbering(36,4) = 115;
          numbering(36,5) = 114;
          numbering(37,0) = 109;
          numbering(37,1) = 112;
          numbering(37,2) = 111;
          numbering(37,3) = 114;
          numbering(37,4) = 116;
          numbering(37,5) = 27;
          numbering(38,0) = 110;
          numbering(38,1) = 113;
          numbering(38,2) = 112;
          numbering(38,3) = 115;
          numbering(38,4) = 28;
          numbering(38,5) = 116;
          numbering(39,0) = 109;
          numbering(39,1) = 110;
          numbering(39,2) = 112;
          numbering(39,3) = 114;
          numbering(39,4) = 115;
          numbering(39,5) = 116;
          numbering(40,0) = 21;
          numbering(40,1) = 96;
          numbering(40,2) = 95;
          numbering(40,3) = 111;
          numbering(40,4) = 118;
          numbering(40,5) = 117;
          numbering(41,0) = 95;
          numbering(41,1) = 97;
          numbering(41,2) = 24;
          numbering(41,3) = 117;
          numbering(41,4) = 120;
          numbering(41,5) = 119;
          numbering(42,0) = 96;
          numbering(42,1) = 25;
          numbering(42,2) = 97;
          numbering(42,3) = 118;
          numbering(42,4) = 121;
          numbering(42,5) = 120;
          numbering(43,0) = 95;
          numbering(43,1) = 96;
          numbering(43,2) = 97;
          numbering(43,3) = 117;
          numbering(43,4) = 118;
          numbering(43,5) = 120;
          numbering(44,0) = 111;
          numbering(44,1) = 118;
          numbering(44,2) = 117;
          numbering(44,3) = 27;
          numbering(44,4) = 123;
          numbering(44,5) = 122;
          numbering(45,0) = 117;
          numbering(45,1) = 120;
          numbering(45,2) = 119;
          numbering(45,3) = 122;
          numbering(45,4) = 124;
          numbering(45,5) = 9;
          numbering(46,0) = 118;
          numbering(46,1) = 121;
          numbering(46,2) = 120;
          numbering(46,3) = 123;
          numbering(46,4) = 29;
          numbering(46,5) = 124;
          numbering(47,0) = 117;
          numbering(47,1) = 118;
          numbering(47,2) = 120;
          numbering(47,3) = 122;
          numbering(47,4) = 123;
          numbering(47,5) = 124;
          numbering(48,0) = 22;
          numbering(48,1) = 106;
          numbering(48,2) = 105;
          numbering(48,3) = 113;
          numbering(48,4) = 126;
          numbering(48,5) = 125;
          numbering(49,0) = 105;
          numbering(49,1) = 107;
          numbering(49,2) = 25;
          numbering(49,3) = 125;
          numbering(49,4) = 127;
          numbering(49,5) = 121;
          numbering(50,0) = 106;
          numbering(50,1) = 26;
          numbering(50,2) = 107;
          numbering(50,3) = 126;
          numbering(50,4) = 128;
          numbering(50,5) = 127;
          numbering(51,0) = 105;
          numbering(51,1) = 106;
          numbering(51,2) = 107;
          numbering(51,3) = 125;
          numbering(51,4) = 126;
          numbering(51,5) = 127;
          numbering(52,0) = 113;
          numbering(52,1) = 126;
          numbering(52,2) = 125;
          numbering(52,3) = 28;
          numbering(52,4) = 130;
          numbering(52,5) = 129;
          numbering(53,0) = 125;
          numbering(53,1) = 127;
          numbering(53,2) = 121;
          numbering(53,3) = 129;
          numbering(53,4) = 131;
          numbering(53,5) = 29;
          numbering(54,0) = 126;
          numbering(54,1) = 128;
          numbering(54,2) = 127;
          numbering(54,3) = 130;
          numbering(54,4) = 10;
          numbering(54,5) = 131;
          numbering(55,0) = 125;
          numbering(55,1) = 126;
          numbering(55,2) = 127;
          numbering(55,3) = 129;
          numbering(55,4) = 130;
          numbering(55,5) = 131;
          numbering(56,0) = 21;
          numbering(56,1) = 86;
          numbering(56,2) = 96;
          numbering(56,3) = 111;
          numbering(56,4) = 112;
          numbering(56,5) = 118;
          numbering(57,0) = 96;
          numbering(57,1) = 105;
          numbering(57,2) = 25;
          numbering(57,3) = 118;
          numbering(57,4) = 125;
          numbering(57,5) = 121;
          numbering(58,0) = 86;
          numbering(58,1) = 22;
          numbering(58,2) = 105;
          numbering(58,3) = 112;
          numbering(58,4) = 113;
          numbering(58,5) = 125;
          numbering(59,0) = 96;
          numbering(59,1) = 86;
          numbering(59,2) = 105;
          numbering(59,3) = 118;
          numbering(59,4) = 112;
          numbering(59,5) = 125;
          numbering(60,0) = 111;
          numbering(60,1) = 112;
          numbering(60,2) = 118;
          numbering(60,3) = 27;
          numbering(60,4) = 116;
          numbering(60,5) = 123;
          numbering(61,0) = 118;
          numbering(61,1) = 125;
          numbering(61,2) = 121;
          numbering(61,3) = 123;
          numbering(61,4) = 129;
          numbering(61,5) = 29;
          numbering(62,0) = 112;
          numbering(62,1) = 113;
          numbering(62,2) = 125;
          numbering(62,3) = 116;
          numbering(62,4) = 28;
          numbering(62,5) = 129;
          numbering(63,0) = 118;
          numbering(63,1) = 112;
          numbering(63,2) = 125;
          numbering(63,3) = 123;
          numbering(63,4) = 116;
          numbering(63,5) = 129;
          numbering(64,0) = 6;
          numbering(64,1) = 133;
          numbering(64,2) = 132;
          numbering(64,3) = 92;
          numbering(64,4) = 135;
          numbering(64,5) = 134;
          numbering(65,0) = 132;
          numbering(65,1) = 136;
          numbering(65,2) = 30;
          numbering(65,3) = 134;
          numbering(65,4) = 138;
          numbering(65,5) = 137;
          numbering(66,0) = 133;
          numbering(66,1) = 31;
          numbering(66,2) = 136;
          numbering(66,3) = 135;
          numbering(66,4) = 139;
          numbering(66,5) = 138;
          numbering(67,0) = 132;
          numbering(67,1) = 133;
          numbering(67,2) = 136;
          numbering(67,3) = 134;
          numbering(67,4) = 135;
          numbering(67,5) = 138;
          numbering(68,0) = 92;
          numbering(68,1) = 135;
          numbering(68,2) = 134;
          numbering(68,3) = 24;
          numbering(68,4) = 141;
          numbering(68,5) = 140;
          numbering(69,0) = 134;
          numbering(69,1) = 138;
          numbering(69,2) = 137;
          numbering(69,3) = 140;
          numbering(69,4) = 142;
          numbering(69,5) = 32;
          numbering(70,0) = 135;
          numbering(70,1) = 139;
          numbering(70,2) = 138;
          numbering(70,3) = 141;
          numbering(70,4) = 33;
          numbering(70,5) = 142;
          numbering(71,0) = 134;
          numbering(71,1) = 135;
          numbering(71,2) = 138;
          numbering(71,3) = 140;
          numbering(71,4) = 141;
          numbering(71,5) = 142;
          numbering(72,0) = 30;
          numbering(72,1) = 144;
          numbering(72,2) = 143;
          numbering(72,3) = 137;
          numbering(72,4) = 146;
          numbering(72,5) = 145;
          numbering(73,0) = 143;
          numbering(73,1) = 147;
          numbering(73,2) = 1;
          numbering(73,3) = 145;
          numbering(73,4) = 149;
          numbering(73,5) = 148;
          numbering(74,0) = 144;
          numbering(74,1) = 34;
          numbering(74,2) = 147;
          numbering(74,3) = 146;
          numbering(74,4) = 150;
          numbering(74,5) = 149;
          numbering(75,0) = 143;
          numbering(75,1) = 144;
          numbering(75,2) = 147;
          numbering(75,3) = 145;
          numbering(75,4) = 146;
          numbering(75,5) = 149;
          numbering(76,0) = 137;
          numbering(76,1) = 146;
          numbering(76,2) = 145;
          numbering(76,3) = 32;
          numbering(76,4) = 152;
          numbering(76,5) = 151;
          numbering(77,0) = 145;
          numbering(77,1) = 149;
          numbering(77,2) = 148;
          numbering(77,3) = 151;
          numbering(77,4) = 153;
          numbering(77,5) = 35;
          numbering(78,0) = 146;
          numbering(78,1) = 150;
          numbering(78,2) = 149;
          numbering(78,3) = 152;
          numbering(78,4) = 36;
          numbering(78,5) = 153;
          numbering(79,0) = 145;
          numbering(79,1) = 146;
          numbering(79,2) = 149;
          numbering(79,3) = 151;
          numbering(79,4) = 152;
          numbering(79,5) = 153;
          numbering(80,0) = 31;
          numbering(80,1) = 155;
          numbering(80,2) = 154;
          numbering(80,3) = 139;
          numbering(80,4) = 157;
          numbering(80,5) = 156;
          numbering(81,0) = 154;
          numbering(81,1) = 158;
          numbering(81,2) = 34;
          numbering(81,3) = 156;
          numbering(81,4) = 159;
          numbering(81,5) = 150;
          numbering(82,0) = 155;
          numbering(82,1) = 11;
          numbering(82,2) = 158;
          numbering(82,3) = 157;
          numbering(82,4) = 160;
          numbering(82,5) = 159;
          numbering(83,0) = 154;
          numbering(83,1) = 155;
          numbering(83,2) = 158;
          numbering(83,3) = 156;
          numbering(83,4) = 157;
          numbering(83,5) = 159;
          numbering(84,0) = 139;
          numbering(84,1) = 157;
          numbering(84,2) = 156;
          numbering(84,3) = 33;
          numbering(84,4) = 162;
          numbering(84,5) = 161;
          numbering(85,0) = 156;
          numbering(85,1) = 159;
          numbering(85,2) = 150;
          numbering(85,3) = 161;
          numbering(85,4) = 163;
          numbering(85,5) = 36;
          numbering(86,0) = 157;
          numbering(86,1) = 160;
          numbering(86,2) = 159;
          numbering(86,3) = 162;
          numbering(86,4) = 37;
          numbering(86,5) = 163;
          numbering(87,0) = 156;
          numbering(87,1) = 157;
          numbering(87,2) = 159;
          numbering(87,3) = 161;
          numbering(87,4) = 162;
          numbering(87,5) = 163;
          numbering(88,0) = 30;
          numbering(88,1) = 136;
          numbering(88,2) = 144;
          numbering(88,3) = 137;
          numbering(88,4) = 138;
          numbering(88,5) = 146;
          numbering(89,0) = 144;
          numbering(89,1) = 154;
          numbering(89,2) = 34;
          numbering(89,3) = 146;
          numbering(89,4) = 156;
          numbering(89,5) = 150;
          numbering(90,0) = 136;
          numbering(90,1) = 31;
          numbering(90,2) = 154;
          numbering(90,3) = 138;
          numbering(90,4) = 139;
          numbering(90,5) = 156;
          numbering(91,0) = 144;
          numbering(91,1) = 136;
          numbering(91,2) = 154;
          numbering(91,3) = 146;
          numbering(91,4) = 138;
          numbering(91,5) = 156;
          numbering(92,0) = 137;
          numbering(92,1) = 138;
          numbering(92,2) = 146;
          numbering(92,3) = 32;
          numbering(92,4) = 142;
          numbering(92,5) = 152;
          numbering(93,0) = 146;
          numbering(93,1) = 156;
          numbering(93,2) = 150;
          numbering(93,3) = 152;
          numbering(93,4) = 161;
          numbering(93,5) = 36;
          numbering(94,0) = 138;
          numbering(94,1) = 139;
          numbering(94,2) = 156;
          numbering(94,3) = 142;
          numbering(94,4) = 33;
          numbering(94,5) = 161;
          numbering(95,0) = 146;
          numbering(95,1) = 138;
          numbering(95,2) = 156;
          numbering(95,3) = 152;
          numbering(95,4) = 142;
          numbering(95,5) = 161;
          numbering(96,0) = 24;
          numbering(96,1) = 141;
          numbering(96,2) = 140;
          numbering(96,3) = 119;
          numbering(96,4) = 165;
          numbering(96,5) = 164;
          numbering(97,0) = 140;
          numbering(97,1) = 142;
          numbering(97,2) = 32;
          numbering(97,3) = 164;
          numbering(97,4) = 167;
          numbering(97,5) = 166;
          numbering(98,0) = 141;
          numbering(98,1) = 33;
          numbering(98,2) = 142;
          numbering(98,3) = 165;
          numbering(98,4) = 168;
          numbering(98,5) = 167;
          numbering(99,0) = 140;
          numbering(99,1) = 141;
          numbering(99,2) = 142;
          numbering(99,3) = 164;
          numbering(99,4) = 165;
          numbering(99,5) = 167;
          numbering(100,0) = 119;
          numbering(100,1) = 165;
          numbering(100,2) = 164;
          numbering(100,3) = 9;
          numbering(100,4) = 170;
          numbering(100,5) = 169;
          numbering(101,0) = 164;
          numbering(101,1) = 167;
          numbering(101,2) = 166;
          numbering(101,3) = 169;
          numbering(101,4) = 171;
          numbering(101,5) = 38;
          numbering(102,0) = 165;
          numbering(102,1) = 168;
          numbering(102,2) = 167;
          numbering(102,3) = 170;
          numbering(102,4) = 39;
          numbering(102,5) = 171;
          numbering(103,0) = 164;
          numbering(103,1) = 165;
          numbering(103,2) = 167;
          numbering(103,3) = 169;
          numbering(103,4) = 170;
          numbering(103,5) = 171;
          numbering(104,0) = 32;
          numbering(104,1) = 152;
          numbering(104,2) = 151;
          numbering(104,3) = 166;
          numbering(104,4) = 173;
          numbering(104,5) = 172;
          numbering(105,0) = 151;
          numbering(105,1) = 153;
          numbering(105,2) = 35;
          numbering(105,3) = 172;
          numbering(105,4) = 175;
          numbering(105,5) = 174;
          numbering(106,0) = 152;
          numbering(106,1) = 36;
          numbering(106,2) = 153;
          numbering(106,3) = 173;
          numbering(106,4) = 176;
          numbering(106,5) = 175;
          numbering(107,0) = 151;
          numbering(107,1) = 152;
          numbering(107,2) = 153;
          numbering(107,3) = 172;
          numbering(107,4) = 173;
          numbering(107,5) = 175;
          numbering(108,0) = 166;
          numbering(108,1) = 173;
          numbering(108,2) = 172;
          numbering(108,3) = 38;
          numbering(108,4) = 178;
          numbering(108,5) = 177;
          numbering(109,0) = 172;
          numbering(109,1) = 175;
          numbering(109,2) = 174;
          numbering(109,3) = 177;
          numbering(109,4) = 179;
          numbering(109,5) = 12;
          numbering(110,0) = 173;
          numbering(110,1) = 176;
          numbering(110,2) = 175;
          numbering(110,3) = 178;
          numbering(110,4) = 40;
          numbering(110,5) = 179;
          numbering(111,0) = 172;
          numbering(111,1) = 173;
          numbering(111,2) = 175;
          numbering(111,3) = 177;
          numbering(111,4) = 178;
          numbering(111,5) = 179;
          numbering(112,0) = 33;
          numbering(112,1) = 162;
          numbering(112,2) = 161;
          numbering(112,3) = 168;
          numbering(112,4) = 181;
          numbering(112,5) = 180;
          numbering(113,0) = 161;
          numbering(113,1) = 163;
          numbering(113,2) = 36;
          numbering(113,3) = 180;
          numbering(113,4) = 182;
          numbering(113,5) = 176;
          numbering(114,0) = 162;
          numbering(114,1) = 37;
          numbering(114,2) = 163;
          numbering(114,3) = 181;
          numbering(114,4) = 183;
          numbering(114,5) = 182;
          numbering(115,0) = 161;
          numbering(115,1) = 162;
          numbering(115,2) = 163;
          numbering(115,3) = 180;
          numbering(115,4) = 181;
          numbering(115,5) = 182;
          numbering(116,0) = 168;
          numbering(116,1) = 181;
          numbering(116,2) = 180;
          numbering(116,3) = 39;
          numbering(116,4) = 185;
          numbering(116,5) = 184;
          numbering(117,0) = 180;
          numbering(117,1) = 182;
          numbering(117,2) = 176;
          numbering(117,3) = 184;
          numbering(117,4) = 186;
          numbering(117,5) = 40;
          numbering(118,0) = 181;
          numbering(118,1) = 183;
          numbering(118,2) = 182;
          numbering(118,3) = 185;
          numbering(118,4) = 13;
          numbering(118,5) = 186;
          numbering(119,0) = 180;
          numbering(119,1) = 181;
          numbering(119,2) = 182;
          numbering(119,3) = 184;
          numbering(119,4) = 185;
          numbering(119,5) = 186;
          numbering(120,0) = 32;
          numbering(120,1) = 142;
          numbering(120,2) = 152;
          numbering(120,3) = 166;
          numbering(120,4) = 167;
          numbering(120,5) = 173;
          numbering(121,0) = 152;
          numbering(121,1) = 161;
          numbering(121,2) = 36;
          numbering(121,3) = 173;
          numbering(121,4) = 180;
          numbering(121,5) = 176;
          numbering(122,0) = 142;
          numbering(122,1) = 33;
          numbering(122,2) = 161;
          numbering(122,3) = 167;
          numbering(122,4) = 168;
          numbering(122,5) = 180;
          numbering(123,0) = 152;
          numbering(123,1) = 142;
          numbering(123,2) = 161;
          numbering(123,3) = 173;
          numbering(123,4) = 167;
          numbering(123,5) = 180;
          numbering(124,0) = 166;
          numbering(124,1) = 167;
          numbering(124,2) = 173;
          numbering(124,3) = 38;
          numbering(124,4) = 171;
          numbering(124,5) = 178;
          numbering(125,0) = 173;
          numbering(125,1) = 180;
          numbering(125,2) = 176;
          numbering(125,3) = 178;
          numbering(125,4) = 184;
          numbering(125,5) = 40;
          numbering(126,0) = 167;
          numbering(126,1) = 168;
          numbering(126,2) = 180;
          numbering(126,3) = 171;
          numbering(126,4) = 39;
          numbering(126,5) = 184;
          numbering(127,0) = 173;
          numbering(127,1) = 167;
          numbering(127,2) = 180;
          numbering(127,3) = 178;
          numbering(127,4) = 171;
          numbering(127,5) = 184;
          numbering(128,0) = 7;
          numbering(128,1) = 188;
          numbering(128,2) = 187;
          numbering(128,3) = 104;
          numbering(128,4) = 190;
          numbering(128,5) = 189;
          numbering(129,0) = 187;
          numbering(129,1) = 191;
          numbering(129,2) = 41;
          numbering(129,3) = 189;
          numbering(129,4) = 193;
          numbering(129,5) = 192;
          numbering(130,0) = 188;
          numbering(130,1) = 42;
          numbering(130,2) = 191;
          numbering(130,3) = 190;
          numbering(130,4) = 194;
          numbering(130,5) = 193;
          numbering(131,0) = 187;
          numbering(131,1) = 188;
          numbering(131,2) = 191;
          numbering(131,3) = 189;
          numbering(131,4) = 190;
          numbering(131,5) = 193;
          numbering(132,0) = 104;
          numbering(132,1) = 190;
          numbering(132,2) = 189;
          numbering(132,3) = 26;
          numbering(132,4) = 196;
          numbering(132,5) = 195;
          numbering(133,0) = 189;
          numbering(133,1) = 193;
          numbering(133,2) = 192;
          numbering(133,3) = 195;
          numbering(133,4) = 197;
          numbering(133,5) = 43;
          numbering(134,0) = 190;
          numbering(134,1) = 194;
          numbering(134,2) = 193;
          numbering(134,3) = 196;
          numbering(134,4) = 44;
          numbering(134,5) = 197;
          numbering(135,0) = 189;
          numbering(135,1) = 190;
          numbering(135,2) = 193;
          numbering(135,3) = 195;
          numbering(135,4) = 196;
          numbering(135,5) = 197;
          numbering(136,0) = 41;
          numbering(136,1) = 199;
          numbering(136,2) = 198;
          numbering(136,3) = 192;
          numbering(136,4) = 201;
          numbering(136,5) = 200;
          numbering(137,0) = 198;
          numbering(137,1) = 202;
          numbering(137,2) = 11;
          numbering(137,3) = 200;
          numbering(137,4) = 203;
          numbering(137,5) = 160;
          numbering(138,0) = 199;
          numbering(138,1) = 45;
          numbering(138,2) = 202;
          numbering(138,3) = 201;
          numbering(138,4) = 204;
          numbering(138,5) = 203;
          numbering(139,0) = 198;
          numbering(139,1) = 199;
          numbering(139,2) = 202;
          numbering(139,3) = 200;
          numbering(139,4) = 201;
          numbering(139,5) = 203;
          numbering(140,0) = 192;
          numbering(140,1) = 201;
          numbering(140,2) = 200;
          numbering(140,3) = 43;
          numbering(140,4) = 206;
          numbering(140,5) = 205;
          numbering(141,0) = 200;
          numbering(141,1) = 203;
          numbering(141,2) = 160;
          numbering(141,3) = 205;
          numbering(141,4) = 207;
          numbering(141,5) = 37;
          numbering(142,0) = 201;
          numbering(142,1) = 204;
          numbering(142,2) = 203;
          numbering(142,3) = 206;
          numbering(142,4) = 46;
          numbering(142,5) = 207;
          numbering(143,0) = 200;
          numbering(143,1) = 201;
          numbering(143,2) = 203;
          numbering(143,3) = 205;
          numbering(143,4) = 206;
          numbering(143,5) = 207;
          numbering(144,0) = 42;
          numbering(144,1) = 209;
          numbering(144,2) = 208;
          numbering(144,3) = 194;
          numbering(144,4) = 211;
          numbering(144,5) = 210;
          numbering(145,0) = 208;
          numbering(145,1) = 212;
          numbering(145,2) = 45;
          numbering(145,3) = 210;
          numbering(145,4) = 213;
          numbering(145,5) = 204;
          numbering(146,0) = 209;
          numbering(146,1) = 2;
          numbering(146,2) = 212;
          numbering(146,3) = 211;
          numbering(146,4) = 214;
          numbering(146,5) = 213;
          numbering(147,0) = 208;
          numbering(147,1) = 209;
          numbering(147,2) = 212;
          numbering(147,3) = 210;
          numbering(147,4) = 211;
          numbering(147,5) = 213;
          numbering(148,0) = 194;
          numbering(148,1) = 211;
          numbering(148,2) = 210;
          numbering(148,3) = 44;
          numbering(148,4) = 216;
          numbering(148,5) = 215;
          numbering(149,0) = 210;
          numbering(149,1) = 213;
          numbering(149,2) = 204;
          numbering(149,3) = 215;
          numbering(149,4) = 217;
          numbering(149,5) = 46;
          numbering(150,0) = 211;
          numbering(150,1) = 214;
          numbering(150,2) = 213;
          numbering(150,3) = 216;
          numbering(150,4) = 47;
          numbering(150,5) = 217;
          numbering(151,0) = 210;
          numbering(151,1) = 211;
          numbering(151,2) = 213;
          numbering(151,3) = 215;
          numbering(151,4) = 216;
          numbering(151,5) = 217;
          numbering(152,0) = 41;
          numbering(152,1) = 191;
          numbering(152,2) = 199;
          numbering(152,3) = 192;
          numbering(152,4) = 193;
          numbering(152,5) = 201;
          numbering(153,0) = 199;
          numbering(153,1) = 208;
          numbering(153,2) = 45;
          numbering(153,3) = 201;
          numbering(153,4) = 210;
          numbering(153,5) = 204;
          numbering(154,0) = 191;
          numbering(154,1) = 42;
          numbering(154,2) = 208;
          numbering(154,3) = 193;
          numbering(154,4) = 194;
          numbering(154,5) = 210;
          numbering(155,0) = 199;
          numbering(155,1) = 191;
          numbering(155,2) = 208;
          numbering(155,3) = 201;
          numbering(155,4) = 193;
          numbering(155,5) = 210;
          numbering(156,0) = 192;
          numbering(156,1) = 193;
          numbering(156,2) = 201;
          numbering(156,3) = 43;
          numbering(156,4) = 197;
          numbering(156,5) = 206;
          numbering(157,0) = 201;
          numbering(157,1) = 210;
          numbering(157,2) = 204;
          numbering(157,3) = 206;
          numbering(157,4) = 215;
          numbering(157,5) = 46;
          numbering(158,0) = 193;
          numbering(158,1) = 194;
          numbering(158,2) = 210;
          numbering(158,3) = 197;
          numbering(158,4) = 44;
          numbering(158,5) = 215;
          numbering(159,0) = 201;
          numbering(159,1) = 193;
          numbering(159,2) = 210;
          numbering(159,3) = 206;
          numbering(159,4) = 197;
          numbering(159,5) = 215;
          numbering(160,0) = 26;
          numbering(160,1) = 196;
          numbering(160,2) = 195;
          numbering(160,3) = 128;
          numbering(160,4) = 219;
          numbering(160,5) = 218;
          numbering(161,0) = 195;
          numbering(161,1) = 197;
          numbering(161,2) = 43;
          numbering(161,3) = 218;
          numbering(161,4) = 221;
          numbering(161,5) = 220;
          numbering(162,0) = 196;
          numbering(162,1) = 44;
          numbering(162,2) = 197;
          numbering(162,3) = 219;
          numbering(162,4) = 222;
          numbering(162,5) = 221;
          numbering(163,0) = 195;
          numbering(163,1) = 196;
          numbering(163,2) = 197;
          numbering(163,3) = 218;
          numbering(163,4) = 219;
          numbering(163,5) = 221;
          numbering(164,0) = 128;
          numbering(164,1) = 219;
          numbering(164,2) = 218;
          numbering(164,3) = 10;
          numbering(164,4) = 224;
          numbering(164,5) = 223;
          numbering(165,0) = 218;
          numbering(165,1) = 221;
          numbering(165,2) = 220;
          numbering(165,3) = 223;
          numbering(165,4) = 225;
          numbering(165,5) = 48;
          numbering(166,0) = 219;
          numbering(166,1) = 222;
          numbering(166,2) = 221;
          numbering(166,3) = 224;
          numbering(166,4) = 49;
          numbering(166,5) = 225;
          numbering(167,0) = 218;
          numbering(167,1) = 219;
          numbering(167,2) = 221;
          numbering(167,3) = 223;
          numbering(167,4) = 224;
          numbering(167,5) = 225;
          numbering(168,0) = 43;
          numbering(168,1) = 206;
          numbering(168,2) = 205;
          numbering(168,3) = 220;
          numbering(168,4) = 227;
          numbering(168,5) = 226;
          numbering(169,0) = 205;
          numbering(169,1) = 207;
          numbering(169,2) = 37;
          numbering(169,3) = 226;
          numbering(169,4) = 228;
          numbering(169,5) = 183;
          numbering(170,0) = 206;
          numbering(170,1) = 46;
          numbering(170,2) = 207;
          numbering(170,3) = 227;
          numbering(170,4) = 229;
          numbering(170,5) = 228;
          numbering(171,0) = 205;
          numbering(171,1) = 206;
          numbering(171,2) = 207;
          numbering(171,3) = 226;
          numbering(171,4) = 227;
          numbering(171,5) = 228;
          numbering(172,0) = 220;
          numbering(172,1) = 227;
          numbering(172,2) = 226;
          numbering(172,3) = 48;
          numbering(172,4) = 231;
          numbering(172,5) = 230;
          numbering(173,0) = 226;
          numbering(173,1) = 228;
          numbering(173,2) = 183;
          numbering(173,3) = 230;
          numbering(173,4) = 232;
          numbering(173,5) = 13;
          numbering(174,0) = 227;
          numbering(174,1) = 229;
          numbering(174,2) = 228;
          numbering(174,3) = 231;
          numbering(174,4) = 50;
          numbering(174,5) = 232;
          numbering(175,0) = 226;
          numbering(175,1) = 227;
          numbering(175,2) = 228;
          numbering(175,3) = 230;
          numbering(175,4) = 231;
          numbering(175,5) = 232;
          numbering(176,0) = 44;
          numbering(176,1) = 216;
          numbering(176,2) = 215;
          numbering(176,3) = 222;
          numbering(176,4) = 234;
          numbering(176,5) = 233;
          numbering(177,0) = 215;
          numbering(177,1) = 217;
          numbering(177,2) = 46;
          numbering(177,3) = 233;
          numbering(177,4) = 235;
          numbering(177,5) = 229;
          numbering(178,0) = 216;
          numbering(178,1) = 47;
          numbering(178,2) = 217;
          numbering(178,3) = 234;
          numbering(178,4) = 236;
          numbering(178,5) = 235;
          numbering(179,0) = 215;
          numbering(179,1) = 216;
          numbering(179,2) = 217;
          numbering(179,3) = 233;
          numbering(179,4) = 234;
          numbering(179,5) = 235;
          numbering(180,0) = 222;
          numbering(180,1) = 234;
          numbering(180,2) = 233;
          numbering(180,3) = 49;
          numbering(180,4) = 238;
          numbering(180,5) = 237;
          numbering(181,0) = 233;
          numbering(181,1) = 235;
          numbering(181,2) = 229;
          numbering(181,3) = 237;
          numbering(181,4) = 239;
          numbering(181,5) = 50;
          numbering(182,0) = 234;
          numbering(182,1) = 236;
          numbering(182,2) = 235;
          numbering(182,3) = 238;
          numbering(182,4) = 14;
          numbering(182,5) = 239;
          numbering(183,0) = 233;
          numbering(183,1) = 234;
          numbering(183,2) = 235;
          numbering(183,3) = 237;
          numbering(183,4) = 238;
          numbering(183,5) = 239;
          numbering(184,0) = 43;
          numbering(184,1) = 197;
          numbering(184,2) = 206;
          numbering(184,3) = 220;
          numbering(184,4) = 221;
          numbering(184,5) = 227;
          numbering(185,0) = 206;
          numbering(185,1) = 215;
          numbering(185,2) = 46;
          numbering(185,3) = 227;
          numbering(185,4) = 233;
          numbering(185,5) = 229;
          numbering(186,0) = 197;
          numbering(186,1) = 44;
          numbering(186,2) = 215;
          numbering(186,3) = 221;
          numbering(186,4) = 222;
          numbering(186,5) = 233;
          numbering(187,0) = 206;
          numbering(187,1) = 197;
          numbering(187,2) = 215;
          numbering(187,3) = 227;
          numbering(187,4) = 221;
          numbering(187,5) = 233;
          numbering(188,0) = 220;
          numbering(188,1) = 221;
          numbering(188,2) = 227;
          numbering(188,3) = 48;
          numbering(188,4) = 225;
          numbering(188,5) = 231;
          numbering(189,0) = 227;
          numbering(189,1) = 233;
          numbering(189,2) = 229;
          numbering(189,3) = 231;
          numbering(189,4) = 237;
          numbering(189,5) = 50;
          numbering(190,0) = 221;
          numbering(190,1) = 222;
          numbering(190,2) = 233;
          numbering(190,3) = 225;
          numbering(190,4) = 49;
          numbering(190,5) = 237;
          numbering(191,0) = 227;
          numbering(191,1) = 221;
          numbering(191,2) = 233;
          numbering(191,3) = 231;
          numbering(191,4) = 225;
          numbering(191,5) = 237;
          numbering(192,0) = 6;
          numbering(192,1) = 91;
          numbering(192,2) = 133;
          numbering(192,3) = 92;
          numbering(192,4) = 93;
          numbering(192,5) = 135;
          numbering(193,0) = 133;
          numbering(193,1) = 240;
          numbering(193,2) = 31;
          numbering(193,3) = 135;
          numbering(193,4) = 241;
          numbering(193,5) = 139;
          numbering(194,0) = 91;
          numbering(194,1) = 23;
          numbering(194,2) = 240;
          numbering(194,3) = 93;
          numbering(194,4) = 94;
          numbering(194,5) = 241;
          numbering(195,0) = 133;
          numbering(195,1) = 91;
          numbering(195,2) = 240;
          numbering(195,3) = 135;
          numbering(195,4) = 93;
          numbering(195,5) = 241;
          numbering(196,0) = 92;
          numbering(196,1) = 93;
          numbering(196,2) = 135;
          numbering(196,3) = 24;
          numbering(196,4) = 97;
          numbering(196,5) = 141;
          numbering(197,0) = 135;
          numbering(197,1) = 241;
          numbering(197,2) = 139;
          numbering(197,3) = 141;
          numbering(197,4) = 242;
          numbering(197,5) = 33;
          numbering(198,0) = 93;
          numbering(198,1) = 94;
          numbering(198,2) = 241;
          numbering(198,3) = 97;
          numbering(198,4) = 25;
          numbering(198,5) = 242;
          numbering(199,0) = 135;
          numbering(199,1) = 93;
          numbering(199,2) = 241;
          numbering(199,3) = 141;
          numbering(199,4) = 97;
          numbering(199,5) = 242;
          numbering(200,0) = 31;
          numbering(200,1) = 243;
          numbering(200,2) = 155;
          numbering(200,3) = 139;
          numbering(200,4) = 244;
          numbering(200,5) = 157;
          numbering(201,0) = 155;
          numbering(201,1) = 198;
          numbering(201,2) = 11;
          numbering(201,3) = 157;
          numbering(201,4) = 200;
          numbering(201,5) = 160;
          numbering(202,0) = 243;
          numbering(202,1) = 41;
          numbering(202,2) = 198;
          numbering(202,3) = 244;
          numbering(202,4) = 192;
          numbering(202,5) = 200;
          numbering(203,0) = 155;
          numbering(203,1) = 243;
          numbering(203,2) = 198;
          numbering(203,3) = 157;
          numbering(203,4) = 244;
          numbering(203,5) = 200;
          numbering(204,0) = 139;
          numbering(204,1) = 244;
          numbering(204,2) = 157;
          numbering(204,3) = 33;
          numbering(204,4) = 245;
          numbering(204,5) = 162;
          numbering(205,0) = 157;
          numbering(205,1) = 200;
          numbering(205,2) = 160;
          numbering(205,3) = 162;
          numbering(205,4) = 205;
          numbering(205,5) = 37;
          numbering(206,0) = 244;
          numbering(206,1) = 192;
          numbering(206,2) = 200;
          numbering(206,3) = 245;
          numbering(206,4) = 43;
          numbering(206,5) = 205;
          numbering(207,0) = 157;
          numbering(207,1) = 244;
          numbering(207,2) = 200;
          numbering(207,3) = 162;
          numbering(207,4) = 245;
          numbering(207,5) = 205;
          numbering(208,0) = 23;
          numbering(208,1) = 102;
          numbering(208,2) = 246;
          numbering(208,3) = 94;
          numbering(208,4) = 103;
          numbering(208,5) = 247;
          numbering(209,0) = 246;
          numbering(209,1) = 187;
          numbering(209,2) = 41;
          numbering(209,3) = 247;
          numbering(209,4) = 189;
          numbering(209,5) = 192;
          numbering(210,0) = 102;
          numbering(210,1) = 7;
          numbering(210,2) = 187;
          numbering(210,3) = 103;
          numbering(210,4) = 104;
          numbering(210,5) = 189;
          numbering(211,0) = 246;
          numbering(211,1) = 102;
          numbering(211,2) = 187;
          numbering(211,3) = 247;
          numbering(211,4) = 103;
          numbering(211,5) = 189;
          numbering(212,0) = 94;
          numbering(212,1) = 103;
          numbering(212,2) = 247;
          numbering(212,3) = 25;
          numbering(212,4) = 107;
          numbering(212,5) = 248;
          numbering(213,0) = 247;
          numbering(213,1) = 189;
          numbering(213,2) = 192;
          numbering(213,3) = 248;
          numbering(213,4) = 195;
          numbering(213,5) = 43;
          numbering(214,0) = 103;
          numbering(214,1) = 104;
          numbering(214,2) = 189;
          numbering(214,3) = 107;
          numbering(214,4) = 26;
          numbering(214,5) = 195;
          numbering(215,0) = 247;
          numbering(215,1) = 103;
          numbering(215,2) = 189;
          numbering(215,3) = 248;
          numbering(215,4) = 107;
          numbering(215,5) = 195;
          numbering(216,0) = 31;
          numbering(216,1) = 240;
          numbering(216,2) = 243;
          numbering(216,3) = 139;
          numbering(216,4) = 241;
          numbering(216,5) = 244;
          numbering(217,0) = 243;
          numbering(217,1) = 246;
          numbering(217,2) = 41;
          numbering(217,3) = 244;
          numbering(217,4) = 247;
          numbering(217,5) = 192;
          numbering(218,0) = 240;
          numbering(218,1) = 23;
          numbering(218,2) = 246;
          numbering(218,3) = 241;
          numbering(218,4) = 94;
          numbering(218,5) = 247;
          numbering(219,0) = 243;
          numbering(219,1) = 240;
          numbering(219,2) = 246;
          numbering(219,3) = 244;
          numbering(219,4) = 241;
          numbering(219,5) = 247;
          numbering(220,0) = 139;
          numbering(220,1) = 241;
          numbering(220,2) = 244;
          numbering(220,3) = 33;
          numbering(220,4) = 242;
          numbering(220,5) = 245;
          numbering(221,0) = 244;
          numbering(221,1) = 247;
          numbering(221,2) = 192;
          numbering(221,3) = 245;
          numbering(221,4) = 248;
          numbering(221,5) = 43;
          numbering(222,0) = 241;
          numbering(222,1) = 94;
          numbering(222,2) = 247;
          numbering(222,3) = 242;
          numbering(222,4) = 25;
          numbering(222,5) = 248;
          numbering(223,0) = 244;
          numbering(223,1) = 241;
          numbering(223,2) = 247;
          numbering(223,3) = 245;
          numbering(223,4) = 242;
          numbering(223,5) = 248;
          numbering(224,0) = 24;
          numbering(224,1) = 97;
          numbering(224,2) = 141;
          numbering(224,3) = 119;
          numbering(224,4) = 120;
          numbering(224,5) = 165;
          numbering(225,0) = 141;
          numbering(225,1) = 242;
          numbering(225,2) = 33;
          numbering(225,3) = 165;
          numbering(225,4) = 249;
          numbering(225,5) = 168;
          numbering(226,0) = 97;
          numbering(226,1) = 25;
          numbering(226,2) = 242;
          numbering(226,3) = 120;
          numbering(226,4) = 121;
          numbering(226,5) = 249;
          numbering(227,0) = 141;
          numbering(227,1) = 97;
          numbering(227,2) = 242;
          numbering(227,3) = 165;
          numbering(227,4) = 120;
          numbering(227,5) = 249;
          numbering(228,0) = 119;
          numbering(228,1) = 120;
          numbering(228,2) = 165;
          numbering(228,3) = 9;
          numbering(228,4) = 124;
          numbering(228,5) = 170;
          numbering(229,0) = 165;
          numbering(229,1) = 249;
          numbering(229,2) = 168;
          numbering(229,3) = 170;
          numbering(229,4) = 250;
          numbering(229,5) = 39;
          numbering(230,0) = 120;
          numbering(230,1) = 121;
          numbering(230,2) = 249;
          numbering(230,3) = 124;
          numbering(230,4) = 29;
          numbering(230,5) = 250;
          numbering(231,0) = 165;
          numbering(231,1) = 120;
          numbering(231,2) = 249;
          numbering(231,3) = 170;
          numbering(231,4) = 124;
          numbering(231,5) = 250;
          numbering(232,0) = 33;
          numbering(232,1) = 245;
          numbering(232,2) = 162;
          numbering(232,3) = 168;
          numbering(232,4) = 251;
          numbering(232,5) = 181;
          numbering(233,0) = 162;
          numbering(233,1) = 205;
          numbering(233,2) = 37;
          numbering(233,3) = 181;
          numbering(233,4) = 226;
          numbering(233,5) = 183;
          numbering(234,0) = 245;
          numbering(234,1) = 43;
          numbering(234,2) = 205;
          numbering(234,3) = 251;
          numbering(234,4) = 220;
          numbering(234,5) = 226;
          numbering(235,0) = 162;
          numbering(235,1) = 245;
          numbering(235,2) = 205;
          numbering(235,3) = 181;
          numbering(235,4) = 251;
          numbering(235,5) = 226;
          numbering(236,0) = 168;
          numbering(236,1) = 251;
          numbering(236,2) = 181;
          numbering(236,3) = 39;
          numbering(236,4) = 252;
          numbering(236,5) = 185;
          numbering(237,0) = 181;
          numbering(237,1) = 226;
          numbering(237,2) = 183;
          numbering(237,3) = 185;
          numbering(237,4) = 230;
          numbering(237,5) = 13;
          numbering(238,0) = 251;
          numbering(238,1) = 220;
          numbering(238,2) = 226;
          numbering(238,3) = 252;
          numbering(238,4) = 48;
          numbering(238,5) = 230;
          numbering(239,0) = 181;
          numbering(239,1) = 251;
          numbering(239,2) = 226;
          numbering(239,3) = 185;
          numbering(239,4) = 252;
          numbering(239,5) = 230;
          numbering(240,0) = 25;
          numbering(240,1) = 107;
          numbering(240,2) = 248;
          numbering(240,3) = 121;
          numbering(240,4) = 127;
          numbering(240,5) = 253;
          numbering(241,0) = 248;
          numbering(241,1) = 195;
          numbering(241,2) = 43;
          numbering(241,3) = 253;
          numbering(241,4) = 218;
          numbering(241,5) = 220;
          numbering(242,0) = 107;
          numbering(242,1) = 26;
          numbering(242,2) = 195;
          numbering(242,3) = 127;
          numbering(242,4) = 128;
          numbering(242,5) = 218;
          numbering(243,0) = 248;
          numbering(243,1) = 107;
          numbering(243,2) = 195;
          numbering(243,3) = 253;
          numbering(243,4) = 127;
          numbering(243,5) = 218;
          numbering(244,0) = 121;
          numbering(244,1) = 127;
          numbering(244,2) = 253;
          numbering(244,3) = 29;
          numbering(244,4) = 131;
          numbering(244,5) = 254;
          numbering(245,0) = 253;
          numbering(245,1) = 218;
          numbering(245,2) = 220;
          numbering(245,3) = 254;
          numbering(245,4) = 223;
          numbering(245,5) = 48;
          numbering(246,0) = 127;
          numbering(246,1) = 128;
          numbering(246,2) = 218;
          numbering(246,3) = 131;
          numbering(246,4) = 10;
          numbering(246,5) = 223;
          numbering(247,0) = 253;
          numbering(247,1) = 127;
          numbering(247,2) = 218;
          numbering(247,3) = 254;
          numbering(247,4) = 131;
          numbering(247,5) = 223;
          numbering(248,0) = 33;
          numbering(248,1) = 242;
          numbering(248,2) = 245;
          numbering(248,3) = 168;
          numbering(248,4) = 249;
          numbering(248,5) = 251;
          numbering(249,0) = 245;
          numbering(249,1) = 248;
          numbering(249,2) = 43;
          numbering(249,3) = 251;
          numbering(249,4) = 253;
          numbering(249,5) = 220;
          numbering(250,0) = 242;
          numbering(250,1) = 25;
          numbering(250,2) = 248;
          numbering(250,3) = 249;
          numbering(250,4) = 121;
          numbering(250,5) = 253;
          numbering(251,0) = 245;
          numbering(251,1) = 242;
          numbering(251,2) = 248;
          numbering(251,3) = 251;
          numbering(251,4) = 249;
          numbering(251,5) = 253;
          numbering(252,0) = 168;
          numbering(252,1) = 249;
          numbering(252,2) = 251;
          numbering(252,3) = 39;
          numbering(252,4) = 250;
          numbering(252,5) = 252;
          numbering(253,0) = 251;
          numbering(253,1) = 253;
          numbering(253,2) = 220;
          numbering(253,3) = 252;
          numbering(253,4) = 254;
          numbering(253,5) = 48;
          numbering(254,0) = 249;
          numbering(254,1) = 121;
          numbering(254,2) = 253;
          numbering(254,3) = 250;
          numbering(254,4) = 29;
          numbering(254,5) = 254;
          numbering(255,0) = 251;
          numbering(255,1) = 249;
          numbering(255,2) = 253;
          numbering(255,3) = 252;
          numbering(255,4) = 250;
          numbering(255,5) = 254;
          numbering(256,0) = 8;
          numbering(256,1) = 115;
          numbering(256,2) = 114;
          numbering(256,3) = 255;
          numbering(256,4) = 257;
          numbering(256,5) = 256;
          numbering(257,0) = 114;
          numbering(257,1) = 116;
          numbering(257,2) = 27;
          numbering(257,3) = 256;
          numbering(257,4) = 259;
          numbering(257,5) = 258;
          numbering(258,0) = 115;
          numbering(258,1) = 28;
          numbering(258,2) = 116;
          numbering(258,3) = 257;
          numbering(258,4) = 260;
          numbering(258,5) = 259;
          numbering(259,0) = 114;
          numbering(259,1) = 115;
          numbering(259,2) = 116;
          numbering(259,3) = 256;
          numbering(259,4) = 257;
          numbering(259,5) = 259;
          numbering(260,0) = 255;
          numbering(260,1) = 257;
          numbering(260,2) = 256;
          numbering(260,3) = 51;
          numbering(260,4) = 262;
          numbering(260,5) = 261;
          numbering(261,0) = 256;
          numbering(261,1) = 259;
          numbering(261,2) = 258;
          numbering(261,3) = 261;
          numbering(261,4) = 263;
          numbering(261,5) = 52;
          numbering(262,0) = 257;
          numbering(262,1) = 260;
          numbering(262,2) = 259;
          numbering(262,3) = 262;
          numbering(262,4) = 53;
          numbering(262,5) = 263;
          numbering(263,0) = 256;
          numbering(263,1) = 257;
          numbering(263,2) = 259;
          numbering(263,3) = 261;
          numbering(263,4) = 262;
          numbering(263,5) = 263;
          numbering(264,0) = 27;
          numbering(264,1) = 123;
          numbering(264,2) = 122;
          numbering(264,3) = 258;
          numbering(264,4) = 265;
          numbering(264,5) = 264;
          numbering(265,0) = 122;
          numbering(265,1) = 124;
          numbering(265,2) = 9;
          numbering(265,3) = 264;
          numbering(265,4) = 267;
          numbering(265,5) = 266;
          numbering(266,0) = 123;
          numbering(266,1) = 29;
          numbering(266,2) = 124;
          numbering(266,3) = 265;
          numbering(266,4) = 268;
          numbering(266,5) = 267;
          numbering(267,0) = 122;
          numbering(267,1) = 123;
          numbering(267,2) = 124;
          numbering(267,3) = 264;
          numbering(267,4) = 265;
          numbering(267,5) = 267;
          numbering(268,0) = 258;
          numbering(268,1) = 265;
          numbering(268,2) = 264;
          numbering(268,3) = 52;
          numbering(268,4) = 270;
          numbering(268,5) = 269;
          numbering(269,0) = 264;
          numbering(269,1) = 267;
          numbering(269,2) = 266;
          numbering(269,3) = 269;
          numbering(269,4) = 271;
          numbering(269,5) = 54;
          numbering(270,0) = 265;
          numbering(270,1) = 268;
          numbering(270,2) = 267;
          numbering(270,3) = 270;
          numbering(270,4) = 55;
          numbering(270,5) = 271;
          numbering(271,0) = 264;
          numbering(271,1) = 265;
          numbering(271,2) = 267;
          numbering(271,3) = 269;
          numbering(271,4) = 270;
          numbering(271,5) = 271;
          numbering(272,0) = 28;
          numbering(272,1) = 130;
          numbering(272,2) = 129;
          numbering(272,3) = 260;
          numbering(272,4) = 273;
          numbering(272,5) = 272;
          numbering(273,0) = 129;
          numbering(273,1) = 131;
          numbering(273,2) = 29;
          numbering(273,3) = 272;
          numbering(273,4) = 274;
          numbering(273,5) = 268;
          numbering(274,0) = 130;
          numbering(274,1) = 10;
          numbering(274,2) = 131;
          numbering(274,3) = 273;
          numbering(274,4) = 275;
          numbering(274,5) = 274;
          numbering(275,0) = 129;
          numbering(275,1) = 130;
          numbering(275,2) = 131;
          numbering(275,3) = 272;
          numbering(275,4) = 273;
          numbering(275,5) = 274;
          numbering(276,0) = 260;
          numbering(276,1) = 273;
          numbering(276,2) = 272;
          numbering(276,3) = 53;
          numbering(276,4) = 277;
          numbering(276,5) = 276;
          numbering(277,0) = 272;
          numbering(277,1) = 274;
          numbering(277,2) = 268;
          numbering(277,3) = 276;
          numbering(277,4) = 278;
          numbering(277,5) = 55;
          numbering(278,0) = 273;
          numbering(278,1) = 275;
          numbering(278,2) = 274;
          numbering(278,3) = 277;
          numbering(278,4) = 56;
          numbering(278,5) = 278;
          numbering(279,0) = 272;
          numbering(279,1) = 273;
          numbering(279,2) = 274;
          numbering(279,3) = 276;
          numbering(279,4) = 277;
          numbering(279,5) = 278;
          numbering(280,0) = 27;
          numbering(280,1) = 116;
          numbering(280,2) = 123;
          numbering(280,3) = 258;
          numbering(280,4) = 259;
          numbering(280,5) = 265;
          numbering(281,0) = 123;
          numbering(281,1) = 129;
          numbering(281,2) = 29;
          numbering(281,3) = 265;
          numbering(281,4) = 272;
          numbering(281,5) = 268;
          numbering(282,0) = 116;
          numbering(282,1) = 28;
          numbering(282,2) = 129;
          numbering(282,3) = 259;
          numbering(282,4) = 260;
          numbering(282,5) = 272;
          numbering(283,0) = 123;
          numbering(283,1) = 116;
          numbering(283,2) = 129;
          numbering(283,3) = 265;
          numbering(283,4) = 259;
          numbering(283,5) = 272;
          numbering(284,0) = 258;
          numbering(284,1) = 259;
          numbering(284,2) = 265;
          numbering(284,3) = 52;
          numbering(284,4) = 263;
          numbering(284,5) = 270;
          numbering(285,0) = 265;
          numbering(285,1) = 272;
          numbering(285,2) = 268;
          numbering(285,3) = 270;
          numbering(285,4) = 276;
          numbering(285,5) = 55;
          numbering(286,0) = 259;
          numbering(286,1) = 260;
          numbering(286,2) = 272;
          numbering(286,3) = 263;
          numbering(286,4) = 53;
          numbering(286,5) = 276;
          numbering(287,0) = 265;
          numbering(287,1) = 259;
          numbering(287,2) = 272;
          numbering(287,3) = 270;
          numbering(287,4) = 263;
          numbering(287,5) = 276;
          numbering(288,0) = 51;
          numbering(288,1) = 262;
          numbering(288,2) = 261;
          numbering(288,3) = 279;
          numbering(288,4) = 281;
          numbering(288,5) = 280;
          numbering(289,0) = 261;
          numbering(289,1) = 263;
          numbering(289,2) = 52;
          numbering(289,3) = 280;
          numbering(289,4) = 283;
          numbering(289,5) = 282;
          numbering(290,0) = 262;
          numbering(290,1) = 53;
          numbering(290,2) = 263;
          numbering(290,3) = 281;
          numbering(290,4) = 284;
          numbering(290,5) = 283;
          numbering(291,0) = 261;
          numbering(291,1) = 262;
          numbering(291,2) = 263;
          numbering(291,3) = 280;
          numbering(291,4) = 281;
          numbering(291,5) = 283;
          numbering(292,0) = 279;
          numbering(292,1) = 281;
          numbering(292,2) = 280;
          numbering(292,3) = 3;
          numbering(292,4) = 286;
          numbering(292,5) = 285;
          numbering(293,0) = 280;
          numbering(293,1) = 283;
          numbering(293,2) = 282;
          numbering(293,3) = 285;
          numbering(293,4) = 287;
          numbering(293,5) = 57;
          numbering(294,0) = 281;
          numbering(294,1) = 284;
          numbering(294,2) = 283;
          numbering(294,3) = 286;
          numbering(294,4) = 58;
          numbering(294,5) = 287;
          numbering(295,0) = 280;
          numbering(295,1) = 281;
          numbering(295,2) = 283;
          numbering(295,3) = 285;
          numbering(295,4) = 286;
          numbering(295,5) = 287;
          numbering(296,0) = 52;
          numbering(296,1) = 270;
          numbering(296,2) = 269;
          numbering(296,3) = 282;
          numbering(296,4) = 289;
          numbering(296,5) = 288;
          numbering(297,0) = 269;
          numbering(297,1) = 271;
          numbering(297,2) = 54;
          numbering(297,3) = 288;
          numbering(297,4) = 291;
          numbering(297,5) = 290;
          numbering(298,0) = 270;
          numbering(298,1) = 55;
          numbering(298,2) = 271;
          numbering(298,3) = 289;
          numbering(298,4) = 292;
          numbering(298,5) = 291;
          numbering(299,0) = 269;
          numbering(299,1) = 270;
          numbering(299,2) = 271;
          numbering(299,3) = 288;
          numbering(299,4) = 289;
          numbering(299,5) = 291;
          numbering(300,0) = 282;
          numbering(300,1) = 289;
          numbering(300,2) = 288;
          numbering(300,3) = 57;
          numbering(300,4) = 294;
          numbering(300,5) = 293;
          numbering(301,0) = 288;
          numbering(301,1) = 291;
          numbering(301,2) = 290;
          numbering(301,3) = 293;
          numbering(301,4) = 295;
          numbering(301,5) = 15;
          numbering(302,0) = 289;
          numbering(302,1) = 292;
          numbering(302,2) = 291;
          numbering(302,3) = 294;
          numbering(302,4) = 59;
          numbering(302,5) = 295;
          numbering(303,0) = 288;
          numbering(303,1) = 289;
          numbering(303,2) = 291;
          numbering(303,3) = 293;
          numbering(303,4) = 294;
          numbering(303,5) = 295;
          numbering(304,0) = 53;
          numbering(304,1) = 277;
          numbering(304,2) = 276;
          numbering(304,3) = 284;
          numbering(304,4) = 297;
          numbering(304,5) = 296;
          numbering(305,0) = 276;
          numbering(305,1) = 278;
          numbering(305,2) = 55;
          numbering(305,3) = 296;
          numbering(305,4) = 298;
          numbering(305,5) = 292;
          numbering(306,0) = 277;
          numbering(306,1) = 56;
          numbering(306,2) = 278;
          numbering(306,3) = 297;
          numbering(306,4) = 299;
          numbering(306,5) = 298;
          numbering(307,0) = 276;
          numbering(307,1) = 277;
          numbering(307,2) = 278;
          numbering(307,3) = 296;
          numbering(307,4) = 297;
          numbering(307,5) = 298;
          numbering(308,0) = 284;
          numbering(308,1) = 297;
          numbering(308,2) = 296;
          numbering(308,3) = 58;
          numbering(308,4) = 301;
          numbering(308,5) = 300;
          numbering(309,0) = 296;
          numbering(309,1) = 298;
          numbering(309,2) = 292;
          numbering(309,3) = 300;
          numbering(309,4) = 302;
          numbering(309,5) = 59;
          numbering(310,0) = 297;
          numbering(310,1) = 299;
          numbering(310,2) = 298;
          numbering(310,3) = 301;
          numbering(310,4) = 16;
          numbering(310,5) = 302;
          numbering(311,0) = 296;
          numbering(311,1) = 297;
          numbering(311,2) = 298;
          numbering(311,3) = 300;
          numbering(311,4) = 301;
          numbering(311,5) = 302;
          numbering(312,0) = 52;
          numbering(312,1) = 263;
          numbering(312,2) = 270;
          numbering(312,3) = 282;
          numbering(312,4) = 283;
          numbering(312,5) = 289;
          numbering(313,0) = 270;
          numbering(313,1) = 276;
          numbering(313,2) = 55;
          numbering(313,3) = 289;
          numbering(313,4) = 296;
          numbering(313,5) = 292;
          numbering(314,0) = 263;
          numbering(314,1) = 53;
          numbering(314,2) = 276;
          numbering(314,3) = 283;
          numbering(314,4) = 284;
          numbering(314,5) = 296;
          numbering(315,0) = 270;
          numbering(315,1) = 263;
          numbering(315,2) = 276;
          numbering(315,3) = 289;
          numbering(315,4) = 283;
          numbering(315,5) = 296;
          numbering(316,0) = 282;
          numbering(316,1) = 283;
          numbering(316,2) = 289;
          numbering(316,3) = 57;
          numbering(316,4) = 287;
          numbering(316,5) = 294;
          numbering(317,0) = 289;
          numbering(317,1) = 296;
          numbering(317,2) = 292;
          numbering(317,3) = 294;
          numbering(317,4) = 300;
          numbering(317,5) = 59;
          numbering(318,0) = 283;
          numbering(318,1) = 284;
          numbering(318,2) = 296;
          numbering(318,3) = 287;
          numbering(318,4) = 58;
          numbering(318,5) = 300;
          numbering(319,0) = 289;
          numbering(319,1) = 283;
          numbering(319,2) = 296;
          numbering(319,3) = 294;
          numbering(319,4) = 287;
          numbering(319,5) = 300;
          numbering(320,0) = 9;
          numbering(320,1) = 170;
          numbering(320,2) = 169;
          numbering(320,3) = 266;
          numbering(320,4) = 304;
          numbering(320,5) = 303;
          numbering(321,0) = 169;
          numbering(321,1) = 171;
          numbering(321,2) = 38;
          numbering(321,3) = 303;
          numbering(321,4) = 306;
          numbering(321,5) = 305;
          numbering(322,0) = 170;
          numbering(322,1) = 39;
          numbering(322,2) = 171;
          numbering(322,3) = 304;
          numbering(322,4) = 307;
          numbering(322,5) = 306;
          numbering(323,0) = 169;
          numbering(323,1) = 170;
          numbering(323,2) = 171;
          numbering(323,3) = 303;
          numbering(323,4) = 304;
          numbering(323,5) = 306;
          numbering(324,0) = 266;
          numbering(324,1) = 304;
          numbering(324,2) = 303;
          numbering(324,3) = 54;
          numbering(324,4) = 309;
          numbering(324,5) = 308;
          numbering(325,0) = 303;
          numbering(325,1) = 306;
          numbering(325,2) = 305;
          numbering(325,3) = 308;
          numbering(325,4) = 310;
          numbering(325,5) = 60;
          numbering(326,0) = 304;
          numbering(326,1) = 307;
          numbering(326,2) = 306;
          numbering(326,3) = 309;
          numbering(326,4) = 61;
          numbering(326,5) = 310;
          numbering(327,0) = 303;
          numbering(327,1) = 304;
          numbering(327,2) = 306;
          numbering(327,3) = 308;
          numbering(327,4) = 309;
          numbering(327,5) = 310;
          numbering(328,0) = 38;
          numbering(328,1) = 178;
          numbering(328,2) = 177;
          numbering(328,3) = 305;
          numbering(328,4) = 312;
          numbering(328,5) = 311;
          numbering(329,0) = 177;
          numbering(329,1) = 179;
          numbering(329,2) = 12;
          numbering(329,3) = 311;
          numbering(329,4) = 314;
          numbering(329,5) = 313;
          numbering(330,0) = 178;
          numbering(330,1) = 40;
          numbering(330,2) = 179;
          numbering(330,3) = 312;
          numbering(330,4) = 315;
          numbering(330,5) = 314;
          numbering(331,0) = 177;
          numbering(331,1) = 178;
          numbering(331,2) = 179;
          numbering(331,3) = 311;
          numbering(331,4) = 312;
          numbering(331,5) = 314;
          numbering(332,0) = 305;
          numbering(332,1) = 312;
          numbering(332,2) = 311;
          numbering(332,3) = 60;
          numbering(332,4) = 317;
          numbering(332,5) = 316;
          numbering(333,0) = 311;
          numbering(333,1) = 314;
          numbering(333,2) = 313;
          numbering(333,3) = 316;
          numbering(333,4) = 318;
          numbering(333,5) = 62;
          numbering(334,0) = 312;
          numbering(334,1) = 315;
          numbering(334,2) = 314;
          numbering(334,3) = 317;
          numbering(334,4) = 63;
          numbering(334,5) = 318;
          numbering(335,0) = 311;
          numbering(335,1) = 312;
          numbering(335,2) = 314;
          numbering(335,3) = 316;
          numbering(335,4) = 317;
          numbering(335,5) = 318;
          numbering(336,0) = 39;
          numbering(336,1) = 185;
          numbering(336,2) = 184;
          numbering(336,3) = 307;
          numbering(336,4) = 320;
          numbering(336,5) = 319;
          numbering(337,0) = 184;
          numbering(337,1) = 186;
          numbering(337,2) = 40;
          numbering(337,3) = 319;
          numbering(337,4) = 321;
          numbering(337,5) = 315;
          numbering(338,0) = 185;
          numbering(338,1) = 13;
          numbering(338,2) = 186;
          numbering(338,3) = 320;
          numbering(338,4) = 322;
          numbering(338,5) = 321;
          numbering(339,0) = 184;
          numbering(339,1) = 185;
          numbering(339,2) = 186;
          numbering(339,3) = 319;
          numbering(339,4) = 320;
          numbering(339,5) = 321;
          numbering(340,0) = 307;
          numbering(340,1) = 320;
          numbering(340,2) = 319;
          numbering(340,3) = 61;
          numbering(340,4) = 324;
          numbering(340,5) = 323;
          numbering(341,0) = 319;
          numbering(341,1) = 321;
          numbering(341,2) = 315;
          numbering(341,3) = 323;
          numbering(341,4) = 325;
          numbering(341,5) = 63;
          numbering(342,0) = 320;
          numbering(342,1) = 322;
          numbering(342,2) = 321;
          numbering(342,3) = 324;
          numbering(342,4) = 64;
          numbering(342,5) = 325;
          numbering(343,0) = 319;
          numbering(343,1) = 320;
          numbering(343,2) = 321;
          numbering(343,3) = 323;
          numbering(343,4) = 324;
          numbering(343,5) = 325;
          numbering(344,0) = 38;
          numbering(344,1) = 171;
          numbering(344,2) = 178;
          numbering(344,3) = 305;
          numbering(344,4) = 306;
          numbering(344,5) = 312;
          numbering(345,0) = 178;
          numbering(345,1) = 184;
          numbering(345,2) = 40;
          numbering(345,3) = 312;
          numbering(345,4) = 319;
          numbering(345,5) = 315;
          numbering(346,0) = 171;
          numbering(346,1) = 39;
          numbering(346,2) = 184;
          numbering(346,3) = 306;
          numbering(346,4) = 307;
          numbering(346,5) = 319;
          numbering(347,0) = 178;
          numbering(347,1) = 171;
          numbering(347,2) = 184;
          numbering(347,3) = 312;
          numbering(347,4) = 306;
          numbering(347,5) = 319;
          numbering(348,0) = 305;
          numbering(348,1) = 306;
          numbering(348,2) = 312;
          numbering(348,3) = 60;
          numbering(348,4) = 310;
          numbering(348,5) = 317;
          numbering(349,0) = 312;
          numbering(349,1) = 319;
          numbering(349,2) = 315;
          numbering(349,3) = 317;
          numbering(349,4) = 323;
          numbering(349,5) = 63;
          numbering(350,0) = 306;
          numbering(350,1) = 307;
          numbering(350,2) = 319;
          numbering(350,3) = 310;
          numbering(350,4) = 61;
          numbering(350,5) = 323;
          numbering(351,0) = 312;
          numbering(351,1) = 306;
          numbering(351,2) = 319;
          numbering(351,3) = 317;
          numbering(351,4) = 310;
          numbering(351,5) = 323;
          numbering(352,0) = 54;
          numbering(352,1) = 309;
          numbering(352,2) = 308;
          numbering(352,3) = 290;
          numbering(352,4) = 327;
          numbering(352,5) = 326;
          numbering(353,0) = 308;
          numbering(353,1) = 310;
          numbering(353,2) = 60;
          numbering(353,3) = 326;
          numbering(353,4) = 329;
          numbering(353,5) = 328;
          numbering(354,0) = 309;
          numbering(354,1) = 61;
          numbering(354,2) = 310;
          numbering(354,3) = 327;
          numbering(354,4) = 330;
          numbering(354,5) = 329;
          numbering(355,0) = 308;
          numbering(355,1) = 309;
          numbering(355,2) = 310;
          numbering(355,3) = 326;
          numbering(355,4) = 327;
          numbering(355,5) = 329;
          numbering(356,0) = 290;
          numbering(356,1) = 327;
          numbering(356,2) = 326;
          numbering(356,3) = 15;
          numbering(356,4) = 332;
          numbering(356,5) = 331;
          numbering(357,0) = 326;
          numbering(357,1) = 329;
          numbering(357,2) = 328;
          numbering(357,3) = 331;
          numbering(357,4) = 333;
          numbering(357,5) = 65;
          numbering(358,0) = 327;
          numbering(358,1) = 330;
          numbering(358,2) = 329;
          numbering(358,3) = 332;
          numbering(358,4) = 66;
          numbering(358,5) = 333;
          numbering(359,0) = 326;
          numbering(359,1) = 327;
          numbering(359,2) = 329;
          numbering(359,3) = 331;
          numbering(359,4) = 332;
          numbering(359,5) = 333;
          numbering(360,0) = 60;
          numbering(360,1) = 317;
          numbering(360,2) = 316;
          numbering(360,3) = 328;
          numbering(360,4) = 335;
          numbering(360,5) = 334;
          numbering(361,0) = 316;
          numbering(361,1) = 318;
          numbering(361,2) = 62;
          numbering(361,3) = 334;
          numbering(361,4) = 337;
          numbering(361,5) = 336;
          numbering(362,0) = 317;
          numbering(362,1) = 63;
          numbering(362,2) = 318;
          numbering(362,3) = 335;
          numbering(362,4) = 338;
          numbering(362,5) = 337;
          numbering(363,0) = 316;
          numbering(363,1) = 317;
          numbering(363,2) = 318;
          numbering(363,3) = 334;
          numbering(363,4) = 335;
          numbering(363,5) = 337;
          numbering(364,0) = 328;
          numbering(364,1) = 335;
          numbering(364,2) = 334;
          numbering(364,3) = 65;
          numbering(364,4) = 340;
          numbering(364,5) = 339;
          numbering(365,0) = 334;
          numbering(365,1) = 337;
          numbering(365,2) = 336;
          numbering(365,3) = 339;
          numbering(365,4) = 341;
          numbering(365,5) = 4;
          numbering(366,0) = 335;
          numbering(366,1) = 338;
          numbering(366,2) = 337;
          numbering(366,3) = 340;
          numbering(366,4) = 67;
          numbering(366,5) = 341;
          numbering(367,0) = 334;
          numbering(367,1) = 335;
          numbering(367,2) = 337;
          numbering(367,3) = 339;
          numbering(367,4) = 340;
          numbering(367,5) = 341;
          numbering(368,0) = 61;
          numbering(368,1) = 324;
          numbering(368,2) = 323;
          numbering(368,3) = 330;
          numbering(368,4) = 343;
          numbering(368,5) = 342;
          numbering(369,0) = 323;
          numbering(369,1) = 325;
          numbering(369,2) = 63;
          numbering(369,3) = 342;
          numbering(369,4) = 344;
          numbering(369,5) = 338;
          numbering(370,0) = 324;
          numbering(370,1) = 64;
          numbering(370,2) = 325;
          numbering(370,3) = 343;
          numbering(370,4) = 345;
          numbering(370,5) = 344;
          numbering(371,0) = 323;
          numbering(371,1) = 324;
          numbering(371,2) = 325;
          numbering(371,3) = 342;
          numbering(371,4) = 343;
          numbering(371,5) = 344;
          numbering(372,0) = 330;
          numbering(372,1) = 343;
          numbering(372,2) = 342;
          numbering(372,3) = 66;
          numbering(372,4) = 347;
          numbering(372,5) = 346;
          numbering(373,0) = 342;
          numbering(373,1) = 344;
          numbering(373,2) = 338;
          numbering(373,3) = 346;
          numbering(373,4) = 348;
          numbering(373,5) = 67;
          numbering(374,0) = 343;
          numbering(374,1) = 345;
          numbering(374,2) = 344;
          numbering(374,3) = 347;
          numbering(374,4) = 17;
          numbering(374,5) = 348;
          numbering(375,0) = 342;
          numbering(375,1) = 343;
          numbering(375,2) = 344;
          numbering(375,3) = 346;
          numbering(375,4) = 347;
          numbering(375,5) = 348;
          numbering(376,0) = 60;
          numbering(376,1) = 310;
          numbering(376,2) = 317;
          numbering(376,3) = 328;
          numbering(376,4) = 329;
          numbering(376,5) = 335;
          numbering(377,0) = 317;
          numbering(377,1) = 323;
          numbering(377,2) = 63;
          numbering(377,3) = 335;
          numbering(377,4) = 342;
          numbering(377,5) = 338;
          numbering(378,0) = 310;
          numbering(378,1) = 61;
          numbering(378,2) = 323;
          numbering(378,3) = 329;
          numbering(378,4) = 330;
          numbering(378,5) = 342;
          numbering(379,0) = 317;
          numbering(379,1) = 310;
          numbering(379,2) = 323;
          numbering(379,3) = 335;
          numbering(379,4) = 329;
          numbering(379,5) = 342;
          numbering(380,0) = 328;
          numbering(380,1) = 329;
          numbering(380,2) = 335;
          numbering(380,3) = 65;
          numbering(380,4) = 333;
          numbering(380,5) = 340;
          numbering(381,0) = 335;
          numbering(381,1) = 342;
          numbering(381,2) = 338;
          numbering(381,3) = 340;
          numbering(381,4) = 346;
          numbering(381,5) = 67;
          numbering(382,0) = 329;
          numbering(382,1) = 330;
          numbering(382,2) = 342;
          numbering(382,3) = 333;
          numbering(382,4) = 66;
          numbering(382,5) = 346;
          numbering(383,0) = 335;
          numbering(383,1) = 329;
          numbering(383,2) = 342;
          numbering(383,3) = 340;
          numbering(383,4) = 333;
          numbering(383,5) = 346;
          numbering(384,0) = 10;
          numbering(384,1) = 224;
          numbering(384,2) = 223;
          numbering(384,3) = 275;
          numbering(384,4) = 350;
          numbering(384,5) = 349;
          numbering(385,0) = 223;
          numbering(385,1) = 225;
          numbering(385,2) = 48;
          numbering(385,3) = 349;
          numbering(385,4) = 352;
          numbering(385,5) = 351;
          numbering(386,0) = 224;
          numbering(386,1) = 49;
          numbering(386,2) = 225;
          numbering(386,3) = 350;
          numbering(386,4) = 353;
          numbering(386,5) = 352;
          numbering(387,0) = 223;
          numbering(387,1) = 224;
          numbering(387,2) = 225;
          numbering(387,3) = 349;
          numbering(387,4) = 350;
          numbering(387,5) = 352;
          numbering(388,0) = 275;
          numbering(388,1) = 350;
          numbering(388,2) = 349;
          numbering(388,3) = 56;
          numbering(388,4) = 355;
          numbering(388,5) = 354;
          numbering(389,0) = 349;
          numbering(389,1) = 352;
          numbering(389,2) = 351;
          numbering(389,3) = 354;
          numbering(389,4) = 356;
          numbering(389,5) = 68;
          numbering(390,0) = 350;
          numbering(390,1) = 353;
          numbering(390,2) = 352;
          numbering(390,3) = 355;
          numbering(390,4) = 69;
          numbering(390,5) = 356;
          numbering(391,0) = 349;
          numbering(391,1) = 350;
          numbering(391,2) = 352;
          numbering(391,3) = 354;
          numbering(391,4) = 355;
          numbering(391,5) = 356;
          numbering(392,0) = 48;
          numbering(392,1) = 231;
          numbering(392,2) = 230;
          numbering(392,3) = 351;
          numbering(392,4) = 358;
          numbering(392,5) = 357;
          numbering(393,0) = 230;
          numbering(393,1) = 232;
          numbering(393,2) = 13;
          numbering(393,3) = 357;
          numbering(393,4) = 359;
          numbering(393,5) = 322;
          numbering(394,0) = 231;
          numbering(394,1) = 50;
          numbering(394,2) = 232;
          numbering(394,3) = 358;
          numbering(394,4) = 360;
          numbering(394,5) = 359;
          numbering(395,0) = 230;
          numbering(395,1) = 231;
          numbering(395,2) = 232;
          numbering(395,3) = 357;
          numbering(395,4) = 358;
          numbering(395,5) = 359;
          numbering(396,0) = 351;
          numbering(396,1) = 358;
          numbering(396,2) = 357;
          numbering(396,3) = 68;
          numbering(396,4) = 362;
          numbering(396,5) = 361;
          numbering(397,0) = 357;
          numbering(397,1) = 359;
          numbering(397,2) = 322;
          numbering(397,3) = 361;
          numbering(397,4) = 363;
          numbering(397,5) = 64;
          numbering(398,0) = 358;
          numbering(398,1) = 360;
          numbering(398,2) = 359;
          numbering(398,3) = 362;
          numbering(398,4) = 70;
          numbering(398,5) = 363;
          numbering(399,0) = 357;
          numbering(399,1) = 358;
          numbering(399,2) = 359;
          numbering(399,3) = 361;
          numbering(399,4) = 362;
          numbering(399,5) = 363;
          numbering(400,0) = 49;
          numbering(400,1) = 238;
          numbering(400,2) = 237;
          numbering(400,3) = 353;
          numbering(400,4) = 365;
          numbering(400,5) = 364;
          numbering(401,0) = 237;
          numbering(401,1) = 239;
          numbering(401,2) = 50;
          numbering(401,3) = 364;
          numbering(401,4) = 366;
          numbering(401,5) = 360;
          numbering(402,0) = 238;
          numbering(402,1) = 14;
          numbering(402,2) = 239;
          numbering(402,3) = 365;
          numbering(402,4) = 367;
          numbering(402,5) = 366;
          numbering(403,0) = 237;
          numbering(403,1) = 238;
          numbering(403,2) = 239;
          numbering(403,3) = 364;
          numbering(403,4) = 365;
          numbering(403,5) = 366;
          numbering(404,0) = 353;
          numbering(404,1) = 365;
          numbering(404,2) = 364;
          numbering(404,3) = 69;
          numbering(404,4) = 369;
          numbering(404,5) = 368;
          numbering(405,0) = 364;
          numbering(405,1) = 366;
          numbering(405,2) = 360;
          numbering(405,3) = 368;
          numbering(405,4) = 370;
          numbering(405,5) = 70;
          numbering(406,0) = 365;
          numbering(406,1) = 367;
          numbering(406,2) = 366;
          numbering(406,3) = 369;
          numbering(406,4) = 71;
          numbering(406,5) = 370;
          numbering(407,0) = 364;
          numbering(407,1) = 365;
          numbering(407,2) = 366;
          numbering(407,3) = 368;
          numbering(407,4) = 369;
          numbering(407,5) = 370;
          numbering(408,0) = 48;
          numbering(408,1) = 225;
          numbering(408,2) = 231;
          numbering(408,3) = 351;
          numbering(408,4) = 352;
          numbering(408,5) = 358;
          numbering(409,0) = 231;
          numbering(409,1) = 237;
          numbering(409,2) = 50;
          numbering(409,3) = 358;
          numbering(409,4) = 364;
          numbering(409,5) = 360;
          numbering(410,0) = 225;
          numbering(410,1) = 49;
          numbering(410,2) = 237;
          numbering(410,3) = 352;
          numbering(410,4) = 353;
          numbering(410,5) = 364;
          numbering(411,0) = 231;
          numbering(411,1) = 225;
          numbering(411,2) = 237;
          numbering(411,3) = 358;
          numbering(411,4) = 352;
          numbering(411,5) = 364;
          numbering(412,0) = 351;
          numbering(412,1) = 352;
          numbering(412,2) = 358;
          numbering(412,3) = 68;
          numbering(412,4) = 356;
          numbering(412,5) = 362;
          numbering(413,0) = 358;
          numbering(413,1) = 364;
          numbering(413,2) = 360;
          numbering(413,3) = 362;
          numbering(413,4) = 368;
          numbering(413,5) = 70;
          numbering(414,0) = 352;
          numbering(414,1) = 353;
          numbering(414,2) = 364;
          numbering(414,3) = 356;
          numbering(414,4) = 69;
          numbering(414,5) = 368;
          numbering(415,0) = 358;
          numbering(415,1) = 352;
          numbering(415,2) = 364;
          numbering(415,3) = 362;
          numbering(415,4) = 356;
          numbering(415,5) = 368;
          numbering(416,0) = 56;
          numbering(416,1) = 355;
          numbering(416,2) = 354;
          numbering(416,3) = 299;
          numbering(416,4) = 372;
          numbering(416,5) = 371;
          numbering(417,0) = 354;
          numbering(417,1) = 356;
          numbering(417,2) = 68;
          numbering(417,3) = 371;
          numbering(417,4) = 374;
          numbering(417,5) = 373;
          numbering(418,0) = 355;
          numbering(418,1) = 69;
          numbering(418,2) = 356;
          numbering(418,3) = 372;
          numbering(418,4) = 375;
          numbering(418,5) = 374;
          numbering(419,0) = 354;
          numbering(419,1) = 355;
          numbering(419,2) = 356;
          numbering(419,3) = 371;
          numbering(419,4) = 372;
          numbering(419,5) = 374;
          numbering(420,0) = 299;
          numbering(420,1) = 372;
          numbering(420,2) = 371;
          numbering(420,3) = 16;
          numbering(420,4) = 377;
          numbering(420,5) = 376;
          numbering(421,0) = 371;
          numbering(421,1) = 374;
          numbering(421,2) = 373;
          numbering(421,3) = 376;
          numbering(421,4) = 378;
          numbering(421,5) = 72;
          numbering(422,0) = 372;
          numbering(422,1) = 375;
          numbering(422,2) = 374;
          numbering(422,3) = 377;
          numbering(422,4) = 73;
          numbering(422,5) = 378;
          numbering(423,0) = 371;
          numbering(423,1) = 372;
          numbering(423,2) = 374;
          numbering(423,3) = 376;
          numbering(423,4) = 377;
          numbering(423,5) = 378;
          numbering(424,0) = 68;
          numbering(424,1) = 362;
          numbering(424,2) = 361;
          numbering(424,3) = 373;
          numbering(424,4) = 380;
          numbering(424,5) = 379;
          numbering(425,0) = 361;
          numbering(425,1) = 363;
          numbering(425,2) = 64;
          numbering(425,3) = 379;
          numbering(425,4) = 381;
          numbering(425,5) = 345;
          numbering(426,0) = 362;
          numbering(426,1) = 70;
          numbering(426,2) = 363;
          numbering(426,3) = 380;
          numbering(426,4) = 382;
          numbering(426,5) = 381;
          numbering(427,0) = 361;
          numbering(427,1) = 362;
          numbering(427,2) = 363;
          numbering(427,3) = 379;
          numbering(427,4) = 380;
          numbering(427,5) = 381;
          numbering(428,0) = 373;
          numbering(428,1) = 380;
          numbering(428,2) = 379;
          numbering(428,3) = 72;
          numbering(428,4) = 384;
          numbering(428,5) = 383;
          numbering(429,0) = 379;
          numbering(429,1) = 381;
          numbering(429,2) = 345;
          numbering(429,3) = 383;
          numbering(429,4) = 385;
          numbering(429,5) = 17;
          numbering(430,0) = 380;
          numbering(430,1) = 382;
          numbering(430,2) = 381;
          numbering(430,3) = 384;
          numbering(430,4) = 74;
          numbering(430,5) = 385;
          numbering(431,0) = 379;
          numbering(431,1) = 380;
          numbering(431,2) = 381;
          numbering(431,3) = 383;
          numbering(431,4) = 384;
          numbering(431,5) = 385;
          numbering(432,0) = 69;
          numbering(432,1) = 369;
          numbering(432,2) = 368;
          numbering(432,3) = 375;
          numbering(432,4) = 387;
          numbering(432,5) = 386;
          numbering(433,0) = 368;
          numbering(433,1) = 370;
          numbering(433,2) = 70;
          numbering(433,3) = 386;
          numbering(433,4) = 388;
          numbering(433,5) = 382;
          numbering(434,0) = 369;
          numbering(434,1) = 71;
          numbering(434,2) = 370;
          numbering(434,3) = 387;
          numbering(434,4) = 389;
          numbering(434,5) = 388;
          numbering(435,0) = 368;
          numbering(435,1) = 369;
          numbering(435,2) = 370;
          numbering(435,3) = 386;
          numbering(435,4) = 387;
          numbering(435,5) = 388;
          numbering(436,0) = 375;
          numbering(436,1) = 387;
          numbering(436,2) = 386;
          numbering(436,3) = 73;
          numbering(436,4) = 391;
          numbering(436,5) = 390;
          numbering(437,0) = 386;
          numbering(437,1) = 388;
          numbering(437,2) = 382;
          numbering(437,3) = 390;
          numbering(437,4) = 392;
          numbering(437,5) = 74;
          numbering(438,0) = 387;
          numbering(438,1) = 389;
          numbering(438,2) = 388;
          numbering(438,3) = 391;
          numbering(438,4) = 5;
          numbering(438,5) = 392;
          numbering(439,0) = 386;
          numbering(439,1) = 387;
          numbering(439,2) = 388;
          numbering(439,3) = 390;
          numbering(439,4) = 391;
          numbering(439,5) = 392;
          numbering(440,0) = 68;
          numbering(440,1) = 356;
          numbering(440,2) = 362;
          numbering(440,3) = 373;
          numbering(440,4) = 374;
          numbering(440,5) = 380;
          numbering(441,0) = 362;
          numbering(441,1) = 368;
          numbering(441,2) = 70;
          numbering(441,3) = 380;
          numbering(441,4) = 386;
          numbering(441,5) = 382;
          numbering(442,0) = 356;
          numbering(442,1) = 69;
          numbering(442,2) = 368;
          numbering(442,3) = 374;
          numbering(442,4) = 375;
          numbering(442,5) = 386;
          numbering(443,0) = 362;
          numbering(443,1) = 356;
          numbering(443,2) = 368;
          numbering(443,3) = 380;
          numbering(443,4) = 374;
          numbering(443,5) = 386;
          numbering(444,0) = 373;
          numbering(444,1) = 374;
          numbering(444,2) = 380;
          numbering(444,3) = 72;
          numbering(444,4) = 378;
          numbering(444,5) = 384;
          numbering(445,0) = 380;
          numbering(445,1) = 386;
          numbering(445,2) = 382;
          numbering(445,3) = 384;
          numbering(445,4) = 390;
          numbering(445,5) = 74;
          numbering(446,0) = 374;
          numbering(446,1) = 375;
          numbering(446,2) = 386;
          numbering(446,3) = 378;
          numbering(446,4) = 73;
          numbering(446,5) = 390;
          numbering(447,0) = 380;
          numbering(447,1) = 374;
          numbering(447,2) = 386;
          numbering(447,3) = 384;
          numbering(447,4) = 378;
          numbering(447,5) = 390;
          numbering(448,0) = 9;
          numbering(448,1) = 124;
          numbering(448,2) = 170;
          numbering(448,3) = 266;
          numbering(448,4) = 267;
          numbering(448,5) = 304;
          numbering(449,0) = 170;
          numbering(449,1) = 250;
          numbering(449,2) = 39;
          numbering(449,3) = 304;
          numbering(449,4) = 393;
          numbering(449,5) = 307;
          numbering(450,0) = 124;
          numbering(450,1) = 29;
          numbering(450,2) = 250;
          numbering(450,3) = 267;
          numbering(450,4) = 268;
          numbering(450,5) = 393;
          numbering(451,0) = 170;
          numbering(451,1) = 124;
          numbering(451,2) = 250;
          numbering(451,3) = 304;
          numbering(451,4) = 267;
          numbering(451,5) = 393;
          numbering(452,0) = 266;
          numbering(452,1) = 267;
          numbering(452,2) = 304;
          numbering(452,3) = 54;
          numbering(452,4) = 271;
          numbering(452,5) = 309;
          numbering(453,0) = 304;
          numbering(453,1) = 393;
          numbering(453,2) = 307;
          numbering(453,3) = 309;
          numbering(453,4) = 394;
          numbering(453,5) = 61;
          numbering(454,0) = 267;
          numbering(454,1) = 268;
          numbering(454,2) = 393;
          numbering(454,3) = 271;
          numbering(454,4) = 55;
          numbering(454,5) = 394;
          numbering(455,0) = 304;
          numbering(455,1) = 267;
          numbering(455,2) = 393;
          numbering(455,3) = 309;
          numbering(455,4) = 271;
          numbering(455,5) = 394;
          numbering(456,0) = 39;
          numbering(456,1) = 252;
          numbering(456,2) = 185;
          numbering(456,3) = 307;
          numbering(456,4) = 395;
          numbering(456,5) = 320;
          numbering(457,0) = 185;
          numbering(457,1) = 230;
          numbering(457,2) = 13;
          numbering(457,3) = 320;
          numbering(457,4) = 357;
          numbering(457,5) = 322;
          numbering(458,0) = 252;
          numbering(458,1) = 48;
          numbering(458,2) = 230;
          numbering(458,3) = 395;
          numbering(458,4) = 351;
          numbering(458,5) = 357;
          numbering(459,0) = 185;
          numbering(459,1) = 252;
          numbering(459,2) = 230;
          numbering(459,3) = 320;
          numbering(459,4) = 395;
          numbering(459,5) = 357;
          numbering(460,0) = 307;
          numbering(460,1) = 395;
          numbering(460,2) = 320;
          numbering(460,3) = 61;
          numbering(460,4) = 396;
          numbering(460,5) = 324;
          numbering(461,0) = 320;
          numbering(461,1) = 357;
          numbering(461,2) = 322;
          numbering(461,3) = 324;
          numbering(461,4) = 361;
          numbering(461,5) = 64;
          numbering(462,0) = 395;
          numbering(462,1) = 351;
          numbering(462,2) = 357;
          numbering(462,3) = 396;
          numbering(462,4) = 68;
          numbering(462,5) = 361;
          numbering(463,0) = 320;
          numbering(463,1) = 395;
          numbering(463,2) = 357;
          numbering(463,3) = 324;
          numbering(463,4) = 396;
          numbering(463,5) = 361;
          numbering(464,0) = 29;
          numbering(464,1) = 131;
          numbering(464,2) = 254;
          numbering(464,3) = 268;
          numbering(464,4) = 274;
          numbering(464,5) = 397;
          numbering(465,0) = 254;
          numbering(465,1) = 223;
          numbering(465,2) = 48;
          numbering(465,3) = 397;
          numbering(465,4) = 349;
          numbering(465,5) = 351;
          numbering(466,0) = 131;
          numbering(466,1) = 10;
          numbering(466,2) = 223;
          numbering(466,3) = 274;
          numbering(466,4) = 275;
          numbering(466,5) = 349;
          numbering(467,0) = 254;
          numbering(467,1) = 131;
          numbering(467,2) = 223;
          numbering(467,3) = 397;
          numbering(467,4) = 274;
          numbering(467,5) = 349;
          numbering(468,0) = 268;
          numbering(468,1) = 274;
          numbering(468,2) = 397;
          numbering(468,3) = 55;
          numbering(468,4) = 278;
          numbering(468,5) = 398;
          numbering(469,0) = 397;
          numbering(469,1) = 349;
          numbering(469,2) = 351;
          numbering(469,3) = 398;
          numbering(469,4) = 354;
          numbering(469,5) = 68;
          numbering(470,0) = 274;
          numbering(470,1) = 275;
          numbering(470,2) = 349;
          numbering(470,3) = 278;
          numbering(470,4) = 56;
          numbering(470,5) = 354;
          numbering(471,0) = 397;
          numbering(471,1) = 274;
          numbering(471,2) = 349;
          numbering(471,3) = 398;
          numbering(471,4) = 278;
          numbering(471,5) = 354;
          numbering(472,0) = 39;
          numbering(472,1) = 250;
          numbering(472,2) = 252;
          numbering(472,3) = 307;
          numbering(472,4) = 393;
          numbering(472,5) = 395;
          numbering(473,0) = 252;
          numbering(473,1) = 254;
          numbering(473,2) = 48;
          numbering(473,3) = 395;
          numbering(473,4) = 397;
          numbering(473,5) = 351;
          numbering(474,0) = 250;
          numbering(474,1) = 29;
          numbering(474,2) = 254;
          numbering(474,3) = 393;
          numbering(474,4) = 268;
          numbering(474,5) = 397;
          numbering(475,0) = 252;
          numbering(475,1) = 250;
          numbering(475,2) = 254;
          numbering(475,3) = 395;
          numbering(475,4) = 393;
          numbering(475,5) = 397;
          numbering(476,0) = 307;
          numbering(476,1) = 393;
          numbering(476,2) = 395;
          numbering(476,3) = 61;
          numbering(476,4) = 394;
          numbering(476,5) = 396;
          numbering(477,0) = 395;
          numbering(477,1) = 397;
          numbering(477,2) = 351;
          numbering(477,3) = 396;
          numbering(477,4) = 398;
          numbering(477,5) = 68;
          numbering(478,0) = 393;
          numbering(478,1) = 268;
          numbering(478,2) = 397;
          numbering(478,3) = 394;
          numbering(478,4) = 55;
          numbering(478,5) = 398;
          numbering(479,0) = 395;
          numbering(479,1) = 393;
          numbering(479,2) = 397;
          numbering(479,3) = 396;
          numbering(479,4) = 394;
          numbering(479,5) = 398;
          numbering(480,0) = 54;
          numbering(480,1) = 271;
          numbering(480,2) = 309;
          numbering(480,3) = 290;
          numbering(480,4) = 291;
          numbering(480,5) = 327;
          numbering(481,0) = 309;
          numbering(481,1) = 394;
          numbering(481,2) = 61;
          numbering(481,3) = 327;
          numbering(481,4) = 399;
          numbering(481,5) = 330;
          numbering(482,0) = 271;
          numbering(482,1) = 55;
          numbering(482,2) = 394;
          numbering(482,3) = 291;
          numbering(482,4) = 292;
          numbering(482,5) = 399;
          numbering(483,0) = 309;
          numbering(483,1) = 271;
          numbering(483,2) = 394;
          numbering(483,3) = 327;
          numbering(483,4) = 291;
          numbering(483,5) = 399;
          numbering(484,0) = 290;
          numbering(484,1) = 291;
          numbering(484,2) = 327;
          numbering(484,3) = 15;
          numbering(484,4) = 295;
          numbering(484,5) = 332;
          numbering(485,0) = 327;
          numbering(485,1) = 399;
          numbering(485,2) = 330;
          numbering(485,3) = 332;
          numbering(485,4) = 400;
          numbering(485,5) = 66;
          numbering(486,0) = 291;
          numbering(486,1) = 292;
          numbering(486,2) = 399;
          numbering(486,3) = 295;
          numbering(486,4) = 59;
          numbering(486,5) = 400;
          numbering(487,0) = 327;
          numbering(487,1) = 291;
          numbering(487,2) = 399;
          numbering(487,3) = 332;
          numbering(487,4) = 295;
          numbering(487,5) = 400;
          numbering(488,0) = 61;
          numbering(488,1) = 396;
          numbering(488,2) = 324;
          numbering(488,3) = 330;
          numbering(488,4) = 401;
          numbering(488,5) = 343;
          numbering(489,0) = 324;
          numbering(489,1) = 361;
          numbering(489,2) = 64;
          numbering(489,3) = 343;
          numbering(489,4) = 379;
          numbering(489,5) = 345;
          numbering(490,0) = 396;
          numbering(490,1) = 68;
          numbering(490,2) = 361;
          numbering(490,3) = 401;
          numbering(490,4) = 373;
          numbering(490,5) = 379;
          numbering(491,0) = 324;
          numbering(491,1) = 396;
          numbering(491,2) = 361;
          numbering(491,3) = 343;
          numbering(491,4) = 401;
          numbering(491,5) = 379;
          numbering(492,0) = 330;
          numbering(492,1) = 401;
          numbering(492,2) = 343;
          numbering(492,3) = 66;
          numbering(492,4) = 402;
          numbering(492,5) = 347;
          numbering(493,0) = 343;
          numbering(493,1) = 379;
          numbering(493,2) = 345;
          numbering(493,3) = 347;
          numbering(493,4) = 383;
          numbering(493,5) = 17;
          numbering(494,0) = 401;
          numbering(494,1) = 373;
          numbering(494,2) = 379;
          numbering(494,3) = 402;
          numbering(494,4) = 72;
          numbering(494,5) = 383;
          numbering(495,0) = 343;
          numbering(495,1) = 401;
          numbering(495,2) = 379;
          numbering(495,3) = 347;
          numbering(495,4) = 402;
          numbering(495,5) = 383;
          numbering(496,0) = 55;
          numbering(496,1) = 278;
          numbering(496,2) = 398;
          numbering(496,3) = 292;
          numbering(496,4) = 298;
          numbering(496,5) = 403;
          numbering(497,0) = 398;
          numbering(497,1) = 354;
          numbering(497,2) = 68;
          numbering(497,3) = 403;
          numbering(497,4) = 371;
          numbering(497,5) = 373;
          numbering(498,0) = 278;
          numbering(498,1) = 56;
          numbering(498,2) = 354;
          numbering(498,3) = 298;
          numbering(498,4) = 299;
          numbering(498,5) = 371;
          numbering(499,0) = 398;
          numbering(499,1) = 278;
          numbering(499,2) = 354;
          numbering(499,3) = 403;
          numbering(499,4) = 298;
          numbering(499,5) = 371;
          numbering(500,0) = 292;
          numbering(500,1) = 298;
          numbering(500,2) = 403;
          numbering(500,3) = 59;
          numbering(500,4) = 302;
          numbering(500,5) = 404;
          numbering(501,0) = 403;
          numbering(501,1) = 371;
          numbering(501,2) = 373;
          numbering(501,3) = 404;
          numbering(501,4) = 376;
          numbering(501,5) = 72;
          numbering(502,0) = 298;
          numbering(502,1) = 299;
          numbering(502,2) = 371;
          numbering(502,3) = 302;
          numbering(502,4) = 16;
          numbering(502,5) = 376;
          numbering(503,0) = 403;
          numbering(503,1) = 298;
          numbering(503,2) = 371;
          numbering(503,3) = 404;
          numbering(503,4) = 302;
          numbering(503,5) = 376;
          numbering(504,0) = 61;
          numbering(504,1) = 394;
          numbering(504,2) = 396;
          numbering(504,3) = 330;
          numbering(504,4) = 399;
          numbering(504,5) = 401;
          numbering(505,0) = 396;
          numbering(505,1) = 398;
          numbering(505,2) = 68;
          numbering(505,3) = 401;
          numbering(505,4) = 403;
          numbering(505,5) = 373;
          numbering(506,0) = 394;
          numbering(506,1) = 55;
          numbering(506,2) = 398;
          numbering(506,3) = 399;
          numbering(506,4) = 292;
          numbering(506,5) = 403;
          numbering(507,0) = 396;
          numbering(507,1) = 394;
          numbering(507,2) = 398;
          numbering(507,3) = 401;
          numbering(507,4) = 399;
          numbering(507,5) = 403;
          numbering(508,0) = 330;
          numbering(508,1) = 399;
          numbering(508,2) = 401;
          numbering(508,3) = 66;
          numbering(508,4) = 400;
          numbering(508,5) = 402;
          numbering(509,0) = 401;
          numbering(509,1) = 403;
          numbering(509,2) = 373;
          numbering(509,3) = 402;
          numbering(509,4) = 404;
          numbering(509,5) = 72;
          numbering(510,0) = 399;
          numbering(510,1) = 292;
          numbering(510,2) = 403;
          numbering(510,3) = 400;
          numbering(510,4) = 59;
          numbering(510,5) = 404;
          numbering(511,0) = 401;
          numbering(511,1) = 399;
          numbering(511,2) = 403;
          numbering(511,3) = 402;
          numbering(511,4) = 400;
          numbering(511,5) = 404;
          return;
        }
      }
    } 
    case SpaMesh::pyram:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(2,4);
          numbering(0,0) = 0;
          numbering(0,1) = 1;
          numbering(0,2) = 2;
          numbering(0,3) = 3;
          numbering(1,0) = 0;
          numbering(1,1) = 2;
          numbering(1,2) = 4;
          numbering(1,3) = 3;
          return;
        }
        case 2:
        {
          numbering.resize(16,4);
          numbering(0,0) = 0;
          numbering(0,1) = 5;
          numbering(0,2) = 6;
          numbering(0,3) = 7;
          numbering(1,0) = 5;
          numbering(1,1) = 1;
          numbering(1,2) = 8;
          numbering(1,3) = 9;
          numbering(2,0) = 6;
          numbering(2,1) = 8;
          numbering(2,2) = 2;
          numbering(2,3) = 10;
          numbering(3,0) = 7;
          numbering(3,1) = 9;
          numbering(3,2) = 10;
          numbering(3,3) = 3;
          numbering(4,0) = 5;
          numbering(4,1) = 9;
          numbering(4,2) = 6;
          numbering(4,3) = 7;
          numbering(5,0) = 5;
          numbering(5,1) = 8;
          numbering(5,2) = 6;
          numbering(5,3) = 9;
          numbering(6,0) = 6;
          numbering(6,1) = 8;
          numbering(6,2) = 10;
          numbering(6,3) = 9;
          numbering(7,0) = 6;
          numbering(7,1) = 9;
          numbering(7,2) = 10;
          numbering(7,3) = 7;
          numbering(8,0) = 0;
          numbering(8,1) = 6;
          numbering(8,2) = 11;
          numbering(8,3) = 7;
          numbering(9,0) = 6;
          numbering(9,1) = 2;
          numbering(9,2) = 12;
          numbering(9,3) = 10;
          numbering(10,0) = 11;
          numbering(10,1) = 12;
          numbering(10,2) = 4;
          numbering(10,3) = 13;
          numbering(11,0) = 7;
          numbering(11,1) = 10;
          numbering(11,2) = 13;
          numbering(11,3) = 3;
          numbering(12,0) = 6;
          numbering(12,1) = 10;
          numbering(12,2) = 11;
          numbering(12,3) = 7;
          numbering(13,0) = 6;
          numbering(13,1) = 12;
          numbering(13,2) = 11;
          numbering(13,3) = 10;
          numbering(14,0) = 11;
          numbering(14,1) = 12;
          numbering(14,2) = 13;
          numbering(14,3) = 10;
          numbering(15,0) = 11;
          numbering(15,1) = 10;
          numbering(15,2) = 13;
          numbering(15,3) = 7;
          return;
        }
        case 3:
        case 4:
        {
          numbering.resize(128,4);
          numbering(0,0) = 0;
          numbering(0,1) = 14;
          numbering(0,2) = 15;
          numbering(0,3) = 16;
          numbering(1,0) = 14;
          numbering(1,1) = 5;
          numbering(1,2) = 17;
          numbering(1,3) = 18;
          numbering(2,0) = 15;
          numbering(2,1) = 17;
          numbering(2,2) = 6;
          numbering(2,3) = 19;
          numbering(3,0) = 16;
          numbering(3,1) = 18;
          numbering(3,2) = 19;
          numbering(3,3) = 7;
          numbering(4,0) = 14;
          numbering(4,1) = 18;
          numbering(4,2) = 15;
          numbering(4,3) = 16;
          numbering(5,0) = 14;
          numbering(5,1) = 17;
          numbering(5,2) = 15;
          numbering(5,3) = 18;
          numbering(6,0) = 15;
          numbering(6,1) = 17;
          numbering(6,2) = 19;
          numbering(6,3) = 18;
          numbering(7,0) = 15;
          numbering(7,1) = 18;
          numbering(7,2) = 19;
          numbering(7,3) = 16;
          numbering(8,0) = 5;
          numbering(8,1) = 20;
          numbering(8,2) = 21;
          numbering(8,3) = 22;
          numbering(9,0) = 20;
          numbering(9,1) = 1;
          numbering(9,2) = 23;
          numbering(9,3) = 24;
          numbering(10,0) = 21;
          numbering(10,1) = 23;
          numbering(10,2) = 8;
          numbering(10,3) = 25;
          numbering(11,0) = 22;
          numbering(11,1) = 24;
          numbering(11,2) = 25;
          numbering(11,3) = 9;
          numbering(12,0) = 20;
          numbering(12,1) = 24;
          numbering(12,2) = 21;
          numbering(12,3) = 22;
          numbering(13,0) = 20;
          numbering(13,1) = 23;
          numbering(13,2) = 21;
          numbering(13,3) = 24;
          numbering(14,0) = 21;
          numbering(14,1) = 23;
          numbering(14,2) = 25;
          numbering(14,3) = 24;
          numbering(15,0) = 21;
          numbering(15,1) = 24;
          numbering(15,2) = 25;
          numbering(15,3) = 22;
          numbering(16,0) = 6;
          numbering(16,1) = 26;
          numbering(16,2) = 27;
          numbering(16,3) = 28;
          numbering(17,0) = 26;
          numbering(17,1) = 8;
          numbering(17,2) = 29;
          numbering(17,3) = 30;
          numbering(18,0) = 27;
          numbering(18,1) = 29;
          numbering(18,2) = 2;
          numbering(18,3) = 31;
          numbering(19,0) = 28;
          numbering(19,1) = 30;
          numbering(19,2) = 31;
          numbering(19,3) = 10;
          numbering(20,0) = 26;
          numbering(20,1) = 30;
          numbering(20,2) = 27;
          numbering(20,3) = 28;
          numbering(21,0) = 26;
          numbering(21,1) = 29;
          numbering(21,2) = 27;
          numbering(21,3) = 30;
          numbering(22,0) = 27;
          numbering(22,1) = 29;
          numbering(22,2) = 31;
          numbering(22,3) = 30;
          numbering(23,0) = 27;
          numbering(23,1) = 30;
          numbering(23,2) = 31;
          numbering(23,3) = 28;
          numbering(24,0) = 7;
          numbering(24,1) = 32;
          numbering(24,2) = 33;
          numbering(24,3) = 34;
          numbering(25,0) = 32;
          numbering(25,1) = 9;
          numbering(25,2) = 35;
          numbering(25,3) = 36;
          numbering(26,0) = 33;
          numbering(26,1) = 35;
          numbering(26,2) = 10;
          numbering(26,3) = 37;
          numbering(27,0) = 34;
          numbering(27,1) = 36;
          numbering(27,2) = 37;
          numbering(27,3) = 3;
          numbering(28,0) = 32;
          numbering(28,1) = 36;
          numbering(28,2) = 33;
          numbering(28,3) = 34;
          numbering(29,0) = 32;
          numbering(29,1) = 35;
          numbering(29,2) = 33;
          numbering(29,3) = 36;
          numbering(30,0) = 33;
          numbering(30,1) = 35;
          numbering(30,2) = 37;
          numbering(30,3) = 36;
          numbering(31,0) = 33;
          numbering(31,1) = 36;
          numbering(31,2) = 37;
          numbering(31,3) = 34;
          numbering(32,0) = 5;
          numbering(32,1) = 22;
          numbering(32,2) = 17;
          numbering(32,3) = 18;
          numbering(33,0) = 22;
          numbering(33,1) = 9;
          numbering(33,2) = 38;
          numbering(33,3) = 32;
          numbering(34,0) = 17;
          numbering(34,1) = 38;
          numbering(34,2) = 6;
          numbering(34,3) = 19;
          numbering(35,0) = 18;
          numbering(35,1) = 32;
          numbering(35,2) = 19;
          numbering(35,3) = 7;
          numbering(36,0) = 18;
          numbering(36,1) = 38;
          numbering(36,2) = 17;
          numbering(36,3) = 19;
          numbering(37,0) = 22;
          numbering(37,1) = 38;
          numbering(37,2) = 17;
          numbering(37,3) = 18;
          numbering(38,0) = 22;
          numbering(38,1) = 32;
          numbering(38,2) = 38;
          numbering(38,3) = 18;
          numbering(39,0) = 18;
          numbering(39,1) = 32;
          numbering(39,2) = 38;
          numbering(39,3) = 19;
          numbering(40,0) = 5;
          numbering(40,1) = 21;
          numbering(40,2) = 17;
          numbering(40,3) = 22;
          numbering(41,0) = 21;
          numbering(41,1) = 8;
          numbering(41,2) = 26;
          numbering(41,3) = 25;
          numbering(42,0) = 17;
          numbering(42,1) = 26;
          numbering(42,2) = 6;
          numbering(42,3) = 38;
          numbering(43,0) = 22;
          numbering(43,1) = 25;
          numbering(43,2) = 38;
          numbering(43,3) = 9;
          numbering(44,0) = 21;
          numbering(44,1) = 25;
          numbering(44,2) = 17;
          numbering(44,3) = 22;
          numbering(45,0) = 21;
          numbering(45,1) = 26;
          numbering(45,2) = 17;
          numbering(45,3) = 25;
          numbering(46,0) = 17;
          numbering(46,1) = 26;
          numbering(46,2) = 38;
          numbering(46,3) = 25;
          numbering(47,0) = 17;
          numbering(47,1) = 25;
          numbering(47,2) = 38;
          numbering(47,3) = 22;
          numbering(48,0) = 6;
          numbering(48,1) = 26;
          numbering(48,2) = 28;
          numbering(48,3) = 38;
          numbering(49,0) = 26;
          numbering(49,1) = 8;
          numbering(49,2) = 30;
          numbering(49,3) = 25;
          numbering(50,0) = 28;
          numbering(50,1) = 30;
          numbering(50,2) = 10;
          numbering(50,3) = 35;
          numbering(51,0) = 38;
          numbering(51,1) = 25;
          numbering(51,2) = 35;
          numbering(51,3) = 9;
          numbering(52,0) = 26;
          numbering(52,1) = 25;
          numbering(52,2) = 28;
          numbering(52,3) = 38;
          numbering(53,0) = 26;
          numbering(53,1) = 30;
          numbering(53,2) = 28;
          numbering(53,3) = 25;
          numbering(54,0) = 28;
          numbering(54,1) = 30;
          numbering(54,2) = 35;
          numbering(54,3) = 25;
          numbering(55,0) = 28;
          numbering(55,1) = 25;
          numbering(55,2) = 35;
          numbering(55,3) = 38;
          numbering(56,0) = 6;
          numbering(56,1) = 38;
          numbering(56,2) = 28;
          numbering(56,3) = 19;
          numbering(57,0) = 38;
          numbering(57,1) = 9;
          numbering(57,2) = 35;
          numbering(57,3) = 32;
          numbering(58,0) = 28;
          numbering(58,1) = 35;
          numbering(58,2) = 10;
          numbering(58,3) = 33;
          numbering(59,0) = 19;
          numbering(59,1) = 32;
          numbering(59,2) = 33;
          numbering(59,3) = 7;
          numbering(60,0) = 19;
          numbering(60,1) = 35;
          numbering(60,2) = 28;
          numbering(60,3) = 33;
          numbering(61,0) = 38;
          numbering(61,1) = 35;
          numbering(61,2) = 28;
          numbering(61,3) = 19;
          numbering(62,0) = 38;
          numbering(62,1) = 32;
          numbering(62,2) = 35;
          numbering(62,3) = 19;
          numbering(63,0) = 19;
          numbering(63,1) = 32;
          numbering(63,2) = 35;
          numbering(63,3) = 33;
          numbering(64,0) = 0;
          numbering(64,1) = 15;
          numbering(64,2) = 39;
          numbering(64,3) = 16;
          numbering(65,0) = 15;
          numbering(65,1) = 6;
          numbering(65,2) = 40;
          numbering(65,3) = 19;
          numbering(66,0) = 39;
          numbering(66,1) = 40;
          numbering(66,2) = 11;
          numbering(66,3) = 41;
          numbering(67,0) = 16;
          numbering(67,1) = 19;
          numbering(67,2) = 41;
          numbering(67,3) = 7;
          numbering(68,0) = 15;
          numbering(68,1) = 19;
          numbering(68,2) = 39;
          numbering(68,3) = 16;
          numbering(69,0) = 15;
          numbering(69,1) = 40;
          numbering(69,2) = 39;
          numbering(69,3) = 19;
          numbering(70,0) = 39;
          numbering(70,1) = 40;
          numbering(70,2) = 41;
          numbering(70,3) = 19;
          numbering(71,0) = 39;
          numbering(71,1) = 19;
          numbering(71,2) = 41;
          numbering(71,3) = 16;
          numbering(72,0) = 6;
          numbering(72,1) = 27;
          numbering(72,2) = 42;
          numbering(72,3) = 28;
          numbering(73,0) = 27;
          numbering(73,1) = 2;
          numbering(73,2) = 43;
          numbering(73,3) = 31;
          numbering(74,0) = 42;
          numbering(74,1) = 43;
          numbering(74,2) = 12;
          numbering(74,3) = 44;
          numbering(75,0) = 28;
          numbering(75,1) = 31;
          numbering(75,2) = 44;
          numbering(75,3) = 10;
          numbering(76,0) = 27;
          numbering(76,1) = 31;
          numbering(76,2) = 42;
          numbering(76,3) = 28;
          numbering(77,0) = 27;
          numbering(77,1) = 43;
          numbering(77,2) = 42;
          numbering(77,3) = 31;
          numbering(78,0) = 42;
          numbering(78,1) = 43;
          numbering(78,2) = 44;
          numbering(78,3) = 31;
          numbering(79,0) = 42;
          numbering(79,1) = 31;
          numbering(79,2) = 44;
          numbering(79,3) = 28;
          numbering(80,0) = 11;
          numbering(80,1) = 45;
          numbering(80,2) = 46;
          numbering(80,3) = 47;
          numbering(81,0) = 45;
          numbering(81,1) = 12;
          numbering(81,2) = 48;
          numbering(81,3) = 49;
          numbering(82,0) = 46;
          numbering(82,1) = 48;
          numbering(82,2) = 4;
          numbering(82,3) = 50;
          numbering(83,0) = 47;
          numbering(83,1) = 49;
          numbering(83,2) = 50;
          numbering(83,3) = 13;
          numbering(84,0) = 45;
          numbering(84,1) = 49;
          numbering(84,2) = 46;
          numbering(84,3) = 47;
          numbering(85,0) = 45;
          numbering(85,1) = 48;
          numbering(85,2) = 46;
          numbering(85,3) = 49;
          numbering(86,0) = 46;
          numbering(86,1) = 48;
          numbering(86,2) = 50;
          numbering(86,3) = 49;
          numbering(87,0) = 46;
          numbering(87,1) = 49;
          numbering(87,2) = 50;
          numbering(87,3) = 47;
          numbering(88,0) = 7;
          numbering(88,1) = 33;
          numbering(88,2) = 51;
          numbering(88,3) = 34;
          numbering(89,0) = 33;
          numbering(89,1) = 10;
          numbering(89,2) = 52;
          numbering(89,3) = 37;
          numbering(90,0) = 51;
          numbering(90,1) = 52;
          numbering(90,2) = 13;
          numbering(90,3) = 53;
          numbering(91,0) = 34;
          numbering(91,1) = 37;
          numbering(91,2) = 53;
          numbering(91,3) = 3;
          numbering(92,0) = 33;
          numbering(92,1) = 37;
          numbering(92,2) = 51;
          numbering(92,3) = 34;
          numbering(93,0) = 33;
          numbering(93,1) = 52;
          numbering(93,2) = 51;
          numbering(93,3) = 37;
          numbering(94,0) = 51;
          numbering(94,1) = 52;
          numbering(94,2) = 53;
          numbering(94,3) = 37;
          numbering(95,0) = 51;
          numbering(95,1) = 37;
          numbering(95,2) = 53;
          numbering(95,3) = 34;
          numbering(96,0) = 6;
          numbering(96,1) = 28;
          numbering(96,2) = 40;
          numbering(96,3) = 19;
          numbering(97,0) = 28;
          numbering(97,1) = 10;
          numbering(97,2) = 54;
          numbering(97,3) = 33;
          numbering(98,0) = 40;
          numbering(98,1) = 54;
          numbering(98,2) = 11;
          numbering(98,3) = 41;
          numbering(99,0) = 19;
          numbering(99,1) = 33;
          numbering(99,2) = 41;
          numbering(99,3) = 7;
          numbering(100,0) = 19;
          numbering(100,1) = 54;
          numbering(100,2) = 40;
          numbering(100,3) = 41;
          numbering(101,0) = 28;
          numbering(101,1) = 54;
          numbering(101,2) = 40;
          numbering(101,3) = 19;
          numbering(102,0) = 28;
          numbering(102,1) = 33;
          numbering(102,2) = 54;
          numbering(102,3) = 19;
          numbering(103,0) = 19;
          numbering(103,1) = 33;
          numbering(103,2) = 54;
          numbering(103,3) = 41;
          numbering(104,0) = 6;
          numbering(104,1) = 42;
          numbering(104,2) = 40;
          numbering(104,3) = 28;
          numbering(105,0) = 42;
          numbering(105,1) = 12;
          numbering(105,2) = 45;
          numbering(105,3) = 44;
          numbering(106,0) = 40;
          numbering(106,1) = 45;
          numbering(106,2) = 11;
          numbering(106,3) = 54;
          numbering(107,0) = 28;
          numbering(107,1) = 44;
          numbering(107,2) = 54;
          numbering(107,3) = 10;
          numbering(108,0) = 42;
          numbering(108,1) = 44;
          numbering(108,2) = 40;
          numbering(108,3) = 28;
          numbering(109,0) = 42;
          numbering(109,1) = 45;
          numbering(109,2) = 40;
          numbering(109,3) = 44;
          numbering(110,0) = 40;
          numbering(110,1) = 45;
          numbering(110,2) = 54;
          numbering(110,3) = 44;
          numbering(111,0) = 40;
          numbering(111,1) = 44;
          numbering(111,2) = 54;
          numbering(111,3) = 28;
          numbering(112,0) = 11;
          numbering(112,1) = 45;
          numbering(112,2) = 47;
          numbering(112,3) = 54;
          numbering(113,0) = 45;
          numbering(113,1) = 12;
          numbering(113,2) = 49;
          numbering(113,3) = 44;
          numbering(114,0) = 47;
          numbering(114,1) = 49;
          numbering(114,2) = 13;
          numbering(114,3) = 52;
          numbering(115,0) = 54;
          numbering(115,1) = 44;
          numbering(115,2) = 52;
          numbering(115,3) = 10;
          numbering(116,0) = 54;
          numbering(116,1) = 49;
          numbering(116,2) = 47;
          numbering(116,3) = 52;
          numbering(117,0) = 45;
          numbering(117,1) = 49;
          numbering(117,2) = 47;
          numbering(117,3) = 54;
          numbering(118,0) = 45;
          numbering(118,1) = 44;
          numbering(118,2) = 49;
          numbering(118,3) = 54;
          numbering(119,0) = 54;
          numbering(119,1) = 44;
          numbering(119,2) = 49;
          numbering(119,3) = 52;
          numbering(120,0) = 11;
          numbering(120,1) = 54;
          numbering(120,2) = 47;
          numbering(120,3) = 41;
          numbering(121,0) = 54;
          numbering(121,1) = 10;
          numbering(121,2) = 52;
          numbering(121,3) = 33;
          numbering(122,0) = 47;
          numbering(122,1) = 52;
          numbering(122,2) = 13;
          numbering(122,3) = 51;
          numbering(123,0) = 41;
          numbering(123,1) = 33;
          numbering(123,2) = 51;
          numbering(123,3) = 7;
          numbering(124,0) = 54;
          numbering(124,1) = 33;
          numbering(124,2) = 47;
          numbering(124,3) = 41;
          numbering(125,0) = 54;
          numbering(125,1) = 52;
          numbering(125,2) = 47;
          numbering(125,3) = 33;
          numbering(126,0) = 47;
          numbering(126,1) = 52;
          numbering(126,2) = 51;
          numbering(126,3) = 33;
          numbering(127,0) = 47;
          numbering(127,1) = 33;
          numbering(127,2) = 51;
          numbering(127,3) = 41;
          return;
        }
        default:
        {
          numbering.resize(1024,4);
          numbering(0,0) = 0;
          numbering(0,1) = 55;
          numbering(0,2) = 56;
          numbering(0,3) = 57;
          numbering(1,0) = 55;
          numbering(1,1) = 14;
          numbering(1,2) = 58;
          numbering(1,3) = 59;
          numbering(2,0) = 56;
          numbering(2,1) = 58;
          numbering(2,2) = 15;
          numbering(2,3) = 60;
          numbering(3,0) = 57;
          numbering(3,1) = 59;
          numbering(3,2) = 60;
          numbering(3,3) = 16;
          numbering(4,0) = 55;
          numbering(4,1) = 59;
          numbering(4,2) = 56;
          numbering(4,3) = 57;
          numbering(5,0) = 55;
          numbering(5,1) = 58;
          numbering(5,2) = 56;
          numbering(5,3) = 59;
          numbering(6,0) = 56;
          numbering(6,1) = 58;
          numbering(6,2) = 60;
          numbering(6,3) = 59;
          numbering(7,0) = 56;
          numbering(7,1) = 59;
          numbering(7,2) = 60;
          numbering(7,3) = 57;
          numbering(8,0) = 14;
          numbering(8,1) = 61;
          numbering(8,2) = 62;
          numbering(8,3) = 63;
          numbering(9,0) = 61;
          numbering(9,1) = 5;
          numbering(9,2) = 64;
          numbering(9,3) = 65;
          numbering(10,0) = 62;
          numbering(10,1) = 64;
          numbering(10,2) = 17;
          numbering(10,3) = 66;
          numbering(11,0) = 63;
          numbering(11,1) = 65;
          numbering(11,2) = 66;
          numbering(11,3) = 18;
          numbering(12,0) = 61;
          numbering(12,1) = 65;
          numbering(12,2) = 62;
          numbering(12,3) = 63;
          numbering(13,0) = 61;
          numbering(13,1) = 64;
          numbering(13,2) = 62;
          numbering(13,3) = 65;
          numbering(14,0) = 62;
          numbering(14,1) = 64;
          numbering(14,2) = 66;
          numbering(14,3) = 65;
          numbering(15,0) = 62;
          numbering(15,1) = 65;
          numbering(15,2) = 66;
          numbering(15,3) = 63;
          numbering(16,0) = 15;
          numbering(16,1) = 67;
          numbering(16,2) = 68;
          numbering(16,3) = 69;
          numbering(17,0) = 67;
          numbering(17,1) = 17;
          numbering(17,2) = 70;
          numbering(17,3) = 71;
          numbering(18,0) = 68;
          numbering(18,1) = 70;
          numbering(18,2) = 6;
          numbering(18,3) = 72;
          numbering(19,0) = 69;
          numbering(19,1) = 71;
          numbering(19,2) = 72;
          numbering(19,3) = 19;
          numbering(20,0) = 67;
          numbering(20,1) = 71;
          numbering(20,2) = 68;
          numbering(20,3) = 69;
          numbering(21,0) = 67;
          numbering(21,1) = 70;
          numbering(21,2) = 68;
          numbering(21,3) = 71;
          numbering(22,0) = 68;
          numbering(22,1) = 70;
          numbering(22,2) = 72;
          numbering(22,3) = 71;
          numbering(23,0) = 68;
          numbering(23,1) = 71;
          numbering(23,2) = 72;
          numbering(23,3) = 69;
          numbering(24,0) = 16;
          numbering(24,1) = 73;
          numbering(24,2) = 74;
          numbering(24,3) = 75;
          numbering(25,0) = 73;
          numbering(25,1) = 18;
          numbering(25,2) = 76;
          numbering(25,3) = 77;
          numbering(26,0) = 74;
          numbering(26,1) = 76;
          numbering(26,2) = 19;
          numbering(26,3) = 78;
          numbering(27,0) = 75;
          numbering(27,1) = 77;
          numbering(27,2) = 78;
          numbering(27,3) = 7;
          numbering(28,0) = 73;
          numbering(28,1) = 77;
          numbering(28,2) = 74;
          numbering(28,3) = 75;
          numbering(29,0) = 73;
          numbering(29,1) = 76;
          numbering(29,2) = 74;
          numbering(29,3) = 77;
          numbering(30,0) = 74;
          numbering(30,1) = 76;
          numbering(30,2) = 78;
          numbering(30,3) = 77;
          numbering(31,0) = 74;
          numbering(31,1) = 77;
          numbering(31,2) = 78;
          numbering(31,3) = 75;
          numbering(32,0) = 14;
          numbering(32,1) = 63;
          numbering(32,2) = 58;
          numbering(32,3) = 59;
          numbering(33,0) = 63;
          numbering(33,1) = 18;
          numbering(33,2) = 79;
          numbering(33,3) = 73;
          numbering(34,0) = 58;
          numbering(34,1) = 79;
          numbering(34,2) = 15;
          numbering(34,3) = 60;
          numbering(35,0) = 59;
          numbering(35,1) = 73;
          numbering(35,2) = 60;
          numbering(35,3) = 16;
          numbering(36,0) = 59;
          numbering(36,1) = 79;
          numbering(36,2) = 58;
          numbering(36,3) = 60;
          numbering(37,0) = 63;
          numbering(37,1) = 79;
          numbering(37,2) = 58;
          numbering(37,3) = 59;
          numbering(38,0) = 63;
          numbering(38,1) = 73;
          numbering(38,2) = 79;
          numbering(38,3) = 59;
          numbering(39,0) = 59;
          numbering(39,1) = 73;
          numbering(39,2) = 79;
          numbering(39,3) = 60;
          numbering(40,0) = 14;
          numbering(40,1) = 62;
          numbering(40,2) = 58;
          numbering(40,3) = 63;
          numbering(41,0) = 62;
          numbering(41,1) = 17;
          numbering(41,2) = 67;
          numbering(41,3) = 66;
          numbering(42,0) = 58;
          numbering(42,1) = 67;
          numbering(42,2) = 15;
          numbering(42,3) = 79;
          numbering(43,0) = 63;
          numbering(43,1) = 66;
          numbering(43,2) = 79;
          numbering(43,3) = 18;
          numbering(44,0) = 62;
          numbering(44,1) = 66;
          numbering(44,2) = 58;
          numbering(44,3) = 63;
          numbering(45,0) = 62;
          numbering(45,1) = 67;
          numbering(45,2) = 58;
          numbering(45,3) = 66;
          numbering(46,0) = 58;
          numbering(46,1) = 67;
          numbering(46,2) = 79;
          numbering(46,3) = 66;
          numbering(47,0) = 58;
          numbering(47,1) = 66;
          numbering(47,2) = 79;
          numbering(47,3) = 63;
          numbering(48,0) = 15;
          numbering(48,1) = 67;
          numbering(48,2) = 69;
          numbering(48,3) = 79;
          numbering(49,0) = 67;
          numbering(49,1) = 17;
          numbering(49,2) = 71;
          numbering(49,3) = 66;
          numbering(50,0) = 69;
          numbering(50,1) = 71;
          numbering(50,2) = 19;
          numbering(50,3) = 76;
          numbering(51,0) = 79;
          numbering(51,1) = 66;
          numbering(51,2) = 76;
          numbering(51,3) = 18;
          numbering(52,0) = 67;
          numbering(52,1) = 66;
          numbering(52,2) = 69;
          numbering(52,3) = 79;
          numbering(53,0) = 67;
          numbering(53,1) = 71;
          numbering(53,2) = 69;
          numbering(53,3) = 66;
          numbering(54,0) = 69;
          numbering(54,1) = 71;
          numbering(54,2) = 76;
          numbering(54,3) = 66;
          numbering(55,0) = 69;
          numbering(55,1) = 66;
          numbering(55,2) = 76;
          numbering(55,3) = 79;
          numbering(56,0) = 15;
          numbering(56,1) = 79;
          numbering(56,2) = 69;
          numbering(56,3) = 60;
          numbering(57,0) = 79;
          numbering(57,1) = 18;
          numbering(57,2) = 76;
          numbering(57,3) = 73;
          numbering(58,0) = 69;
          numbering(58,1) = 76;
          numbering(58,2) = 19;
          numbering(58,3) = 74;
          numbering(59,0) = 60;
          numbering(59,1) = 73;
          numbering(59,2) = 74;
          numbering(59,3) = 16;
          numbering(60,0) = 60;
          numbering(60,1) = 76;
          numbering(60,2) = 69;
          numbering(60,3) = 74;
          numbering(61,0) = 79;
          numbering(61,1) = 76;
          numbering(61,2) = 69;
          numbering(61,3) = 60;
          numbering(62,0) = 79;
          numbering(62,1) = 73;
          numbering(62,2) = 76;
          numbering(62,3) = 60;
          numbering(63,0) = 60;
          numbering(63,1) = 73;
          numbering(63,2) = 76;
          numbering(63,3) = 74;
          numbering(64,0) = 5;
          numbering(64,1) = 80;
          numbering(64,2) = 81;
          numbering(64,3) = 82;
          numbering(65,0) = 80;
          numbering(65,1) = 20;
          numbering(65,2) = 83;
          numbering(65,3) = 84;
          numbering(66,0) = 81;
          numbering(66,1) = 83;
          numbering(66,2) = 21;
          numbering(66,3) = 85;
          numbering(67,0) = 82;
          numbering(67,1) = 84;
          numbering(67,2) = 85;
          numbering(67,3) = 22;
          numbering(68,0) = 80;
          numbering(68,1) = 84;
          numbering(68,2) = 81;
          numbering(68,3) = 82;
          numbering(69,0) = 80;
          numbering(69,1) = 83;
          numbering(69,2) = 81;
          numbering(69,3) = 84;
          numbering(70,0) = 81;
          numbering(70,1) = 83;
          numbering(70,2) = 85;
          numbering(70,3) = 84;
          numbering(71,0) = 81;
          numbering(71,1) = 84;
          numbering(71,2) = 85;
          numbering(71,3) = 82;
          numbering(72,0) = 20;
          numbering(72,1) = 86;
          numbering(72,2) = 87;
          numbering(72,3) = 88;
          numbering(73,0) = 86;
          numbering(73,1) = 1;
          numbering(73,2) = 89;
          numbering(73,3) = 90;
          numbering(74,0) = 87;
          numbering(74,1) = 89;
          numbering(74,2) = 23;
          numbering(74,3) = 91;
          numbering(75,0) = 88;
          numbering(75,1) = 90;
          numbering(75,2) = 91;
          numbering(75,3) = 24;
          numbering(76,0) = 86;
          numbering(76,1) = 90;
          numbering(76,2) = 87;
          numbering(76,3) = 88;
          numbering(77,0) = 86;
          numbering(77,1) = 89;
          numbering(77,2) = 87;
          numbering(77,3) = 90;
          numbering(78,0) = 87;
          numbering(78,1) = 89;
          numbering(78,2) = 91;
          numbering(78,3) = 90;
          numbering(79,0) = 87;
          numbering(79,1) = 90;
          numbering(79,2) = 91;
          numbering(79,3) = 88;
          numbering(80,0) = 21;
          numbering(80,1) = 92;
          numbering(80,2) = 93;
          numbering(80,3) = 94;
          numbering(81,0) = 92;
          numbering(81,1) = 23;
          numbering(81,2) = 95;
          numbering(81,3) = 96;
          numbering(82,0) = 93;
          numbering(82,1) = 95;
          numbering(82,2) = 8;
          numbering(82,3) = 97;
          numbering(83,0) = 94;
          numbering(83,1) = 96;
          numbering(83,2) = 97;
          numbering(83,3) = 25;
          numbering(84,0) = 92;
          numbering(84,1) = 96;
          numbering(84,2) = 93;
          numbering(84,3) = 94;
          numbering(85,0) = 92;
          numbering(85,1) = 95;
          numbering(85,2) = 93;
          numbering(85,3) = 96;
          numbering(86,0) = 93;
          numbering(86,1) = 95;
          numbering(86,2) = 97;
          numbering(86,3) = 96;
          numbering(87,0) = 93;
          numbering(87,1) = 96;
          numbering(87,2) = 97;
          numbering(87,3) = 94;
          numbering(88,0) = 22;
          numbering(88,1) = 98;
          numbering(88,2) = 99;
          numbering(88,3) = 100;
          numbering(89,0) = 98;
          numbering(89,1) = 24;
          numbering(89,2) = 101;
          numbering(89,3) = 102;
          numbering(90,0) = 99;
          numbering(90,1) = 101;
          numbering(90,2) = 25;
          numbering(90,3) = 103;
          numbering(91,0) = 100;
          numbering(91,1) = 102;
          numbering(91,2) = 103;
          numbering(91,3) = 9;
          numbering(92,0) = 98;
          numbering(92,1) = 102;
          numbering(92,2) = 99;
          numbering(92,3) = 100;
          numbering(93,0) = 98;
          numbering(93,1) = 101;
          numbering(93,2) = 99;
          numbering(93,3) = 102;
          numbering(94,0) = 99;
          numbering(94,1) = 101;
          numbering(94,2) = 103;
          numbering(94,3) = 102;
          numbering(95,0) = 99;
          numbering(95,1) = 102;
          numbering(95,2) = 103;
          numbering(95,3) = 100;
          numbering(96,0) = 20;
          numbering(96,1) = 88;
          numbering(96,2) = 83;
          numbering(96,3) = 84;
          numbering(97,0) = 88;
          numbering(97,1) = 24;
          numbering(97,2) = 104;
          numbering(97,3) = 98;
          numbering(98,0) = 83;
          numbering(98,1) = 104;
          numbering(98,2) = 21;
          numbering(98,3) = 85;
          numbering(99,0) = 84;
          numbering(99,1) = 98;
          numbering(99,2) = 85;
          numbering(99,3) = 22;
          numbering(100,0) = 84;
          numbering(100,1) = 104;
          numbering(100,2) = 83;
          numbering(100,3) = 85;
          numbering(101,0) = 88;
          numbering(101,1) = 104;
          numbering(101,2) = 83;
          numbering(101,3) = 84;
          numbering(102,0) = 88;
          numbering(102,1) = 98;
          numbering(102,2) = 104;
          numbering(102,3) = 84;
          numbering(103,0) = 84;
          numbering(103,1) = 98;
          numbering(103,2) = 104;
          numbering(103,3) = 85;
          numbering(104,0) = 20;
          numbering(104,1) = 87;
          numbering(104,2) = 83;
          numbering(104,3) = 88;
          numbering(105,0) = 87;
          numbering(105,1) = 23;
          numbering(105,2) = 92;
          numbering(105,3) = 91;
          numbering(106,0) = 83;
          numbering(106,1) = 92;
          numbering(106,2) = 21;
          numbering(106,3) = 104;
          numbering(107,0) = 88;
          numbering(107,1) = 91;
          numbering(107,2) = 104;
          numbering(107,3) = 24;
          numbering(108,0) = 87;
          numbering(108,1) = 91;
          numbering(108,2) = 83;
          numbering(108,3) = 88;
          numbering(109,0) = 87;
          numbering(109,1) = 92;
          numbering(109,2) = 83;
          numbering(109,3) = 91;
          numbering(110,0) = 83;
          numbering(110,1) = 92;
          numbering(110,2) = 104;
          numbering(110,3) = 91;
          numbering(111,0) = 83;
          numbering(111,1) = 91;
          numbering(111,2) = 104;
          numbering(111,3) = 88;
          numbering(112,0) = 21;
          numbering(112,1) = 92;
          numbering(112,2) = 94;
          numbering(112,3) = 104;
          numbering(113,0) = 92;
          numbering(113,1) = 23;
          numbering(113,2) = 96;
          numbering(113,3) = 91;
          numbering(114,0) = 94;
          numbering(114,1) = 96;
          numbering(114,2) = 25;
          numbering(114,3) = 101;
          numbering(115,0) = 104;
          numbering(115,1) = 91;
          numbering(115,2) = 101;
          numbering(115,3) = 24;
          numbering(116,0) = 92;
          numbering(116,1) = 91;
          numbering(116,2) = 94;
          numbering(116,3) = 104;
          numbering(117,0) = 92;
          numbering(117,1) = 96;
          numbering(117,2) = 94;
          numbering(117,3) = 91;
          numbering(118,0) = 94;
          numbering(118,1) = 96;
          numbering(118,2) = 101;
          numbering(118,3) = 91;
          numbering(119,0) = 94;
          numbering(119,1) = 91;
          numbering(119,2) = 101;
          numbering(119,3) = 104;
          numbering(120,0) = 21;
          numbering(120,1) = 104;
          numbering(120,2) = 94;
          numbering(120,3) = 85;
          numbering(121,0) = 104;
          numbering(121,1) = 24;
          numbering(121,2) = 101;
          numbering(121,3) = 98;
          numbering(122,0) = 94;
          numbering(122,1) = 101;
          numbering(122,2) = 25;
          numbering(122,3) = 99;
          numbering(123,0) = 85;
          numbering(123,1) = 98;
          numbering(123,2) = 99;
          numbering(123,3) = 22;
          numbering(124,0) = 85;
          numbering(124,1) = 101;
          numbering(124,2) = 94;
          numbering(124,3) = 99;
          numbering(125,0) = 104;
          numbering(125,1) = 101;
          numbering(125,2) = 94;
          numbering(125,3) = 85;
          numbering(126,0) = 104;
          numbering(126,1) = 98;
          numbering(126,2) = 101;
          numbering(126,3) = 85;
          numbering(127,0) = 85;
          numbering(127,1) = 98;
          numbering(127,2) = 101;
          numbering(127,3) = 99;
          numbering(128,0) = 6;
          numbering(128,1) = 105;
          numbering(128,2) = 106;
          numbering(128,3) = 107;
          numbering(129,0) = 105;
          numbering(129,1) = 26;
          numbering(129,2) = 108;
          numbering(129,3) = 109;
          numbering(130,0) = 106;
          numbering(130,1) = 108;
          numbering(130,2) = 27;
          numbering(130,3) = 110;
          numbering(131,0) = 107;
          numbering(131,1) = 109;
          numbering(131,2) = 110;
          numbering(131,3) = 28;
          numbering(132,0) = 105;
          numbering(132,1) = 109;
          numbering(132,2) = 106;
          numbering(132,3) = 107;
          numbering(133,0) = 105;
          numbering(133,1) = 108;
          numbering(133,2) = 106;
          numbering(133,3) = 109;
          numbering(134,0) = 106;
          numbering(134,1) = 108;
          numbering(134,2) = 110;
          numbering(134,3) = 109;
          numbering(135,0) = 106;
          numbering(135,1) = 109;
          numbering(135,2) = 110;
          numbering(135,3) = 107;
          numbering(136,0) = 26;
          numbering(136,1) = 111;
          numbering(136,2) = 112;
          numbering(136,3) = 113;
          numbering(137,0) = 111;
          numbering(137,1) = 8;
          numbering(137,2) = 114;
          numbering(137,3) = 115;
          numbering(138,0) = 112;
          numbering(138,1) = 114;
          numbering(138,2) = 29;
          numbering(138,3) = 116;
          numbering(139,0) = 113;
          numbering(139,1) = 115;
          numbering(139,2) = 116;
          numbering(139,3) = 30;
          numbering(140,0) = 111;
          numbering(140,1) = 115;
          numbering(140,2) = 112;
          numbering(140,3) = 113;
          numbering(141,0) = 111;
          numbering(141,1) = 114;
          numbering(141,2) = 112;
          numbering(141,3) = 115;
          numbering(142,0) = 112;
          numbering(142,1) = 114;
          numbering(142,2) = 116;
          numbering(142,3) = 115;
          numbering(143,0) = 112;
          numbering(143,1) = 115;
          numbering(143,2) = 116;
          numbering(143,3) = 113;
          numbering(144,0) = 27;
          numbering(144,1) = 117;
          numbering(144,2) = 118;
          numbering(144,3) = 119;
          numbering(145,0) = 117;
          numbering(145,1) = 29;
          numbering(145,2) = 120;
          numbering(145,3) = 121;
          numbering(146,0) = 118;
          numbering(146,1) = 120;
          numbering(146,2) = 2;
          numbering(146,3) = 122;
          numbering(147,0) = 119;
          numbering(147,1) = 121;
          numbering(147,2) = 122;
          numbering(147,3) = 31;
          numbering(148,0) = 117;
          numbering(148,1) = 121;
          numbering(148,2) = 118;
          numbering(148,3) = 119;
          numbering(149,0) = 117;
          numbering(149,1) = 120;
          numbering(149,2) = 118;
          numbering(149,3) = 121;
          numbering(150,0) = 118;
          numbering(150,1) = 120;
          numbering(150,2) = 122;
          numbering(150,3) = 121;
          numbering(151,0) = 118;
          numbering(151,1) = 121;
          numbering(151,2) = 122;
          numbering(151,3) = 119;
          numbering(152,0) = 28;
          numbering(152,1) = 123;
          numbering(152,2) = 124;
          numbering(152,3) = 125;
          numbering(153,0) = 123;
          numbering(153,1) = 30;
          numbering(153,2) = 126;
          numbering(153,3) = 127;
          numbering(154,0) = 124;
          numbering(154,1) = 126;
          numbering(154,2) = 31;
          numbering(154,3) = 128;
          numbering(155,0) = 125;
          numbering(155,1) = 127;
          numbering(155,2) = 128;
          numbering(155,3) = 10;
          numbering(156,0) = 123;
          numbering(156,1) = 127;
          numbering(156,2) = 124;
          numbering(156,3) = 125;
          numbering(157,0) = 123;
          numbering(157,1) = 126;
          numbering(157,2) = 124;
          numbering(157,3) = 127;
          numbering(158,0) = 124;
          numbering(158,1) = 126;
          numbering(158,2) = 128;
          numbering(158,3) = 127;
          numbering(159,0) = 124;
          numbering(159,1) = 127;
          numbering(159,2) = 128;
          numbering(159,3) = 125;
          numbering(160,0) = 26;
          numbering(160,1) = 113;
          numbering(160,2) = 108;
          numbering(160,3) = 109;
          numbering(161,0) = 113;
          numbering(161,1) = 30;
          numbering(161,2) = 129;
          numbering(161,3) = 123;
          numbering(162,0) = 108;
          numbering(162,1) = 129;
          numbering(162,2) = 27;
          numbering(162,3) = 110;
          numbering(163,0) = 109;
          numbering(163,1) = 123;
          numbering(163,2) = 110;
          numbering(163,3) = 28;
          numbering(164,0) = 109;
          numbering(164,1) = 129;
          numbering(164,2) = 108;
          numbering(164,3) = 110;
          numbering(165,0) = 113;
          numbering(165,1) = 129;
          numbering(165,2) = 108;
          numbering(165,3) = 109;
          numbering(166,0) = 113;
          numbering(166,1) = 123;
          numbering(166,2) = 129;
          numbering(166,3) = 109;
          numbering(167,0) = 109;
          numbering(167,1) = 123;
          numbering(167,2) = 129;
          numbering(167,3) = 110;
          numbering(168,0) = 26;
          numbering(168,1) = 112;
          numbering(168,2) = 108;
          numbering(168,3) = 113;
          numbering(169,0) = 112;
          numbering(169,1) = 29;
          numbering(169,2) = 117;
          numbering(169,3) = 116;
          numbering(170,0) = 108;
          numbering(170,1) = 117;
          numbering(170,2) = 27;
          numbering(170,3) = 129;
          numbering(171,0) = 113;
          numbering(171,1) = 116;
          numbering(171,2) = 129;
          numbering(171,3) = 30;
          numbering(172,0) = 112;
          numbering(172,1) = 116;
          numbering(172,2) = 108;
          numbering(172,3) = 113;
          numbering(173,0) = 112;
          numbering(173,1) = 117;
          numbering(173,2) = 108;
          numbering(173,3) = 116;
          numbering(174,0) = 108;
          numbering(174,1) = 117;
          numbering(174,2) = 129;
          numbering(174,3) = 116;
          numbering(175,0) = 108;
          numbering(175,1) = 116;
          numbering(175,2) = 129;
          numbering(175,3) = 113;
          numbering(176,0) = 27;
          numbering(176,1) = 117;
          numbering(176,2) = 119;
          numbering(176,3) = 129;
          numbering(177,0) = 117;
          numbering(177,1) = 29;
          numbering(177,2) = 121;
          numbering(177,3) = 116;
          numbering(178,0) = 119;
          numbering(178,1) = 121;
          numbering(178,2) = 31;
          numbering(178,3) = 126;
          numbering(179,0) = 129;
          numbering(179,1) = 116;
          numbering(179,2) = 126;
          numbering(179,3) = 30;
          numbering(180,0) = 117;
          numbering(180,1) = 116;
          numbering(180,2) = 119;
          numbering(180,3) = 129;
          numbering(181,0) = 117;
          numbering(181,1) = 121;
          numbering(181,2) = 119;
          numbering(181,3) = 116;
          numbering(182,0) = 119;
          numbering(182,1) = 121;
          numbering(182,2) = 126;
          numbering(182,3) = 116;
          numbering(183,0) = 119;
          numbering(183,1) = 116;
          numbering(183,2) = 126;
          numbering(183,3) = 129;
          numbering(184,0) = 27;
          numbering(184,1) = 129;
          numbering(184,2) = 119;
          numbering(184,3) = 110;
          numbering(185,0) = 129;
          numbering(185,1) = 30;
          numbering(185,2) = 126;
          numbering(185,3) = 123;
          numbering(186,0) = 119;
          numbering(186,1) = 126;
          numbering(186,2) = 31;
          numbering(186,3) = 124;
          numbering(187,0) = 110;
          numbering(187,1) = 123;
          numbering(187,2) = 124;
          numbering(187,3) = 28;
          numbering(188,0) = 110;
          numbering(188,1) = 126;
          numbering(188,2) = 119;
          numbering(188,3) = 124;
          numbering(189,0) = 129;
          numbering(189,1) = 126;
          numbering(189,2) = 119;
          numbering(189,3) = 110;
          numbering(190,0) = 129;
          numbering(190,1) = 123;
          numbering(190,2) = 126;
          numbering(190,3) = 110;
          numbering(191,0) = 110;
          numbering(191,1) = 123;
          numbering(191,2) = 126;
          numbering(191,3) = 124;
          numbering(192,0) = 7;
          numbering(192,1) = 130;
          numbering(192,2) = 131;
          numbering(192,3) = 132;
          numbering(193,0) = 130;
          numbering(193,1) = 32;
          numbering(193,2) = 133;
          numbering(193,3) = 134;
          numbering(194,0) = 131;
          numbering(194,1) = 133;
          numbering(194,2) = 33;
          numbering(194,3) = 135;
          numbering(195,0) = 132;
          numbering(195,1) = 134;
          numbering(195,2) = 135;
          numbering(195,3) = 34;
          numbering(196,0) = 130;
          numbering(196,1) = 134;
          numbering(196,2) = 131;
          numbering(196,3) = 132;
          numbering(197,0) = 130;
          numbering(197,1) = 133;
          numbering(197,2) = 131;
          numbering(197,3) = 134;
          numbering(198,0) = 131;
          numbering(198,1) = 133;
          numbering(198,2) = 135;
          numbering(198,3) = 134;
          numbering(199,0) = 131;
          numbering(199,1) = 134;
          numbering(199,2) = 135;
          numbering(199,3) = 132;
          numbering(200,0) = 32;
          numbering(200,1) = 136;
          numbering(200,2) = 137;
          numbering(200,3) = 138;
          numbering(201,0) = 136;
          numbering(201,1) = 9;
          numbering(201,2) = 139;
          numbering(201,3) = 140;
          numbering(202,0) = 137;
          numbering(202,1) = 139;
          numbering(202,2) = 35;
          numbering(202,3) = 141;
          numbering(203,0) = 138;
          numbering(203,1) = 140;
          numbering(203,2) = 141;
          numbering(203,3) = 36;
          numbering(204,0) = 136;
          numbering(204,1) = 140;
          numbering(204,2) = 137;
          numbering(204,3) = 138;
          numbering(205,0) = 136;
          numbering(205,1) = 139;
          numbering(205,2) = 137;
          numbering(205,3) = 140;
          numbering(206,0) = 137;
          numbering(206,1) = 139;
          numbering(206,2) = 141;
          numbering(206,3) = 140;
          numbering(207,0) = 137;
          numbering(207,1) = 140;
          numbering(207,2) = 141;
          numbering(207,3) = 138;
          numbering(208,0) = 33;
          numbering(208,1) = 142;
          numbering(208,2) = 143;
          numbering(208,3) = 144;
          numbering(209,0) = 142;
          numbering(209,1) = 35;
          numbering(209,2) = 145;
          numbering(209,3) = 146;
          numbering(210,0) = 143;
          numbering(210,1) = 145;
          numbering(210,2) = 10;
          numbering(210,3) = 147;
          numbering(211,0) = 144;
          numbering(211,1) = 146;
          numbering(211,2) = 147;
          numbering(211,3) = 37;
          numbering(212,0) = 142;
          numbering(212,1) = 146;
          numbering(212,2) = 143;
          numbering(212,3) = 144;
          numbering(213,0) = 142;
          numbering(213,1) = 145;
          numbering(213,2) = 143;
          numbering(213,3) = 146;
          numbering(214,0) = 143;
          numbering(214,1) = 145;
          numbering(214,2) = 147;
          numbering(214,3) = 146;
          numbering(215,0) = 143;
          numbering(215,1) = 146;
          numbering(215,2) = 147;
          numbering(215,3) = 144;
          numbering(216,0) = 34;
          numbering(216,1) = 148;
          numbering(216,2) = 149;
          numbering(216,3) = 150;
          numbering(217,0) = 148;
          numbering(217,1) = 36;
          numbering(217,2) = 151;
          numbering(217,3) = 152;
          numbering(218,0) = 149;
          numbering(218,1) = 151;
          numbering(218,2) = 37;
          numbering(218,3) = 153;
          numbering(219,0) = 150;
          numbering(219,1) = 152;
          numbering(219,2) = 153;
          numbering(219,3) = 3;
          numbering(220,0) = 148;
          numbering(220,1) = 152;
          numbering(220,2) = 149;
          numbering(220,3) = 150;
          numbering(221,0) = 148;
          numbering(221,1) = 151;
          numbering(221,2) = 149;
          numbering(221,3) = 152;
          numbering(222,0) = 149;
          numbering(222,1) = 151;
          numbering(222,2) = 153;
          numbering(222,3) = 152;
          numbering(223,0) = 149;
          numbering(223,1) = 152;
          numbering(223,2) = 153;
          numbering(223,3) = 150;
          numbering(224,0) = 32;
          numbering(224,1) = 138;
          numbering(224,2) = 133;
          numbering(224,3) = 134;
          numbering(225,0) = 138;
          numbering(225,1) = 36;
          numbering(225,2) = 154;
          numbering(225,3) = 148;
          numbering(226,0) = 133;
          numbering(226,1) = 154;
          numbering(226,2) = 33;
          numbering(226,3) = 135;
          numbering(227,0) = 134;
          numbering(227,1) = 148;
          numbering(227,2) = 135;
          numbering(227,3) = 34;
          numbering(228,0) = 134;
          numbering(228,1) = 154;
          numbering(228,2) = 133;
          numbering(228,3) = 135;
          numbering(229,0) = 138;
          numbering(229,1) = 154;
          numbering(229,2) = 133;
          numbering(229,3) = 134;
          numbering(230,0) = 138;
          numbering(230,1) = 148;
          numbering(230,2) = 154;
          numbering(230,3) = 134;
          numbering(231,0) = 134;
          numbering(231,1) = 148;
          numbering(231,2) = 154;
          numbering(231,3) = 135;
          numbering(232,0) = 32;
          numbering(232,1) = 137;
          numbering(232,2) = 133;
          numbering(232,3) = 138;
          numbering(233,0) = 137;
          numbering(233,1) = 35;
          numbering(233,2) = 142;
          numbering(233,3) = 141;
          numbering(234,0) = 133;
          numbering(234,1) = 142;
          numbering(234,2) = 33;
          numbering(234,3) = 154;
          numbering(235,0) = 138;
          numbering(235,1) = 141;
          numbering(235,2) = 154;
          numbering(235,3) = 36;
          numbering(236,0) = 137;
          numbering(236,1) = 141;
          numbering(236,2) = 133;
          numbering(236,3) = 138;
          numbering(237,0) = 137;
          numbering(237,1) = 142;
          numbering(237,2) = 133;
          numbering(237,3) = 141;
          numbering(238,0) = 133;
          numbering(238,1) = 142;
          numbering(238,2) = 154;
          numbering(238,3) = 141;
          numbering(239,0) = 133;
          numbering(239,1) = 141;
          numbering(239,2) = 154;
          numbering(239,3) = 138;
          numbering(240,0) = 33;
          numbering(240,1) = 142;
          numbering(240,2) = 144;
          numbering(240,3) = 154;
          numbering(241,0) = 142;
          numbering(241,1) = 35;
          numbering(241,2) = 146;
          numbering(241,3) = 141;
          numbering(242,0) = 144;
          numbering(242,1) = 146;
          numbering(242,2) = 37;
          numbering(242,3) = 151;
          numbering(243,0) = 154;
          numbering(243,1) = 141;
          numbering(243,2) = 151;
          numbering(243,3) = 36;
          numbering(244,0) = 142;
          numbering(244,1) = 141;
          numbering(244,2) = 144;
          numbering(244,3) = 154;
          numbering(245,0) = 142;
          numbering(245,1) = 146;
          numbering(245,2) = 144;
          numbering(245,3) = 141;
          numbering(246,0) = 144;
          numbering(246,1) = 146;
          numbering(246,2) = 151;
          numbering(246,3) = 141;
          numbering(247,0) = 144;
          numbering(247,1) = 141;
          numbering(247,2) = 151;
          numbering(247,3) = 154;
          numbering(248,0) = 33;
          numbering(248,1) = 154;
          numbering(248,2) = 144;
          numbering(248,3) = 135;
          numbering(249,0) = 154;
          numbering(249,1) = 36;
          numbering(249,2) = 151;
          numbering(249,3) = 148;
          numbering(250,0) = 144;
          numbering(250,1) = 151;
          numbering(250,2) = 37;
          numbering(250,3) = 149;
          numbering(251,0) = 135;
          numbering(251,1) = 148;
          numbering(251,2) = 149;
          numbering(251,3) = 34;
          numbering(252,0) = 135;
          numbering(252,1) = 151;
          numbering(252,2) = 144;
          numbering(252,3) = 149;
          numbering(253,0) = 154;
          numbering(253,1) = 151;
          numbering(253,2) = 144;
          numbering(253,3) = 135;
          numbering(254,0) = 154;
          numbering(254,1) = 148;
          numbering(254,2) = 151;
          numbering(254,3) = 135;
          numbering(255,0) = 135;
          numbering(255,1) = 148;
          numbering(255,2) = 151;
          numbering(255,3) = 149;
          numbering(256,0) = 5;
          numbering(256,1) = 82;
          numbering(256,2) = 64;
          numbering(256,3) = 65;
          numbering(257,0) = 82;
          numbering(257,1) = 22;
          numbering(257,2) = 155;
          numbering(257,3) = 156;
          numbering(258,0) = 64;
          numbering(258,1) = 155;
          numbering(258,2) = 17;
          numbering(258,3) = 66;
          numbering(259,0) = 65;
          numbering(259,1) = 156;
          numbering(259,2) = 66;
          numbering(259,3) = 18;
          numbering(260,0) = 65;
          numbering(260,1) = 155;
          numbering(260,2) = 64;
          numbering(260,3) = 66;
          numbering(261,0) = 82;
          numbering(261,1) = 155;
          numbering(261,2) = 64;
          numbering(261,3) = 65;
          numbering(262,0) = 82;
          numbering(262,1) = 156;
          numbering(262,2) = 155;
          numbering(262,3) = 65;
          numbering(263,0) = 65;
          numbering(263,1) = 156;
          numbering(263,2) = 155;
          numbering(263,3) = 66;
          numbering(264,0) = 22;
          numbering(264,1) = 100;
          numbering(264,2) = 157;
          numbering(264,3) = 158;
          numbering(265,0) = 100;
          numbering(265,1) = 9;
          numbering(265,2) = 159;
          numbering(265,3) = 136;
          numbering(266,0) = 157;
          numbering(266,1) = 159;
          numbering(266,2) = 38;
          numbering(266,3) = 160;
          numbering(267,0) = 158;
          numbering(267,1) = 136;
          numbering(267,2) = 160;
          numbering(267,3) = 32;
          numbering(268,0) = 158;
          numbering(268,1) = 159;
          numbering(268,2) = 157;
          numbering(268,3) = 160;
          numbering(269,0) = 100;
          numbering(269,1) = 159;
          numbering(269,2) = 157;
          numbering(269,3) = 158;
          numbering(270,0) = 100;
          numbering(270,1) = 136;
          numbering(270,2) = 159;
          numbering(270,3) = 158;
          numbering(271,0) = 158;
          numbering(271,1) = 136;
          numbering(271,2) = 159;
          numbering(271,3) = 160;
          numbering(272,0) = 17;
          numbering(272,1) = 161;
          numbering(272,2) = 70;
          numbering(272,3) = 71;
          numbering(273,0) = 161;
          numbering(273,1) = 38;
          numbering(273,2) = 162;
          numbering(273,3) = 163;
          numbering(274,0) = 70;
          numbering(274,1) = 162;
          numbering(274,2) = 6;
          numbering(274,3) = 72;
          numbering(275,0) = 71;
          numbering(275,1) = 163;
          numbering(275,2) = 72;
          numbering(275,3) = 19;
          numbering(276,0) = 71;
          numbering(276,1) = 162;
          numbering(276,2) = 70;
          numbering(276,3) = 72;
          numbering(277,0) = 161;
          numbering(277,1) = 162;
          numbering(277,2) = 70;
          numbering(277,3) = 71;
          numbering(278,0) = 161;
          numbering(278,1) = 163;
          numbering(278,2) = 162;
          numbering(278,3) = 71;
          numbering(279,0) = 71;
          numbering(279,1) = 163;
          numbering(279,2) = 162;
          numbering(279,3) = 72;
          numbering(280,0) = 18;
          numbering(280,1) = 164;
          numbering(280,2) = 76;
          numbering(280,3) = 77;
          numbering(281,0) = 164;
          numbering(281,1) = 32;
          numbering(281,2) = 165;
          numbering(281,3) = 130;
          numbering(282,0) = 76;
          numbering(282,1) = 165;
          numbering(282,2) = 19;
          numbering(282,3) = 78;
          numbering(283,0) = 77;
          numbering(283,1) = 130;
          numbering(283,2) = 78;
          numbering(283,3) = 7;
          numbering(284,0) = 77;
          numbering(284,1) = 165;
          numbering(284,2) = 76;
          numbering(284,3) = 78;
          numbering(285,0) = 164;
          numbering(285,1) = 165;
          numbering(285,2) = 76;
          numbering(285,3) = 77;
          numbering(286,0) = 164;
          numbering(286,1) = 130;
          numbering(286,2) = 165;
          numbering(286,3) = 77;
          numbering(287,0) = 77;
          numbering(287,1) = 130;
          numbering(287,2) = 165;
          numbering(287,3) = 78;
          numbering(288,0) = 18;
          numbering(288,1) = 166;
          numbering(288,2) = 66;
          numbering(288,3) = 76;
          numbering(289,0) = 166;
          numbering(289,1) = 38;
          numbering(289,2) = 161;
          numbering(289,3) = 163;
          numbering(290,0) = 66;
          numbering(290,1) = 161;
          numbering(290,2) = 17;
          numbering(290,3) = 71;
          numbering(291,0) = 76;
          numbering(291,1) = 163;
          numbering(291,2) = 71;
          numbering(291,3) = 19;
          numbering(292,0) = 166;
          numbering(292,1) = 163;
          numbering(292,2) = 66;
          numbering(292,3) = 76;
          numbering(293,0) = 166;
          numbering(293,1) = 161;
          numbering(293,2) = 66;
          numbering(293,3) = 163;
          numbering(294,0) = 66;
          numbering(294,1) = 161;
          numbering(294,2) = 71;
          numbering(294,3) = 163;
          numbering(295,0) = 66;
          numbering(295,1) = 163;
          numbering(295,2) = 71;
          numbering(295,3) = 76;
          numbering(296,0) = 22;
          numbering(296,1) = 157;
          numbering(296,2) = 155;
          numbering(296,3) = 156;
          numbering(297,0) = 157;
          numbering(297,1) = 38;
          numbering(297,2) = 161;
          numbering(297,3) = 166;
          numbering(298,0) = 155;
          numbering(298,1) = 161;
          numbering(298,2) = 17;
          numbering(298,3) = 66;
          numbering(299,0) = 156;
          numbering(299,1) = 166;
          numbering(299,2) = 66;
          numbering(299,3) = 18;
          numbering(300,0) = 157;
          numbering(300,1) = 166;
          numbering(300,2) = 155;
          numbering(300,3) = 156;
          numbering(301,0) = 157;
          numbering(301,1) = 161;
          numbering(301,2) = 155;
          numbering(301,3) = 166;
          numbering(302,0) = 155;
          numbering(302,1) = 161;
          numbering(302,2) = 66;
          numbering(302,3) = 166;
          numbering(303,0) = 155;
          numbering(303,1) = 166;
          numbering(303,2) = 66;
          numbering(303,3) = 156;
          numbering(304,0) = 22;
          numbering(304,1) = 158;
          numbering(304,2) = 157;
          numbering(304,3) = 156;
          numbering(305,0) = 158;
          numbering(305,1) = 32;
          numbering(305,2) = 160;
          numbering(305,3) = 164;
          numbering(306,0) = 157;
          numbering(306,1) = 160;
          numbering(306,2) = 38;
          numbering(306,3) = 166;
          numbering(307,0) = 156;
          numbering(307,1) = 164;
          numbering(307,2) = 166;
          numbering(307,3) = 18;
          numbering(308,0) = 156;
          numbering(308,1) = 160;
          numbering(308,2) = 157;
          numbering(308,3) = 166;
          numbering(309,0) = 158;
          numbering(309,1) = 160;
          numbering(309,2) = 157;
          numbering(309,3) = 156;
          numbering(310,0) = 158;
          numbering(310,1) = 164;
          numbering(310,2) = 160;
          numbering(310,3) = 156;
          numbering(311,0) = 156;
          numbering(311,1) = 164;
          numbering(311,2) = 160;
          numbering(311,3) = 166;
          numbering(312,0) = 18;
          numbering(312,1) = 164;
          numbering(312,2) = 166;
          numbering(312,3) = 76;
          numbering(313,0) = 164;
          numbering(313,1) = 32;
          numbering(313,2) = 160;
          numbering(313,3) = 165;
          numbering(314,0) = 166;
          numbering(314,1) = 160;
          numbering(314,2) = 38;
          numbering(314,3) = 163;
          numbering(315,0) = 76;
          numbering(315,1) = 165;
          numbering(315,2) = 163;
          numbering(315,3) = 19;
          numbering(316,0) = 164;
          numbering(316,1) = 165;
          numbering(316,2) = 166;
          numbering(316,3) = 76;
          numbering(317,0) = 164;
          numbering(317,1) = 160;
          numbering(317,2) = 166;
          numbering(317,3) = 165;
          numbering(318,0) = 166;
          numbering(318,1) = 160;
          numbering(318,2) = 163;
          numbering(318,3) = 165;
          numbering(319,0) = 166;
          numbering(319,1) = 165;
          numbering(319,2) = 163;
          numbering(319,3) = 76;
          numbering(320,0) = 5;
          numbering(320,1) = 81;
          numbering(320,2) = 64;
          numbering(320,3) = 82;
          numbering(321,0) = 81;
          numbering(321,1) = 21;
          numbering(321,2) = 167;
          numbering(321,3) = 85;
          numbering(322,0) = 64;
          numbering(322,1) = 167;
          numbering(322,2) = 17;
          numbering(322,3) = 155;
          numbering(323,0) = 82;
          numbering(323,1) = 85;
          numbering(323,2) = 155;
          numbering(323,3) = 22;
          numbering(324,0) = 81;
          numbering(324,1) = 85;
          numbering(324,2) = 64;
          numbering(324,3) = 82;
          numbering(325,0) = 81;
          numbering(325,1) = 167;
          numbering(325,2) = 64;
          numbering(325,3) = 85;
          numbering(326,0) = 64;
          numbering(326,1) = 167;
          numbering(326,2) = 155;
          numbering(326,3) = 85;
          numbering(327,0) = 64;
          numbering(327,1) = 85;
          numbering(327,2) = 155;
          numbering(327,3) = 82;
          numbering(328,0) = 21;
          numbering(328,1) = 93;
          numbering(328,2) = 168;
          numbering(328,3) = 94;
          numbering(329,0) = 93;
          numbering(329,1) = 8;
          numbering(329,2) = 111;
          numbering(329,3) = 97;
          numbering(330,0) = 168;
          numbering(330,1) = 111;
          numbering(330,2) = 26;
          numbering(330,3) = 169;
          numbering(331,0) = 94;
          numbering(331,1) = 97;
          numbering(331,2) = 169;
          numbering(331,3) = 25;
          numbering(332,0) = 93;
          numbering(332,1) = 97;
          numbering(332,2) = 168;
          numbering(332,3) = 94;
          numbering(333,0) = 93;
          numbering(333,1) = 111;
          numbering(333,2) = 168;
          numbering(333,3) = 97;
          numbering(334,0) = 168;
          numbering(334,1) = 111;
          numbering(334,2) = 169;
          numbering(334,3) = 97;
          numbering(335,0) = 168;
          numbering(335,1) = 97;
          numbering(335,2) = 169;
          numbering(335,3) = 94;
          numbering(336,0) = 17;
          numbering(336,1) = 170;
          numbering(336,2) = 70;
          numbering(336,3) = 161;
          numbering(337,0) = 170;
          numbering(337,1) = 26;
          numbering(337,2) = 105;
          numbering(337,3) = 171;
          numbering(338,0) = 70;
          numbering(338,1) = 105;
          numbering(338,2) = 6;
          numbering(338,3) = 162;
          numbering(339,0) = 161;
          numbering(339,1) = 171;
          numbering(339,2) = 162;
          numbering(339,3) = 38;
          numbering(340,0) = 170;
          numbering(340,1) = 171;
          numbering(340,2) = 70;
          numbering(340,3) = 161;
          numbering(341,0) = 170;
          numbering(341,1) = 105;
          numbering(341,2) = 70;
          numbering(341,3) = 171;
          numbering(342,0) = 70;
          numbering(342,1) = 105;
          numbering(342,2) = 162;
          numbering(342,3) = 171;
          numbering(343,0) = 70;
          numbering(343,1) = 171;
          numbering(343,2) = 162;
          numbering(343,3) = 161;
          numbering(344,0) = 22;
          numbering(344,1) = 99;
          numbering(344,2) = 157;
          numbering(344,3) = 100;
          numbering(345,0) = 99;
          numbering(345,1) = 25;
          numbering(345,2) = 172;
          numbering(345,3) = 103;
          numbering(346,0) = 157;
          numbering(346,1) = 172;
          numbering(346,2) = 38;
          numbering(346,3) = 159;
          numbering(347,0) = 100;
          numbering(347,1) = 103;
          numbering(347,2) = 159;
          numbering(347,3) = 9;
          numbering(348,0) = 99;
          numbering(348,1) = 103;
          numbering(348,2) = 157;
          numbering(348,3) = 100;
          numbering(349,0) = 99;
          numbering(349,1) = 172;
          numbering(349,2) = 157;
          numbering(349,3) = 103;
          numbering(350,0) = 157;
          numbering(350,1) = 172;
          numbering(350,2) = 159;
          numbering(350,3) = 103;
          numbering(351,0) = 157;
          numbering(351,1) = 103;
          numbering(351,2) = 159;
          numbering(351,3) = 100;
          numbering(352,0) = 21;
          numbering(352,1) = 94;
          numbering(352,2) = 167;
          numbering(352,3) = 85;
          numbering(353,0) = 94;
          numbering(353,1) = 25;
          numbering(353,2) = 173;
          numbering(353,3) = 99;
          numbering(354,0) = 167;
          numbering(354,1) = 173;
          numbering(354,2) = 17;
          numbering(354,3) = 155;
          numbering(355,0) = 85;
          numbering(355,1) = 99;
          numbering(355,2) = 155;
          numbering(355,3) = 22;
          numbering(356,0) = 85;
          numbering(356,1) = 173;
          numbering(356,2) = 167;
          numbering(356,3) = 155;
          numbering(357,0) = 94;
          numbering(357,1) = 173;
          numbering(357,2) = 167;
          numbering(357,3) = 85;
          numbering(358,0) = 94;
          numbering(358,1) = 99;
          numbering(358,2) = 173;
          numbering(358,3) = 85;
          numbering(359,0) = 85;
          numbering(359,1) = 99;
          numbering(359,2) = 173;
          numbering(359,3) = 155;
          numbering(360,0) = 21;
          numbering(360,1) = 168;
          numbering(360,2) = 167;
          numbering(360,3) = 94;
          numbering(361,0) = 168;
          numbering(361,1) = 26;
          numbering(361,2) = 170;
          numbering(361,3) = 169;
          numbering(362,0) = 167;
          numbering(362,1) = 170;
          numbering(362,2) = 17;
          numbering(362,3) = 173;
          numbering(363,0) = 94;
          numbering(363,1) = 169;
          numbering(363,2) = 173;
          numbering(363,3) = 25;
          numbering(364,0) = 168;
          numbering(364,1) = 169;
          numbering(364,2) = 167;
          numbering(364,3) = 94;
          numbering(365,0) = 168;
          numbering(365,1) = 170;
          numbering(365,2) = 167;
          numbering(365,3) = 169;
          numbering(366,0) = 167;
          numbering(366,1) = 170;
          numbering(366,2) = 173;
          numbering(366,3) = 169;
          numbering(367,0) = 167;
          numbering(367,1) = 169;
          numbering(367,2) = 173;
          numbering(367,3) = 94;
          numbering(368,0) = 17;
          numbering(368,1) = 170;
          numbering(368,2) = 161;
          numbering(368,3) = 173;
          numbering(369,0) = 170;
          numbering(369,1) = 26;
          numbering(369,2) = 171;
          numbering(369,3) = 169;
          numbering(370,0) = 161;
          numbering(370,1) = 171;
          numbering(370,2) = 38;
          numbering(370,3) = 172;
          numbering(371,0) = 173;
          numbering(371,1) = 169;
          numbering(371,2) = 172;
          numbering(371,3) = 25;
          numbering(372,0) = 173;
          numbering(372,1) = 171;
          numbering(372,2) = 161;
          numbering(372,3) = 172;
          numbering(373,0) = 170;
          numbering(373,1) = 171;
          numbering(373,2) = 161;
          numbering(373,3) = 173;
          numbering(374,0) = 170;
          numbering(374,1) = 169;
          numbering(374,2) = 171;
          numbering(374,3) = 173;
          numbering(375,0) = 173;
          numbering(375,1) = 169;
          numbering(375,2) = 171;
          numbering(375,3) = 172;
          numbering(376,0) = 17;
          numbering(376,1) = 173;
          numbering(376,2) = 161;
          numbering(376,3) = 155;
          numbering(377,0) = 173;
          numbering(377,1) = 25;
          numbering(377,2) = 172;
          numbering(377,3) = 99;
          numbering(378,0) = 161;
          numbering(378,1) = 172;
          numbering(378,2) = 38;
          numbering(378,3) = 157;
          numbering(379,0) = 155;
          numbering(379,1) = 99;
          numbering(379,2) = 157;
          numbering(379,3) = 22;
          numbering(380,0) = 173;
          numbering(380,1) = 99;
          numbering(380,2) = 161;
          numbering(380,3) = 155;
          numbering(381,0) = 173;
          numbering(381,1) = 172;
          numbering(381,2) = 161;
          numbering(381,3) = 99;
          numbering(382,0) = 161;
          numbering(382,1) = 172;
          numbering(382,2) = 157;
          numbering(382,3) = 99;
          numbering(383,0) = 161;
          numbering(383,1) = 99;
          numbering(383,2) = 157;
          numbering(383,3) = 155;
          numbering(384,0) = 6;
          numbering(384,1) = 105;
          numbering(384,2) = 107;
          numbering(384,3) = 162;
          numbering(385,0) = 105;
          numbering(385,1) = 26;
          numbering(385,2) = 109;
          numbering(385,3) = 171;
          numbering(386,0) = 107;
          numbering(386,1) = 109;
          numbering(386,2) = 28;
          numbering(386,3) = 174;
          numbering(387,0) = 162;
          numbering(387,1) = 171;
          numbering(387,2) = 174;
          numbering(387,3) = 38;
          numbering(388,0) = 105;
          numbering(388,1) = 171;
          numbering(388,2) = 107;
          numbering(388,3) = 162;
          numbering(389,0) = 105;
          numbering(389,1) = 109;
          numbering(389,2) = 107;
          numbering(389,3) = 171;
          numbering(390,0) = 107;
          numbering(390,1) = 109;
          numbering(390,2) = 174;
          numbering(390,3) = 171;
          numbering(391,0) = 107;
          numbering(391,1) = 171;
          numbering(391,2) = 174;
          numbering(391,3) = 162;
          numbering(392,0) = 26;
          numbering(392,1) = 111;
          numbering(392,2) = 113;
          numbering(392,3) = 169;
          numbering(393,0) = 111;
          numbering(393,1) = 8;
          numbering(393,2) = 115;
          numbering(393,3) = 97;
          numbering(394,0) = 113;
          numbering(394,1) = 115;
          numbering(394,2) = 30;
          numbering(394,3) = 175;
          numbering(395,0) = 169;
          numbering(395,1) = 97;
          numbering(395,2) = 175;
          numbering(395,3) = 25;
          numbering(396,0) = 111;
          numbering(396,1) = 97;
          numbering(396,2) = 113;
          numbering(396,3) = 169;
          numbering(397,0) = 111;
          numbering(397,1) = 115;
          numbering(397,2) = 113;
          numbering(397,3) = 97;
          numbering(398,0) = 113;
          numbering(398,1) = 115;
          numbering(398,2) = 175;
          numbering(398,3) = 97;
          numbering(399,0) = 113;
          numbering(399,1) = 97;
          numbering(399,2) = 175;
          numbering(399,3) = 169;
          numbering(400,0) = 28;
          numbering(400,1) = 123;
          numbering(400,2) = 125;
          numbering(400,3) = 176;
          numbering(401,0) = 123;
          numbering(401,1) = 30;
          numbering(401,2) = 127;
          numbering(401,3) = 177;
          numbering(402,0) = 125;
          numbering(402,1) = 127;
          numbering(402,2) = 10;
          numbering(402,3) = 145;
          numbering(403,0) = 176;
          numbering(403,1) = 177;
          numbering(403,2) = 145;
          numbering(403,3) = 35;
          numbering(404,0) = 123;
          numbering(404,1) = 177;
          numbering(404,2) = 125;
          numbering(404,3) = 176;
          numbering(405,0) = 123;
          numbering(405,1) = 127;
          numbering(405,2) = 125;
          numbering(405,3) = 177;
          numbering(406,0) = 125;
          numbering(406,1) = 127;
          numbering(406,2) = 145;
          numbering(406,3) = 177;
          numbering(407,0) = 125;
          numbering(407,1) = 177;
          numbering(407,2) = 145;
          numbering(407,3) = 176;
          numbering(408,0) = 38;
          numbering(408,1) = 172;
          numbering(408,2) = 178;
          numbering(408,3) = 159;
          numbering(409,0) = 172;
          numbering(409,1) = 25;
          numbering(409,2) = 179;
          numbering(409,3) = 103;
          numbering(410,0) = 178;
          numbering(410,1) = 179;
          numbering(410,2) = 35;
          numbering(410,3) = 139;
          numbering(411,0) = 159;
          numbering(411,1) = 103;
          numbering(411,2) = 139;
          numbering(411,3) = 9;
          numbering(412,0) = 172;
          numbering(412,1) = 103;
          numbering(412,2) = 178;
          numbering(412,3) = 159;
          numbering(413,0) = 172;
          numbering(413,1) = 179;
          numbering(413,2) = 178;
          numbering(413,3) = 103;
          numbering(414,0) = 178;
          numbering(414,1) = 179;
          numbering(414,2) = 139;
          numbering(414,3) = 103;
          numbering(415,0) = 178;
          numbering(415,1) = 103;
          numbering(415,2) = 139;
          numbering(415,3) = 159;
          numbering(416,0) = 26;
          numbering(416,1) = 169;
          numbering(416,2) = 109;
          numbering(416,3) = 171;
          numbering(417,0) = 169;
          numbering(417,1) = 25;
          numbering(417,2) = 180;
          numbering(417,3) = 172;
          numbering(418,0) = 109;
          numbering(418,1) = 180;
          numbering(418,2) = 28;
          numbering(418,3) = 174;
          numbering(419,0) = 171;
          numbering(419,1) = 172;
          numbering(419,2) = 174;
          numbering(419,3) = 38;
          numbering(420,0) = 171;
          numbering(420,1) = 180;
          numbering(420,2) = 109;
          numbering(420,3) = 174;
          numbering(421,0) = 169;
          numbering(421,1) = 180;
          numbering(421,2) = 109;
          numbering(421,3) = 171;
          numbering(422,0) = 169;
          numbering(422,1) = 172;
          numbering(422,2) = 180;
          numbering(422,3) = 171;
          numbering(423,0) = 171;
          numbering(423,1) = 172;
          numbering(423,2) = 180;
          numbering(423,3) = 174;
          numbering(424,0) = 26;
          numbering(424,1) = 113;
          numbering(424,2) = 109;
          numbering(424,3) = 169;
          numbering(425,0) = 113;
          numbering(425,1) = 30;
          numbering(425,2) = 123;
          numbering(425,3) = 175;
          numbering(426,0) = 109;
          numbering(426,1) = 123;
          numbering(426,2) = 28;
          numbering(426,3) = 180;
          numbering(427,0) = 169;
          numbering(427,1) = 175;
          numbering(427,2) = 180;
          numbering(427,3) = 25;
          numbering(428,0) = 113;
          numbering(428,1) = 175;
          numbering(428,2) = 109;
          numbering(428,3) = 169;
          numbering(429,0) = 113;
          numbering(429,1) = 123;
          numbering(429,2) = 109;
          numbering(429,3) = 175;
          numbering(430,0) = 109;
          numbering(430,1) = 123;
          numbering(430,2) = 180;
          numbering(430,3) = 175;
          numbering(431,0) = 109;
          numbering(431,1) = 175;
          numbering(431,2) = 180;
          numbering(431,3) = 169;
          numbering(432,0) = 28;
          numbering(432,1) = 123;
          numbering(432,2) = 176;
          numbering(432,3) = 180;
          numbering(433,0) = 123;
          numbering(433,1) = 30;
          numbering(433,2) = 177;
          numbering(433,3) = 175;
          numbering(434,0) = 176;
          numbering(434,1) = 177;
          numbering(434,2) = 35;
          numbering(434,3) = 179;
          numbering(435,0) = 180;
          numbering(435,1) = 175;
          numbering(435,2) = 179;
          numbering(435,3) = 25;
          numbering(436,0) = 180;
          numbering(436,1) = 177;
          numbering(436,2) = 176;
          numbering(436,3) = 179;
          numbering(437,0) = 123;
          numbering(437,1) = 177;
          numbering(437,2) = 176;
          numbering(437,3) = 180;
          numbering(438,0) = 123;
          numbering(438,1) = 175;
          numbering(438,2) = 177;
          numbering(438,3) = 180;
          numbering(439,0) = 180;
          numbering(439,1) = 175;
          numbering(439,2) = 177;
          numbering(439,3) = 179;
          numbering(440,0) = 28;
          numbering(440,1) = 180;
          numbering(440,2) = 176;
          numbering(440,3) = 174;
          numbering(441,0) = 180;
          numbering(441,1) = 25;
          numbering(441,2) = 179;
          numbering(441,3) = 172;
          numbering(442,0) = 176;
          numbering(442,1) = 179;
          numbering(442,2) = 35;
          numbering(442,3) = 178;
          numbering(443,0) = 174;
          numbering(443,1) = 172;
          numbering(443,2) = 178;
          numbering(443,3) = 38;
          numbering(444,0) = 180;
          numbering(444,1) = 172;
          numbering(444,2) = 176;
          numbering(444,3) = 174;
          numbering(445,0) = 180;
          numbering(445,1) = 179;
          numbering(445,2) = 176;
          numbering(445,3) = 172;
          numbering(446,0) = 176;
          numbering(446,1) = 179;
          numbering(446,2) = 178;
          numbering(446,3) = 172;
          numbering(447,0) = 176;
          numbering(447,1) = 172;
          numbering(447,2) = 178;
          numbering(447,3) = 174;
          numbering(448,0) = 6;
          numbering(448,1) = 162;
          numbering(448,2) = 107;
          numbering(448,3) = 72;
          numbering(449,0) = 162;
          numbering(449,1) = 38;
          numbering(449,2) = 174;
          numbering(449,3) = 163;
          numbering(450,0) = 107;
          numbering(450,1) = 174;
          numbering(450,2) = 28;
          numbering(450,3) = 181;
          numbering(451,0) = 72;
          numbering(451,1) = 163;
          numbering(451,2) = 181;
          numbering(451,3) = 19;
          numbering(452,0) = 72;
          numbering(452,1) = 174;
          numbering(452,2) = 107;
          numbering(452,3) = 181;
          numbering(453,0) = 162;
          numbering(453,1) = 174;
          numbering(453,2) = 107;
          numbering(453,3) = 72;
          numbering(454,0) = 162;
          numbering(454,1) = 163;
          numbering(454,2) = 174;
          numbering(454,3) = 72;
          numbering(455,0) = 72;
          numbering(455,1) = 163;
          numbering(455,2) = 174;
          numbering(455,3) = 181;
          numbering(456,0) = 38;
          numbering(456,1) = 159;
          numbering(456,2) = 178;
          numbering(456,3) = 160;
          numbering(457,0) = 159;
          numbering(457,1) = 9;
          numbering(457,2) = 139;
          numbering(457,3) = 136;
          numbering(458,0) = 178;
          numbering(458,1) = 139;
          numbering(458,2) = 35;
          numbering(458,3) = 137;
          numbering(459,0) = 160;
          numbering(459,1) = 136;
          numbering(459,2) = 137;
          numbering(459,3) = 32;
          numbering(460,0) = 160;
          numbering(460,1) = 139;
          numbering(460,2) = 178;
          numbering(460,3) = 137;
          numbering(461,0) = 159;
          numbering(461,1) = 139;
          numbering(461,2) = 178;
          numbering(461,3) = 160;
          numbering(462,0) = 159;
          numbering(462,1) = 136;
          numbering(462,2) = 139;
          numbering(462,3) = 160;
          numbering(463,0) = 160;
          numbering(463,1) = 136;
          numbering(463,2) = 139;
          numbering(463,3) = 137;
          numbering(464,0) = 28;
          numbering(464,1) = 176;
          numbering(464,2) = 125;
          numbering(464,3) = 182;
          numbering(465,0) = 176;
          numbering(465,1) = 35;
          numbering(465,2) = 145;
          numbering(465,3) = 142;
          numbering(466,0) = 125;
          numbering(466,1) = 145;
          numbering(466,2) = 10;
          numbering(466,3) = 143;
          numbering(467,0) = 182;
          numbering(467,1) = 142;
          numbering(467,2) = 143;
          numbering(467,3) = 33;
          numbering(468,0) = 182;
          numbering(468,1) = 145;
          numbering(468,2) = 125;
          numbering(468,3) = 143;
          numbering(469,0) = 176;
          numbering(469,1) = 145;
          numbering(469,2) = 125;
          numbering(469,3) = 182;
          numbering(470,0) = 176;
          numbering(470,1) = 142;
          numbering(470,2) = 145;
          numbering(470,3) = 182;
          numbering(471,0) = 182;
          numbering(471,1) = 142;
          numbering(471,2) = 145;
          numbering(471,3) = 143;
          numbering(472,0) = 19;
          numbering(472,1) = 165;
          numbering(472,2) = 183;
          numbering(472,3) = 78;
          numbering(473,0) = 165;
          numbering(473,1) = 32;
          numbering(473,2) = 133;
          numbering(473,3) = 130;
          numbering(474,0) = 183;
          numbering(474,1) = 133;
          numbering(474,2) = 33;
          numbering(474,3) = 131;
          numbering(475,0) = 78;
          numbering(475,1) = 130;
          numbering(475,2) = 131;
          numbering(475,3) = 7;
          numbering(476,0) = 78;
          numbering(476,1) = 133;
          numbering(476,2) = 183;
          numbering(476,3) = 131;
          numbering(477,0) = 165;
          numbering(477,1) = 133;
          numbering(477,2) = 183;
          numbering(477,3) = 78;
          numbering(478,0) = 165;
          numbering(478,1) = 130;
          numbering(478,2) = 133;
          numbering(478,3) = 78;
          numbering(479,0) = 78;
          numbering(479,1) = 130;
          numbering(479,2) = 133;
          numbering(479,3) = 131;
          numbering(480,0) = 19;
          numbering(480,1) = 184;
          numbering(480,2) = 181;
          numbering(480,3) = 183;
          numbering(481,0) = 184;
          numbering(481,1) = 35;
          numbering(481,2) = 176;
          numbering(481,3) = 142;
          numbering(482,0) = 181;
          numbering(482,1) = 176;
          numbering(482,2) = 28;
          numbering(482,3) = 182;
          numbering(483,0) = 183;
          numbering(483,1) = 142;
          numbering(483,2) = 182;
          numbering(483,3) = 33;
          numbering(484,0) = 184;
          numbering(484,1) = 176;
          numbering(484,2) = 182;
          numbering(484,3) = 142;
          numbering(485,0) = 184;
          numbering(485,1) = 176;
          numbering(485,2) = 181;
          numbering(485,3) = 182;
          numbering(486,0) = 183;
          numbering(486,1) = 184;
          numbering(486,2) = 181;
          numbering(486,3) = 182;
          numbering(487,0) = 183;
          numbering(487,1) = 184;
          numbering(487,2) = 182;
          numbering(487,3) = 142;
          numbering(488,0) = 38;
          numbering(488,1) = 178;
          numbering(488,2) = 174;
          numbering(488,3) = 163;
          numbering(489,0) = 178;
          numbering(489,1) = 35;
          numbering(489,2) = 176;
          numbering(489,3) = 184;
          numbering(490,0) = 174;
          numbering(490,1) = 176;
          numbering(490,2) = 28;
          numbering(490,3) = 181;
          numbering(491,0) = 163;
          numbering(491,1) = 184;
          numbering(491,2) = 181;
          numbering(491,3) = 19;
          numbering(492,0) = 178;
          numbering(492,1) = 184;
          numbering(492,2) = 174;
          numbering(492,3) = 163;
          numbering(493,0) = 178;
          numbering(493,1) = 176;
          numbering(493,2) = 174;
          numbering(493,3) = 184;
          numbering(494,0) = 174;
          numbering(494,1) = 176;
          numbering(494,2) = 181;
          numbering(494,3) = 184;
          numbering(495,0) = 174;
          numbering(495,1) = 184;
          numbering(495,2) = 181;
          numbering(495,3) = 163;
          numbering(496,0) = 38;
          numbering(496,1) = 160;
          numbering(496,2) = 178;
          numbering(496,3) = 163;
          numbering(497,0) = 160;
          numbering(497,1) = 32;
          numbering(497,2) = 137;
          numbering(497,3) = 165;
          numbering(498,0) = 178;
          numbering(498,1) = 137;
          numbering(498,2) = 35;
          numbering(498,3) = 184;
          numbering(499,0) = 163;
          numbering(499,1) = 165;
          numbering(499,2) = 184;
          numbering(499,3) = 19;
          numbering(500,0) = 160;
          numbering(500,1) = 137;
          numbering(500,2) = 184;
          numbering(500,3) = 165;
          numbering(501,0) = 160;
          numbering(501,1) = 137;
          numbering(501,2) = 178;
          numbering(501,3) = 184;
          numbering(502,0) = 163;
          numbering(502,1) = 160;
          numbering(502,2) = 178;
          numbering(502,3) = 184;
          numbering(503,0) = 163;
          numbering(503,1) = 160;
          numbering(503,2) = 184;
          numbering(503,3) = 165;
          numbering(504,0) = 19;
          numbering(504,1) = 165;
          numbering(504,2) = 184;
          numbering(504,3) = 183;
          numbering(505,0) = 165;
          numbering(505,1) = 32;
          numbering(505,2) = 137;
          numbering(505,3) = 133;
          numbering(506,0) = 184;
          numbering(506,1) = 137;
          numbering(506,2) = 35;
          numbering(506,3) = 142;
          numbering(507,0) = 183;
          numbering(507,1) = 133;
          numbering(507,2) = 142;
          numbering(507,3) = 33;
          numbering(508,0) = 165;
          numbering(508,1) = 133;
          numbering(508,2) = 184;
          numbering(508,3) = 183;
          numbering(509,0) = 165;
          numbering(509,1) = 137;
          numbering(509,2) = 184;
          numbering(509,3) = 133;
          numbering(510,0) = 184;
          numbering(510,1) = 137;
          numbering(510,2) = 142;
          numbering(510,3) = 133;
          numbering(511,0) = 184;
          numbering(511,1) = 133;
          numbering(511,2) = 142;
          numbering(511,3) = 183;
          numbering(512,0) = 0;
          numbering(512,1) = 56;
          numbering(512,2) = 185;
          numbering(512,3) = 57;
          numbering(513,0) = 56;
          numbering(513,1) = 15;
          numbering(513,2) = 186;
          numbering(513,3) = 60;
          numbering(514,0) = 185;
          numbering(514,1) = 186;
          numbering(514,2) = 39;
          numbering(514,3) = 187;
          numbering(515,0) = 57;
          numbering(515,1) = 60;
          numbering(515,2) = 187;
          numbering(515,3) = 16;
          numbering(516,0) = 56;
          numbering(516,1) = 60;
          numbering(516,2) = 185;
          numbering(516,3) = 57;
          numbering(517,0) = 56;
          numbering(517,1) = 186;
          numbering(517,2) = 185;
          numbering(517,3) = 60;
          numbering(518,0) = 185;
          numbering(518,1) = 186;
          numbering(518,2) = 187;
          numbering(518,3) = 60;
          numbering(519,0) = 185;
          numbering(519,1) = 60;
          numbering(519,2) = 187;
          numbering(519,3) = 57;
          numbering(520,0) = 15;
          numbering(520,1) = 68;
          numbering(520,2) = 188;
          numbering(520,3) = 69;
          numbering(521,0) = 68;
          numbering(521,1) = 6;
          numbering(521,2) = 189;
          numbering(521,3) = 72;
          numbering(522,0) = 188;
          numbering(522,1) = 189;
          numbering(522,2) = 40;
          numbering(522,3) = 190;
          numbering(523,0) = 69;
          numbering(523,1) = 72;
          numbering(523,2) = 190;
          numbering(523,3) = 19;
          numbering(524,0) = 68;
          numbering(524,1) = 72;
          numbering(524,2) = 188;
          numbering(524,3) = 69;
          numbering(525,0) = 68;
          numbering(525,1) = 189;
          numbering(525,2) = 188;
          numbering(525,3) = 72;
          numbering(526,0) = 188;
          numbering(526,1) = 189;
          numbering(526,2) = 190;
          numbering(526,3) = 72;
          numbering(527,0) = 188;
          numbering(527,1) = 72;
          numbering(527,2) = 190;
          numbering(527,3) = 69;
          numbering(528,0) = 39;
          numbering(528,1) = 191;
          numbering(528,2) = 192;
          numbering(528,3) = 193;
          numbering(529,0) = 191;
          numbering(529,1) = 40;
          numbering(529,2) = 194;
          numbering(529,3) = 195;
          numbering(530,0) = 192;
          numbering(530,1) = 194;
          numbering(530,2) = 11;
          numbering(530,3) = 196;
          numbering(531,0) = 193;
          numbering(531,1) = 195;
          numbering(531,2) = 196;
          numbering(531,3) = 41;
          numbering(532,0) = 191;
          numbering(532,1) = 195;
          numbering(532,2) = 192;
          numbering(532,3) = 193;
          numbering(533,0) = 191;
          numbering(533,1) = 194;
          numbering(533,2) = 192;
          numbering(533,3) = 195;
          numbering(534,0) = 192;
          numbering(534,1) = 194;
          numbering(534,2) = 196;
          numbering(534,3) = 195;
          numbering(535,0) = 192;
          numbering(535,1) = 195;
          numbering(535,2) = 196;
          numbering(535,3) = 193;
          numbering(536,0) = 16;
          numbering(536,1) = 74;
          numbering(536,2) = 197;
          numbering(536,3) = 75;
          numbering(537,0) = 74;
          numbering(537,1) = 19;
          numbering(537,2) = 198;
          numbering(537,3) = 78;
          numbering(538,0) = 197;
          numbering(538,1) = 198;
          numbering(538,2) = 41;
          numbering(538,3) = 199;
          numbering(539,0) = 75;
          numbering(539,1) = 78;
          numbering(539,2) = 199;
          numbering(539,3) = 7;
          numbering(540,0) = 74;
          numbering(540,1) = 78;
          numbering(540,2) = 197;
          numbering(540,3) = 75;
          numbering(541,0) = 74;
          numbering(541,1) = 198;
          numbering(541,2) = 197;
          numbering(541,3) = 78;
          numbering(542,0) = 197;
          numbering(542,1) = 198;
          numbering(542,2) = 199;
          numbering(542,3) = 78;
          numbering(543,0) = 197;
          numbering(543,1) = 78;
          numbering(543,2) = 199;
          numbering(543,3) = 75;
          numbering(544,0) = 15;
          numbering(544,1) = 69;
          numbering(544,2) = 186;
          numbering(544,3) = 60;
          numbering(545,0) = 69;
          numbering(545,1) = 19;
          numbering(545,2) = 200;
          numbering(545,3) = 74;
          numbering(546,0) = 186;
          numbering(546,1) = 200;
          numbering(546,2) = 39;
          numbering(546,3) = 187;
          numbering(547,0) = 60;
          numbering(547,1) = 74;
          numbering(547,2) = 187;
          numbering(547,3) = 16;
          numbering(548,0) = 60;
          numbering(548,1) = 200;
          numbering(548,2) = 186;
          numbering(548,3) = 187;
          numbering(549,0) = 69;
          numbering(549,1) = 200;
          numbering(549,2) = 186;
          numbering(549,3) = 60;
          numbering(550,0) = 69;
          numbering(550,1) = 74;
          numbering(550,2) = 200;
          numbering(550,3) = 60;
          numbering(551,0) = 60;
          numbering(551,1) = 74;
          numbering(551,2) = 200;
          numbering(551,3) = 187;
          numbering(552,0) = 15;
          numbering(552,1) = 188;
          numbering(552,2) = 186;
          numbering(552,3) = 69;
          numbering(553,0) = 188;
          numbering(553,1) = 40;
          numbering(553,2) = 191;
          numbering(553,3) = 190;
          numbering(554,0) = 186;
          numbering(554,1) = 191;
          numbering(554,2) = 39;
          numbering(554,3) = 200;
          numbering(555,0) = 69;
          numbering(555,1) = 190;
          numbering(555,2) = 200;
          numbering(555,3) = 19;
          numbering(556,0) = 188;
          numbering(556,1) = 190;
          numbering(556,2) = 186;
          numbering(556,3) = 69;
          numbering(557,0) = 188;
          numbering(557,1) = 191;
          numbering(557,2) = 186;
          numbering(557,3) = 190;
          numbering(558,0) = 186;
          numbering(558,1) = 191;
          numbering(558,2) = 200;
          numbering(558,3) = 190;
          numbering(559,0) = 186;
          numbering(559,1) = 190;
          numbering(559,2) = 200;
          numbering(559,3) = 69;
          numbering(560,0) = 39;
          numbering(560,1) = 191;
          numbering(560,2) = 193;
          numbering(560,3) = 200;
          numbering(561,0) = 191;
          numbering(561,1) = 40;
          numbering(561,2) = 195;
          numbering(561,3) = 190;
          numbering(562,0) = 193;
          numbering(562,1) = 195;
          numbering(562,2) = 41;
          numbering(562,3) = 198;
          numbering(563,0) = 200;
          numbering(563,1) = 190;
          numbering(563,2) = 198;
          numbering(563,3) = 19;
          numbering(564,0) = 200;
          numbering(564,1) = 195;
          numbering(564,2) = 193;
          numbering(564,3) = 198;
          numbering(565,0) = 191;
          numbering(565,1) = 195;
          numbering(565,2) = 193;
          numbering(565,3) = 200;
          numbering(566,0) = 191;
          numbering(566,1) = 190;
          numbering(566,2) = 195;
          numbering(566,3) = 200;
          numbering(567,0) = 200;
          numbering(567,1) = 190;
          numbering(567,2) = 195;
          numbering(567,3) = 198;
          numbering(568,0) = 39;
          numbering(568,1) = 200;
          numbering(568,2) = 193;
          numbering(568,3) = 187;
          numbering(569,0) = 200;
          numbering(569,1) = 19;
          numbering(569,2) = 198;
          numbering(569,3) = 74;
          numbering(570,0) = 193;
          numbering(570,1) = 198;
          numbering(570,2) = 41;
          numbering(570,3) = 197;
          numbering(571,0) = 187;
          numbering(571,1) = 74;
          numbering(571,2) = 197;
          numbering(571,3) = 16;
          numbering(572,0) = 200;
          numbering(572,1) = 74;
          numbering(572,2) = 193;
          numbering(572,3) = 187;
          numbering(573,0) = 200;
          numbering(573,1) = 198;
          numbering(573,2) = 193;
          numbering(573,3) = 74;
          numbering(574,0) = 193;
          numbering(574,1) = 198;
          numbering(574,2) = 197;
          numbering(574,3) = 74;
          numbering(575,0) = 193;
          numbering(575,1) = 74;
          numbering(575,2) = 197;
          numbering(575,3) = 187;
          numbering(576,0) = 6;
          numbering(576,1) = 106;
          numbering(576,2) = 201;
          numbering(576,3) = 107;
          numbering(577,0) = 106;
          numbering(577,1) = 27;
          numbering(577,2) = 202;
          numbering(577,3) = 110;
          numbering(578,0) = 201;
          numbering(578,1) = 202;
          numbering(578,2) = 42;
          numbering(578,3) = 203;
          numbering(579,0) = 107;
          numbering(579,1) = 110;
          numbering(579,2) = 203;
          numbering(579,3) = 28;
          numbering(580,0) = 106;
          numbering(580,1) = 110;
          numbering(580,2) = 201;
          numbering(580,3) = 107;
          numbering(581,0) = 106;
          numbering(581,1) = 202;
          numbering(581,2) = 201;
          numbering(581,3) = 110;
          numbering(582,0) = 201;
          numbering(582,1) = 202;
          numbering(582,2) = 203;
          numbering(582,3) = 110;
          numbering(583,0) = 201;
          numbering(583,1) = 110;
          numbering(583,2) = 203;
          numbering(583,3) = 107;
          numbering(584,0) = 27;
          numbering(584,1) = 118;
          numbering(584,2) = 204;
          numbering(584,3) = 119;
          numbering(585,0) = 118;
          numbering(585,1) = 2;
          numbering(585,2) = 205;
          numbering(585,3) = 122;
          numbering(586,0) = 204;
          numbering(586,1) = 205;
          numbering(586,2) = 43;
          numbering(586,3) = 206;
          numbering(587,0) = 119;
          numbering(587,1) = 122;
          numbering(587,2) = 206;
          numbering(587,3) = 31;
          numbering(588,0) = 118;
          numbering(588,1) = 122;
          numbering(588,2) = 204;
          numbering(588,3) = 119;
          numbering(589,0) = 118;
          numbering(589,1) = 205;
          numbering(589,2) = 204;
          numbering(589,3) = 122;
          numbering(590,0) = 204;
          numbering(590,1) = 205;
          numbering(590,2) = 206;
          numbering(590,3) = 122;
          numbering(591,0) = 204;
          numbering(591,1) = 122;
          numbering(591,2) = 206;
          numbering(591,3) = 119;
          numbering(592,0) = 42;
          numbering(592,1) = 207;
          numbering(592,2) = 208;
          numbering(592,3) = 209;
          numbering(593,0) = 207;
          numbering(593,1) = 43;
          numbering(593,2) = 210;
          numbering(593,3) = 211;
          numbering(594,0) = 208;
          numbering(594,1) = 210;
          numbering(594,2) = 12;
          numbering(594,3) = 212;
          numbering(595,0) = 209;
          numbering(595,1) = 211;
          numbering(595,2) = 212;
          numbering(595,3) = 44;
          numbering(596,0) = 207;
          numbering(596,1) = 211;
          numbering(596,2) = 208;
          numbering(596,3) = 209;
          numbering(597,0) = 207;
          numbering(597,1) = 210;
          numbering(597,2) = 208;
          numbering(597,3) = 211;
          numbering(598,0) = 208;
          numbering(598,1) = 210;
          numbering(598,2) = 212;
          numbering(598,3) = 211;
          numbering(599,0) = 208;
          numbering(599,1) = 211;
          numbering(599,2) = 212;
          numbering(599,3) = 209;
          numbering(600,0) = 28;
          numbering(600,1) = 124;
          numbering(600,2) = 213;
          numbering(600,3) = 125;
          numbering(601,0) = 124;
          numbering(601,1) = 31;
          numbering(601,2) = 214;
          numbering(601,3) = 128;
          numbering(602,0) = 213;
          numbering(602,1) = 214;
          numbering(602,2) = 44;
          numbering(602,3) = 215;
          numbering(603,0) = 125;
          numbering(603,1) = 128;
          numbering(603,2) = 215;
          numbering(603,3) = 10;
          numbering(604,0) = 124;
          numbering(604,1) = 128;
          numbering(604,2) = 213;
          numbering(604,3) = 125;
          numbering(605,0) = 124;
          numbering(605,1) = 214;
          numbering(605,2) = 213;
          numbering(605,3) = 128;
          numbering(606,0) = 213;
          numbering(606,1) = 214;
          numbering(606,2) = 215;
          numbering(606,3) = 128;
          numbering(607,0) = 213;
          numbering(607,1) = 128;
          numbering(607,2) = 215;
          numbering(607,3) = 125;
          numbering(608,0) = 27;
          numbering(608,1) = 119;
          numbering(608,2) = 202;
          numbering(608,3) = 110;
          numbering(609,0) = 119;
          numbering(609,1) = 31;
          numbering(609,2) = 216;
          numbering(609,3) = 124;
          numbering(610,0) = 202;
          numbering(610,1) = 216;
          numbering(610,2) = 42;
          numbering(610,3) = 203;
          numbering(611,0) = 110;
          numbering(611,1) = 124;
          numbering(611,2) = 203;
          numbering(611,3) = 28;
          numbering(612,0) = 110;
          numbering(612,1) = 216;
          numbering(612,2) = 202;
          numbering(612,3) = 203;
          numbering(613,0) = 119;
          numbering(613,1) = 216;
          numbering(613,2) = 202;
          numbering(613,3) = 110;
          numbering(614,0) = 119;
          numbering(614,1) = 124;
          numbering(614,2) = 216;
          numbering(614,3) = 110;
          numbering(615,0) = 110;
          numbering(615,1) = 124;
          numbering(615,2) = 216;
          numbering(615,3) = 203;
          numbering(616,0) = 27;
          numbering(616,1) = 204;
          numbering(616,2) = 202;
          numbering(616,3) = 119;
          numbering(617,0) = 204;
          numbering(617,1) = 43;
          numbering(617,2) = 207;
          numbering(617,3) = 206;
          numbering(618,0) = 202;
          numbering(618,1) = 207;
          numbering(618,2) = 42;
          numbering(618,3) = 216;
          numbering(619,0) = 119;
          numbering(619,1) = 206;
          numbering(619,2) = 216;
          numbering(619,3) = 31;
          numbering(620,0) = 204;
          numbering(620,1) = 206;
          numbering(620,2) = 202;
          numbering(620,3) = 119;
          numbering(621,0) = 204;
          numbering(621,1) = 207;
          numbering(621,2) = 202;
          numbering(621,3) = 206;
          numbering(622,0) = 202;
          numbering(622,1) = 207;
          numbering(622,2) = 216;
          numbering(622,3) = 206;
          numbering(623,0) = 202;
          numbering(623,1) = 206;
          numbering(623,2) = 216;
          numbering(623,3) = 119;
          numbering(624,0) = 42;
          numbering(624,1) = 207;
          numbering(624,2) = 209;
          numbering(624,3) = 216;
          numbering(625,0) = 207;
          numbering(625,1) = 43;
          numbering(625,2) = 211;
          numbering(625,3) = 206;
          numbering(626,0) = 209;
          numbering(626,1) = 211;
          numbering(626,2) = 44;
          numbering(626,3) = 214;
          numbering(627,0) = 216;
          numbering(627,1) = 206;
          numbering(627,2) = 214;
          numbering(627,3) = 31;
          numbering(628,0) = 216;
          numbering(628,1) = 211;
          numbering(628,2) = 209;
          numbering(628,3) = 214;
          numbering(629,0) = 207;
          numbering(629,1) = 211;
          numbering(629,2) = 209;
          numbering(629,3) = 216;
          numbering(630,0) = 207;
          numbering(630,1) = 206;
          numbering(630,2) = 211;
          numbering(630,3) = 216;
          numbering(631,0) = 216;
          numbering(631,1) = 206;
          numbering(631,2) = 211;
          numbering(631,3) = 214;
          numbering(632,0) = 42;
          numbering(632,1) = 216;
          numbering(632,2) = 209;
          numbering(632,3) = 203;
          numbering(633,0) = 216;
          numbering(633,1) = 31;
          numbering(633,2) = 214;
          numbering(633,3) = 124;
          numbering(634,0) = 209;
          numbering(634,1) = 214;
          numbering(634,2) = 44;
          numbering(634,3) = 213;
          numbering(635,0) = 203;
          numbering(635,1) = 124;
          numbering(635,2) = 213;
          numbering(635,3) = 28;
          numbering(636,0) = 216;
          numbering(636,1) = 124;
          numbering(636,2) = 209;
          numbering(636,3) = 203;
          numbering(637,0) = 216;
          numbering(637,1) = 214;
          numbering(637,2) = 209;
          numbering(637,3) = 124;
          numbering(638,0) = 209;
          numbering(638,1) = 214;
          numbering(638,2) = 213;
          numbering(638,3) = 124;
          numbering(639,0) = 209;
          numbering(639,1) = 124;
          numbering(639,2) = 213;
          numbering(639,3) = 203;
          numbering(640,0) = 11;
          numbering(640,1) = 217;
          numbering(640,2) = 218;
          numbering(640,3) = 219;
          numbering(641,0) = 217;
          numbering(641,1) = 45;
          numbering(641,2) = 220;
          numbering(641,3) = 221;
          numbering(642,0) = 218;
          numbering(642,1) = 220;
          numbering(642,2) = 46;
          numbering(642,3) = 222;
          numbering(643,0) = 219;
          numbering(643,1) = 221;
          numbering(643,2) = 222;
          numbering(643,3) = 47;
          numbering(644,0) = 217;
          numbering(644,1) = 221;
          numbering(644,2) = 218;
          numbering(644,3) = 219;
          numbering(645,0) = 217;
          numbering(645,1) = 220;
          numbering(645,2) = 218;
          numbering(645,3) = 221;
          numbering(646,0) = 218;
          numbering(646,1) = 220;
          numbering(646,2) = 222;
          numbering(646,3) = 221;
          numbering(647,0) = 218;
          numbering(647,1) = 221;
          numbering(647,2) = 222;
          numbering(647,3) = 219;
          numbering(648,0) = 45;
          numbering(648,1) = 223;
          numbering(648,2) = 224;
          numbering(648,3) = 225;
          numbering(649,0) = 223;
          numbering(649,1) = 12;
          numbering(649,2) = 226;
          numbering(649,3) = 227;
          numbering(650,0) = 224;
          numbering(650,1) = 226;
          numbering(650,2) = 48;
          numbering(650,3) = 228;
          numbering(651,0) = 225;
          numbering(651,1) = 227;
          numbering(651,2) = 228;
          numbering(651,3) = 49;
          numbering(652,0) = 223;
          numbering(652,1) = 227;
          numbering(652,2) = 224;
          numbering(652,3) = 225;
          numbering(653,0) = 223;
          numbering(653,1) = 226;
          numbering(653,2) = 224;
          numbering(653,3) = 227;
          numbering(654,0) = 224;
          numbering(654,1) = 226;
          numbering(654,2) = 228;
          numbering(654,3) = 227;
          numbering(655,0) = 224;
          numbering(655,1) = 227;
          numbering(655,2) = 228;
          numbering(655,3) = 225;
          numbering(656,0) = 46;
          numbering(656,1) = 229;
          numbering(656,2) = 230;
          numbering(656,3) = 231;
          numbering(657,0) = 229;
          numbering(657,1) = 48;
          numbering(657,2) = 232;
          numbering(657,3) = 233;
          numbering(658,0) = 230;
          numbering(658,1) = 232;
          numbering(658,2) = 4;
          numbering(658,3) = 234;
          numbering(659,0) = 231;
          numbering(659,1) = 233;
          numbering(659,2) = 234;
          numbering(659,3) = 50;
          numbering(660,0) = 229;
          numbering(660,1) = 233;
          numbering(660,2) = 230;
          numbering(660,3) = 231;
          numbering(661,0) = 229;
          numbering(661,1) = 232;
          numbering(661,2) = 230;
          numbering(661,3) = 233;
          numbering(662,0) = 230;
          numbering(662,1) = 232;
          numbering(662,2) = 234;
          numbering(662,3) = 233;
          numbering(663,0) = 230;
          numbering(663,1) = 233;
          numbering(663,2) = 234;
          numbering(663,3) = 231;
          numbering(664,0) = 47;
          numbering(664,1) = 235;
          numbering(664,2) = 236;
          numbering(664,3) = 237;
          numbering(665,0) = 235;
          numbering(665,1) = 49;
          numbering(665,2) = 238;
          numbering(665,3) = 239;
          numbering(666,0) = 236;
          numbering(666,1) = 238;
          numbering(666,2) = 50;
          numbering(666,3) = 240;
          numbering(667,0) = 237;
          numbering(667,1) = 239;
          numbering(667,2) = 240;
          numbering(667,3) = 13;
          numbering(668,0) = 235;
          numbering(668,1) = 239;
          numbering(668,2) = 236;
          numbering(668,3) = 237;
          numbering(669,0) = 235;
          numbering(669,1) = 238;
          numbering(669,2) = 236;
          numbering(669,3) = 239;
          numbering(670,0) = 236;
          numbering(670,1) = 238;
          numbering(670,2) = 240;
          numbering(670,3) = 239;
          numbering(671,0) = 236;
          numbering(671,1) = 239;
          numbering(671,2) = 240;
          numbering(671,3) = 237;
          numbering(672,0) = 45;
          numbering(672,1) = 225;
          numbering(672,2) = 220;
          numbering(672,3) = 221;
          numbering(673,0) = 225;
          numbering(673,1) = 49;
          numbering(673,2) = 241;
          numbering(673,3) = 235;
          numbering(674,0) = 220;
          numbering(674,1) = 241;
          numbering(674,2) = 46;
          numbering(674,3) = 222;
          numbering(675,0) = 221;
          numbering(675,1) = 235;
          numbering(675,2) = 222;
          numbering(675,3) = 47;
          numbering(676,0) = 221;
          numbering(676,1) = 241;
          numbering(676,2) = 220;
          numbering(676,3) = 222;
          numbering(677,0) = 225;
          numbering(677,1) = 241;
          numbering(677,2) = 220;
          numbering(677,3) = 221;
          numbering(678,0) = 225;
          numbering(678,1) = 235;
          numbering(678,2) = 241;
          numbering(678,3) = 221;
          numbering(679,0) = 221;
          numbering(679,1) = 235;
          numbering(679,2) = 241;
          numbering(679,3) = 222;
          numbering(680,0) = 45;
          numbering(680,1) = 224;
          numbering(680,2) = 220;
          numbering(680,3) = 225;
          numbering(681,0) = 224;
          numbering(681,1) = 48;
          numbering(681,2) = 229;
          numbering(681,3) = 228;
          numbering(682,0) = 220;
          numbering(682,1) = 229;
          numbering(682,2) = 46;
          numbering(682,3) = 241;
          numbering(683,0) = 225;
          numbering(683,1) = 228;
          numbering(683,2) = 241;
          numbering(683,3) = 49;
          numbering(684,0) = 224;
          numbering(684,1) = 228;
          numbering(684,2) = 220;
          numbering(684,3) = 225;
          numbering(685,0) = 224;
          numbering(685,1) = 229;
          numbering(685,2) = 220;
          numbering(685,3) = 228;
          numbering(686,0) = 220;
          numbering(686,1) = 229;
          numbering(686,2) = 241;
          numbering(686,3) = 228;
          numbering(687,0) = 220;
          numbering(687,1) = 228;
          numbering(687,2) = 241;
          numbering(687,3) = 225;
          numbering(688,0) = 46;
          numbering(688,1) = 229;
          numbering(688,2) = 231;
          numbering(688,3) = 241;
          numbering(689,0) = 229;
          numbering(689,1) = 48;
          numbering(689,2) = 233;
          numbering(689,3) = 228;
          numbering(690,0) = 231;
          numbering(690,1) = 233;
          numbering(690,2) = 50;
          numbering(690,3) = 238;
          numbering(691,0) = 241;
          numbering(691,1) = 228;
          numbering(691,2) = 238;
          numbering(691,3) = 49;
          numbering(692,0) = 241;
          numbering(692,1) = 233;
          numbering(692,2) = 231;
          numbering(692,3) = 238;
          numbering(693,0) = 229;
          numbering(693,1) = 233;
          numbering(693,2) = 231;
          numbering(693,3) = 241;
          numbering(694,0) = 229;
          numbering(694,1) = 228;
          numbering(694,2) = 233;
          numbering(694,3) = 241;
          numbering(695,0) = 241;
          numbering(695,1) = 228;
          numbering(695,2) = 233;
          numbering(695,3) = 238;
          numbering(696,0) = 46;
          numbering(696,1) = 241;
          numbering(696,2) = 231;
          numbering(696,3) = 222;
          numbering(697,0) = 241;
          numbering(697,1) = 49;
          numbering(697,2) = 238;
          numbering(697,3) = 235;
          numbering(698,0) = 231;
          numbering(698,1) = 238;
          numbering(698,2) = 50;
          numbering(698,3) = 236;
          numbering(699,0) = 222;
          numbering(699,1) = 235;
          numbering(699,2) = 236;
          numbering(699,3) = 47;
          numbering(700,0) = 241;
          numbering(700,1) = 235;
          numbering(700,2) = 231;
          numbering(700,3) = 222;
          numbering(701,0) = 241;
          numbering(701,1) = 238;
          numbering(701,2) = 231;
          numbering(701,3) = 235;
          numbering(702,0) = 231;
          numbering(702,1) = 238;
          numbering(702,2) = 236;
          numbering(702,3) = 235;
          numbering(703,0) = 231;
          numbering(703,1) = 235;
          numbering(703,2) = 236;
          numbering(703,3) = 222;
          numbering(704,0) = 7;
          numbering(704,1) = 131;
          numbering(704,2) = 242;
          numbering(704,3) = 132;
          numbering(705,0) = 131;
          numbering(705,1) = 33;
          numbering(705,2) = 243;
          numbering(705,3) = 135;
          numbering(706,0) = 242;
          numbering(706,1) = 243;
          numbering(706,2) = 51;
          numbering(706,3) = 244;
          numbering(707,0) = 132;
          numbering(707,1) = 135;
          numbering(707,2) = 244;
          numbering(707,3) = 34;
          numbering(708,0) = 131;
          numbering(708,1) = 135;
          numbering(708,2) = 242;
          numbering(708,3) = 132;
          numbering(709,0) = 131;
          numbering(709,1) = 243;
          numbering(709,2) = 242;
          numbering(709,3) = 135;
          numbering(710,0) = 242;
          numbering(710,1) = 243;
          numbering(710,2) = 244;
          numbering(710,3) = 135;
          numbering(711,0) = 242;
          numbering(711,1) = 135;
          numbering(711,2) = 244;
          numbering(711,3) = 132;
          numbering(712,0) = 33;
          numbering(712,1) = 143;
          numbering(712,2) = 245;
          numbering(712,3) = 144;
          numbering(713,0) = 143;
          numbering(713,1) = 10;
          numbering(713,2) = 246;
          numbering(713,3) = 147;
          numbering(714,0) = 245;
          numbering(714,1) = 246;
          numbering(714,2) = 52;
          numbering(714,3) = 247;
          numbering(715,0) = 144;
          numbering(715,1) = 147;
          numbering(715,2) = 247;
          numbering(715,3) = 37;
          numbering(716,0) = 143;
          numbering(716,1) = 147;
          numbering(716,2) = 245;
          numbering(716,3) = 144;
          numbering(717,0) = 143;
          numbering(717,1) = 246;
          numbering(717,2) = 245;
          numbering(717,3) = 147;
          numbering(718,0) = 245;
          numbering(718,1) = 246;
          numbering(718,2) = 247;
          numbering(718,3) = 147;
          numbering(719,0) = 245;
          numbering(719,1) = 147;
          numbering(719,2) = 247;
          numbering(719,3) = 144;
          numbering(720,0) = 51;
          numbering(720,1) = 248;
          numbering(720,2) = 249;
          numbering(720,3) = 250;
          numbering(721,0) = 248;
          numbering(721,1) = 52;
          numbering(721,2) = 251;
          numbering(721,3) = 252;
          numbering(722,0) = 249;
          numbering(722,1) = 251;
          numbering(722,2) = 13;
          numbering(722,3) = 253;
          numbering(723,0) = 250;
          numbering(723,1) = 252;
          numbering(723,2) = 253;
          numbering(723,3) = 53;
          numbering(724,0) = 248;
          numbering(724,1) = 252;
          numbering(724,2) = 249;
          numbering(724,3) = 250;
          numbering(725,0) = 248;
          numbering(725,1) = 251;
          numbering(725,2) = 249;
          numbering(725,3) = 252;
          numbering(726,0) = 249;
          numbering(726,1) = 251;
          numbering(726,2) = 253;
          numbering(726,3) = 252;
          numbering(727,0) = 249;
          numbering(727,1) = 252;
          numbering(727,2) = 253;
          numbering(727,3) = 250;
          numbering(728,0) = 34;
          numbering(728,1) = 149;
          numbering(728,2) = 254;
          numbering(728,3) = 150;
          numbering(729,0) = 149;
          numbering(729,1) = 37;
          numbering(729,2) = 255;
          numbering(729,3) = 153;
          numbering(730,0) = 254;
          numbering(730,1) = 255;
          numbering(730,2) = 53;
          numbering(730,3) = 256;
          numbering(731,0) = 150;
          numbering(731,1) = 153;
          numbering(731,2) = 256;
          numbering(731,3) = 3;
          numbering(732,0) = 149;
          numbering(732,1) = 153;
          numbering(732,2) = 254;
          numbering(732,3) = 150;
          numbering(733,0) = 149;
          numbering(733,1) = 255;
          numbering(733,2) = 254;
          numbering(733,3) = 153;
          numbering(734,0) = 254;
          numbering(734,1) = 255;
          numbering(734,2) = 256;
          numbering(734,3) = 153;
          numbering(735,0) = 254;
          numbering(735,1) = 153;
          numbering(735,2) = 256;
          numbering(735,3) = 150;
          numbering(736,0) = 33;
          numbering(736,1) = 144;
          numbering(736,2) = 243;
          numbering(736,3) = 135;
          numbering(737,0) = 144;
          numbering(737,1) = 37;
          numbering(737,2) = 257;
          numbering(737,3) = 149;
          numbering(738,0) = 243;
          numbering(738,1) = 257;
          numbering(738,2) = 51;
          numbering(738,3) = 244;
          numbering(739,0) = 135;
          numbering(739,1) = 149;
          numbering(739,2) = 244;
          numbering(739,3) = 34;
          numbering(740,0) = 135;
          numbering(740,1) = 257;
          numbering(740,2) = 243;
          numbering(740,3) = 244;
          numbering(741,0) = 144;
          numbering(741,1) = 257;
          numbering(741,2) = 243;
          numbering(741,3) = 135;
          numbering(742,0) = 144;
          numbering(742,1) = 149;
          numbering(742,2) = 257;
          numbering(742,3) = 135;
          numbering(743,0) = 135;
          numbering(743,1) = 149;
          numbering(743,2) = 257;
          numbering(743,3) = 244;
          numbering(744,0) = 33;
          numbering(744,1) = 245;
          numbering(744,2) = 243;
          numbering(744,3) = 144;
          numbering(745,0) = 245;
          numbering(745,1) = 52;
          numbering(745,2) = 248;
          numbering(745,3) = 247;
          numbering(746,0) = 243;
          numbering(746,1) = 248;
          numbering(746,2) = 51;
          numbering(746,3) = 257;
          numbering(747,0) = 144;
          numbering(747,1) = 247;
          numbering(747,2) = 257;
          numbering(747,3) = 37;
          numbering(748,0) = 245;
          numbering(748,1) = 247;
          numbering(748,2) = 243;
          numbering(748,3) = 144;
          numbering(749,0) = 245;
          numbering(749,1) = 248;
          numbering(749,2) = 243;
          numbering(749,3) = 247;
          numbering(750,0) = 243;
          numbering(750,1) = 248;
          numbering(750,2) = 257;
          numbering(750,3) = 247;
          numbering(751,0) = 243;
          numbering(751,1) = 247;
          numbering(751,2) = 257;
          numbering(751,3) = 144;
          numbering(752,0) = 51;
          numbering(752,1) = 248;
          numbering(752,2) = 250;
          numbering(752,3) = 257;
          numbering(753,0) = 248;
          numbering(753,1) = 52;
          numbering(753,2) = 252;
          numbering(753,3) = 247;
          numbering(754,0) = 250;
          numbering(754,1) = 252;
          numbering(754,2) = 53;
          numbering(754,3) = 255;
          numbering(755,0) = 257;
          numbering(755,1) = 247;
          numbering(755,2) = 255;
          numbering(755,3) = 37;
          numbering(756,0) = 257;
          numbering(756,1) = 252;
          numbering(756,2) = 250;
          numbering(756,3) = 255;
          numbering(757,0) = 248;
          numbering(757,1) = 252;
          numbering(757,2) = 250;
          numbering(757,3) = 257;
          numbering(758,0) = 248;
          numbering(758,1) = 247;
          numbering(758,2) = 252;
          numbering(758,3) = 257;
          numbering(759,0) = 257;
          numbering(759,1) = 247;
          numbering(759,2) = 252;
          numbering(759,3) = 255;
          numbering(760,0) = 51;
          numbering(760,1) = 257;
          numbering(760,2) = 250;
          numbering(760,3) = 244;
          numbering(761,0) = 257;
          numbering(761,1) = 37;
          numbering(761,2) = 255;
          numbering(761,3) = 149;
          numbering(762,0) = 250;
          numbering(762,1) = 255;
          numbering(762,2) = 53;
          numbering(762,3) = 254;
          numbering(763,0) = 244;
          numbering(763,1) = 149;
          numbering(763,2) = 254;
          numbering(763,3) = 34;
          numbering(764,0) = 257;
          numbering(764,1) = 149;
          numbering(764,2) = 250;
          numbering(764,3) = 244;
          numbering(765,0) = 257;
          numbering(765,1) = 255;
          numbering(765,2) = 250;
          numbering(765,3) = 149;
          numbering(766,0) = 250;
          numbering(766,1) = 255;
          numbering(766,2) = 254;
          numbering(766,3) = 149;
          numbering(767,0) = 250;
          numbering(767,1) = 149;
          numbering(767,2) = 254;
          numbering(767,3) = 244;
          numbering(768,0) = 6;
          numbering(768,1) = 107;
          numbering(768,2) = 189;
          numbering(768,3) = 72;
          numbering(769,0) = 107;
          numbering(769,1) = 28;
          numbering(769,2) = 258;
          numbering(769,3) = 181;
          numbering(770,0) = 189;
          numbering(770,1) = 258;
          numbering(770,2) = 40;
          numbering(770,3) = 190;
          numbering(771,0) = 72;
          numbering(771,1) = 181;
          numbering(771,2) = 190;
          numbering(771,3) = 19;
          numbering(772,0) = 72;
          numbering(772,1) = 258;
          numbering(772,2) = 189;
          numbering(772,3) = 190;
          numbering(773,0) = 107;
          numbering(773,1) = 258;
          numbering(773,2) = 189;
          numbering(773,3) = 72;
          numbering(774,0) = 107;
          numbering(774,1) = 181;
          numbering(774,2) = 258;
          numbering(774,3) = 72;
          numbering(775,0) = 72;
          numbering(775,1) = 181;
          numbering(775,2) = 258;
          numbering(775,3) = 190;
          numbering(776,0) = 28;
          numbering(776,1) = 125;
          numbering(776,2) = 259;
          numbering(776,3) = 182;
          numbering(777,0) = 125;
          numbering(777,1) = 10;
          numbering(777,2) = 260;
          numbering(777,3) = 143;
          numbering(778,0) = 259;
          numbering(778,1) = 260;
          numbering(778,2) = 54;
          numbering(778,3) = 261;
          numbering(779,0) = 182;
          numbering(779,1) = 143;
          numbering(779,2) = 261;
          numbering(779,3) = 33;
          numbering(780,0) = 182;
          numbering(780,1) = 260;
          numbering(780,2) = 259;
          numbering(780,3) = 261;
          numbering(781,0) = 125;
          numbering(781,1) = 260;
          numbering(781,2) = 259;
          numbering(781,3) = 182;
          numbering(782,0) = 125;
          numbering(782,1) = 143;
          numbering(782,2) = 260;
          numbering(782,3) = 182;
          numbering(783,0) = 182;
          numbering(783,1) = 143;
          numbering(783,2) = 260;
          numbering(783,3) = 261;
          numbering(784,0) = 40;
          numbering(784,1) = 262;
          numbering(784,2) = 194;
          numbering(784,3) = 195;
          numbering(785,0) = 262;
          numbering(785,1) = 54;
          numbering(785,2) = 263;
          numbering(785,3) = 264;
          numbering(786,0) = 194;
          numbering(786,1) = 263;
          numbering(786,2) = 11;
          numbering(786,3) = 196;
          numbering(787,0) = 195;
          numbering(787,1) = 264;
          numbering(787,2) = 196;
          numbering(787,3) = 41;
          numbering(788,0) = 195;
          numbering(788,1) = 263;
          numbering(788,2) = 194;
          numbering(788,3) = 196;
          numbering(789,0) = 262;
          numbering(789,1) = 263;
          numbering(789,2) = 194;
          numbering(789,3) = 195;
          numbering(790,0) = 262;
          numbering(790,1) = 264;
          numbering(790,2) = 263;
          numbering(790,3) = 195;
          numbering(791,0) = 195;
          numbering(791,1) = 264;
          numbering(791,2) = 263;
          numbering(791,3) = 196;
          numbering(792,0) = 19;
          numbering(792,1) = 183;
          numbering(792,2) = 198;
          numbering(792,3) = 78;
          numbering(793,0) = 183;
          numbering(793,1) = 33;
          numbering(793,2) = 265;
          numbering(793,3) = 131;
          numbering(794,0) = 198;
          numbering(794,1) = 265;
          numbering(794,2) = 41;
          numbering(794,3) = 199;
          numbering(795,0) = 78;
          numbering(795,1) = 131;
          numbering(795,2) = 199;
          numbering(795,3) = 7;
          numbering(796,0) = 78;
          numbering(796,1) = 265;
          numbering(796,2) = 198;
          numbering(796,3) = 199;
          numbering(797,0) = 183;
          numbering(797,1) = 265;
          numbering(797,2) = 198;
          numbering(797,3) = 78;
          numbering(798,0) = 183;
          numbering(798,1) = 131;
          numbering(798,2) = 265;
          numbering(798,3) = 78;
          numbering(799,0) = 78;
          numbering(799,1) = 131;
          numbering(799,2) = 265;
          numbering(799,3) = 199;
          numbering(800,0) = 19;
          numbering(800,1) = 266;
          numbering(800,2) = 190;
          numbering(800,3) = 198;
          numbering(801,0) = 266;
          numbering(801,1) = 54;
          numbering(801,2) = 262;
          numbering(801,3) = 264;
          numbering(802,0) = 190;
          numbering(802,1) = 262;
          numbering(802,2) = 40;
          numbering(802,3) = 195;
          numbering(803,0) = 198;
          numbering(803,1) = 264;
          numbering(803,2) = 195;
          numbering(803,3) = 41;
          numbering(804,0) = 266;
          numbering(804,1) = 264;
          numbering(804,2) = 190;
          numbering(804,3) = 198;
          numbering(805,0) = 266;
          numbering(805,1) = 262;
          numbering(805,2) = 190;
          numbering(805,3) = 264;
          numbering(806,0) = 190;
          numbering(806,1) = 262;
          numbering(806,2) = 195;
          numbering(806,3) = 264;
          numbering(807,0) = 190;
          numbering(807,1) = 264;
          numbering(807,2) = 195;
          numbering(807,3) = 198;
          numbering(808,0) = 28;
          numbering(808,1) = 259;
          numbering(808,2) = 258;
          numbering(808,3) = 181;
          numbering(809,0) = 259;
          numbering(809,1) = 54;
          numbering(809,2) = 262;
          numbering(809,3) = 266;
          numbering(810,0) = 258;
          numbering(810,1) = 262;
          numbering(810,2) = 40;
          numbering(810,3) = 190;
          numbering(811,0) = 181;
          numbering(811,1) = 266;
          numbering(811,2) = 190;
          numbering(811,3) = 19;
          numbering(812,0) = 259;
          numbering(812,1) = 266;
          numbering(812,2) = 258;
          numbering(812,3) = 181;
          numbering(813,0) = 259;
          numbering(813,1) = 262;
          numbering(813,2) = 258;
          numbering(813,3) = 266;
          numbering(814,0) = 258;
          numbering(814,1) = 262;
          numbering(814,2) = 190;
          numbering(814,3) = 266;
          numbering(815,0) = 258;
          numbering(815,1) = 266;
          numbering(815,2) = 190;
          numbering(815,3) = 181;
          numbering(816,0) = 28;
          numbering(816,1) = 182;
          numbering(816,2) = 259;
          numbering(816,3) = 181;
          numbering(817,0) = 182;
          numbering(817,1) = 33;
          numbering(817,2) = 261;
          numbering(817,3) = 183;
          numbering(818,0) = 259;
          numbering(818,1) = 261;
          numbering(818,2) = 54;
          numbering(818,3) = 266;
          numbering(819,0) = 181;
          numbering(819,1) = 183;
          numbering(819,2) = 266;
          numbering(819,3) = 19;
          numbering(820,0) = 181;
          numbering(820,1) = 261;
          numbering(820,2) = 259;
          numbering(820,3) = 266;
          numbering(821,0) = 182;
          numbering(821,1) = 261;
          numbering(821,2) = 259;
          numbering(821,3) = 181;
          numbering(822,0) = 182;
          numbering(822,1) = 183;
          numbering(822,2) = 261;
          numbering(822,3) = 181;
          numbering(823,0) = 181;
          numbering(823,1) = 183;
          numbering(823,2) = 261;
          numbering(823,3) = 266;
          numbering(824,0) = 19;
          numbering(824,1) = 183;
          numbering(824,2) = 266;
          numbering(824,3) = 198;
          numbering(825,0) = 183;
          numbering(825,1) = 33;
          numbering(825,2) = 261;
          numbering(825,3) = 265;
          numbering(826,0) = 266;
          numbering(826,1) = 261;
          numbering(826,2) = 54;
          numbering(826,3) = 264;
          numbering(827,0) = 198;
          numbering(827,1) = 265;
          numbering(827,2) = 264;
          numbering(827,3) = 41;
          numbering(828,0) = 183;
          numbering(828,1) = 265;
          numbering(828,2) = 266;
          numbering(828,3) = 198;
          numbering(829,0) = 183;
          numbering(829,1) = 261;
          numbering(829,2) = 266;
          numbering(829,3) = 265;
          numbering(830,0) = 266;
          numbering(830,1) = 261;
          numbering(830,2) = 264;
          numbering(830,3) = 265;
          numbering(831,0) = 266;
          numbering(831,1) = 265;
          numbering(831,2) = 264;
          numbering(831,3) = 198;
          numbering(832,0) = 6;
          numbering(832,1) = 201;
          numbering(832,2) = 189;
          numbering(832,3) = 107;
          numbering(833,0) = 201;
          numbering(833,1) = 42;
          numbering(833,2) = 267;
          numbering(833,3) = 203;
          numbering(834,0) = 189;
          numbering(834,1) = 267;
          numbering(834,2) = 40;
          numbering(834,3) = 258;
          numbering(835,0) = 107;
          numbering(835,1) = 203;
          numbering(835,2) = 258;
          numbering(835,3) = 28;
          numbering(836,0) = 201;
          numbering(836,1) = 203;
          numbering(836,2) = 189;
          numbering(836,3) = 107;
          numbering(837,0) = 201;
          numbering(837,1) = 267;
          numbering(837,2) = 189;
          numbering(837,3) = 203;
          numbering(838,0) = 189;
          numbering(838,1) = 267;
          numbering(838,2) = 258;
          numbering(838,3) = 203;
          numbering(839,0) = 189;
          numbering(839,1) = 203;
          numbering(839,2) = 258;
          numbering(839,3) = 107;
          numbering(840,0) = 42;
          numbering(840,1) = 208;
          numbering(840,2) = 268;
          numbering(840,3) = 209;
          numbering(841,0) = 208;
          numbering(841,1) = 12;
          numbering(841,2) = 223;
          numbering(841,3) = 212;
          numbering(842,0) = 268;
          numbering(842,1) = 223;
          numbering(842,2) = 45;
          numbering(842,3) = 269;
          numbering(843,0) = 209;
          numbering(843,1) = 212;
          numbering(843,2) = 269;
          numbering(843,3) = 44;
          numbering(844,0) = 208;
          numbering(844,1) = 212;
          numbering(844,2) = 268;
          numbering(844,3) = 209;
          numbering(845,0) = 208;
          numbering(845,1) = 223;
          numbering(845,2) = 268;
          numbering(845,3) = 212;
          numbering(846,0) = 268;
          numbering(846,1) = 223;
          numbering(846,2) = 269;
          numbering(846,3) = 212;
          numbering(847,0) = 268;
          numbering(847,1) = 212;
          numbering(847,2) = 269;
          numbering(847,3) = 209;
          numbering(848,0) = 40;
          numbering(848,1) = 270;
          numbering(848,2) = 194;
          numbering(848,3) = 262;
          numbering(849,0) = 270;
          numbering(849,1) = 45;
          numbering(849,2) = 217;
          numbering(849,3) = 271;
          numbering(850,0) = 194;
          numbering(850,1) = 217;
          numbering(850,2) = 11;
          numbering(850,3) = 263;
          numbering(851,0) = 262;
          numbering(851,1) = 271;
          numbering(851,2) = 263;
          numbering(851,3) = 54;
          numbering(852,0) = 270;
          numbering(852,1) = 271;
          numbering(852,2) = 194;
          numbering(852,3) = 262;
          numbering(853,0) = 270;
          numbering(853,1) = 217;
          numbering(853,2) = 194;
          numbering(853,3) = 271;
          numbering(854,0) = 194;
          numbering(854,1) = 217;
          numbering(854,2) = 263;
          numbering(854,3) = 271;
          numbering(855,0) = 194;
          numbering(855,1) = 271;
          numbering(855,2) = 263;
          numbering(855,3) = 262;
          numbering(856,0) = 28;
          numbering(856,1) = 213;
          numbering(856,2) = 259;
          numbering(856,3) = 125;
          numbering(857,0) = 213;
          numbering(857,1) = 44;
          numbering(857,2) = 272;
          numbering(857,3) = 215;
          numbering(858,0) = 259;
          numbering(858,1) = 272;
          numbering(858,2) = 54;
          numbering(858,3) = 260;
          numbering(859,0) = 125;
          numbering(859,1) = 215;
          numbering(859,2) = 260;
          numbering(859,3) = 10;
          numbering(860,0) = 213;
          numbering(860,1) = 215;
          numbering(860,2) = 259;
          numbering(860,3) = 125;
          numbering(861,0) = 213;
          numbering(861,1) = 272;
          numbering(861,2) = 259;
          numbering(861,3) = 215;
          numbering(862,0) = 259;
          numbering(862,1) = 272;
          numbering(862,2) = 260;
          numbering(862,3) = 215;
          numbering(863,0) = 259;
          numbering(863,1) = 215;
          numbering(863,2) = 260;
          numbering(863,3) = 125;
          numbering(864,0) = 42;
          numbering(864,1) = 209;
          numbering(864,2) = 267;
          numbering(864,3) = 203;
          numbering(865,0) = 209;
          numbering(865,1) = 44;
          numbering(865,2) = 273;
          numbering(865,3) = 213;
          numbering(866,0) = 267;
          numbering(866,1) = 273;
          numbering(866,2) = 40;
          numbering(866,3) = 258;
          numbering(867,0) = 203;
          numbering(867,1) = 213;
          numbering(867,2) = 258;
          numbering(867,3) = 28;
          numbering(868,0) = 203;
          numbering(868,1) = 273;
          numbering(868,2) = 267;
          numbering(868,3) = 258;
          numbering(869,0) = 209;
          numbering(869,1) = 273;
          numbering(869,2) = 267;
          numbering(869,3) = 203;
          numbering(870,0) = 209;
          numbering(870,1) = 213;
          numbering(870,2) = 273;
          numbering(870,3) = 203;
          numbering(871,0) = 203;
          numbering(871,1) = 213;
          numbering(871,2) = 273;
          numbering(871,3) = 258;
          numbering(872,0) = 42;
          numbering(872,1) = 268;
          numbering(872,2) = 267;
          numbering(872,3) = 209;
          numbering(873,0) = 268;
          numbering(873,1) = 45;
          numbering(873,2) = 270;
          numbering(873,3) = 269;
          numbering(874,0) = 267;
          numbering(874,1) = 270;
          numbering(874,2) = 40;
          numbering(874,3) = 273;
          numbering(875,0) = 209;
          numbering(875,1) = 269;
          numbering(875,2) = 273;
          numbering(875,3) = 44;
          numbering(876,0) = 268;
          numbering(876,1) = 269;
          numbering(876,2) = 267;
          numbering(876,3) = 209;
          numbering(877,0) = 268;
          numbering(877,1) = 270;
          numbering(877,2) = 267;
          numbering(877,3) = 269;
          numbering(878,0) = 267;
          numbering(878,1) = 270;
          numbering(878,2) = 273;
          numbering(878,3) = 269;
          numbering(879,0) = 267;
          numbering(879,1) = 269;
          numbering(879,2) = 273;
          numbering(879,3) = 209;
          numbering(880,0) = 40;
          numbering(880,1) = 270;
          numbering(880,2) = 262;
          numbering(880,3) = 273;
          numbering(881,0) = 270;
          numbering(881,1) = 45;
          numbering(881,2) = 271;
          numbering(881,3) = 269;
          numbering(882,0) = 262;
          numbering(882,1) = 271;
          numbering(882,2) = 54;
          numbering(882,3) = 272;
          numbering(883,0) = 273;
          numbering(883,1) = 269;
          numbering(883,2) = 272;
          numbering(883,3) = 44;
          numbering(884,0) = 273;
          numbering(884,1) = 271;
          numbering(884,2) = 262;
          numbering(884,3) = 272;
          numbering(885,0) = 270;
          numbering(885,1) = 271;
          numbering(885,2) = 262;
          numbering(885,3) = 273;
          numbering(886,0) = 270;
          numbering(886,1) = 269;
          numbering(886,2) = 271;
          numbering(886,3) = 273;
          numbering(887,0) = 273;
          numbering(887,1) = 269;
          numbering(887,2) = 271;
          numbering(887,3) = 272;
          numbering(888,0) = 40;
          numbering(888,1) = 273;
          numbering(888,2) = 262;
          numbering(888,3) = 258;
          numbering(889,0) = 273;
          numbering(889,1) = 44;
          numbering(889,2) = 272;
          numbering(889,3) = 213;
          numbering(890,0) = 262;
          numbering(890,1) = 272;
          numbering(890,2) = 54;
          numbering(890,3) = 259;
          numbering(891,0) = 258;
          numbering(891,1) = 213;
          numbering(891,2) = 259;
          numbering(891,3) = 28;
          numbering(892,0) = 258;
          numbering(892,1) = 272;
          numbering(892,2) = 262;
          numbering(892,3) = 259;
          numbering(893,0) = 273;
          numbering(893,1) = 272;
          numbering(893,2) = 262;
          numbering(893,3) = 258;
          numbering(894,0) = 273;
          numbering(894,1) = 213;
          numbering(894,2) = 272;
          numbering(894,3) = 258;
          numbering(895,0) = 258;
          numbering(895,1) = 213;
          numbering(895,2) = 272;
          numbering(895,3) = 259;
          numbering(896,0) = 11;
          numbering(896,1) = 217;
          numbering(896,2) = 219;
          numbering(896,3) = 263;
          numbering(897,0) = 217;
          numbering(897,1) = 45;
          numbering(897,2) = 221;
          numbering(897,3) = 271;
          numbering(898,0) = 219;
          numbering(898,1) = 221;
          numbering(898,2) = 47;
          numbering(898,3) = 274;
          numbering(899,0) = 263;
          numbering(899,1) = 271;
          numbering(899,2) = 274;
          numbering(899,3) = 54;
          numbering(900,0) = 263;
          numbering(900,1) = 221;
          numbering(900,2) = 219;
          numbering(900,3) = 274;
          numbering(901,0) = 217;
          numbering(901,1) = 221;
          numbering(901,2) = 219;
          numbering(901,3) = 263;
          numbering(902,0) = 217;
          numbering(902,1) = 271;
          numbering(902,2) = 221;
          numbering(902,3) = 263;
          numbering(903,0) = 263;
          numbering(903,1) = 271;
          numbering(903,2) = 221;
          numbering(903,3) = 274;
          numbering(904,0) = 45;
          numbering(904,1) = 223;
          numbering(904,2) = 225;
          numbering(904,3) = 269;
          numbering(905,0) = 223;
          numbering(905,1) = 12;
          numbering(905,2) = 227;
          numbering(905,3) = 212;
          numbering(906,0) = 225;
          numbering(906,1) = 227;
          numbering(906,2) = 49;
          numbering(906,3) = 275;
          numbering(907,0) = 269;
          numbering(907,1) = 212;
          numbering(907,2) = 275;
          numbering(907,3) = 44;
          numbering(908,0) = 269;
          numbering(908,1) = 227;
          numbering(908,2) = 225;
          numbering(908,3) = 275;
          numbering(909,0) = 223;
          numbering(909,1) = 227;
          numbering(909,2) = 225;
          numbering(909,3) = 269;
          numbering(910,0) = 223;
          numbering(910,1) = 212;
          numbering(910,2) = 227;
          numbering(910,3) = 269;
          numbering(911,0) = 269;
          numbering(911,1) = 212;
          numbering(911,2) = 227;
          numbering(911,3) = 275;
          numbering(912,0) = 47;
          numbering(912,1) = 235;
          numbering(912,2) = 237;
          numbering(912,3) = 276;
          numbering(913,0) = 235;
          numbering(913,1) = 49;
          numbering(913,2) = 239;
          numbering(913,3) = 277;
          numbering(914,0) = 237;
          numbering(914,1) = 239;
          numbering(914,2) = 13;
          numbering(914,3) = 251;
          numbering(915,0) = 276;
          numbering(915,1) = 277;
          numbering(915,2) = 251;
          numbering(915,3) = 52;
          numbering(916,0) = 276;
          numbering(916,1) = 239;
          numbering(916,2) = 237;
          numbering(916,3) = 251;
          numbering(917,0) = 235;
          numbering(917,1) = 239;
          numbering(917,2) = 237;
          numbering(917,3) = 276;
          numbering(918,0) = 235;
          numbering(918,1) = 277;
          numbering(918,2) = 239;
          numbering(918,3) = 276;
          numbering(919,0) = 276;
          numbering(919,1) = 277;
          numbering(919,2) = 239;
          numbering(919,3) = 251;
          numbering(920,0) = 54;
          numbering(920,1) = 272;
          numbering(920,2) = 278;
          numbering(920,3) = 260;
          numbering(921,0) = 272;
          numbering(921,1) = 44;
          numbering(921,2) = 279;
          numbering(921,3) = 215;
          numbering(922,0) = 278;
          numbering(922,1) = 279;
          numbering(922,2) = 52;
          numbering(922,3) = 246;
          numbering(923,0) = 260;
          numbering(923,1) = 215;
          numbering(923,2) = 246;
          numbering(923,3) = 10;
          numbering(924,0) = 260;
          numbering(924,1) = 279;
          numbering(924,2) = 278;
          numbering(924,3) = 246;
          numbering(925,0) = 272;
          numbering(925,1) = 279;
          numbering(925,2) = 278;
          numbering(925,3) = 260;
          numbering(926,0) = 272;
          numbering(926,1) = 215;
          numbering(926,2) = 279;
          numbering(926,3) = 260;
          numbering(927,0) = 260;
          numbering(927,1) = 215;
          numbering(927,2) = 279;
          numbering(927,3) = 246;
          numbering(928,0) = 54;
          numbering(928,1) = 280;
          numbering(928,2) = 274;
          numbering(928,3) = 278;
          numbering(929,0) = 280;
          numbering(929,1) = 49;
          numbering(929,2) = 235;
          numbering(929,3) = 277;
          numbering(930,0) = 274;
          numbering(930,1) = 235;
          numbering(930,2) = 47;
          numbering(930,3) = 276;
          numbering(931,0) = 278;
          numbering(931,1) = 277;
          numbering(931,2) = 276;
          numbering(931,3) = 52;
          numbering(932,0) = 280;
          numbering(932,1) = 277;
          numbering(932,2) = 274;
          numbering(932,3) = 278;
          numbering(933,0) = 280;
          numbering(933,1) = 235;
          numbering(933,2) = 274;
          numbering(933,3) = 277;
          numbering(934,0) = 274;
          numbering(934,1) = 235;
          numbering(934,2) = 276;
          numbering(934,3) = 277;
          numbering(935,0) = 274;
          numbering(935,1) = 277;
          numbering(935,2) = 276;
          numbering(935,3) = 278;
          numbering(936,0) = 45;
          numbering(936,1) = 225;
          numbering(936,2) = 221;
          numbering(936,3) = 271;
          numbering(937,0) = 225;
          numbering(937,1) = 49;
          numbering(937,2) = 235;
          numbering(937,3) = 280;
          numbering(938,0) = 221;
          numbering(938,1) = 235;
          numbering(938,2) = 47;
          numbering(938,3) = 274;
          numbering(939,0) = 271;
          numbering(939,1) = 280;
          numbering(939,2) = 274;
          numbering(939,3) = 54;
          numbering(940,0) = 225;
          numbering(940,1) = 280;
          numbering(940,2) = 221;
          numbering(940,3) = 271;
          numbering(941,0) = 225;
          numbering(941,1) = 235;
          numbering(941,2) = 221;
          numbering(941,3) = 280;
          numbering(942,0) = 221;
          numbering(942,1) = 235;
          numbering(942,2) = 274;
          numbering(942,3) = 280;
          numbering(943,0) = 221;
          numbering(943,1) = 280;
          numbering(943,2) = 274;
          numbering(943,3) = 271;
          numbering(944,0) = 45;
          numbering(944,1) = 269;
          numbering(944,2) = 225;
          numbering(944,3) = 271;
          numbering(945,0) = 269;
          numbering(945,1) = 44;
          numbering(945,2) = 275;
          numbering(945,3) = 272;
          numbering(946,0) = 225;
          numbering(946,1) = 275;
          numbering(946,2) = 49;
          numbering(946,3) = 280;
          numbering(947,0) = 271;
          numbering(947,1) = 272;
          numbering(947,2) = 280;
          numbering(947,3) = 54;
          numbering(948,0) = 269;
          numbering(948,1) = 272;
          numbering(948,2) = 225;
          numbering(948,3) = 271;
          numbering(949,0) = 269;
          numbering(949,1) = 275;
          numbering(949,2) = 225;
          numbering(949,3) = 272;
          numbering(950,0) = 225;
          numbering(950,1) = 275;
          numbering(950,2) = 280;
          numbering(950,3) = 272;
          numbering(951,0) = 225;
          numbering(951,1) = 272;
          numbering(951,2) = 280;
          numbering(951,3) = 271;
          numbering(952,0) = 54;
          numbering(952,1) = 272;
          numbering(952,2) = 280;
          numbering(952,3) = 278;
          numbering(953,0) = 272;
          numbering(953,1) = 44;
          numbering(953,2) = 275;
          numbering(953,3) = 279;
          numbering(954,0) = 280;
          numbering(954,1) = 275;
          numbering(954,2) = 49;
          numbering(954,3) = 277;
          numbering(955,0) = 278;
          numbering(955,1) = 279;
          numbering(955,2) = 277;
          numbering(955,3) = 52;
          numbering(956,0) = 272;
          numbering(956,1) = 279;
          numbering(956,2) = 280;
          numbering(956,3) = 278;
          numbering(957,0) = 272;
          numbering(957,1) = 275;
          numbering(957,2) = 280;
          numbering(957,3) = 279;
          numbering(958,0) = 280;
          numbering(958,1) = 275;
          numbering(958,2) = 277;
          numbering(958,3) = 279;
          numbering(959,0) = 280;
          numbering(959,1) = 279;
          numbering(959,2) = 277;
          numbering(959,3) = 278;
          numbering(960,0) = 11;
          numbering(960,1) = 263;
          numbering(960,2) = 219;
          numbering(960,3) = 196;
          numbering(961,0) = 263;
          numbering(961,1) = 54;
          numbering(961,2) = 274;
          numbering(961,3) = 264;
          numbering(962,0) = 219;
          numbering(962,1) = 274;
          numbering(962,2) = 47;
          numbering(962,3) = 281;
          numbering(963,0) = 196;
          numbering(963,1) = 264;
          numbering(963,2) = 281;
          numbering(963,3) = 41;
          numbering(964,0) = 263;
          numbering(964,1) = 264;
          numbering(964,2) = 219;
          numbering(964,3) = 196;
          numbering(965,0) = 263;
          numbering(965,1) = 274;
          numbering(965,2) = 219;
          numbering(965,3) = 264;
          numbering(966,0) = 219;
          numbering(966,1) = 274;
          numbering(966,2) = 281;
          numbering(966,3) = 264;
          numbering(967,0) = 219;
          numbering(967,1) = 264;
          numbering(967,2) = 281;
          numbering(967,3) = 196;
          numbering(968,0) = 54;
          numbering(968,1) = 260;
          numbering(968,2) = 278;
          numbering(968,3) = 261;
          numbering(969,0) = 260;
          numbering(969,1) = 10;
          numbering(969,2) = 246;
          numbering(969,3) = 143;
          numbering(970,0) = 278;
          numbering(970,1) = 246;
          numbering(970,2) = 52;
          numbering(970,3) = 245;
          numbering(971,0) = 261;
          numbering(971,1) = 143;
          numbering(971,2) = 245;
          numbering(971,3) = 33;
          numbering(972,0) = 260;
          numbering(972,1) = 143;
          numbering(972,2) = 278;
          numbering(972,3) = 261;
          numbering(973,0) = 260;
          numbering(973,1) = 246;
          numbering(973,2) = 278;
          numbering(973,3) = 143;
          numbering(974,0) = 278;
          numbering(974,1) = 246;
          numbering(974,2) = 245;
          numbering(974,3) = 143;
          numbering(975,0) = 278;
          numbering(975,1) = 143;
          numbering(975,2) = 245;
          numbering(975,3) = 261;
          numbering(976,0) = 47;
          numbering(976,1) = 276;
          numbering(976,2) = 237;
          numbering(976,3) = 282;
          numbering(977,0) = 276;
          numbering(977,1) = 52;
          numbering(977,2) = 251;
          numbering(977,3) = 248;
          numbering(978,0) = 237;
          numbering(978,1) = 251;
          numbering(978,2) = 13;
          numbering(978,3) = 249;
          numbering(979,0) = 282;
          numbering(979,1) = 248;
          numbering(979,2) = 249;
          numbering(979,3) = 51;
          numbering(980,0) = 276;
          numbering(980,1) = 248;
          numbering(980,2) = 237;
          numbering(980,3) = 282;
          numbering(981,0) = 276;
          numbering(981,1) = 251;
          numbering(981,2) = 237;
          numbering(981,3) = 248;
          numbering(982,0) = 237;
          numbering(982,1) = 251;
          numbering(982,2) = 249;
          numbering(982,3) = 248;
          numbering(983,0) = 237;
          numbering(983,1) = 248;
          numbering(983,2) = 249;
          numbering(983,3) = 282;
          numbering(984,0) = 41;
          numbering(984,1) = 265;
          numbering(984,2) = 283;
          numbering(984,3) = 199;
          numbering(985,0) = 265;
          numbering(985,1) = 33;
          numbering(985,2) = 243;
          numbering(985,3) = 131;
          numbering(986,0) = 283;
          numbering(986,1) = 243;
          numbering(986,2) = 51;
          numbering(986,3) = 242;
          numbering(987,0) = 199;
          numbering(987,1) = 131;
          numbering(987,2) = 242;
          numbering(987,3) = 7;
          numbering(988,0) = 265;
          numbering(988,1) = 131;
          numbering(988,2) = 283;
          numbering(988,3) = 199;
          numbering(989,0) = 265;
          numbering(989,1) = 243;
          numbering(989,2) = 283;
          numbering(989,3) = 131;
          numbering(990,0) = 283;
          numbering(990,1) = 243;
          numbering(990,2) = 242;
          numbering(990,3) = 131;
          numbering(991,0) = 283;
          numbering(991,1) = 131;
          numbering(991,2) = 242;
          numbering(991,3) = 199;
          numbering(992,0) = 54;
          numbering(992,1) = 261;
          numbering(992,2) = 274;
          numbering(992,3) = 264;
          numbering(993,0) = 261;
          numbering(993,1) = 33;
          numbering(993,2) = 284;
          numbering(993,3) = 265;
          numbering(994,0) = 274;
          numbering(994,1) = 284;
          numbering(994,2) = 47;
          numbering(994,3) = 281;
          numbering(995,0) = 264;
          numbering(995,1) = 265;
          numbering(995,2) = 281;
          numbering(995,3) = 41;
          numbering(996,0) = 264;
          numbering(996,1) = 284;
          numbering(996,2) = 274;
          numbering(996,3) = 281;
          numbering(997,0) = 261;
          numbering(997,1) = 284;
          numbering(997,2) = 274;
          numbering(997,3) = 264;
          numbering(998,0) = 261;
          numbering(998,1) = 265;
          numbering(998,2) = 284;
          numbering(998,3) = 264;
          numbering(999,0) = 264;
          numbering(999,1) = 265;
          numbering(999,2) = 284;
          numbering(999,3) = 281;
          numbering(1000,0) = 54;
          numbering(1000,1) = 278;
          numbering(1000,2) = 274;
          numbering(1000,3) = 261;
          numbering(1001,0) = 278;
          numbering(1001,1) = 52;
          numbering(1001,2) = 276;
          numbering(1001,3) = 245;
          numbering(1002,0) = 274;
          numbering(1002,1) = 276;
          numbering(1002,2) = 47;
          numbering(1002,3) = 284;
          numbering(1003,0) = 261;
          numbering(1003,1) = 245;
          numbering(1003,2) = 284;
          numbering(1003,3) = 33;
          numbering(1004,0) = 261;
          numbering(1004,1) = 276;
          numbering(1004,2) = 274;
          numbering(1004,3) = 284;
          numbering(1005,0) = 278;
          numbering(1005,1) = 276;
          numbering(1005,2) = 274;
          numbering(1005,3) = 261;
          numbering(1006,0) = 278;
          numbering(1006,1) = 245;
          numbering(1006,2) = 276;
          numbering(1006,3) = 261;
          numbering(1007,0) = 261;
          numbering(1007,1) = 245;
          numbering(1007,2) = 276;
          numbering(1007,3) = 284;
          numbering(1008,0) = 47;
          numbering(1008,1) = 276;
          numbering(1008,2) = 282;
          numbering(1008,3) = 284;
          numbering(1009,0) = 276;
          numbering(1009,1) = 52;
          numbering(1009,2) = 248;
          numbering(1009,3) = 245;
          numbering(1010,0) = 282;
          numbering(1010,1) = 248;
          numbering(1010,2) = 51;
          numbering(1010,3) = 243;
          numbering(1011,0) = 284;
          numbering(1011,1) = 245;
          numbering(1011,2) = 243;
          numbering(1011,3) = 33;
          numbering(1012,0) = 284;
          numbering(1012,1) = 248;
          numbering(1012,2) = 282;
          numbering(1012,3) = 243;
          numbering(1013,0) = 276;
          numbering(1013,1) = 248;
          numbering(1013,2) = 282;
          numbering(1013,3) = 284;
          numbering(1014,0) = 276;
          numbering(1014,1) = 245;
          numbering(1014,2) = 248;
          numbering(1014,3) = 284;
          numbering(1015,0) = 284;
          numbering(1015,1) = 245;
          numbering(1015,2) = 248;
          numbering(1015,3) = 243;
          numbering(1016,0) = 47;
          numbering(1016,1) = 284;
          numbering(1016,2) = 282;
          numbering(1016,3) = 281;
          numbering(1017,0) = 284;
          numbering(1017,1) = 33;
          numbering(1017,2) = 243;
          numbering(1017,3) = 265;
          numbering(1018,0) = 282;
          numbering(1018,1) = 243;
          numbering(1018,2) = 51;
          numbering(1018,3) = 283;
          numbering(1019,0) = 281;
          numbering(1019,1) = 265;
          numbering(1019,2) = 283;
          numbering(1019,3) = 41;
          numbering(1020,0) = 284;
          numbering(1020,1) = 265;
          numbering(1020,2) = 282;
          numbering(1020,3) = 281;
          numbering(1021,0) = 284;
          numbering(1021,1) = 243;
          numbering(1021,2) = 282;
          numbering(1021,3) = 265;
          numbering(1022,0) = 282;
          numbering(1022,1) = 243;
          numbering(1022,2) = 283;
          numbering(1022,3) = 265;
          numbering(1023,0) = 282;
          numbering(1023,1) = 265;
          numbering(1023,2) = 283;
          numbering(1023,3) = 281;
          return;
        }
      }
    } 
    case SpaMesh::tet:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(1,4);
          numbering(0,0) = 0;
          numbering(0,1) = 1;
          numbering(0,2) = 2;
          numbering(0,3) = 3;
          return;
        }
        case 2:
        {
          numbering.resize(8,4);
          numbering(0,0) = 0;
          numbering(0,1) = 4;
          numbering(0,2) = 5;
          numbering(0,3) = 6;
          numbering(1,0) = 4;
          numbering(1,1) = 1;
          numbering(1,2) = 7;
          numbering(1,3) = 8;
          numbering(2,0) = 5;
          numbering(2,1) = 7;
          numbering(2,2) = 2;
          numbering(2,3) = 9;
          numbering(3,0) = 6;
          numbering(3,1) = 8;
          numbering(3,2) = 9;
          numbering(3,3) = 3;
          numbering(4,0) = 4;
          numbering(4,1) = 8;
          numbering(4,2) = 5;
          numbering(4,3) = 6;
          numbering(5,0) = 4;
          numbering(5,1) = 7;
          numbering(5,2) = 5;
          numbering(5,3) = 8;
          numbering(6,0) = 5;
          numbering(6,1) = 7;
          numbering(6,2) = 9;
          numbering(6,3) = 8;
          numbering(7,0) = 5;
          numbering(7,1) = 8;
          numbering(7,2) = 9;
          numbering(7,3) = 6;
          return;
        }
        case 3:
        case 4:
        {
          numbering.resize(64,4);
          numbering(0,0) = 0;
          numbering(0,1) = 10;
          numbering(0,2) = 11;
          numbering(0,3) = 12;
          numbering(1,0) = 10;
          numbering(1,1) = 4;
          numbering(1,2) = 13;
          numbering(1,3) = 14;
          numbering(2,0) = 11;
          numbering(2,1) = 13;
          numbering(2,2) = 5;
          numbering(2,3) = 15;
          numbering(3,0) = 12;
          numbering(3,1) = 14;
          numbering(3,2) = 15;
          numbering(3,3) = 6;
          numbering(4,0) = 10;
          numbering(4,1) = 14;
          numbering(4,2) = 11;
          numbering(4,3) = 12;
          numbering(5,0) = 10;
          numbering(5,1) = 13;
          numbering(5,2) = 11;
          numbering(5,3) = 14;
          numbering(6,0) = 11;
          numbering(6,1) = 13;
          numbering(6,2) = 15;
          numbering(6,3) = 14;
          numbering(7,0) = 11;
          numbering(7,1) = 14;
          numbering(7,2) = 15;
          numbering(7,3) = 12;
          numbering(8,0) = 4;
          numbering(8,1) = 16;
          numbering(8,2) = 17;
          numbering(8,3) = 18;
          numbering(9,0) = 16;
          numbering(9,1) = 1;
          numbering(9,2) = 19;
          numbering(9,3) = 20;
          numbering(10,0) = 17;
          numbering(10,1) = 19;
          numbering(10,2) = 7;
          numbering(10,3) = 21;
          numbering(11,0) = 18;
          numbering(11,1) = 20;
          numbering(11,2) = 21;
          numbering(11,3) = 8;
          numbering(12,0) = 16;
          numbering(12,1) = 20;
          numbering(12,2) = 17;
          numbering(12,3) = 18;
          numbering(13,0) = 16;
          numbering(13,1) = 19;
          numbering(13,2) = 17;
          numbering(13,3) = 20;
          numbering(14,0) = 17;
          numbering(14,1) = 19;
          numbering(14,2) = 21;
          numbering(14,3) = 20;
          numbering(15,0) = 17;
          numbering(15,1) = 20;
          numbering(15,2) = 21;
          numbering(15,3) = 18;
          numbering(16,0) = 5;
          numbering(16,1) = 22;
          numbering(16,2) = 23;
          numbering(16,3) = 24;
          numbering(17,0) = 22;
          numbering(17,1) = 7;
          numbering(17,2) = 25;
          numbering(17,3) = 26;
          numbering(18,0) = 23;
          numbering(18,1) = 25;
          numbering(18,2) = 2;
          numbering(18,3) = 27;
          numbering(19,0) = 24;
          numbering(19,1) = 26;
          numbering(19,2) = 27;
          numbering(19,3) = 9;
          numbering(20,0) = 22;
          numbering(20,1) = 26;
          numbering(20,2) = 23;
          numbering(20,3) = 24;
          numbering(21,0) = 22;
          numbering(21,1) = 25;
          numbering(21,2) = 23;
          numbering(21,3) = 26;
          numbering(22,0) = 23;
          numbering(22,1) = 25;
          numbering(22,2) = 27;
          numbering(22,3) = 26;
          numbering(23,0) = 23;
          numbering(23,1) = 26;
          numbering(23,2) = 27;
          numbering(23,3) = 24;
          numbering(24,0) = 6;
          numbering(24,1) = 28;
          numbering(24,2) = 29;
          numbering(24,3) = 30;
          numbering(25,0) = 28;
          numbering(25,1) = 8;
          numbering(25,2) = 31;
          numbering(25,3) = 32;
          numbering(26,0) = 29;
          numbering(26,1) = 31;
          numbering(26,2) = 9;
          numbering(26,3) = 33;
          numbering(27,0) = 30;
          numbering(27,1) = 32;
          numbering(27,2) = 33;
          numbering(27,3) = 3;
          numbering(28,0) = 28;
          numbering(28,1) = 32;
          numbering(28,2) = 29;
          numbering(28,3) = 30;
          numbering(29,0) = 28;
          numbering(29,1) = 31;
          numbering(29,2) = 29;
          numbering(29,3) = 32;
          numbering(30,0) = 29;
          numbering(30,1) = 31;
          numbering(30,2) = 33;
          numbering(30,3) = 32;
          numbering(31,0) = 29;
          numbering(31,1) = 32;
          numbering(31,2) = 33;
          numbering(31,3) = 30;
          numbering(32,0) = 4;
          numbering(32,1) = 18;
          numbering(32,2) = 13;
          numbering(32,3) = 14;
          numbering(33,0) = 18;
          numbering(33,1) = 8;
          numbering(33,2) = 34;
          numbering(33,3) = 28;
          numbering(34,0) = 13;
          numbering(34,1) = 34;
          numbering(34,2) = 5;
          numbering(34,3) = 15;
          numbering(35,0) = 14;
          numbering(35,1) = 28;
          numbering(35,2) = 15;
          numbering(35,3) = 6;
          numbering(36,0) = 14;
          numbering(36,1) = 34;
          numbering(36,2) = 13;
          numbering(36,3) = 15;
          numbering(37,0) = 18;
          numbering(37,1) = 34;
          numbering(37,2) = 13;
          numbering(37,3) = 14;
          numbering(38,0) = 18;
          numbering(38,1) = 28;
          numbering(38,2) = 34;
          numbering(38,3) = 14;
          numbering(39,0) = 14;
          numbering(39,1) = 28;
          numbering(39,2) = 34;
          numbering(39,3) = 15;
          numbering(40,0) = 4;
          numbering(40,1) = 17;
          numbering(40,2) = 13;
          numbering(40,3) = 18;
          numbering(41,0) = 17;
          numbering(41,1) = 7;
          numbering(41,2) = 22;
          numbering(41,3) = 21;
          numbering(42,0) = 13;
          numbering(42,1) = 22;
          numbering(42,2) = 5;
          numbering(42,3) = 34;
          numbering(43,0) = 18;
          numbering(43,1) = 21;
          numbering(43,2) = 34;
          numbering(43,3) = 8;
          numbering(44,0) = 17;
          numbering(44,1) = 21;
          numbering(44,2) = 13;
          numbering(44,3) = 18;
          numbering(45,0) = 17;
          numbering(45,1) = 22;
          numbering(45,2) = 13;
          numbering(45,3) = 21;
          numbering(46,0) = 13;
          numbering(46,1) = 22;
          numbering(46,2) = 34;
          numbering(46,3) = 21;
          numbering(47,0) = 13;
          numbering(47,1) = 21;
          numbering(47,2) = 34;
          numbering(47,3) = 18;
          numbering(48,0) = 5;
          numbering(48,1) = 22;
          numbering(48,2) = 24;
          numbering(48,3) = 34;
          numbering(49,0) = 22;
          numbering(49,1) = 7;
          numbering(49,2) = 26;
          numbering(49,3) = 21;
          numbering(50,0) = 24;
          numbering(50,1) = 26;
          numbering(50,2) = 9;
          numbering(50,3) = 31;
          numbering(51,0) = 34;
          numbering(51,1) = 21;
          numbering(51,2) = 31;
          numbering(51,3) = 8;
          numbering(52,0) = 34;
          numbering(52,1) = 26;
          numbering(52,2) = 24;
          numbering(52,3) = 31;
          numbering(53,0) = 22;
          numbering(53,1) = 26;
          numbering(53,2) = 24;
          numbering(53,3) = 34;
          numbering(54,0) = 22;
          numbering(54,1) = 21;
          numbering(54,2) = 26;
          numbering(54,3) = 34;
          numbering(55,0) = 34;
          numbering(55,1) = 21;
          numbering(55,2) = 26;
          numbering(55,3) = 31;
          numbering(56,0) = 5;
          numbering(56,1) = 34;
          numbering(56,2) = 24;
          numbering(56,3) = 15;
          numbering(57,0) = 34;
          numbering(57,1) = 8;
          numbering(57,2) = 31;
          numbering(57,3) = 28;
          numbering(58,0) = 24;
          numbering(58,1) = 31;
          numbering(58,2) = 9;
          numbering(58,3) = 29;
          numbering(59,0) = 15;
          numbering(59,1) = 28;
          numbering(59,2) = 29;
          numbering(59,3) = 6;
          numbering(60,0) = 15;
          numbering(60,1) = 31;
          numbering(60,2) = 24;
          numbering(60,3) = 29;
          numbering(61,0) = 34;
          numbering(61,1) = 31;
          numbering(61,2) = 24;
          numbering(61,3) = 15;
          numbering(62,0) = 34;
          numbering(62,1) = 28;
          numbering(62,2) = 31;
          numbering(62,3) = 15;
          numbering(63,0) = 15;
          numbering(63,1) = 28;
          numbering(63,2) = 31;
          numbering(63,3) = 29;
          return;
        }
        default:
        {
          numbering.resize(512,4);
          numbering(0,0) = 0;
          numbering(0,1) = 35;
          numbering(0,2) = 36;
          numbering(0,3) = 37;
          numbering(1,0) = 35;
          numbering(1,1) = 10;
          numbering(1,2) = 38;
          numbering(1,3) = 39;
          numbering(2,0) = 36;
          numbering(2,1) = 38;
          numbering(2,2) = 11;
          numbering(2,3) = 40;
          numbering(3,0) = 37;
          numbering(3,1) = 39;
          numbering(3,2) = 40;
          numbering(3,3) = 12;
          numbering(4,0) = 35;
          numbering(4,1) = 39;
          numbering(4,2) = 36;
          numbering(4,3) = 37;
          numbering(5,0) = 35;
          numbering(5,1) = 38;
          numbering(5,2) = 36;
          numbering(5,3) = 39;
          numbering(6,0) = 36;
          numbering(6,1) = 38;
          numbering(6,2) = 40;
          numbering(6,3) = 39;
          numbering(7,0) = 36;
          numbering(7,1) = 39;
          numbering(7,2) = 40;
          numbering(7,3) = 37;
          numbering(8,0) = 10;
          numbering(8,1) = 41;
          numbering(8,2) = 42;
          numbering(8,3) = 43;
          numbering(9,0) = 41;
          numbering(9,1) = 4;
          numbering(9,2) = 44;
          numbering(9,3) = 45;
          numbering(10,0) = 42;
          numbering(10,1) = 44;
          numbering(10,2) = 13;
          numbering(10,3) = 46;
          numbering(11,0) = 43;
          numbering(11,1) = 45;
          numbering(11,2) = 46;
          numbering(11,3) = 14;
          numbering(12,0) = 41;
          numbering(12,1) = 45;
          numbering(12,2) = 42;
          numbering(12,3) = 43;
          numbering(13,0) = 41;
          numbering(13,1) = 44;
          numbering(13,2) = 42;
          numbering(13,3) = 45;
          numbering(14,0) = 42;
          numbering(14,1) = 44;
          numbering(14,2) = 46;
          numbering(14,3) = 45;
          numbering(15,0) = 42;
          numbering(15,1) = 45;
          numbering(15,2) = 46;
          numbering(15,3) = 43;
          numbering(16,0) = 11;
          numbering(16,1) = 47;
          numbering(16,2) = 48;
          numbering(16,3) = 49;
          numbering(17,0) = 47;
          numbering(17,1) = 13;
          numbering(17,2) = 50;
          numbering(17,3) = 51;
          numbering(18,0) = 48;
          numbering(18,1) = 50;
          numbering(18,2) = 5;
          numbering(18,3) = 52;
          numbering(19,0) = 49;
          numbering(19,1) = 51;
          numbering(19,2) = 52;
          numbering(19,3) = 15;
          numbering(20,0) = 47;
          numbering(20,1) = 51;
          numbering(20,2) = 48;
          numbering(20,3) = 49;
          numbering(21,0) = 47;
          numbering(21,1) = 50;
          numbering(21,2) = 48;
          numbering(21,3) = 51;
          numbering(22,0) = 48;
          numbering(22,1) = 50;
          numbering(22,2) = 52;
          numbering(22,3) = 51;
          numbering(23,0) = 48;
          numbering(23,1) = 51;
          numbering(23,2) = 52;
          numbering(23,3) = 49;
          numbering(24,0) = 12;
          numbering(24,1) = 53;
          numbering(24,2) = 54;
          numbering(24,3) = 55;
          numbering(25,0) = 53;
          numbering(25,1) = 14;
          numbering(25,2) = 56;
          numbering(25,3) = 57;
          numbering(26,0) = 54;
          numbering(26,1) = 56;
          numbering(26,2) = 15;
          numbering(26,3) = 58;
          numbering(27,0) = 55;
          numbering(27,1) = 57;
          numbering(27,2) = 58;
          numbering(27,3) = 6;
          numbering(28,0) = 53;
          numbering(28,1) = 57;
          numbering(28,2) = 54;
          numbering(28,3) = 55;
          numbering(29,0) = 53;
          numbering(29,1) = 56;
          numbering(29,2) = 54;
          numbering(29,3) = 57;
          numbering(30,0) = 54;
          numbering(30,1) = 56;
          numbering(30,2) = 58;
          numbering(30,3) = 57;
          numbering(31,0) = 54;
          numbering(31,1) = 57;
          numbering(31,2) = 58;
          numbering(31,3) = 55;
          numbering(32,0) = 10;
          numbering(32,1) = 43;
          numbering(32,2) = 38;
          numbering(32,3) = 39;
          numbering(33,0) = 43;
          numbering(33,1) = 14;
          numbering(33,2) = 59;
          numbering(33,3) = 53;
          numbering(34,0) = 38;
          numbering(34,1) = 59;
          numbering(34,2) = 11;
          numbering(34,3) = 40;
          numbering(35,0) = 39;
          numbering(35,1) = 53;
          numbering(35,2) = 40;
          numbering(35,3) = 12;
          numbering(36,0) = 39;
          numbering(36,1) = 59;
          numbering(36,2) = 38;
          numbering(36,3) = 40;
          numbering(37,0) = 43;
          numbering(37,1) = 59;
          numbering(37,2) = 38;
          numbering(37,3) = 39;
          numbering(38,0) = 43;
          numbering(38,1) = 53;
          numbering(38,2) = 59;
          numbering(38,3) = 39;
          numbering(39,0) = 39;
          numbering(39,1) = 53;
          numbering(39,2) = 59;
          numbering(39,3) = 40;
          numbering(40,0) = 10;
          numbering(40,1) = 42;
          numbering(40,2) = 38;
          numbering(40,3) = 43;
          numbering(41,0) = 42;
          numbering(41,1) = 13;
          numbering(41,2) = 47;
          numbering(41,3) = 46;
          numbering(42,0) = 38;
          numbering(42,1) = 47;
          numbering(42,2) = 11;
          numbering(42,3) = 59;
          numbering(43,0) = 43;
          numbering(43,1) = 46;
          numbering(43,2) = 59;
          numbering(43,3) = 14;
          numbering(44,0) = 42;
          numbering(44,1) = 46;
          numbering(44,2) = 38;
          numbering(44,3) = 43;
          numbering(45,0) = 42;
          numbering(45,1) = 47;
          numbering(45,2) = 38;
          numbering(45,3) = 46;
          numbering(46,0) = 38;
          numbering(46,1) = 47;
          numbering(46,2) = 59;
          numbering(46,3) = 46;
          numbering(47,0) = 38;
          numbering(47,1) = 46;
          numbering(47,2) = 59;
          numbering(47,3) = 43;
          numbering(48,0) = 11;
          numbering(48,1) = 47;
          numbering(48,2) = 49;
          numbering(48,3) = 59;
          numbering(49,0) = 47;
          numbering(49,1) = 13;
          numbering(49,2) = 51;
          numbering(49,3) = 46;
          numbering(50,0) = 49;
          numbering(50,1) = 51;
          numbering(50,2) = 15;
          numbering(50,3) = 56;
          numbering(51,0) = 59;
          numbering(51,1) = 46;
          numbering(51,2) = 56;
          numbering(51,3) = 14;
          numbering(52,0) = 59;
          numbering(52,1) = 51;
          numbering(52,2) = 49;
          numbering(52,3) = 56;
          numbering(53,0) = 47;
          numbering(53,1) = 51;
          numbering(53,2) = 49;
          numbering(53,3) = 59;
          numbering(54,0) = 47;
          numbering(54,1) = 46;
          numbering(54,2) = 51;
          numbering(54,3) = 59;
          numbering(55,0) = 59;
          numbering(55,1) = 46;
          numbering(55,2) = 51;
          numbering(55,3) = 56;
          numbering(56,0) = 11;
          numbering(56,1) = 59;
          numbering(56,2) = 49;
          numbering(56,3) = 40;
          numbering(57,0) = 59;
          numbering(57,1) = 14;
          numbering(57,2) = 56;
          numbering(57,3) = 53;
          numbering(58,0) = 49;
          numbering(58,1) = 56;
          numbering(58,2) = 15;
          numbering(58,3) = 54;
          numbering(59,0) = 40;
          numbering(59,1) = 53;
          numbering(59,2) = 54;
          numbering(59,3) = 12;
          numbering(60,0) = 40;
          numbering(60,1) = 56;
          numbering(60,2) = 49;
          numbering(60,3) = 54;
          numbering(61,0) = 59;
          numbering(61,1) = 56;
          numbering(61,2) = 49;
          numbering(61,3) = 40;
          numbering(62,0) = 59;
          numbering(62,1) = 53;
          numbering(62,2) = 56;
          numbering(62,3) = 40;
          numbering(63,0) = 40;
          numbering(63,1) = 53;
          numbering(63,2) = 56;
          numbering(63,3) = 54;
          numbering(64,0) = 4;
          numbering(64,1) = 60;
          numbering(64,2) = 61;
          numbering(64,3) = 62;
          numbering(65,0) = 60;
          numbering(65,1) = 16;
          numbering(65,2) = 63;
          numbering(65,3) = 64;
          numbering(66,0) = 61;
          numbering(66,1) = 63;
          numbering(66,2) = 17;
          numbering(66,3) = 65;
          numbering(67,0) = 62;
          numbering(67,1) = 64;
          numbering(67,2) = 65;
          numbering(67,3) = 18;
          numbering(68,0) = 60;
          numbering(68,1) = 64;
          numbering(68,2) = 61;
          numbering(68,3) = 62;
          numbering(69,0) = 60;
          numbering(69,1) = 63;
          numbering(69,2) = 61;
          numbering(69,3) = 64;
          numbering(70,0) = 61;
          numbering(70,1) = 63;
          numbering(70,2) = 65;
          numbering(70,3) = 64;
          numbering(71,0) = 61;
          numbering(71,1) = 64;
          numbering(71,2) = 65;
          numbering(71,3) = 62;
          numbering(72,0) = 16;
          numbering(72,1) = 66;
          numbering(72,2) = 67;
          numbering(72,3) = 68;
          numbering(73,0) = 66;
          numbering(73,1) = 1;
          numbering(73,2) = 69;
          numbering(73,3) = 70;
          numbering(74,0) = 67;
          numbering(74,1) = 69;
          numbering(74,2) = 19;
          numbering(74,3) = 71;
          numbering(75,0) = 68;
          numbering(75,1) = 70;
          numbering(75,2) = 71;
          numbering(75,3) = 20;
          numbering(76,0) = 66;
          numbering(76,1) = 70;
          numbering(76,2) = 67;
          numbering(76,3) = 68;
          numbering(77,0) = 66;
          numbering(77,1) = 69;
          numbering(77,2) = 67;
          numbering(77,3) = 70;
          numbering(78,0) = 67;
          numbering(78,1) = 69;
          numbering(78,2) = 71;
          numbering(78,3) = 70;
          numbering(79,0) = 67;
          numbering(79,1) = 70;
          numbering(79,2) = 71;
          numbering(79,3) = 68;
          numbering(80,0) = 17;
          numbering(80,1) = 72;
          numbering(80,2) = 73;
          numbering(80,3) = 74;
          numbering(81,0) = 72;
          numbering(81,1) = 19;
          numbering(81,2) = 75;
          numbering(81,3) = 76;
          numbering(82,0) = 73;
          numbering(82,1) = 75;
          numbering(82,2) = 7;
          numbering(82,3) = 77;
          numbering(83,0) = 74;
          numbering(83,1) = 76;
          numbering(83,2) = 77;
          numbering(83,3) = 21;
          numbering(84,0) = 72;
          numbering(84,1) = 76;
          numbering(84,2) = 73;
          numbering(84,3) = 74;
          numbering(85,0) = 72;
          numbering(85,1) = 75;
          numbering(85,2) = 73;
          numbering(85,3) = 76;
          numbering(86,0) = 73;
          numbering(86,1) = 75;
          numbering(86,2) = 77;
          numbering(86,3) = 76;
          numbering(87,0) = 73;
          numbering(87,1) = 76;
          numbering(87,2) = 77;
          numbering(87,3) = 74;
          numbering(88,0) = 18;
          numbering(88,1) = 78;
          numbering(88,2) = 79;
          numbering(88,3) = 80;
          numbering(89,0) = 78;
          numbering(89,1) = 20;
          numbering(89,2) = 81;
          numbering(89,3) = 82;
          numbering(90,0) = 79;
          numbering(90,1) = 81;
          numbering(90,2) = 21;
          numbering(90,3) = 83;
          numbering(91,0) = 80;
          numbering(91,1) = 82;
          numbering(91,2) = 83;
          numbering(91,3) = 8;
          numbering(92,0) = 78;
          numbering(92,1) = 82;
          numbering(92,2) = 79;
          numbering(92,3) = 80;
          numbering(93,0) = 78;
          numbering(93,1) = 81;
          numbering(93,2) = 79;
          numbering(93,3) = 82;
          numbering(94,0) = 79;
          numbering(94,1) = 81;
          numbering(94,2) = 83;
          numbering(94,3) = 82;
          numbering(95,0) = 79;
          numbering(95,1) = 82;
          numbering(95,2) = 83;
          numbering(95,3) = 80;
          numbering(96,0) = 16;
          numbering(96,1) = 68;
          numbering(96,2) = 63;
          numbering(96,3) = 64;
          numbering(97,0) = 68;
          numbering(97,1) = 20;
          numbering(97,2) = 84;
          numbering(97,3) = 78;
          numbering(98,0) = 63;
          numbering(98,1) = 84;
          numbering(98,2) = 17;
          numbering(98,3) = 65;
          numbering(99,0) = 64;
          numbering(99,1) = 78;
          numbering(99,2) = 65;
          numbering(99,3) = 18;
          numbering(100,0) = 64;
          numbering(100,1) = 84;
          numbering(100,2) = 63;
          numbering(100,3) = 65;
          numbering(101,0) = 68;
          numbering(101,1) = 84;
          numbering(101,2) = 63;
          numbering(101,3) = 64;
          numbering(102,0) = 68;
          numbering(102,1) = 78;
          numbering(102,2) = 84;
          numbering(102,3) = 64;
          numbering(103,0) = 64;
          numbering(103,1) = 78;
          numbering(103,2) = 84;
          numbering(103,3) = 65;
          numbering(104,0) = 16;
          numbering(104,1) = 67;
          numbering(104,2) = 63;
          numbering(104,3) = 68;
          numbering(105,0) = 67;
          numbering(105,1) = 19;
          numbering(105,2) = 72;
          numbering(105,3) = 71;
          numbering(106,0) = 63;
          numbering(106,1) = 72;
          numbering(106,2) = 17;
          numbering(106,3) = 84;
          numbering(107,0) = 68;
          numbering(107,1) = 71;
          numbering(107,2) = 84;
          numbering(107,3) = 20;
          numbering(108,0) = 67;
          numbering(108,1) = 71;
          numbering(108,2) = 63;
          numbering(108,3) = 68;
          numbering(109,0) = 67;
          numbering(109,1) = 72;
          numbering(109,2) = 63;
          numbering(109,3) = 71;
          numbering(110,0) = 63;
          numbering(110,1) = 72;
          numbering(110,2) = 84;
          numbering(110,3) = 71;
          numbering(111,0) = 63;
          numbering(111,1) = 71;
          numbering(111,2) = 84;
          numbering(111,3) = 68;
          numbering(112,0) = 17;
          numbering(112,1) = 72;
          numbering(112,2) = 74;
          numbering(112,3) = 84;
          numbering(113,0) = 72;
          numbering(113,1) = 19;
          numbering(113,2) = 76;
          numbering(113,3) = 71;
          numbering(114,0) = 74;
          numbering(114,1) = 76;
          numbering(114,2) = 21;
          numbering(114,3) = 81;
          numbering(115,0) = 84;
          numbering(115,1) = 71;
          numbering(115,2) = 81;
          numbering(115,3) = 20;
          numbering(116,0) = 84;
          numbering(116,1) = 76;
          numbering(116,2) = 74;
          numbering(116,3) = 81;
          numbering(117,0) = 72;
          numbering(117,1) = 76;
          numbering(117,2) = 74;
          numbering(117,3) = 84;
          numbering(118,0) = 72;
          numbering(118,1) = 71;
          numbering(118,2) = 76;
          numbering(118,3) = 84;
          numbering(119,0) = 84;
          numbering(119,1) = 71;
          numbering(119,2) = 76;
          numbering(119,3) = 81;
          numbering(120,0) = 17;
          numbering(120,1) = 84;
          numbering(120,2) = 74;
          numbering(120,3) = 65;
          numbering(121,0) = 84;
          numbering(121,1) = 20;
          numbering(121,2) = 81;
          numbering(121,3) = 78;
          numbering(122,0) = 74;
          numbering(122,1) = 81;
          numbering(122,2) = 21;
          numbering(122,3) = 79;
          numbering(123,0) = 65;
          numbering(123,1) = 78;
          numbering(123,2) = 79;
          numbering(123,3) = 18;
          numbering(124,0) = 65;
          numbering(124,1) = 81;
          numbering(124,2) = 74;
          numbering(124,3) = 79;
          numbering(125,0) = 84;
          numbering(125,1) = 81;
          numbering(125,2) = 74;
          numbering(125,3) = 65;
          numbering(126,0) = 84;
          numbering(126,1) = 78;
          numbering(126,2) = 81;
          numbering(126,3) = 65;
          numbering(127,0) = 65;
          numbering(127,1) = 78;
          numbering(127,2) = 81;
          numbering(127,3) = 79;
          numbering(128,0) = 5;
          numbering(128,1) = 85;
          numbering(128,2) = 86;
          numbering(128,3) = 87;
          numbering(129,0) = 85;
          numbering(129,1) = 22;
          numbering(129,2) = 88;
          numbering(129,3) = 89;
          numbering(130,0) = 86;
          numbering(130,1) = 88;
          numbering(130,2) = 23;
          numbering(130,3) = 90;
          numbering(131,0) = 87;
          numbering(131,1) = 89;
          numbering(131,2) = 90;
          numbering(131,3) = 24;
          numbering(132,0) = 85;
          numbering(132,1) = 89;
          numbering(132,2) = 86;
          numbering(132,3) = 87;
          numbering(133,0) = 85;
          numbering(133,1) = 88;
          numbering(133,2) = 86;
          numbering(133,3) = 89;
          numbering(134,0) = 86;
          numbering(134,1) = 88;
          numbering(134,2) = 90;
          numbering(134,3) = 89;
          numbering(135,0) = 86;
          numbering(135,1) = 89;
          numbering(135,2) = 90;
          numbering(135,3) = 87;
          numbering(136,0) = 22;
          numbering(136,1) = 91;
          numbering(136,2) = 92;
          numbering(136,3) = 93;
          numbering(137,0) = 91;
          numbering(137,1) = 7;
          numbering(137,2) = 94;
          numbering(137,3) = 95;
          numbering(138,0) = 92;
          numbering(138,1) = 94;
          numbering(138,2) = 25;
          numbering(138,3) = 96;
          numbering(139,0) = 93;
          numbering(139,1) = 95;
          numbering(139,2) = 96;
          numbering(139,3) = 26;
          numbering(140,0) = 91;
          numbering(140,1) = 95;
          numbering(140,2) = 92;
          numbering(140,3) = 93;
          numbering(141,0) = 91;
          numbering(141,1) = 94;
          numbering(141,2) = 92;
          numbering(141,3) = 95;
          numbering(142,0) = 92;
          numbering(142,1) = 94;
          numbering(142,2) = 96;
          numbering(142,3) = 95;
          numbering(143,0) = 92;
          numbering(143,1) = 95;
          numbering(143,2) = 96;
          numbering(143,3) = 93;
          numbering(144,0) = 23;
          numbering(144,1) = 97;
          numbering(144,2) = 98;
          numbering(144,3) = 99;
          numbering(145,0) = 97;
          numbering(145,1) = 25;
          numbering(145,2) = 100;
          numbering(145,3) = 101;
          numbering(146,0) = 98;
          numbering(146,1) = 100;
          numbering(146,2) = 2;
          numbering(146,3) = 102;
          numbering(147,0) = 99;
          numbering(147,1) = 101;
          numbering(147,2) = 102;
          numbering(147,3) = 27;
          numbering(148,0) = 97;
          numbering(148,1) = 101;
          numbering(148,2) = 98;
          numbering(148,3) = 99;
          numbering(149,0) = 97;
          numbering(149,1) = 100;
          numbering(149,2) = 98;
          numbering(149,3) = 101;
          numbering(150,0) = 98;
          numbering(150,1) = 100;
          numbering(150,2) = 102;
          numbering(150,3) = 101;
          numbering(151,0) = 98;
          numbering(151,1) = 101;
          numbering(151,2) = 102;
          numbering(151,3) = 99;
          numbering(152,0) = 24;
          numbering(152,1) = 103;
          numbering(152,2) = 104;
          numbering(152,3) = 105;
          numbering(153,0) = 103;
          numbering(153,1) = 26;
          numbering(153,2) = 106;
          numbering(153,3) = 107;
          numbering(154,0) = 104;
          numbering(154,1) = 106;
          numbering(154,2) = 27;
          numbering(154,3) = 108;
          numbering(155,0) = 105;
          numbering(155,1) = 107;
          numbering(155,2) = 108;
          numbering(155,3) = 9;
          numbering(156,0) = 103;
          numbering(156,1) = 107;
          numbering(156,2) = 104;
          numbering(156,3) = 105;
          numbering(157,0) = 103;
          numbering(157,1) = 106;
          numbering(157,2) = 104;
          numbering(157,3) = 107;
          numbering(158,0) = 104;
          numbering(158,1) = 106;
          numbering(158,2) = 108;
          numbering(158,3) = 107;
          numbering(159,0) = 104;
          numbering(159,1) = 107;
          numbering(159,2) = 108;
          numbering(159,3) = 105;
          numbering(160,0) = 22;
          numbering(160,1) = 93;
          numbering(160,2) = 88;
          numbering(160,3) = 89;
          numbering(161,0) = 93;
          numbering(161,1) = 26;
          numbering(161,2) = 109;
          numbering(161,3) = 103;
          numbering(162,0) = 88;
          numbering(162,1) = 109;
          numbering(162,2) = 23;
          numbering(162,3) = 90;
          numbering(163,0) = 89;
          numbering(163,1) = 103;
          numbering(163,2) = 90;
          numbering(163,3) = 24;
          numbering(164,0) = 89;
          numbering(164,1) = 109;
          numbering(164,2) = 88;
          numbering(164,3) = 90;
          numbering(165,0) = 93;
          numbering(165,1) = 109;
          numbering(165,2) = 88;
          numbering(165,3) = 89;
          numbering(166,0) = 93;
          numbering(166,1) = 103;
          numbering(166,2) = 109;
          numbering(166,3) = 89;
          numbering(167,0) = 89;
          numbering(167,1) = 103;
          numbering(167,2) = 109;
          numbering(167,3) = 90;
          numbering(168,0) = 22;
          numbering(168,1) = 92;
          numbering(168,2) = 88;
          numbering(168,3) = 93;
          numbering(169,0) = 92;
          numbering(169,1) = 25;
          numbering(169,2) = 97;
          numbering(169,3) = 96;
          numbering(170,0) = 88;
          numbering(170,1) = 97;
          numbering(170,2) = 23;
          numbering(170,3) = 109;
          numbering(171,0) = 93;
          numbering(171,1) = 96;
          numbering(171,2) = 109;
          numbering(171,3) = 26;
          numbering(172,0) = 92;
          numbering(172,1) = 96;
          numbering(172,2) = 88;
          numbering(172,3) = 93;
          numbering(173,0) = 92;
          numbering(173,1) = 97;
          numbering(173,2) = 88;
          numbering(173,3) = 96;
          numbering(174,0) = 88;
          numbering(174,1) = 97;
          numbering(174,2) = 109;
          numbering(174,3) = 96;
          numbering(175,0) = 88;
          numbering(175,1) = 96;
          numbering(175,2) = 109;
          numbering(175,3) = 93;
          numbering(176,0) = 23;
          numbering(176,1) = 97;
          numbering(176,2) = 99;
          numbering(176,3) = 109;
          numbering(177,0) = 97;
          numbering(177,1) = 25;
          numbering(177,2) = 101;
          numbering(177,3) = 96;
          numbering(178,0) = 99;
          numbering(178,1) = 101;
          numbering(178,2) = 27;
          numbering(178,3) = 106;
          numbering(179,0) = 109;
          numbering(179,1) = 96;
          numbering(179,2) = 106;
          numbering(179,3) = 26;
          numbering(180,0) = 109;
          numbering(180,1) = 101;
          numbering(180,2) = 99;
          numbering(180,3) = 106;
          numbering(181,0) = 97;
          numbering(181,1) = 101;
          numbering(181,2) = 99;
          numbering(181,3) = 109;
          numbering(182,0) = 97;
          numbering(182,1) = 96;
          numbering(182,2) = 101;
          numbering(182,3) = 109;
          numbering(183,0) = 109;
          numbering(183,1) = 96;
          numbering(183,2) = 101;
          numbering(183,3) = 106;
          numbering(184,0) = 23;
          numbering(184,1) = 109;
          numbering(184,2) = 99;
          numbering(184,3) = 90;
          numbering(185,0) = 109;
          numbering(185,1) = 26;
          numbering(185,2) = 106;
          numbering(185,3) = 103;
          numbering(186,0) = 99;
          numbering(186,1) = 106;
          numbering(186,2) = 27;
          numbering(186,3) = 104;
          numbering(187,0) = 90;
          numbering(187,1) = 103;
          numbering(187,2) = 104;
          numbering(187,3) = 24;
          numbering(188,0) = 90;
          numbering(188,1) = 106;
          numbering(188,2) = 99;
          numbering(188,3) = 104;
          numbering(189,0) = 109;
          numbering(189,1) = 106;
          numbering(189,2) = 99;
          numbering(189,3) = 90;
          numbering(190,0) = 109;
          numbering(190,1) = 103;
          numbering(190,2) = 106;
          numbering(190,3) = 90;
          numbering(191,0) = 90;
          numbering(191,1) = 103;
          numbering(191,2) = 106;
          numbering(191,3) = 104;
          numbering(192,0) = 6;
          numbering(192,1) = 110;
          numbering(192,2) = 111;
          numbering(192,3) = 112;
          numbering(193,0) = 110;
          numbering(193,1) = 28;
          numbering(193,2) = 113;
          numbering(193,3) = 114;
          numbering(194,0) = 111;
          numbering(194,1) = 113;
          numbering(194,2) = 29;
          numbering(194,3) = 115;
          numbering(195,0) = 112;
          numbering(195,1) = 114;
          numbering(195,2) = 115;
          numbering(195,3) = 30;
          numbering(196,0) = 110;
          numbering(196,1) = 114;
          numbering(196,2) = 111;
          numbering(196,3) = 112;
          numbering(197,0) = 110;
          numbering(197,1) = 113;
          numbering(197,2) = 111;
          numbering(197,3) = 114;
          numbering(198,0) = 111;
          numbering(198,1) = 113;
          numbering(198,2) = 115;
          numbering(198,3) = 114;
          numbering(199,0) = 111;
          numbering(199,1) = 114;
          numbering(199,2) = 115;
          numbering(199,3) = 112;
          numbering(200,0) = 28;
          numbering(200,1) = 116;
          numbering(200,2) = 117;
          numbering(200,3) = 118;
          numbering(201,0) = 116;
          numbering(201,1) = 8;
          numbering(201,2) = 119;
          numbering(201,3) = 120;
          numbering(202,0) = 117;
          numbering(202,1) = 119;
          numbering(202,2) = 31;
          numbering(202,3) = 121;
          numbering(203,0) = 118;
          numbering(203,1) = 120;
          numbering(203,2) = 121;
          numbering(203,3) = 32;
          numbering(204,0) = 116;
          numbering(204,1) = 120;
          numbering(204,2) = 117;
          numbering(204,3) = 118;
          numbering(205,0) = 116;
          numbering(205,1) = 119;
          numbering(205,2) = 117;
          numbering(205,3) = 120;
          numbering(206,0) = 117;
          numbering(206,1) = 119;
          numbering(206,2) = 121;
          numbering(206,3) = 120;
          numbering(207,0) = 117;
          numbering(207,1) = 120;
          numbering(207,2) = 121;
          numbering(207,3) = 118;
          numbering(208,0) = 29;
          numbering(208,1) = 122;
          numbering(208,2) = 123;
          numbering(208,3) = 124;
          numbering(209,0) = 122;
          numbering(209,1) = 31;
          numbering(209,2) = 125;
          numbering(209,3) = 126;
          numbering(210,0) = 123;
          numbering(210,1) = 125;
          numbering(210,2) = 9;
          numbering(210,3) = 127;
          numbering(211,0) = 124;
          numbering(211,1) = 126;
          numbering(211,2) = 127;
          numbering(211,3) = 33;
          numbering(212,0) = 122;
          numbering(212,1) = 126;
          numbering(212,2) = 123;
          numbering(212,3) = 124;
          numbering(213,0) = 122;
          numbering(213,1) = 125;
          numbering(213,2) = 123;
          numbering(213,3) = 126;
          numbering(214,0) = 123;
          numbering(214,1) = 125;
          numbering(214,2) = 127;
          numbering(214,3) = 126;
          numbering(215,0) = 123;
          numbering(215,1) = 126;
          numbering(215,2) = 127;
          numbering(215,3) = 124;
          numbering(216,0) = 30;
          numbering(216,1) = 128;
          numbering(216,2) = 129;
          numbering(216,3) = 130;
          numbering(217,0) = 128;
          numbering(217,1) = 32;
          numbering(217,2) = 131;
          numbering(217,3) = 132;
          numbering(218,0) = 129;
          numbering(218,1) = 131;
          numbering(218,2) = 33;
          numbering(218,3) = 133;
          numbering(219,0) = 130;
          numbering(219,1) = 132;
          numbering(219,2) = 133;
          numbering(219,3) = 3;
          numbering(220,0) = 128;
          numbering(220,1) = 132;
          numbering(220,2) = 129;
          numbering(220,3) = 130;
          numbering(221,0) = 128;
          numbering(221,1) = 131;
          numbering(221,2) = 129;
          numbering(221,3) = 132;
          numbering(222,0) = 129;
          numbering(222,1) = 131;
          numbering(222,2) = 133;
          numbering(222,3) = 132;
          numbering(223,0) = 129;
          numbering(223,1) = 132;
          numbering(223,2) = 133;
          numbering(223,3) = 130;
          numbering(224,0) = 28;
          numbering(224,1) = 118;
          numbering(224,2) = 113;
          numbering(224,3) = 114;
          numbering(225,0) = 118;
          numbering(225,1) = 32;
          numbering(225,2) = 134;
          numbering(225,3) = 128;
          numbering(226,0) = 113;
          numbering(226,1) = 134;
          numbering(226,2) = 29;
          numbering(226,3) = 115;
          numbering(227,0) = 114;
          numbering(227,1) = 128;
          numbering(227,2) = 115;
          numbering(227,3) = 30;
          numbering(228,0) = 114;
          numbering(228,1) = 134;
          numbering(228,2) = 113;
          numbering(228,3) = 115;
          numbering(229,0) = 118;
          numbering(229,1) = 134;
          numbering(229,2) = 113;
          numbering(229,3) = 114;
          numbering(230,0) = 118;
          numbering(230,1) = 128;
          numbering(230,2) = 134;
          numbering(230,3) = 114;
          numbering(231,0) = 114;
          numbering(231,1) = 128;
          numbering(231,2) = 134;
          numbering(231,3) = 115;
          numbering(232,0) = 28;
          numbering(232,1) = 117;
          numbering(232,2) = 113;
          numbering(232,3) = 118;
          numbering(233,0) = 117;
          numbering(233,1) = 31;
          numbering(233,2) = 122;
          numbering(233,3) = 121;
          numbering(234,0) = 113;
          numbering(234,1) = 122;
          numbering(234,2) = 29;
          numbering(234,3) = 134;
          numbering(235,0) = 118;
          numbering(235,1) = 121;
          numbering(235,2) = 134;
          numbering(235,3) = 32;
          numbering(236,0) = 117;
          numbering(236,1) = 121;
          numbering(236,2) = 113;
          numbering(236,3) = 118;
          numbering(237,0) = 117;
          numbering(237,1) = 122;
          numbering(237,2) = 113;
          numbering(237,3) = 121;
          numbering(238,0) = 113;
          numbering(238,1) = 122;
          numbering(238,2) = 134;
          numbering(238,3) = 121;
          numbering(239,0) = 113;
          numbering(239,1) = 121;
          numbering(239,2) = 134;
          numbering(239,3) = 118;
          numbering(240,0) = 29;
          numbering(240,1) = 122;
          numbering(240,2) = 124;
          numbering(240,3) = 134;
          numbering(241,0) = 122;
          numbering(241,1) = 31;
          numbering(241,2) = 126;
          numbering(241,3) = 121;
          numbering(242,0) = 124;
          numbering(242,1) = 126;
          numbering(242,2) = 33;
          numbering(242,3) = 131;
          numbering(243,0) = 134;
          numbering(243,1) = 121;
          numbering(243,2) = 131;
          numbering(243,3) = 32;
          numbering(244,0) = 134;
          numbering(244,1) = 126;
          numbering(244,2) = 124;
          numbering(244,3) = 131;
          numbering(245,0) = 122;
          numbering(245,1) = 126;
          numbering(245,2) = 124;
          numbering(245,3) = 134;
          numbering(246,0) = 122;
          numbering(246,1) = 121;
          numbering(246,2) = 126;
          numbering(246,3) = 134;
          numbering(247,0) = 134;
          numbering(247,1) = 121;
          numbering(247,2) = 126;
          numbering(247,3) = 131;
          numbering(248,0) = 29;
          numbering(248,1) = 134;
          numbering(248,2) = 124;
          numbering(248,3) = 115;
          numbering(249,0) = 134;
          numbering(249,1) = 32;
          numbering(249,2) = 131;
          numbering(249,3) = 128;
          numbering(250,0) = 124;
          numbering(250,1) = 131;
          numbering(250,2) = 33;
          numbering(250,3) = 129;
          numbering(251,0) = 115;
          numbering(251,1) = 128;
          numbering(251,2) = 129;
          numbering(251,3) = 30;
          numbering(252,0) = 115;
          numbering(252,1) = 131;
          numbering(252,2) = 124;
          numbering(252,3) = 129;
          numbering(253,0) = 134;
          numbering(253,1) = 131;
          numbering(253,2) = 124;
          numbering(253,3) = 115;
          numbering(254,0) = 134;
          numbering(254,1) = 128;
          numbering(254,2) = 131;
          numbering(254,3) = 115;
          numbering(255,0) = 115;
          numbering(255,1) = 128;
          numbering(255,2) = 131;
          numbering(255,3) = 129;
          numbering(256,0) = 4;
          numbering(256,1) = 62;
          numbering(256,2) = 44;
          numbering(256,3) = 45;
          numbering(257,0) = 62;
          numbering(257,1) = 18;
          numbering(257,2) = 135;
          numbering(257,3) = 136;
          numbering(258,0) = 44;
          numbering(258,1) = 135;
          numbering(258,2) = 13;
          numbering(258,3) = 46;
          numbering(259,0) = 45;
          numbering(259,1) = 136;
          numbering(259,2) = 46;
          numbering(259,3) = 14;
          numbering(260,0) = 45;
          numbering(260,1) = 135;
          numbering(260,2) = 44;
          numbering(260,3) = 46;
          numbering(261,0) = 62;
          numbering(261,1) = 135;
          numbering(261,2) = 44;
          numbering(261,3) = 45;
          numbering(262,0) = 62;
          numbering(262,1) = 136;
          numbering(262,2) = 135;
          numbering(262,3) = 45;
          numbering(263,0) = 45;
          numbering(263,1) = 136;
          numbering(263,2) = 135;
          numbering(263,3) = 46;
          numbering(264,0) = 18;
          numbering(264,1) = 80;
          numbering(264,2) = 137;
          numbering(264,3) = 138;
          numbering(265,0) = 80;
          numbering(265,1) = 8;
          numbering(265,2) = 139;
          numbering(265,3) = 116;
          numbering(266,0) = 137;
          numbering(266,1) = 139;
          numbering(266,2) = 34;
          numbering(266,3) = 140;
          numbering(267,0) = 138;
          numbering(267,1) = 116;
          numbering(267,2) = 140;
          numbering(267,3) = 28;
          numbering(268,0) = 138;
          numbering(268,1) = 139;
          numbering(268,2) = 137;
          numbering(268,3) = 140;
          numbering(269,0) = 80;
          numbering(269,1) = 139;
          numbering(269,2) = 137;
          numbering(269,3) = 138;
          numbering(270,0) = 80;
          numbering(270,1) = 116;
          numbering(270,2) = 139;
          numbering(270,3) = 138;
          numbering(271,0) = 138;
          numbering(271,1) = 116;
          numbering(271,2) = 139;
          numbering(271,3) = 140;
          numbering(272,0) = 13;
          numbering(272,1) = 141;
          numbering(272,2) = 50;
          numbering(272,3) = 51;
          numbering(273,0) = 141;
          numbering(273,1) = 34;
          numbering(273,2) = 142;
          numbering(273,3) = 143;
          numbering(274,0) = 50;
          numbering(274,1) = 142;
          numbering(274,2) = 5;
          numbering(274,3) = 52;
          numbering(275,0) = 51;
          numbering(275,1) = 143;
          numbering(275,2) = 52;
          numbering(275,3) = 15;
          numbering(276,0) = 51;
          numbering(276,1) = 142;
          numbering(276,2) = 50;
          numbering(276,3) = 52;
          numbering(277,0) = 141;
          numbering(277,1) = 142;
          numbering(277,2) = 50;
          numbering(277,3) = 51;
          numbering(278,0) = 141;
          numbering(278,1) = 143;
          numbering(278,2) = 142;
          numbering(278,3) = 51;
          numbering(279,0) = 51;
          numbering(279,1) = 143;
          numbering(279,2) = 142;
          numbering(279,3) = 52;
          numbering(280,0) = 14;
          numbering(280,1) = 144;
          numbering(280,2) = 56;
          numbering(280,3) = 57;
          numbering(281,0) = 144;
          numbering(281,1) = 28;
          numbering(281,2) = 145;
          numbering(281,3) = 110;
          numbering(282,0) = 56;
          numbering(282,1) = 145;
          numbering(282,2) = 15;
          numbering(282,3) = 58;
          numbering(283,0) = 57;
          numbering(283,1) = 110;
          numbering(283,2) = 58;
          numbering(283,3) = 6;
          numbering(284,0) = 57;
          numbering(284,1) = 145;
          numbering(284,2) = 56;
          numbering(284,3) = 58;
          numbering(285,0) = 144;
          numbering(285,1) = 145;
          numbering(285,2) = 56;
          numbering(285,3) = 57;
          numbering(286,0) = 144;
          numbering(286,1) = 110;
          numbering(286,2) = 145;
          numbering(286,3) = 57;
          numbering(287,0) = 57;
          numbering(287,1) = 110;
          numbering(287,2) = 145;
          numbering(287,3) = 58;
          numbering(288,0) = 14;
          numbering(288,1) = 146;
          numbering(288,2) = 46;
          numbering(288,3) = 56;
          numbering(289,0) = 146;
          numbering(289,1) = 34;
          numbering(289,2) = 141;
          numbering(289,3) = 143;
          numbering(290,0) = 46;
          numbering(290,1) = 141;
          numbering(290,2) = 13;
          numbering(290,3) = 51;
          numbering(291,0) = 56;
          numbering(291,1) = 143;
          numbering(291,2) = 51;
          numbering(291,3) = 15;
          numbering(292,0) = 146;
          numbering(292,1) = 143;
          numbering(292,2) = 46;
          numbering(292,3) = 56;
          numbering(293,0) = 146;
          numbering(293,1) = 141;
          numbering(293,2) = 46;
          numbering(293,3) = 143;
          numbering(294,0) = 46;
          numbering(294,1) = 141;
          numbering(294,2) = 51;
          numbering(294,3) = 143;
          numbering(295,0) = 46;
          numbering(295,1) = 143;
          numbering(295,2) = 51;
          numbering(295,3) = 56;
          numbering(296,0) = 18;
          numbering(296,1) = 137;
          numbering(296,2) = 135;
          numbering(296,3) = 136;
          numbering(297,0) = 137;
          numbering(297,1) = 34;
          numbering(297,2) = 141;
          numbering(297,3) = 146;
          numbering(298,0) = 135;
          numbering(298,1) = 141;
          numbering(298,2) = 13;
          numbering(298,3) = 46;
          numbering(299,0) = 136;
          numbering(299,1) = 146;
          numbering(299,2) = 46;
          numbering(299,3) = 14;
          numbering(300,0) = 137;
          numbering(300,1) = 146;
          numbering(300,2) = 135;
          numbering(300,3) = 136;
          numbering(301,0) = 137;
          numbering(301,1) = 141;
          numbering(301,2) = 135;
          numbering(301,3) = 146;
          numbering(302,0) = 135;
          numbering(302,1) = 141;
          numbering(302,2) = 46;
          numbering(302,3) = 146;
          numbering(303,0) = 135;
          numbering(303,1) = 146;
          numbering(303,2) = 46;
          numbering(303,3) = 136;
          numbering(304,0) = 18;
          numbering(304,1) = 138;
          numbering(304,2) = 137;
          numbering(304,3) = 136;
          numbering(305,0) = 138;
          numbering(305,1) = 28;
          numbering(305,2) = 140;
          numbering(305,3) = 144;
          numbering(306,0) = 137;
          numbering(306,1) = 140;
          numbering(306,2) = 34;
          numbering(306,3) = 146;
          numbering(307,0) = 136;
          numbering(307,1) = 144;
          numbering(307,2) = 146;
          numbering(307,3) = 14;
          numbering(308,0) = 138;
          numbering(308,1) = 144;
          numbering(308,2) = 137;
          numbering(308,3) = 136;
          numbering(309,0) = 138;
          numbering(309,1) = 140;
          numbering(309,2) = 137;
          numbering(309,3) = 144;
          numbering(310,0) = 137;
          numbering(310,1) = 140;
          numbering(310,2) = 146;
          numbering(310,3) = 144;
          numbering(311,0) = 137;
          numbering(311,1) = 144;
          numbering(311,2) = 146;
          numbering(311,3) = 136;
          numbering(312,0) = 14;
          numbering(312,1) = 144;
          numbering(312,2) = 146;
          numbering(312,3) = 56;
          numbering(313,0) = 144;
          numbering(313,1) = 28;
          numbering(313,2) = 140;
          numbering(313,3) = 145;
          numbering(314,0) = 146;
          numbering(314,1) = 140;
          numbering(314,2) = 34;
          numbering(314,3) = 143;
          numbering(315,0) = 56;
          numbering(315,1) = 145;
          numbering(315,2) = 143;
          numbering(315,3) = 15;
          numbering(316,0) = 144;
          numbering(316,1) = 145;
          numbering(316,2) = 146;
          numbering(316,3) = 56;
          numbering(317,0) = 144;
          numbering(317,1) = 140;
          numbering(317,2) = 146;
          numbering(317,3) = 145;
          numbering(318,0) = 146;
          numbering(318,1) = 140;
          numbering(318,2) = 143;
          numbering(318,3) = 145;
          numbering(319,0) = 146;
          numbering(319,1) = 145;
          numbering(319,2) = 143;
          numbering(319,3) = 56;
          numbering(320,0) = 4;
          numbering(320,1) = 61;
          numbering(320,2) = 44;
          numbering(320,3) = 62;
          numbering(321,0) = 61;
          numbering(321,1) = 17;
          numbering(321,2) = 147;
          numbering(321,3) = 65;
          numbering(322,0) = 44;
          numbering(322,1) = 147;
          numbering(322,2) = 13;
          numbering(322,3) = 135;
          numbering(323,0) = 62;
          numbering(323,1) = 65;
          numbering(323,2) = 135;
          numbering(323,3) = 18;
          numbering(324,0) = 61;
          numbering(324,1) = 65;
          numbering(324,2) = 44;
          numbering(324,3) = 62;
          numbering(325,0) = 61;
          numbering(325,1) = 147;
          numbering(325,2) = 44;
          numbering(325,3) = 65;
          numbering(326,0) = 44;
          numbering(326,1) = 147;
          numbering(326,2) = 135;
          numbering(326,3) = 65;
          numbering(327,0) = 44;
          numbering(327,1) = 65;
          numbering(327,2) = 135;
          numbering(327,3) = 62;
          numbering(328,0) = 17;
          numbering(328,1) = 73;
          numbering(328,2) = 148;
          numbering(328,3) = 74;
          numbering(329,0) = 73;
          numbering(329,1) = 7;
          numbering(329,2) = 91;
          numbering(329,3) = 77;
          numbering(330,0) = 148;
          numbering(330,1) = 91;
          numbering(330,2) = 22;
          numbering(330,3) = 149;
          numbering(331,0) = 74;
          numbering(331,1) = 77;
          numbering(331,2) = 149;
          numbering(331,3) = 21;
          numbering(332,0) = 73;
          numbering(332,1) = 77;
          numbering(332,2) = 148;
          numbering(332,3) = 74;
          numbering(333,0) = 73;
          numbering(333,1) = 91;
          numbering(333,2) = 148;
          numbering(333,3) = 77;
          numbering(334,0) = 148;
          numbering(334,1) = 91;
          numbering(334,2) = 149;
          numbering(334,3) = 77;
          numbering(335,0) = 148;
          numbering(335,1) = 77;
          numbering(335,2) = 149;
          numbering(335,3) = 74;
          numbering(336,0) = 13;
          numbering(336,1) = 150;
          numbering(336,2) = 50;
          numbering(336,3) = 141;
          numbering(337,0) = 150;
          numbering(337,1) = 22;
          numbering(337,2) = 85;
          numbering(337,3) = 151;
          numbering(338,0) = 50;
          numbering(338,1) = 85;
          numbering(338,2) = 5;
          numbering(338,3) = 142;
          numbering(339,0) = 141;
          numbering(339,1) = 151;
          numbering(339,2) = 142;
          numbering(339,3) = 34;
          numbering(340,0) = 150;
          numbering(340,1) = 151;
          numbering(340,2) = 50;
          numbering(340,3) = 141;
          numbering(341,0) = 150;
          numbering(341,1) = 85;
          numbering(341,2) = 50;
          numbering(341,3) = 151;
          numbering(342,0) = 50;
          numbering(342,1) = 85;
          numbering(342,2) = 142;
          numbering(342,3) = 151;
          numbering(343,0) = 50;
          numbering(343,1) = 151;
          numbering(343,2) = 142;
          numbering(343,3) = 141;
          numbering(344,0) = 18;
          numbering(344,1) = 79;
          numbering(344,2) = 137;
          numbering(344,3) = 80;
          numbering(345,0) = 79;
          numbering(345,1) = 21;
          numbering(345,2) = 152;
          numbering(345,3) = 83;
          numbering(346,0) = 137;
          numbering(346,1) = 152;
          numbering(346,2) = 34;
          numbering(346,3) = 139;
          numbering(347,0) = 80;
          numbering(347,1) = 83;
          numbering(347,2) = 139;
          numbering(347,3) = 8;
          numbering(348,0) = 79;
          numbering(348,1) = 83;
          numbering(348,2) = 137;
          numbering(348,3) = 80;
          numbering(349,0) = 79;
          numbering(349,1) = 152;
          numbering(349,2) = 137;
          numbering(349,3) = 83;
          numbering(350,0) = 137;
          numbering(350,1) = 152;
          numbering(350,2) = 139;
          numbering(350,3) = 83;
          numbering(351,0) = 137;
          numbering(351,1) = 83;
          numbering(351,2) = 139;
          numbering(351,3) = 80;
          numbering(352,0) = 17;
          numbering(352,1) = 74;
          numbering(352,2) = 147;
          numbering(352,3) = 65;
          numbering(353,0) = 74;
          numbering(353,1) = 21;
          numbering(353,2) = 153;
          numbering(353,3) = 79;
          numbering(354,0) = 147;
          numbering(354,1) = 153;
          numbering(354,2) = 13;
          numbering(354,3) = 135;
          numbering(355,0) = 65;
          numbering(355,1) = 79;
          numbering(355,2) = 135;
          numbering(355,3) = 18;
          numbering(356,0) = 65;
          numbering(356,1) = 153;
          numbering(356,2) = 147;
          numbering(356,3) = 135;
          numbering(357,0) = 74;
          numbering(357,1) = 153;
          numbering(357,2) = 147;
          numbering(357,3) = 65;
          numbering(358,0) = 74;
          numbering(358,1) = 79;
          numbering(358,2) = 153;
          numbering(358,3) = 65;
          numbering(359,0) = 65;
          numbering(359,1) = 79;
          numbering(359,2) = 153;
          numbering(359,3) = 135;
          numbering(360,0) = 17;
          numbering(360,1) = 148;
          numbering(360,2) = 147;
          numbering(360,3) = 74;
          numbering(361,0) = 148;
          numbering(361,1) = 22;
          numbering(361,2) = 150;
          numbering(361,3) = 149;
          numbering(362,0) = 147;
          numbering(362,1) = 150;
          numbering(362,2) = 13;
          numbering(362,3) = 153;
          numbering(363,0) = 74;
          numbering(363,1) = 149;
          numbering(363,2) = 153;
          numbering(363,3) = 21;
          numbering(364,0) = 148;
          numbering(364,1) = 149;
          numbering(364,2) = 147;
          numbering(364,3) = 74;
          numbering(365,0) = 148;
          numbering(365,1) = 150;
          numbering(365,2) = 147;
          numbering(365,3) = 149;
          numbering(366,0) = 147;
          numbering(366,1) = 150;
          numbering(366,2) = 153;
          numbering(366,3) = 149;
          numbering(367,0) = 147;
          numbering(367,1) = 149;
          numbering(367,2) = 153;
          numbering(367,3) = 74;
          numbering(368,0) = 13;
          numbering(368,1) = 150;
          numbering(368,2) = 141;
          numbering(368,3) = 153;
          numbering(369,0) = 150;
          numbering(369,1) = 22;
          numbering(369,2) = 151;
          numbering(369,3) = 149;
          numbering(370,0) = 141;
          numbering(370,1) = 151;
          numbering(370,2) = 34;
          numbering(370,3) = 152;
          numbering(371,0) = 153;
          numbering(371,1) = 149;
          numbering(371,2) = 152;
          numbering(371,3) = 21;
          numbering(372,0) = 150;
          numbering(372,1) = 149;
          numbering(372,2) = 141;
          numbering(372,3) = 153;
          numbering(373,0) = 150;
          numbering(373,1) = 151;
          numbering(373,2) = 141;
          numbering(373,3) = 149;
          numbering(374,0) = 141;
          numbering(374,1) = 151;
          numbering(374,2) = 152;
          numbering(374,3) = 149;
          numbering(375,0) = 141;
          numbering(375,1) = 149;
          numbering(375,2) = 152;
          numbering(375,3) = 153;
          numbering(376,0) = 13;
          numbering(376,1) = 153;
          numbering(376,2) = 141;
          numbering(376,3) = 135;
          numbering(377,0) = 153;
          numbering(377,1) = 21;
          numbering(377,2) = 152;
          numbering(377,3) = 79;
          numbering(378,0) = 141;
          numbering(378,1) = 152;
          numbering(378,2) = 34;
          numbering(378,3) = 137;
          numbering(379,0) = 135;
          numbering(379,1) = 79;
          numbering(379,2) = 137;
          numbering(379,3) = 18;
          numbering(380,0) = 135;
          numbering(380,1) = 152;
          numbering(380,2) = 141;
          numbering(380,3) = 137;
          numbering(381,0) = 153;
          numbering(381,1) = 152;
          numbering(381,2) = 141;
          numbering(381,3) = 135;
          numbering(382,0) = 153;
          numbering(382,1) = 79;
          numbering(382,2) = 152;
          numbering(382,3) = 135;
          numbering(383,0) = 135;
          numbering(383,1) = 79;
          numbering(383,2) = 152;
          numbering(383,3) = 137;
          numbering(384,0) = 5;
          numbering(384,1) = 85;
          numbering(384,2) = 87;
          numbering(384,3) = 142;
          numbering(385,0) = 85;
          numbering(385,1) = 22;
          numbering(385,2) = 89;
          numbering(385,3) = 151;
          numbering(386,0) = 87;
          numbering(386,1) = 89;
          numbering(386,2) = 24;
          numbering(386,3) = 154;
          numbering(387,0) = 142;
          numbering(387,1) = 151;
          numbering(387,2) = 154;
          numbering(387,3) = 34;
          numbering(388,0) = 142;
          numbering(388,1) = 89;
          numbering(388,2) = 87;
          numbering(388,3) = 154;
          numbering(389,0) = 85;
          numbering(389,1) = 89;
          numbering(389,2) = 87;
          numbering(389,3) = 142;
          numbering(390,0) = 85;
          numbering(390,1) = 151;
          numbering(390,2) = 89;
          numbering(390,3) = 142;
          numbering(391,0) = 142;
          numbering(391,1) = 151;
          numbering(391,2) = 89;
          numbering(391,3) = 154;
          numbering(392,0) = 22;
          numbering(392,1) = 91;
          numbering(392,2) = 93;
          numbering(392,3) = 149;
          numbering(393,0) = 91;
          numbering(393,1) = 7;
          numbering(393,2) = 95;
          numbering(393,3) = 77;
          numbering(394,0) = 93;
          numbering(394,1) = 95;
          numbering(394,2) = 26;
          numbering(394,3) = 155;
          numbering(395,0) = 149;
          numbering(395,1) = 77;
          numbering(395,2) = 155;
          numbering(395,3) = 21;
          numbering(396,0) = 149;
          numbering(396,1) = 95;
          numbering(396,2) = 93;
          numbering(396,3) = 155;
          numbering(397,0) = 91;
          numbering(397,1) = 95;
          numbering(397,2) = 93;
          numbering(397,3) = 149;
          numbering(398,0) = 91;
          numbering(398,1) = 77;
          numbering(398,2) = 95;
          numbering(398,3) = 149;
          numbering(399,0) = 149;
          numbering(399,1) = 77;
          numbering(399,2) = 95;
          numbering(399,3) = 155;
          numbering(400,0) = 24;
          numbering(400,1) = 103;
          numbering(400,2) = 105;
          numbering(400,3) = 156;
          numbering(401,0) = 103;
          numbering(401,1) = 26;
          numbering(401,2) = 107;
          numbering(401,3) = 157;
          numbering(402,0) = 105;
          numbering(402,1) = 107;
          numbering(402,2) = 9;
          numbering(402,3) = 125;
          numbering(403,0) = 156;
          numbering(403,1) = 157;
          numbering(403,2) = 125;
          numbering(403,3) = 31;
          numbering(404,0) = 156;
          numbering(404,1) = 107;
          numbering(404,2) = 105;
          numbering(404,3) = 125;
          numbering(405,0) = 103;
          numbering(405,1) = 107;
          numbering(405,2) = 105;
          numbering(405,3) = 156;
          numbering(406,0) = 103;
          numbering(406,1) = 157;
          numbering(406,2) = 107;
          numbering(406,3) = 156;
          numbering(407,0) = 156;
          numbering(407,1) = 157;
          numbering(407,2) = 107;
          numbering(407,3) = 125;
          numbering(408,0) = 34;
          numbering(408,1) = 152;
          numbering(408,2) = 158;
          numbering(408,3) = 139;
          numbering(409,0) = 152;
          numbering(409,1) = 21;
          numbering(409,2) = 159;
          numbering(409,3) = 83;
          numbering(410,0) = 158;
          numbering(410,1) = 159;
          numbering(410,2) = 31;
          numbering(410,3) = 119;
          numbering(411,0) = 139;
          numbering(411,1) = 83;
          numbering(411,2) = 119;
          numbering(411,3) = 8;
          numbering(412,0) = 139;
          numbering(412,1) = 159;
          numbering(412,2) = 158;
          numbering(412,3) = 119;
          numbering(413,0) = 152;
          numbering(413,1) = 159;
          numbering(413,2) = 158;
          numbering(413,3) = 139;
          numbering(414,0) = 152;
          numbering(414,1) = 83;
          numbering(414,2) = 159;
          numbering(414,3) = 139;
          numbering(415,0) = 139;
          numbering(415,1) = 83;
          numbering(415,2) = 159;
          numbering(415,3) = 119;
          numbering(416,0) = 34;
          numbering(416,1) = 160;
          numbering(416,2) = 154;
          numbering(416,3) = 158;
          numbering(417,0) = 160;
          numbering(417,1) = 26;
          numbering(417,2) = 103;
          numbering(417,3) = 157;
          numbering(418,0) = 154;
          numbering(418,1) = 103;
          numbering(418,2) = 24;
          numbering(418,3) = 156;
          numbering(419,0) = 158;
          numbering(419,1) = 157;
          numbering(419,2) = 156;
          numbering(419,3) = 31;
          numbering(420,0) = 160;
          numbering(420,1) = 157;
          numbering(420,2) = 154;
          numbering(420,3) = 158;
          numbering(421,0) = 160;
          numbering(421,1) = 103;
          numbering(421,2) = 154;
          numbering(421,3) = 157;
          numbering(422,0) = 154;
          numbering(422,1) = 103;
          numbering(422,2) = 156;
          numbering(422,3) = 157;
          numbering(423,0) = 154;
          numbering(423,1) = 157;
          numbering(423,2) = 156;
          numbering(423,3) = 158;
          numbering(424,0) = 22;
          numbering(424,1) = 93;
          numbering(424,2) = 89;
          numbering(424,3) = 151;
          numbering(425,0) = 93;
          numbering(425,1) = 26;
          numbering(425,2) = 103;
          numbering(425,3) = 160;
          numbering(426,0) = 89;
          numbering(426,1) = 103;
          numbering(426,2) = 24;
          numbering(426,3) = 154;
          numbering(427,0) = 151;
          numbering(427,1) = 160;
          numbering(427,2) = 154;
          numbering(427,3) = 34;
          numbering(428,0) = 93;
          numbering(428,1) = 160;
          numbering(428,2) = 89;
          numbering(428,3) = 151;
          numbering(429,0) = 93;
          numbering(429,1) = 103;
          numbering(429,2) = 89;
          numbering(429,3) = 160;
          numbering(430,0) = 89;
          numbering(430,1) = 103;
          numbering(430,2) = 154;
          numbering(430,3) = 160;
          numbering(431,0) = 89;
          numbering(431,1) = 160;
          numbering(431,2) = 154;
          numbering(431,3) = 151;
          numbering(432,0) = 22;
          numbering(432,1) = 149;
          numbering(432,2) = 93;
          numbering(432,3) = 151;
          numbering(433,0) = 149;
          numbering(433,1) = 21;
          numbering(433,2) = 155;
          numbering(433,3) = 152;
          numbering(434,0) = 93;
          numbering(434,1) = 155;
          numbering(434,2) = 26;
          numbering(434,3) = 160;
          numbering(435,0) = 151;
          numbering(435,1) = 152;
          numbering(435,2) = 160;
          numbering(435,3) = 34;
          numbering(436,0) = 151;
          numbering(436,1) = 155;
          numbering(436,2) = 93;
          numbering(436,3) = 160;
          numbering(437,0) = 149;
          numbering(437,1) = 155;
          numbering(437,2) = 93;
          numbering(437,3) = 151;
          numbering(438,0) = 149;
          numbering(438,1) = 152;
          numbering(438,2) = 155;
          numbering(438,3) = 151;
          numbering(439,0) = 151;
          numbering(439,1) = 152;
          numbering(439,2) = 155;
          numbering(439,3) = 160;
          numbering(440,0) = 34;
          numbering(440,1) = 152;
          numbering(440,2) = 160;
          numbering(440,3) = 158;
          numbering(441,0) = 152;
          numbering(441,1) = 21;
          numbering(441,2) = 155;
          numbering(441,3) = 159;
          numbering(442,0) = 160;
          numbering(442,1) = 155;
          numbering(442,2) = 26;
          numbering(442,3) = 157;
          numbering(443,0) = 158;
          numbering(443,1) = 159;
          numbering(443,2) = 157;
          numbering(443,3) = 31;
          numbering(444,0) = 152;
          numbering(444,1) = 159;
          numbering(444,2) = 160;
          numbering(444,3) = 158;
          numbering(445,0) = 152;
          numbering(445,1) = 155;
          numbering(445,2) = 160;
          numbering(445,3) = 159;
          numbering(446,0) = 160;
          numbering(446,1) = 155;
          numbering(446,2) = 157;
          numbering(446,3) = 159;
          numbering(447,0) = 160;
          numbering(447,1) = 159;
          numbering(447,2) = 157;
          numbering(447,3) = 158;
          numbering(448,0) = 5;
          numbering(448,1) = 142;
          numbering(448,2) = 87;
          numbering(448,3) = 52;
          numbering(449,0) = 142;
          numbering(449,1) = 34;
          numbering(449,2) = 154;
          numbering(449,3) = 143;
          numbering(450,0) = 87;
          numbering(450,1) = 154;
          numbering(450,2) = 24;
          numbering(450,3) = 161;
          numbering(451,0) = 52;
          numbering(451,1) = 143;
          numbering(451,2) = 161;
          numbering(451,3) = 15;
          numbering(452,0) = 52;
          numbering(452,1) = 154;
          numbering(452,2) = 87;
          numbering(452,3) = 161;
          numbering(453,0) = 142;
          numbering(453,1) = 154;
          numbering(453,2) = 87;
          numbering(453,3) = 52;
          numbering(454,0) = 142;
          numbering(454,1) = 143;
          numbering(454,2) = 154;
          numbering(454,3) = 52;
          numbering(455,0) = 52;
          numbering(455,1) = 143;
          numbering(455,2) = 154;
          numbering(455,3) = 161;
          numbering(456,0) = 34;
          numbering(456,1) = 139;
          numbering(456,2) = 158;
          numbering(456,3) = 140;
          numbering(457,0) = 139;
          numbering(457,1) = 8;
          numbering(457,2) = 119;
          numbering(457,3) = 116;
          numbering(458,0) = 158;
          numbering(458,1) = 119;
          numbering(458,2) = 31;
          numbering(458,3) = 117;
          numbering(459,0) = 140;
          numbering(459,1) = 116;
          numbering(459,2) = 117;
          numbering(459,3) = 28;
          numbering(460,0) = 140;
          numbering(460,1) = 119;
          numbering(460,2) = 158;
          numbering(460,3) = 117;
          numbering(461,0) = 139;
          numbering(461,1) = 119;
          numbering(461,2) = 158;
          numbering(461,3) = 140;
          numbering(462,0) = 139;
          numbering(462,1) = 116;
          numbering(462,2) = 119;
          numbering(462,3) = 140;
          numbering(463,0) = 140;
          numbering(463,1) = 116;
          numbering(463,2) = 119;
          numbering(463,3) = 117;
          numbering(464,0) = 24;
          numbering(464,1) = 156;
          numbering(464,2) = 105;
          numbering(464,3) = 162;
          numbering(465,0) = 156;
          numbering(465,1) = 31;
          numbering(465,2) = 125;
          numbering(465,3) = 122;
          numbering(466,0) = 105;
          numbering(466,1) = 125;
          numbering(466,2) = 9;
          numbering(466,3) = 123;
          numbering(467,0) = 162;
          numbering(467,1) = 122;
          numbering(467,2) = 123;
          numbering(467,3) = 29;
          numbering(468,0) = 162;
          numbering(468,1) = 125;
          numbering(468,2) = 105;
          numbering(468,3) = 123;
          numbering(469,0) = 156;
          numbering(469,1) = 125;
          numbering(469,2) = 105;
          numbering(469,3) = 162;
          numbering(470,0) = 156;
          numbering(470,1) = 122;
          numbering(470,2) = 125;
          numbering(470,3) = 162;
          numbering(471,0) = 162;
          numbering(471,1) = 122;
          numbering(471,2) = 125;
          numbering(471,3) = 123;
          numbering(472,0) = 15;
          numbering(472,1) = 145;
          numbering(472,2) = 163;
          numbering(472,3) = 58;
          numbering(473,0) = 145;
          numbering(473,1) = 28;
          numbering(473,2) = 113;
          numbering(473,3) = 110;
          numbering(474,0) = 163;
          numbering(474,1) = 113;
          numbering(474,2) = 29;
          numbering(474,3) = 111;
          numbering(475,0) = 58;
          numbering(475,1) = 110;
          numbering(475,2) = 111;
          numbering(475,3) = 6;
          numbering(476,0) = 58;
          numbering(476,1) = 113;
          numbering(476,2) = 163;
          numbering(476,3) = 111;
          numbering(477,0) = 145;
          numbering(477,1) = 113;
          numbering(477,2) = 163;
          numbering(477,3) = 58;
          numbering(478,0) = 145;
          numbering(478,1) = 110;
          numbering(478,2) = 113;
          numbering(478,3) = 58;
          numbering(479,0) = 58;
          numbering(479,1) = 110;
          numbering(479,2) = 113;
          numbering(479,3) = 111;
          numbering(480,0) = 15;
          numbering(480,1) = 164;
          numbering(480,2) = 161;
          numbering(480,3) = 163;
          numbering(481,0) = 164;
          numbering(481,1) = 31;
          numbering(481,2) = 156;
          numbering(481,3) = 122;
          numbering(482,0) = 161;
          numbering(482,1) = 156;
          numbering(482,2) = 24;
          numbering(482,3) = 162;
          numbering(483,0) = 163;
          numbering(483,1) = 122;
          numbering(483,2) = 162;
          numbering(483,3) = 29;
          numbering(484,0) = 163;
          numbering(484,1) = 156;
          numbering(484,2) = 161;
          numbering(484,3) = 162;
          numbering(485,0) = 164;
          numbering(485,1) = 156;
          numbering(485,2) = 161;
          numbering(485,3) = 163;
          numbering(486,0) = 164;
          numbering(486,1) = 122;
          numbering(486,2) = 156;
          numbering(486,3) = 163;
          numbering(487,0) = 163;
          numbering(487,1) = 122;
          numbering(487,2) = 156;
          numbering(487,3) = 162;
          numbering(488,0) = 34;
          numbering(488,1) = 158;
          numbering(488,2) = 154;
          numbering(488,3) = 143;
          numbering(489,0) = 158;
          numbering(489,1) = 31;
          numbering(489,2) = 156;
          numbering(489,3) = 164;
          numbering(490,0) = 154;
          numbering(490,1) = 156;
          numbering(490,2) = 24;
          numbering(490,3) = 161;
          numbering(491,0) = 143;
          numbering(491,1) = 164;
          numbering(491,2) = 161;
          numbering(491,3) = 15;
          numbering(492,0) = 158;
          numbering(492,1) = 164;
          numbering(492,2) = 154;
          numbering(492,3) = 143;
          numbering(493,0) = 158;
          numbering(493,1) = 156;
          numbering(493,2) = 154;
          numbering(493,3) = 164;
          numbering(494,0) = 154;
          numbering(494,1) = 156;
          numbering(494,2) = 161;
          numbering(494,3) = 164;
          numbering(495,0) = 154;
          numbering(495,1) = 164;
          numbering(495,2) = 161;
          numbering(495,3) = 143;
          numbering(496,0) = 34;
          numbering(496,1) = 140;
          numbering(496,2) = 158;
          numbering(496,3) = 143;
          numbering(497,0) = 140;
          numbering(497,1) = 28;
          numbering(497,2) = 117;
          numbering(497,3) = 145;
          numbering(498,0) = 158;
          numbering(498,1) = 117;
          numbering(498,2) = 31;
          numbering(498,3) = 164;
          numbering(499,0) = 143;
          numbering(499,1) = 145;
          numbering(499,2) = 164;
          numbering(499,3) = 15;
          numbering(500,0) = 140;
          numbering(500,1) = 145;
          numbering(500,2) = 158;
          numbering(500,3) = 143;
          numbering(501,0) = 140;
          numbering(501,1) = 117;
          numbering(501,2) = 158;
          numbering(501,3) = 145;
          numbering(502,0) = 158;
          numbering(502,1) = 117;
          numbering(502,2) = 164;
          numbering(502,3) = 145;
          numbering(503,0) = 158;
          numbering(503,1) = 145;
          numbering(503,2) = 164;
          numbering(503,3) = 143;
          numbering(504,0) = 15;
          numbering(504,1) = 145;
          numbering(504,2) = 164;
          numbering(504,3) = 163;
          numbering(505,0) = 145;
          numbering(505,1) = 28;
          numbering(505,2) = 117;
          numbering(505,3) = 113;
          numbering(506,0) = 164;
          numbering(506,1) = 117;
          numbering(506,2) = 31;
          numbering(506,3) = 122;
          numbering(507,0) = 163;
          numbering(507,1) = 113;
          numbering(507,2) = 122;
          numbering(507,3) = 29;
          numbering(508,0) = 145;
          numbering(508,1) = 113;
          numbering(508,2) = 164;
          numbering(508,3) = 163;
          numbering(509,0) = 145;
          numbering(509,1) = 117;
          numbering(509,2) = 164;
          numbering(509,3) = 113;
          numbering(510,0) = 164;
          numbering(510,1) = 117;
          numbering(510,2) = 122;
          numbering(510,3) = 113;
          numbering(511,0) = 164;
          numbering(511,1) = 113;
          numbering(511,2) = 122;
          numbering(511,3) = 163;
          return;
        }
      }
    } 
    case SpaMesh::hex:
    {
      switch(order)
      {
        case 0:
        case 1:
        {
          numbering.resize(1,8);
          numbering(0,0) = 0;
          numbering(0,1) = 1;
          numbering(0,2) = 2;
          numbering(0,3) = 3;
          numbering(0,4) = 4;
          numbering(0,5) = 5;
          numbering(0,6) = 6;
          numbering(0,7) = 7;
          return;
        }
        case 2:
        {
          numbering.resize(8,8);
          numbering(0,0) = 0;
          numbering(0,1) = 8;
          numbering(0,2) = 9;
          numbering(0,3) = 10;
          numbering(0,4) = 11;
          numbering(0,5) = 12;
          numbering(0,6) = 13;
          numbering(0,7) = 14;
          numbering(1,0) = 8;
          numbering(1,1) = 1;
          numbering(1,2) = 15;
          numbering(1,3) = 9;
          numbering(1,4) = 12;
          numbering(1,5) = 16;
          numbering(1,6) = 17;
          numbering(1,7) = 13;
          numbering(2,0) = 10;
          numbering(2,1) = 9;
          numbering(2,2) = 18;
          numbering(2,3) = 3;
          numbering(2,4) = 14;
          numbering(2,5) = 13;
          numbering(2,6) = 19;
          numbering(2,7) = 20;
          numbering(3,0) = 9;
          numbering(3,1) = 15;
          numbering(3,2) = 2;
          numbering(3,3) = 18;
          numbering(3,4) = 13;
          numbering(3,5) = 17;
          numbering(3,6) = 21;
          numbering(3,7) = 19;
          numbering(4,0) = 11;
          numbering(4,1) = 12;
          numbering(4,2) = 13;
          numbering(4,3) = 14;
          numbering(4,4) = 4;
          numbering(4,5) = 22;
          numbering(4,6) = 23;
          numbering(4,7) = 24;
          numbering(5,0) = 12;
          numbering(5,1) = 16;
          numbering(5,2) = 17;
          numbering(5,3) = 13;
          numbering(5,4) = 22;
          numbering(5,5) = 5;
          numbering(5,6) = 25;
          numbering(5,7) = 23;
          numbering(6,0) = 14;
          numbering(6,1) = 13;
          numbering(6,2) = 19;
          numbering(6,3) = 20;
          numbering(6,4) = 24;
          numbering(6,5) = 23;
          numbering(6,6) = 26;
          numbering(6,7) = 7;
          numbering(7,0) = 13;
          numbering(7,1) = 17;
          numbering(7,2) = 21;
          numbering(7,3) = 19;
          numbering(7,4) = 23;
          numbering(7,5) = 25;
          numbering(7,6) = 6;
          numbering(7,7) = 26;
          return;
        }
        case 3:
        case 4:
        {
          numbering.resize(64,8);
          numbering(0,0) = 0;
          numbering(0,1) = 27;
          numbering(0,2) = 28;
          numbering(0,3) = 29;
          numbering(0,4) = 30;
          numbering(0,5) = 31;
          numbering(0,6) = 32;
          numbering(0,7) = 33;
          numbering(1,0) = 27;
          numbering(1,1) = 8;
          numbering(1,2) = 34;
          numbering(1,3) = 28;
          numbering(1,4) = 31;
          numbering(1,5) = 35;
          numbering(1,6) = 36;
          numbering(1,7) = 32;
          numbering(2,0) = 29;
          numbering(2,1) = 28;
          numbering(2,2) = 37;
          numbering(2,3) = 10;
          numbering(2,4) = 33;
          numbering(2,5) = 32;
          numbering(2,6) = 38;
          numbering(2,7) = 39;
          numbering(3,0) = 28;
          numbering(3,1) = 34;
          numbering(3,2) = 9;
          numbering(3,3) = 37;
          numbering(3,4) = 32;
          numbering(3,5) = 36;
          numbering(3,6) = 40;
          numbering(3,7) = 38;
          numbering(4,0) = 30;
          numbering(4,1) = 31;
          numbering(4,2) = 32;
          numbering(4,3) = 33;
          numbering(4,4) = 11;
          numbering(4,5) = 41;
          numbering(4,6) = 42;
          numbering(4,7) = 43;
          numbering(5,0) = 31;
          numbering(5,1) = 35;
          numbering(5,2) = 36;
          numbering(5,3) = 32;
          numbering(5,4) = 41;
          numbering(5,5) = 12;
          numbering(5,6) = 44;
          numbering(5,7) = 42;
          numbering(6,0) = 33;
          numbering(6,1) = 32;
          numbering(6,2) = 38;
          numbering(6,3) = 39;
          numbering(6,4) = 43;
          numbering(6,5) = 42;
          numbering(6,6) = 45;
          numbering(6,7) = 14;
          numbering(7,0) = 32;
          numbering(7,1) = 36;
          numbering(7,2) = 40;
          numbering(7,3) = 38;
          numbering(7,4) = 42;
          numbering(7,5) = 44;
          numbering(7,6) = 13;
          numbering(7,7) = 45;
          numbering(8,0) = 8;
          numbering(8,1) = 46;
          numbering(8,2) = 47;
          numbering(8,3) = 34;
          numbering(8,4) = 35;
          numbering(8,5) = 48;
          numbering(8,6) = 49;
          numbering(8,7) = 36;
          numbering(9,0) = 46;
          numbering(9,1) = 1;
          numbering(9,2) = 50;
          numbering(9,3) = 47;
          numbering(9,4) = 48;
          numbering(9,5) = 51;
          numbering(9,6) = 52;
          numbering(9,7) = 49;
          numbering(10,0) = 34;
          numbering(10,1) = 47;
          numbering(10,2) = 53;
          numbering(10,3) = 9;
          numbering(10,4) = 36;
          numbering(10,5) = 49;
          numbering(10,6) = 54;
          numbering(10,7) = 40;
          numbering(11,0) = 47;
          numbering(11,1) = 50;
          numbering(11,2) = 15;
          numbering(11,3) = 53;
          numbering(11,4) = 49;
          numbering(11,5) = 52;
          numbering(11,6) = 55;
          numbering(11,7) = 54;
          numbering(12,0) = 35;
          numbering(12,1) = 48;
          numbering(12,2) = 49;
          numbering(12,3) = 36;
          numbering(12,4) = 12;
          numbering(12,5) = 56;
          numbering(12,6) = 57;
          numbering(12,7) = 44;
          numbering(13,0) = 48;
          numbering(13,1) = 51;
          numbering(13,2) = 52;
          numbering(13,3) = 49;
          numbering(13,4) = 56;
          numbering(13,5) = 16;
          numbering(13,6) = 58;
          numbering(13,7) = 57;
          numbering(14,0) = 36;
          numbering(14,1) = 49;
          numbering(14,2) = 54;
          numbering(14,3) = 40;
          numbering(14,4) = 44;
          numbering(14,5) = 57;
          numbering(14,6) = 59;
          numbering(14,7) = 13;
          numbering(15,0) = 49;
          numbering(15,1) = 52;
          numbering(15,2) = 55;
          numbering(15,3) = 54;
          numbering(15,4) = 57;
          numbering(15,5) = 58;
          numbering(15,6) = 17;
          numbering(15,7) = 59;
          numbering(16,0) = 10;
          numbering(16,1) = 37;
          numbering(16,2) = 60;
          numbering(16,3) = 61;
          numbering(16,4) = 39;
          numbering(16,5) = 38;
          numbering(16,6) = 62;
          numbering(16,7) = 63;
          numbering(17,0) = 37;
          numbering(17,1) = 9;
          numbering(17,2) = 64;
          numbering(17,3) = 60;
          numbering(17,4) = 38;
          numbering(17,5) = 40;
          numbering(17,6) = 65;
          numbering(17,7) = 62;
          numbering(18,0) = 61;
          numbering(18,1) = 60;
          numbering(18,2) = 66;
          numbering(18,3) = 3;
          numbering(18,4) = 63;
          numbering(18,5) = 62;
          numbering(18,6) = 67;
          numbering(18,7) = 68;
          numbering(19,0) = 60;
          numbering(19,1) = 64;
          numbering(19,2) = 18;
          numbering(19,3) = 66;
          numbering(19,4) = 62;
          numbering(19,5) = 65;
          numbering(19,6) = 69;
          numbering(19,7) = 67;
          numbering(20,0) = 39;
          numbering(20,1) = 38;
          numbering(20,2) = 62;
          numbering(20,3) = 63;
          numbering(20,4) = 14;
          numbering(20,5) = 45;
          numbering(20,6) = 70;
          numbering(20,7) = 71;
          numbering(21,0) = 38;
          numbering(21,1) = 40;
          numbering(21,2) = 65;
          numbering(21,3) = 62;
          numbering(21,4) = 45;
          numbering(21,5) = 13;
          numbering(21,6) = 72;
          numbering(21,7) = 70;
          numbering(22,0) = 63;
          numbering(22,1) = 62;
          numbering(22,2) = 67;
          numbering(22,3) = 68;
          numbering(22,4) = 71;
          numbering(22,5) = 70;
          numbering(22,6) = 73;
          numbering(22,7) = 20;
          numbering(23,0) = 62;
          numbering(23,1) = 65;
          numbering(23,2) = 69;
          numbering(23,3) = 67;
          numbering(23,4) = 70;
          numbering(23,5) = 72;
          numbering(23,6) = 19;
          numbering(23,7) = 73;
          numbering(24,0) = 9;
          numbering(24,1) = 53;
          numbering(24,2) = 74;
          numbering(24,3) = 64;
          numbering(24,4) = 40;
          numbering(24,5) = 54;
          numbering(24,6) = 75;
          numbering(24,7) = 65;
          numbering(25,0) = 53;
          numbering(25,1) = 15;
          numbering(25,2) = 76;
          numbering(25,3) = 74;
          numbering(25,4) = 54;
          numbering(25,5) = 55;
          numbering(25,6) = 77;
          numbering(25,7) = 75;
          numbering(26,0) = 64;
          numbering(26,1) = 74;
          numbering(26,2) = 78;
          numbering(26,3) = 18;
          numbering(26,4) = 65;
          numbering(26,5) = 75;
          numbering(26,6) = 79;
          numbering(26,7) = 69;
          numbering(27,0) = 74;
          numbering(27,1) = 76;
          numbering(27,2) = 2;
          numbering(27,3) = 78;
          numbering(27,4) = 75;
          numbering(27,5) = 77;
          numbering(27,6) = 80;
          numbering(27,7) = 79;
          numbering(28,0) = 40;
          numbering(28,1) = 54;
          numbering(28,2) = 75;
          numbering(28,3) = 65;
          numbering(28,4) = 13;
          numbering(28,5) = 59;
          numbering(28,6) = 81;
          numbering(28,7) = 72;
          numbering(29,0) = 54;
          numbering(29,1) = 55;
          numbering(29,2) = 77;
          numbering(29,3) = 75;
          numbering(29,4) = 59;
          numbering(29,5) = 17;
          numbering(29,6) = 82;
          numbering(29,7) = 81;
          numbering(30,0) = 65;
          numbering(30,1) = 75;
          numbering(30,2) = 79;
          numbering(30,3) = 69;
          numbering(30,4) = 72;
          numbering(30,5) = 81;
          numbering(30,6) = 83;
          numbering(30,7) = 19;
          numbering(31,0) = 75;
          numbering(31,1) = 77;
          numbering(31,2) = 80;
          numbering(31,3) = 79;
          numbering(31,4) = 81;
          numbering(31,5) = 82;
          numbering(31,6) = 21;
          numbering(31,7) = 83;
          numbering(32,0) = 11;
          numbering(32,1) = 41;
          numbering(32,2) = 42;
          numbering(32,3) = 43;
          numbering(32,4) = 84;
          numbering(32,5) = 85;
          numbering(32,6) = 86;
          numbering(32,7) = 87;
          numbering(33,0) = 41;
          numbering(33,1) = 12;
          numbering(33,2) = 44;
          numbering(33,3) = 42;
          numbering(33,4) = 85;
          numbering(33,5) = 88;
          numbering(33,6) = 89;
          numbering(33,7) = 86;
          numbering(34,0) = 43;
          numbering(34,1) = 42;
          numbering(34,2) = 45;
          numbering(34,3) = 14;
          numbering(34,4) = 87;
          numbering(34,5) = 86;
          numbering(34,6) = 90;
          numbering(34,7) = 91;
          numbering(35,0) = 42;
          numbering(35,1) = 44;
          numbering(35,2) = 13;
          numbering(35,3) = 45;
          numbering(35,4) = 86;
          numbering(35,5) = 89;
          numbering(35,6) = 92;
          numbering(35,7) = 90;
          numbering(36,0) = 84;
          numbering(36,1) = 85;
          numbering(36,2) = 86;
          numbering(36,3) = 87;
          numbering(36,4) = 4;
          numbering(36,5) = 93;
          numbering(36,6) = 94;
          numbering(36,7) = 95;
          numbering(37,0) = 85;
          numbering(37,1) = 88;
          numbering(37,2) = 89;
          numbering(37,3) = 86;
          numbering(37,4) = 93;
          numbering(37,5) = 22;
          numbering(37,6) = 96;
          numbering(37,7) = 94;
          numbering(38,0) = 87;
          numbering(38,1) = 86;
          numbering(38,2) = 90;
          numbering(38,3) = 91;
          numbering(38,4) = 95;
          numbering(38,5) = 94;
          numbering(38,6) = 97;
          numbering(38,7) = 24;
          numbering(39,0) = 86;
          numbering(39,1) = 89;
          numbering(39,2) = 92;
          numbering(39,3) = 90;
          numbering(39,4) = 94;
          numbering(39,5) = 96;
          numbering(39,6) = 23;
          numbering(39,7) = 97;
          numbering(40,0) = 12;
          numbering(40,1) = 56;
          numbering(40,2) = 57;
          numbering(40,3) = 44;
          numbering(40,4) = 88;
          numbering(40,5) = 98;
          numbering(40,6) = 99;
          numbering(40,7) = 89;
          numbering(41,0) = 56;
          numbering(41,1) = 16;
          numbering(41,2) = 58;
          numbering(41,3) = 57;
          numbering(41,4) = 98;
          numbering(41,5) = 100;
          numbering(41,6) = 101;
          numbering(41,7) = 99;
          numbering(42,0) = 44;
          numbering(42,1) = 57;
          numbering(42,2) = 59;
          numbering(42,3) = 13;
          numbering(42,4) = 89;
          numbering(42,5) = 99;
          numbering(42,6) = 102;
          numbering(42,7) = 92;
          numbering(43,0) = 57;
          numbering(43,1) = 58;
          numbering(43,2) = 17;
          numbering(43,3) = 59;
          numbering(43,4) = 99;
          numbering(43,5) = 101;
          numbering(43,6) = 103;
          numbering(43,7) = 102;
          numbering(44,0) = 88;
          numbering(44,1) = 98;
          numbering(44,2) = 99;
          numbering(44,3) = 89;
          numbering(44,4) = 22;
          numbering(44,5) = 104;
          numbering(44,6) = 105;
          numbering(44,7) = 96;
          numbering(45,0) = 98;
          numbering(45,1) = 100;
          numbering(45,2) = 101;
          numbering(45,3) = 99;
          numbering(45,4) = 104;
          numbering(45,5) = 5;
          numbering(45,6) = 106;
          numbering(45,7) = 105;
          numbering(46,0) = 89;
          numbering(46,1) = 99;
          numbering(46,2) = 102;
          numbering(46,3) = 92;
          numbering(46,4) = 96;
          numbering(46,5) = 105;
          numbering(46,6) = 107;
          numbering(46,7) = 23;
          numbering(47,0) = 99;
          numbering(47,1) = 101;
          numbering(47,2) = 103;
          numbering(47,3) = 102;
          numbering(47,4) = 105;
          numbering(47,5) = 106;
          numbering(47,6) = 25;
          numbering(47,7) = 107;
          numbering(48,0) = 14;
          numbering(48,1) = 45;
          numbering(48,2) = 70;
          numbering(48,3) = 71;
          numbering(48,4) = 91;
          numbering(48,5) = 90;
          numbering(48,6) = 108;
          numbering(48,7) = 109;
          numbering(49,0) = 45;
          numbering(49,1) = 13;
          numbering(49,2) = 72;
          numbering(49,3) = 70;
          numbering(49,4) = 90;
          numbering(49,5) = 92;
          numbering(49,6) = 110;
          numbering(49,7) = 108;
          numbering(50,0) = 71;
          numbering(50,1) = 70;
          numbering(50,2) = 73;
          numbering(50,3) = 20;
          numbering(50,4) = 109;
          numbering(50,5) = 108;
          numbering(50,6) = 111;
          numbering(50,7) = 112;
          numbering(51,0) = 70;
          numbering(51,1) = 72;
          numbering(51,2) = 19;
          numbering(51,3) = 73;
          numbering(51,4) = 108;
          numbering(51,5) = 110;
          numbering(51,6) = 113;
          numbering(51,7) = 111;
          numbering(52,0) = 91;
          numbering(52,1) = 90;
          numbering(52,2) = 108;
          numbering(52,3) = 109;
          numbering(52,4) = 24;
          numbering(52,5) = 97;
          numbering(52,6) = 114;
          numbering(52,7) = 115;
          numbering(53,0) = 90;
          numbering(53,1) = 92;
          numbering(53,2) = 110;
          numbering(53,3) = 108;
          numbering(53,4) = 97;
          numbering(53,5) = 23;
          numbering(53,6) = 116;
          numbering(53,7) = 114;
          numbering(54,0) = 109;
          numbering(54,1) = 108;
          numbering(54,2) = 111;
          numbering(54,3) = 112;
          numbering(54,4) = 115;
          numbering(54,5) = 114;
          numbering(54,6) = 117;
          numbering(54,7) = 7;
          numbering(55,0) = 108;
          numbering(55,1) = 110;
          numbering(55,2) = 113;
          numbering(55,3) = 111;
          numbering(55,4) = 114;
          numbering(55,5) = 116;
          numbering(55,6) = 26;
          numbering(55,7) = 117;
          numbering(56,0) = 13;
          numbering(56,1) = 59;
          numbering(56,2) = 81;
          numbering(56,3) = 72;
          numbering(56,4) = 92;
          numbering(56,5) = 102;
          numbering(56,6) = 118;
          numbering(56,7) = 110;
          numbering(57,0) = 59;
          numbering(57,1) = 17;
          numbering(57,2) = 82;
          numbering(57,3) = 81;
          numbering(57,4) = 102;
          numbering(57,5) = 103;
          numbering(57,6) = 119;
          numbering(57,7) = 118;
          numbering(58,0) = 72;
          numbering(58,1) = 81;
          numbering(58,2) = 83;
          numbering(58,3) = 19;
          numbering(58,4) = 110;
          numbering(58,5) = 118;
          numbering(58,6) = 120;
          numbering(58,7) = 113;
          numbering(59,0) = 81;
          numbering(59,1) = 82;
          numbering(59,2) = 21;
          numbering(59,3) = 83;
          numbering(59,4) = 118;
          numbering(59,5) = 119;
          numbering(59,6) = 121;
          numbering(59,7) = 120;
          numbering(60,0) = 92;
          numbering(60,1) = 102;
          numbering(60,2) = 118;
          numbering(60,3) = 110;
          numbering(60,4) = 23;
          numbering(60,5) = 107;
          numbering(60,6) = 122;
          numbering(60,7) = 116;
          numbering(61,0) = 102;
          numbering(61,1) = 103;
          numbering(61,2) = 119;
          numbering(61,3) = 118;
          numbering(61,4) = 107;
          numbering(61,5) = 25;
          numbering(61,6) = 123;
          numbering(61,7) = 122;
          numbering(62,0) = 110;
          numbering(62,1) = 118;
          numbering(62,2) = 120;
          numbering(62,3) = 113;
          numbering(62,4) = 116;
          numbering(62,5) = 122;
          numbering(62,6) = 124;
          numbering(62,7) = 26;
          numbering(63,0) = 118;
          numbering(63,1) = 119;
          numbering(63,2) = 121;
          numbering(63,3) = 120;
          numbering(63,4) = 122;
          numbering(63,5) = 123;
          numbering(63,6) = 6;
          numbering(63,7) = 124;
          return;
        }
        default:
        {
          numbering.resize(512,8);
          numbering(0,0) = 0;
          numbering(0,1) = 125;
          numbering(0,2) = 126;
          numbering(0,3) = 127;
          numbering(0,4) = 128;
          numbering(0,5) = 129;
          numbering(0,6) = 130;
          numbering(0,7) = 131;
          numbering(1,0) = 125;
          numbering(1,1) = 27;
          numbering(1,2) = 132;
          numbering(1,3) = 126;
          numbering(1,4) = 129;
          numbering(1,5) = 133;
          numbering(1,6) = 134;
          numbering(1,7) = 130;
          numbering(2,0) = 127;
          numbering(2,1) = 126;
          numbering(2,2) = 135;
          numbering(2,3) = 29;
          numbering(2,4) = 131;
          numbering(2,5) = 130;
          numbering(2,6) = 136;
          numbering(2,7) = 137;
          numbering(3,0) = 126;
          numbering(3,1) = 132;
          numbering(3,2) = 28;
          numbering(3,3) = 135;
          numbering(3,4) = 130;
          numbering(3,5) = 134;
          numbering(3,6) = 138;
          numbering(3,7) = 136;
          numbering(4,0) = 128;
          numbering(4,1) = 129;
          numbering(4,2) = 130;
          numbering(4,3) = 131;
          numbering(4,4) = 30;
          numbering(4,5) = 139;
          numbering(4,6) = 140;
          numbering(4,7) = 141;
          numbering(5,0) = 129;
          numbering(5,1) = 133;
          numbering(5,2) = 134;
          numbering(5,3) = 130;
          numbering(5,4) = 139;
          numbering(5,5) = 31;
          numbering(5,6) = 142;
          numbering(5,7) = 140;
          numbering(6,0) = 131;
          numbering(6,1) = 130;
          numbering(6,2) = 136;
          numbering(6,3) = 137;
          numbering(6,4) = 141;
          numbering(6,5) = 140;
          numbering(6,6) = 143;
          numbering(6,7) = 33;
          numbering(7,0) = 130;
          numbering(7,1) = 134;
          numbering(7,2) = 138;
          numbering(7,3) = 136;
          numbering(7,4) = 140;
          numbering(7,5) = 142;
          numbering(7,6) = 32;
          numbering(7,7) = 143;
          numbering(8,0) = 27;
          numbering(8,1) = 144;
          numbering(8,2) = 145;
          numbering(8,3) = 132;
          numbering(8,4) = 133;
          numbering(8,5) = 146;
          numbering(8,6) = 147;
          numbering(8,7) = 134;
          numbering(9,0) = 144;
          numbering(9,1) = 8;
          numbering(9,2) = 148;
          numbering(9,3) = 145;
          numbering(9,4) = 146;
          numbering(9,5) = 149;
          numbering(9,6) = 150;
          numbering(9,7) = 147;
          numbering(10,0) = 132;
          numbering(10,1) = 145;
          numbering(10,2) = 151;
          numbering(10,3) = 28;
          numbering(10,4) = 134;
          numbering(10,5) = 147;
          numbering(10,6) = 152;
          numbering(10,7) = 138;
          numbering(11,0) = 145;
          numbering(11,1) = 148;
          numbering(11,2) = 34;
          numbering(11,3) = 151;
          numbering(11,4) = 147;
          numbering(11,5) = 150;
          numbering(11,6) = 153;
          numbering(11,7) = 152;
          numbering(12,0) = 133;
          numbering(12,1) = 146;
          numbering(12,2) = 147;
          numbering(12,3) = 134;
          numbering(12,4) = 31;
          numbering(12,5) = 154;
          numbering(12,6) = 155;
          numbering(12,7) = 142;
          numbering(13,0) = 146;
          numbering(13,1) = 149;
          numbering(13,2) = 150;
          numbering(13,3) = 147;
          numbering(13,4) = 154;
          numbering(13,5) = 35;
          numbering(13,6) = 156;
          numbering(13,7) = 155;
          numbering(14,0) = 134;
          numbering(14,1) = 147;
          numbering(14,2) = 152;
          numbering(14,3) = 138;
          numbering(14,4) = 142;
          numbering(14,5) = 155;
          numbering(14,6) = 157;
          numbering(14,7) = 32;
          numbering(15,0) = 147;
          numbering(15,1) = 150;
          numbering(15,2) = 153;
          numbering(15,3) = 152;
          numbering(15,4) = 155;
          numbering(15,5) = 156;
          numbering(15,6) = 36;
          numbering(15,7) = 157;
          numbering(16,0) = 29;
          numbering(16,1) = 135;
          numbering(16,2) = 158;
          numbering(16,3) = 159;
          numbering(16,4) = 137;
          numbering(16,5) = 136;
          numbering(16,6) = 160;
          numbering(16,7) = 161;
          numbering(17,0) = 135;
          numbering(17,1) = 28;
          numbering(17,2) = 162;
          numbering(17,3) = 158;
          numbering(17,4) = 136;
          numbering(17,5) = 138;
          numbering(17,6) = 163;
          numbering(17,7) = 160;
          numbering(18,0) = 159;
          numbering(18,1) = 158;
          numbering(18,2) = 164;
          numbering(18,3) = 10;
          numbering(18,4) = 161;
          numbering(18,5) = 160;
          numbering(18,6) = 165;
          numbering(18,7) = 166;
          numbering(19,0) = 158;
          numbering(19,1) = 162;
          numbering(19,2) = 37;
          numbering(19,3) = 164;
          numbering(19,4) = 160;
          numbering(19,5) = 163;
          numbering(19,6) = 167;
          numbering(19,7) = 165;
          numbering(20,0) = 137;
          numbering(20,1) = 136;
          numbering(20,2) = 160;
          numbering(20,3) = 161;
          numbering(20,4) = 33;
          numbering(20,5) = 143;
          numbering(20,6) = 168;
          numbering(20,7) = 169;
          numbering(21,0) = 136;
          numbering(21,1) = 138;
          numbering(21,2) = 163;
          numbering(21,3) = 160;
          numbering(21,4) = 143;
          numbering(21,5) = 32;
          numbering(21,6) = 170;
          numbering(21,7) = 168;
          numbering(22,0) = 161;
          numbering(22,1) = 160;
          numbering(22,2) = 165;
          numbering(22,3) = 166;
          numbering(22,4) = 169;
          numbering(22,5) = 168;
          numbering(22,6) = 171;
          numbering(22,7) = 39;
          numbering(23,0) = 160;
          numbering(23,1) = 163;
          numbering(23,2) = 167;
          numbering(23,3) = 165;
          numbering(23,4) = 168;
          numbering(23,5) = 170;
          numbering(23,6) = 38;
          numbering(23,7) = 171;
          numbering(24,0) = 28;
          numbering(24,1) = 151;
          numbering(24,2) = 172;
          numbering(24,3) = 162;
          numbering(24,4) = 138;
          numbering(24,5) = 152;
          numbering(24,6) = 173;
          numbering(24,7) = 163;
          numbering(25,0) = 151;
          numbering(25,1) = 34;
          numbering(25,2) = 174;
          numbering(25,3) = 172;
          numbering(25,4) = 152;
          numbering(25,5) = 153;
          numbering(25,6) = 175;
          numbering(25,7) = 173;
          numbering(26,0) = 162;
          numbering(26,1) = 172;
          numbering(26,2) = 176;
          numbering(26,3) = 37;
          numbering(26,4) = 163;
          numbering(26,5) = 173;
          numbering(26,6) = 177;
          numbering(26,7) = 167;
          numbering(27,0) = 172;
          numbering(27,1) = 174;
          numbering(27,2) = 9;
          numbering(27,3) = 176;
          numbering(27,4) = 173;
          numbering(27,5) = 175;
          numbering(27,6) = 178;
          numbering(27,7) = 177;
          numbering(28,0) = 138;
          numbering(28,1) = 152;
          numbering(28,2) = 173;
          numbering(28,3) = 163;
          numbering(28,4) = 32;
          numbering(28,5) = 157;
          numbering(28,6) = 179;
          numbering(28,7) = 170;
          numbering(29,0) = 152;
          numbering(29,1) = 153;
          numbering(29,2) = 175;
          numbering(29,3) = 173;
          numbering(29,4) = 157;
          numbering(29,5) = 36;
          numbering(29,6) = 180;
          numbering(29,7) = 179;
          numbering(30,0) = 163;
          numbering(30,1) = 173;
          numbering(30,2) = 177;
          numbering(30,3) = 167;
          numbering(30,4) = 170;
          numbering(30,5) = 179;
          numbering(30,6) = 181;
          numbering(30,7) = 38;
          numbering(31,0) = 173;
          numbering(31,1) = 175;
          numbering(31,2) = 178;
          numbering(31,3) = 177;
          numbering(31,4) = 179;
          numbering(31,5) = 180;
          numbering(31,6) = 40;
          numbering(31,7) = 181;
          numbering(32,0) = 30;
          numbering(32,1) = 139;
          numbering(32,2) = 140;
          numbering(32,3) = 141;
          numbering(32,4) = 182;
          numbering(32,5) = 183;
          numbering(32,6) = 184;
          numbering(32,7) = 185;
          numbering(33,0) = 139;
          numbering(33,1) = 31;
          numbering(33,2) = 142;
          numbering(33,3) = 140;
          numbering(33,4) = 183;
          numbering(33,5) = 186;
          numbering(33,6) = 187;
          numbering(33,7) = 184;
          numbering(34,0) = 141;
          numbering(34,1) = 140;
          numbering(34,2) = 143;
          numbering(34,3) = 33;
          numbering(34,4) = 185;
          numbering(34,5) = 184;
          numbering(34,6) = 188;
          numbering(34,7) = 189;
          numbering(35,0) = 140;
          numbering(35,1) = 142;
          numbering(35,2) = 32;
          numbering(35,3) = 143;
          numbering(35,4) = 184;
          numbering(35,5) = 187;
          numbering(35,6) = 190;
          numbering(35,7) = 188;
          numbering(36,0) = 182;
          numbering(36,1) = 183;
          numbering(36,2) = 184;
          numbering(36,3) = 185;
          numbering(36,4) = 11;
          numbering(36,5) = 191;
          numbering(36,6) = 192;
          numbering(36,7) = 193;
          numbering(37,0) = 183;
          numbering(37,1) = 186;
          numbering(37,2) = 187;
          numbering(37,3) = 184;
          numbering(37,4) = 191;
          numbering(37,5) = 41;
          numbering(37,6) = 194;
          numbering(37,7) = 192;
          numbering(38,0) = 185;
          numbering(38,1) = 184;
          numbering(38,2) = 188;
          numbering(38,3) = 189;
          numbering(38,4) = 193;
          numbering(38,5) = 192;
          numbering(38,6) = 195;
          numbering(38,7) = 43;
          numbering(39,0) = 184;
          numbering(39,1) = 187;
          numbering(39,2) = 190;
          numbering(39,3) = 188;
          numbering(39,4) = 192;
          numbering(39,5) = 194;
          numbering(39,6) = 42;
          numbering(39,7) = 195;
          numbering(40,0) = 31;
          numbering(40,1) = 154;
          numbering(40,2) = 155;
          numbering(40,3) = 142;
          numbering(40,4) = 186;
          numbering(40,5) = 196;
          numbering(40,6) = 197;
          numbering(40,7) = 187;
          numbering(41,0) = 154;
          numbering(41,1) = 35;
          numbering(41,2) = 156;
          numbering(41,3) = 155;
          numbering(41,4) = 196;
          numbering(41,5) = 198;
          numbering(41,6) = 199;
          numbering(41,7) = 197;
          numbering(42,0) = 142;
          numbering(42,1) = 155;
          numbering(42,2) = 157;
          numbering(42,3) = 32;
          numbering(42,4) = 187;
          numbering(42,5) = 197;
          numbering(42,6) = 200;
          numbering(42,7) = 190;
          numbering(43,0) = 155;
          numbering(43,1) = 156;
          numbering(43,2) = 36;
          numbering(43,3) = 157;
          numbering(43,4) = 197;
          numbering(43,5) = 199;
          numbering(43,6) = 201;
          numbering(43,7) = 200;
          numbering(44,0) = 186;
          numbering(44,1) = 196;
          numbering(44,2) = 197;
          numbering(44,3) = 187;
          numbering(44,4) = 41;
          numbering(44,5) = 202;
          numbering(44,6) = 203;
          numbering(44,7) = 194;
          numbering(45,0) = 196;
          numbering(45,1) = 198;
          numbering(45,2) = 199;
          numbering(45,3) = 197;
          numbering(45,4) = 202;
          numbering(45,5) = 12;
          numbering(45,6) = 204;
          numbering(45,7) = 203;
          numbering(46,0) = 187;
          numbering(46,1) = 197;
          numbering(46,2) = 200;
          numbering(46,3) = 190;
          numbering(46,4) = 194;
          numbering(46,5) = 203;
          numbering(46,6) = 205;
          numbering(46,7) = 42;
          numbering(47,0) = 197;
          numbering(47,1) = 199;
          numbering(47,2) = 201;
          numbering(47,3) = 200;
          numbering(47,4) = 203;
          numbering(47,5) = 204;
          numbering(47,6) = 44;
          numbering(47,7) = 205;
          numbering(48,0) = 33;
          numbering(48,1) = 143;
          numbering(48,2) = 168;
          numbering(48,3) = 169;
          numbering(48,4) = 189;
          numbering(48,5) = 188;
          numbering(48,6) = 206;
          numbering(48,7) = 207;
          numbering(49,0) = 143;
          numbering(49,1) = 32;
          numbering(49,2) = 170;
          numbering(49,3) = 168;
          numbering(49,4) = 188;
          numbering(49,5) = 190;
          numbering(49,6) = 208;
          numbering(49,7) = 206;
          numbering(50,0) = 169;
          numbering(50,1) = 168;
          numbering(50,2) = 171;
          numbering(50,3) = 39;
          numbering(50,4) = 207;
          numbering(50,5) = 206;
          numbering(50,6) = 209;
          numbering(50,7) = 210;
          numbering(51,0) = 168;
          numbering(51,1) = 170;
          numbering(51,2) = 38;
          numbering(51,3) = 171;
          numbering(51,4) = 206;
          numbering(51,5) = 208;
          numbering(51,6) = 211;
          numbering(51,7) = 209;
          numbering(52,0) = 189;
          numbering(52,1) = 188;
          numbering(52,2) = 206;
          numbering(52,3) = 207;
          numbering(52,4) = 43;
          numbering(52,5) = 195;
          numbering(52,6) = 212;
          numbering(52,7) = 213;
          numbering(53,0) = 188;
          numbering(53,1) = 190;
          numbering(53,2) = 208;
          numbering(53,3) = 206;
          numbering(53,4) = 195;
          numbering(53,5) = 42;
          numbering(53,6) = 214;
          numbering(53,7) = 212;
          numbering(54,0) = 207;
          numbering(54,1) = 206;
          numbering(54,2) = 209;
          numbering(54,3) = 210;
          numbering(54,4) = 213;
          numbering(54,5) = 212;
          numbering(54,6) = 215;
          numbering(54,7) = 14;
          numbering(55,0) = 206;
          numbering(55,1) = 208;
          numbering(55,2) = 211;
          numbering(55,3) = 209;
          numbering(55,4) = 212;
          numbering(55,5) = 214;
          numbering(55,6) = 45;
          numbering(55,7) = 215;
          numbering(56,0) = 32;
          numbering(56,1) = 157;
          numbering(56,2) = 179;
          numbering(56,3) = 170;
          numbering(56,4) = 190;
          numbering(56,5) = 200;
          numbering(56,6) = 216;
          numbering(56,7) = 208;
          numbering(57,0) = 157;
          numbering(57,1) = 36;
          numbering(57,2) = 180;
          numbering(57,3) = 179;
          numbering(57,4) = 200;
          numbering(57,5) = 201;
          numbering(57,6) = 217;
          numbering(57,7) = 216;
          numbering(58,0) = 170;
          numbering(58,1) = 179;
          numbering(58,2) = 181;
          numbering(58,3) = 38;
          numbering(58,4) = 208;
          numbering(58,5) = 216;
          numbering(58,6) = 218;
          numbering(58,7) = 211;
          numbering(59,0) = 179;
          numbering(59,1) = 180;
          numbering(59,2) = 40;
          numbering(59,3) = 181;
          numbering(59,4) = 216;
          numbering(59,5) = 217;
          numbering(59,6) = 219;
          numbering(59,7) = 218;
          numbering(60,0) = 190;
          numbering(60,1) = 200;
          numbering(60,2) = 216;
          numbering(60,3) = 208;
          numbering(60,4) = 42;
          numbering(60,5) = 205;
          numbering(60,6) = 220;
          numbering(60,7) = 214;
          numbering(61,0) = 200;
          numbering(61,1) = 201;
          numbering(61,2) = 217;
          numbering(61,3) = 216;
          numbering(61,4) = 205;
          numbering(61,5) = 44;
          numbering(61,6) = 221;
          numbering(61,7) = 220;
          numbering(62,0) = 208;
          numbering(62,1) = 216;
          numbering(62,2) = 218;
          numbering(62,3) = 211;
          numbering(62,4) = 214;
          numbering(62,5) = 220;
          numbering(62,6) = 222;
          numbering(62,7) = 45;
          numbering(63,0) = 216;
          numbering(63,1) = 217;
          numbering(63,2) = 219;
          numbering(63,3) = 218;
          numbering(63,4) = 220;
          numbering(63,5) = 221;
          numbering(63,6) = 13;
          numbering(63,7) = 222;
          numbering(64,0) = 8;
          numbering(64,1) = 223;
          numbering(64,2) = 224;
          numbering(64,3) = 148;
          numbering(64,4) = 149;
          numbering(64,5) = 225;
          numbering(64,6) = 226;
          numbering(64,7) = 150;
          numbering(65,0) = 223;
          numbering(65,1) = 46;
          numbering(65,2) = 227;
          numbering(65,3) = 224;
          numbering(65,4) = 225;
          numbering(65,5) = 228;
          numbering(65,6) = 229;
          numbering(65,7) = 226;
          numbering(66,0) = 148;
          numbering(66,1) = 224;
          numbering(66,2) = 230;
          numbering(66,3) = 34;
          numbering(66,4) = 150;
          numbering(66,5) = 226;
          numbering(66,6) = 231;
          numbering(66,7) = 153;
          numbering(67,0) = 224;
          numbering(67,1) = 227;
          numbering(67,2) = 47;
          numbering(67,3) = 230;
          numbering(67,4) = 226;
          numbering(67,5) = 229;
          numbering(67,6) = 232;
          numbering(67,7) = 231;
          numbering(68,0) = 149;
          numbering(68,1) = 225;
          numbering(68,2) = 226;
          numbering(68,3) = 150;
          numbering(68,4) = 35;
          numbering(68,5) = 233;
          numbering(68,6) = 234;
          numbering(68,7) = 156;
          numbering(69,0) = 225;
          numbering(69,1) = 228;
          numbering(69,2) = 229;
          numbering(69,3) = 226;
          numbering(69,4) = 233;
          numbering(69,5) = 48;
          numbering(69,6) = 235;
          numbering(69,7) = 234;
          numbering(70,0) = 150;
          numbering(70,1) = 226;
          numbering(70,2) = 231;
          numbering(70,3) = 153;
          numbering(70,4) = 156;
          numbering(70,5) = 234;
          numbering(70,6) = 236;
          numbering(70,7) = 36;
          numbering(71,0) = 226;
          numbering(71,1) = 229;
          numbering(71,2) = 232;
          numbering(71,3) = 231;
          numbering(71,4) = 234;
          numbering(71,5) = 235;
          numbering(71,6) = 49;
          numbering(71,7) = 236;
          numbering(72,0) = 46;
          numbering(72,1) = 237;
          numbering(72,2) = 238;
          numbering(72,3) = 227;
          numbering(72,4) = 228;
          numbering(72,5) = 239;
          numbering(72,6) = 240;
          numbering(72,7) = 229;
          numbering(73,0) = 237;
          numbering(73,1) = 1;
          numbering(73,2) = 241;
          numbering(73,3) = 238;
          numbering(73,4) = 239;
          numbering(73,5) = 242;
          numbering(73,6) = 243;
          numbering(73,7) = 240;
          numbering(74,0) = 227;
          numbering(74,1) = 238;
          numbering(74,2) = 244;
          numbering(74,3) = 47;
          numbering(74,4) = 229;
          numbering(74,5) = 240;
          numbering(74,6) = 245;
          numbering(74,7) = 232;
          numbering(75,0) = 238;
          numbering(75,1) = 241;
          numbering(75,2) = 50;
          numbering(75,3) = 244;
          numbering(75,4) = 240;
          numbering(75,5) = 243;
          numbering(75,6) = 246;
          numbering(75,7) = 245;
          numbering(76,0) = 228;
          numbering(76,1) = 239;
          numbering(76,2) = 240;
          numbering(76,3) = 229;
          numbering(76,4) = 48;
          numbering(76,5) = 247;
          numbering(76,6) = 248;
          numbering(76,7) = 235;
          numbering(77,0) = 239;
          numbering(77,1) = 242;
          numbering(77,2) = 243;
          numbering(77,3) = 240;
          numbering(77,4) = 247;
          numbering(77,5) = 51;
          numbering(77,6) = 249;
          numbering(77,7) = 248;
          numbering(78,0) = 229;
          numbering(78,1) = 240;
          numbering(78,2) = 245;
          numbering(78,3) = 232;
          numbering(78,4) = 235;
          numbering(78,5) = 248;
          numbering(78,6) = 250;
          numbering(78,7) = 49;
          numbering(79,0) = 240;
          numbering(79,1) = 243;
          numbering(79,2) = 246;
          numbering(79,3) = 245;
          numbering(79,4) = 248;
          numbering(79,5) = 249;
          numbering(79,6) = 52;
          numbering(79,7) = 250;
          numbering(80,0) = 34;
          numbering(80,1) = 230;
          numbering(80,2) = 251;
          numbering(80,3) = 174;
          numbering(80,4) = 153;
          numbering(80,5) = 231;
          numbering(80,6) = 252;
          numbering(80,7) = 175;
          numbering(81,0) = 230;
          numbering(81,1) = 47;
          numbering(81,2) = 253;
          numbering(81,3) = 251;
          numbering(81,4) = 231;
          numbering(81,5) = 232;
          numbering(81,6) = 254;
          numbering(81,7) = 252;
          numbering(82,0) = 174;
          numbering(82,1) = 251;
          numbering(82,2) = 255;
          numbering(82,3) = 9;
          numbering(82,4) = 175;
          numbering(82,5) = 252;
          numbering(82,6) = 256;
          numbering(82,7) = 178;
          numbering(83,0) = 251;
          numbering(83,1) = 253;
          numbering(83,2) = 53;
          numbering(83,3) = 255;
          numbering(83,4) = 252;
          numbering(83,5) = 254;
          numbering(83,6) = 257;
          numbering(83,7) = 256;
          numbering(84,0) = 153;
          numbering(84,1) = 231;
          numbering(84,2) = 252;
          numbering(84,3) = 175;
          numbering(84,4) = 36;
          numbering(84,5) = 236;
          numbering(84,6) = 258;
          numbering(84,7) = 180;
          numbering(85,0) = 231;
          numbering(85,1) = 232;
          numbering(85,2) = 254;
          numbering(85,3) = 252;
          numbering(85,4) = 236;
          numbering(85,5) = 49;
          numbering(85,6) = 259;
          numbering(85,7) = 258;
          numbering(86,0) = 175;
          numbering(86,1) = 252;
          numbering(86,2) = 256;
          numbering(86,3) = 178;
          numbering(86,4) = 180;
          numbering(86,5) = 258;
          numbering(86,6) = 260;
          numbering(86,7) = 40;
          numbering(87,0) = 252;
          numbering(87,1) = 254;
          numbering(87,2) = 257;
          numbering(87,3) = 256;
          numbering(87,4) = 258;
          numbering(87,5) = 259;
          numbering(87,6) = 54;
          numbering(87,7) = 260;
          numbering(88,0) = 47;
          numbering(88,1) = 244;
          numbering(88,2) = 261;
          numbering(88,3) = 253;
          numbering(88,4) = 232;
          numbering(88,5) = 245;
          numbering(88,6) = 262;
          numbering(88,7) = 254;
          numbering(89,0) = 244;
          numbering(89,1) = 50;
          numbering(89,2) = 263;
          numbering(89,3) = 261;
          numbering(89,4) = 245;
          numbering(89,5) = 246;
          numbering(89,6) = 264;
          numbering(89,7) = 262;
          numbering(90,0) = 253;
          numbering(90,1) = 261;
          numbering(90,2) = 265;
          numbering(90,3) = 53;
          numbering(90,4) = 254;
          numbering(90,5) = 262;
          numbering(90,6) = 266;
          numbering(90,7) = 257;
          numbering(91,0) = 261;
          numbering(91,1) = 263;
          numbering(91,2) = 15;
          numbering(91,3) = 265;
          numbering(91,4) = 262;
          numbering(91,5) = 264;
          numbering(91,6) = 267;
          numbering(91,7) = 266;
          numbering(92,0) = 232;
          numbering(92,1) = 245;
          numbering(92,2) = 262;
          numbering(92,3) = 254;
          numbering(92,4) = 49;
          numbering(92,5) = 250;
          numbering(92,6) = 268;
          numbering(92,7) = 259;
          numbering(93,0) = 245;
          numbering(93,1) = 246;
          numbering(93,2) = 264;
          numbering(93,3) = 262;
          numbering(93,4) = 250;
          numbering(93,5) = 52;
          numbering(93,6) = 269;
          numbering(93,7) = 268;
          numbering(94,0) = 254;
          numbering(94,1) = 262;
          numbering(94,2) = 266;
          numbering(94,3) = 257;
          numbering(94,4) = 259;
          numbering(94,5) = 268;
          numbering(94,6) = 270;
          numbering(94,7) = 54;
          numbering(95,0) = 262;
          numbering(95,1) = 264;
          numbering(95,2) = 267;
          numbering(95,3) = 266;
          numbering(95,4) = 268;
          numbering(95,5) = 269;
          numbering(95,6) = 55;
          numbering(95,7) = 270;
          numbering(96,0) = 35;
          numbering(96,1) = 233;
          numbering(96,2) = 234;
          numbering(96,3) = 156;
          numbering(96,4) = 198;
          numbering(96,5) = 271;
          numbering(96,6) = 272;
          numbering(96,7) = 199;
          numbering(97,0) = 233;
          numbering(97,1) = 48;
          numbering(97,2) = 235;
          numbering(97,3) = 234;
          numbering(97,4) = 271;
          numbering(97,5) = 273;
          numbering(97,6) = 274;
          numbering(97,7) = 272;
          numbering(98,0) = 156;
          numbering(98,1) = 234;
          numbering(98,2) = 236;
          numbering(98,3) = 36;
          numbering(98,4) = 199;
          numbering(98,5) = 272;
          numbering(98,6) = 275;
          numbering(98,7) = 201;
          numbering(99,0) = 234;
          numbering(99,1) = 235;
          numbering(99,2) = 49;
          numbering(99,3) = 236;
          numbering(99,4) = 272;
          numbering(99,5) = 274;
          numbering(99,6) = 276;
          numbering(99,7) = 275;
          numbering(100,0) = 198;
          numbering(100,1) = 271;
          numbering(100,2) = 272;
          numbering(100,3) = 199;
          numbering(100,4) = 12;
          numbering(100,5) = 277;
          numbering(100,6) = 278;
          numbering(100,7) = 204;
          numbering(101,0) = 271;
          numbering(101,1) = 273;
          numbering(101,2) = 274;
          numbering(101,3) = 272;
          numbering(101,4) = 277;
          numbering(101,5) = 56;
          numbering(101,6) = 279;
          numbering(101,7) = 278;
          numbering(102,0) = 199;
          numbering(102,1) = 272;
          numbering(102,2) = 275;
          numbering(102,3) = 201;
          numbering(102,4) = 204;
          numbering(102,5) = 278;
          numbering(102,6) = 280;
          numbering(102,7) = 44;
          numbering(103,0) = 272;
          numbering(103,1) = 274;
          numbering(103,2) = 276;
          numbering(103,3) = 275;
          numbering(103,4) = 278;
          numbering(103,5) = 279;
          numbering(103,6) = 57;
          numbering(103,7) = 280;
          numbering(104,0) = 48;
          numbering(104,1) = 247;
          numbering(104,2) = 248;
          numbering(104,3) = 235;
          numbering(104,4) = 273;
          numbering(104,5) = 281;
          numbering(104,6) = 282;
          numbering(104,7) = 274;
          numbering(105,0) = 247;
          numbering(105,1) = 51;
          numbering(105,2) = 249;
          numbering(105,3) = 248;
          numbering(105,4) = 281;
          numbering(105,5) = 283;
          numbering(105,6) = 284;
          numbering(105,7) = 282;
          numbering(106,0) = 235;
          numbering(106,1) = 248;
          numbering(106,2) = 250;
          numbering(106,3) = 49;
          numbering(106,4) = 274;
          numbering(106,5) = 282;
          numbering(106,6) = 285;
          numbering(106,7) = 276;
          numbering(107,0) = 248;
          numbering(107,1) = 249;
          numbering(107,2) = 52;
          numbering(107,3) = 250;
          numbering(107,4) = 282;
          numbering(107,5) = 284;
          numbering(107,6) = 286;
          numbering(107,7) = 285;
          numbering(108,0) = 273;
          numbering(108,1) = 281;
          numbering(108,2) = 282;
          numbering(108,3) = 274;
          numbering(108,4) = 56;
          numbering(108,5) = 287;
          numbering(108,6) = 288;
          numbering(108,7) = 279;
          numbering(109,0) = 281;
          numbering(109,1) = 283;
          numbering(109,2) = 284;
          numbering(109,3) = 282;
          numbering(109,4) = 287;
          numbering(109,5) = 16;
          numbering(109,6) = 289;
          numbering(109,7) = 288;
          numbering(110,0) = 274;
          numbering(110,1) = 282;
          numbering(110,2) = 285;
          numbering(110,3) = 276;
          numbering(110,4) = 279;
          numbering(110,5) = 288;
          numbering(110,6) = 290;
          numbering(110,7) = 57;
          numbering(111,0) = 282;
          numbering(111,1) = 284;
          numbering(111,2) = 286;
          numbering(111,3) = 285;
          numbering(111,4) = 288;
          numbering(111,5) = 289;
          numbering(111,6) = 58;
          numbering(111,7) = 290;
          numbering(112,0) = 36;
          numbering(112,1) = 236;
          numbering(112,2) = 258;
          numbering(112,3) = 180;
          numbering(112,4) = 201;
          numbering(112,5) = 275;
          numbering(112,6) = 291;
          numbering(112,7) = 217;
          numbering(113,0) = 236;
          numbering(113,1) = 49;
          numbering(113,2) = 259;
          numbering(113,3) = 258;
          numbering(113,4) = 275;
          numbering(113,5) = 276;
          numbering(113,6) = 292;
          numbering(113,7) = 291;
          numbering(114,0) = 180;
          numbering(114,1) = 258;
          numbering(114,2) = 260;
          numbering(114,3) = 40;
          numbering(114,4) = 217;
          numbering(114,5) = 291;
          numbering(114,6) = 293;
          numbering(114,7) = 219;
          numbering(115,0) = 258;
          numbering(115,1) = 259;
          numbering(115,2) = 54;
          numbering(115,3) = 260;
          numbering(115,4) = 291;
          numbering(115,5) = 292;
          numbering(115,6) = 294;
          numbering(115,7) = 293;
          numbering(116,0) = 201;
          numbering(116,1) = 275;
          numbering(116,2) = 291;
          numbering(116,3) = 217;
          numbering(116,4) = 44;
          numbering(116,5) = 280;
          numbering(116,6) = 295;
          numbering(116,7) = 221;
          numbering(117,0) = 275;
          numbering(117,1) = 276;
          numbering(117,2) = 292;
          numbering(117,3) = 291;
          numbering(117,4) = 280;
          numbering(117,5) = 57;
          numbering(117,6) = 296;
          numbering(117,7) = 295;
          numbering(118,0) = 217;
          numbering(118,1) = 291;
          numbering(118,2) = 293;
          numbering(118,3) = 219;
          numbering(118,4) = 221;
          numbering(118,5) = 295;
          numbering(118,6) = 297;
          numbering(118,7) = 13;
          numbering(119,0) = 291;
          numbering(119,1) = 292;
          numbering(119,2) = 294;
          numbering(119,3) = 293;
          numbering(119,4) = 295;
          numbering(119,5) = 296;
          numbering(119,6) = 59;
          numbering(119,7) = 297;
          numbering(120,0) = 49;
          numbering(120,1) = 250;
          numbering(120,2) = 268;
          numbering(120,3) = 259;
          numbering(120,4) = 276;
          numbering(120,5) = 285;
          numbering(120,6) = 298;
          numbering(120,7) = 292;
          numbering(121,0) = 250;
          numbering(121,1) = 52;
          numbering(121,2) = 269;
          numbering(121,3) = 268;
          numbering(121,4) = 285;
          numbering(121,5) = 286;
          numbering(121,6) = 299;
          numbering(121,7) = 298;
          numbering(122,0) = 259;
          numbering(122,1) = 268;
          numbering(122,2) = 270;
          numbering(122,3) = 54;
          numbering(122,4) = 292;
          numbering(122,5) = 298;
          numbering(122,6) = 300;
          numbering(122,7) = 294;
          numbering(123,0) = 268;
          numbering(123,1) = 269;
          numbering(123,2) = 55;
          numbering(123,3) = 270;
          numbering(123,4) = 298;
          numbering(123,5) = 299;
          numbering(123,6) = 301;
          numbering(123,7) = 300;
          numbering(124,0) = 276;
          numbering(124,1) = 285;
          numbering(124,2) = 298;
          numbering(124,3) = 292;
          numbering(124,4) = 57;
          numbering(124,5) = 290;
          numbering(124,6) = 302;
          numbering(124,7) = 296;
          numbering(125,0) = 285;
          numbering(125,1) = 286;
          numbering(125,2) = 299;
          numbering(125,3) = 298;
          numbering(125,4) = 290;
          numbering(125,5) = 58;
          numbering(125,6) = 303;
          numbering(125,7) = 302;
          numbering(126,0) = 292;
          numbering(126,1) = 298;
          numbering(126,2) = 300;
          numbering(126,3) = 294;
          numbering(126,4) = 296;
          numbering(126,5) = 302;
          numbering(126,6) = 304;
          numbering(126,7) = 59;
          numbering(127,0) = 298;
          numbering(127,1) = 299;
          numbering(127,2) = 301;
          numbering(127,3) = 300;
          numbering(127,4) = 302;
          numbering(127,5) = 303;
          numbering(127,6) = 17;
          numbering(127,7) = 304;
          numbering(128,0) = 10;
          numbering(128,1) = 164;
          numbering(128,2) = 305;
          numbering(128,3) = 306;
          numbering(128,4) = 166;
          numbering(128,5) = 165;
          numbering(128,6) = 307;
          numbering(128,7) = 308;
          numbering(129,0) = 164;
          numbering(129,1) = 37;
          numbering(129,2) = 309;
          numbering(129,3) = 305;
          numbering(129,4) = 165;
          numbering(129,5) = 167;
          numbering(129,6) = 310;
          numbering(129,7) = 307;
          numbering(130,0) = 306;
          numbering(130,1) = 305;
          numbering(130,2) = 311;
          numbering(130,3) = 61;
          numbering(130,4) = 308;
          numbering(130,5) = 307;
          numbering(130,6) = 312;
          numbering(130,7) = 313;
          numbering(131,0) = 305;
          numbering(131,1) = 309;
          numbering(131,2) = 60;
          numbering(131,3) = 311;
          numbering(131,4) = 307;
          numbering(131,5) = 310;
          numbering(131,6) = 314;
          numbering(131,7) = 312;
          numbering(132,0) = 166;
          numbering(132,1) = 165;
          numbering(132,2) = 307;
          numbering(132,3) = 308;
          numbering(132,4) = 39;
          numbering(132,5) = 171;
          numbering(132,6) = 315;
          numbering(132,7) = 316;
          numbering(133,0) = 165;
          numbering(133,1) = 167;
          numbering(133,2) = 310;
          numbering(133,3) = 307;
          numbering(133,4) = 171;
          numbering(133,5) = 38;
          numbering(133,6) = 317;
          numbering(133,7) = 315;
          numbering(134,0) = 308;
          numbering(134,1) = 307;
          numbering(134,2) = 312;
          numbering(134,3) = 313;
          numbering(134,4) = 316;
          numbering(134,5) = 315;
          numbering(134,6) = 318;
          numbering(134,7) = 63;
          numbering(135,0) = 307;
          numbering(135,1) = 310;
          numbering(135,2) = 314;
          numbering(135,3) = 312;
          numbering(135,4) = 315;
          numbering(135,5) = 317;
          numbering(135,6) = 62;
          numbering(135,7) = 318;
          numbering(136,0) = 37;
          numbering(136,1) = 176;
          numbering(136,2) = 319;
          numbering(136,3) = 309;
          numbering(136,4) = 167;
          numbering(136,5) = 177;
          numbering(136,6) = 320;
          numbering(136,7) = 310;
          numbering(137,0) = 176;
          numbering(137,1) = 9;
          numbering(137,2) = 321;
          numbering(137,3) = 319;
          numbering(137,4) = 177;
          numbering(137,5) = 178;
          numbering(137,6) = 322;
          numbering(137,7) = 320;
          numbering(138,0) = 309;
          numbering(138,1) = 319;
          numbering(138,2) = 323;
          numbering(138,3) = 60;
          numbering(138,4) = 310;
          numbering(138,5) = 320;
          numbering(138,6) = 324;
          numbering(138,7) = 314;
          numbering(139,0) = 319;
          numbering(139,1) = 321;
          numbering(139,2) = 64;
          numbering(139,3) = 323;
          numbering(139,4) = 320;
          numbering(139,5) = 322;
          numbering(139,6) = 325;
          numbering(139,7) = 324;
          numbering(140,0) = 167;
          numbering(140,1) = 177;
          numbering(140,2) = 320;
          numbering(140,3) = 310;
          numbering(140,4) = 38;
          numbering(140,5) = 181;
          numbering(140,6) = 326;
          numbering(140,7) = 317;
          numbering(141,0) = 177;
          numbering(141,1) = 178;
          numbering(141,2) = 322;
          numbering(141,3) = 320;
          numbering(141,4) = 181;
          numbering(141,5) = 40;
          numbering(141,6) = 327;
          numbering(141,7) = 326;
          numbering(142,0) = 310;
          numbering(142,1) = 320;
          numbering(142,2) = 324;
          numbering(142,3) = 314;
          numbering(142,4) = 317;
          numbering(142,5) = 326;
          numbering(142,6) = 328;
          numbering(142,7) = 62;
          numbering(143,0) = 320;
          numbering(143,1) = 322;
          numbering(143,2) = 325;
          numbering(143,3) = 324;
          numbering(143,4) = 326;
          numbering(143,5) = 327;
          numbering(143,6) = 65;
          numbering(143,7) = 328;
          numbering(144,0) = 61;
          numbering(144,1) = 311;
          numbering(144,2) = 329;
          numbering(144,3) = 330;
          numbering(144,4) = 313;
          numbering(144,5) = 312;
          numbering(144,6) = 331;
          numbering(144,7) = 332;
          numbering(145,0) = 311;
          numbering(145,1) = 60;
          numbering(145,2) = 333;
          numbering(145,3) = 329;
          numbering(145,4) = 312;
          numbering(145,5) = 314;
          numbering(145,6) = 334;
          numbering(145,7) = 331;
          numbering(146,0) = 330;
          numbering(146,1) = 329;
          numbering(146,2) = 335;
          numbering(146,3) = 3;
          numbering(146,4) = 332;
          numbering(146,5) = 331;
          numbering(146,6) = 336;
          numbering(146,7) = 337;
          numbering(147,0) = 329;
          numbering(147,1) = 333;
          numbering(147,2) = 66;
          numbering(147,3) = 335;
          numbering(147,4) = 331;
          numbering(147,5) = 334;
          numbering(147,6) = 338;
          numbering(147,7) = 336;
          numbering(148,0) = 313;
          numbering(148,1) = 312;
          numbering(148,2) = 331;
          numbering(148,3) = 332;
          numbering(148,4) = 63;
          numbering(148,5) = 318;
          numbering(148,6) = 339;
          numbering(148,7) = 340;
          numbering(149,0) = 312;
          numbering(149,1) = 314;
          numbering(149,2) = 334;
          numbering(149,3) = 331;
          numbering(149,4) = 318;
          numbering(149,5) = 62;
          numbering(149,6) = 341;
          numbering(149,7) = 339;
          numbering(150,0) = 332;
          numbering(150,1) = 331;
          numbering(150,2) = 336;
          numbering(150,3) = 337;
          numbering(150,4) = 340;
          numbering(150,5) = 339;
          numbering(150,6) = 342;
          numbering(150,7) = 68;
          numbering(151,0) = 331;
          numbering(151,1) = 334;
          numbering(151,2) = 338;
          numbering(151,3) = 336;
          numbering(151,4) = 339;
          numbering(151,5) = 341;
          numbering(151,6) = 67;
          numbering(151,7) = 342;
          numbering(152,0) = 60;
          numbering(152,1) = 323;
          numbering(152,2) = 343;
          numbering(152,3) = 333;
          numbering(152,4) = 314;
          numbering(152,5) = 324;
          numbering(152,6) = 344;
          numbering(152,7) = 334;
          numbering(153,0) = 323;
          numbering(153,1) = 64;
          numbering(153,2) = 345;
          numbering(153,3) = 343;
          numbering(153,4) = 324;
          numbering(153,5) = 325;
          numbering(153,6) = 346;
          numbering(153,7) = 344;
          numbering(154,0) = 333;
          numbering(154,1) = 343;
          numbering(154,2) = 347;
          numbering(154,3) = 66;
          numbering(154,4) = 334;
          numbering(154,5) = 344;
          numbering(154,6) = 348;
          numbering(154,7) = 338;
          numbering(155,0) = 343;
          numbering(155,1) = 345;
          numbering(155,2) = 18;
          numbering(155,3) = 347;
          numbering(155,4) = 344;
          numbering(155,5) = 346;
          numbering(155,6) = 349;
          numbering(155,7) = 348;
          numbering(156,0) = 314;
          numbering(156,1) = 324;
          numbering(156,2) = 344;
          numbering(156,3) = 334;
          numbering(156,4) = 62;
          numbering(156,5) = 328;
          numbering(156,6) = 350;
          numbering(156,7) = 341;
          numbering(157,0) = 324;
          numbering(157,1) = 325;
          numbering(157,2) = 346;
          numbering(157,3) = 344;
          numbering(157,4) = 328;
          numbering(157,5) = 65;
          numbering(157,6) = 351;
          numbering(157,7) = 350;
          numbering(158,0) = 334;
          numbering(158,1) = 344;
          numbering(158,2) = 348;
          numbering(158,3) = 338;
          numbering(158,4) = 341;
          numbering(158,5) = 350;
          numbering(158,6) = 352;
          numbering(158,7) = 67;
          numbering(159,0) = 344;
          numbering(159,1) = 346;
          numbering(159,2) = 349;
          numbering(159,3) = 348;
          numbering(159,4) = 350;
          numbering(159,5) = 351;
          numbering(159,6) = 69;
          numbering(159,7) = 352;
          numbering(160,0) = 39;
          numbering(160,1) = 171;
          numbering(160,2) = 315;
          numbering(160,3) = 316;
          numbering(160,4) = 210;
          numbering(160,5) = 209;
          numbering(160,6) = 353;
          numbering(160,7) = 354;
          numbering(161,0) = 171;
          numbering(161,1) = 38;
          numbering(161,2) = 317;
          numbering(161,3) = 315;
          numbering(161,4) = 209;
          numbering(161,5) = 211;
          numbering(161,6) = 355;
          numbering(161,7) = 353;
          numbering(162,0) = 316;
          numbering(162,1) = 315;
          numbering(162,2) = 318;
          numbering(162,3) = 63;
          numbering(162,4) = 354;
          numbering(162,5) = 353;
          numbering(162,6) = 356;
          numbering(162,7) = 357;
          numbering(163,0) = 315;
          numbering(163,1) = 317;
          numbering(163,2) = 62;
          numbering(163,3) = 318;
          numbering(163,4) = 353;
          numbering(163,5) = 355;
          numbering(163,6) = 358;
          numbering(163,7) = 356;
          numbering(164,0) = 210;
          numbering(164,1) = 209;
          numbering(164,2) = 353;
          numbering(164,3) = 354;
          numbering(164,4) = 14;
          numbering(164,5) = 215;
          numbering(164,6) = 359;
          numbering(164,7) = 360;
          numbering(165,0) = 209;
          numbering(165,1) = 211;
          numbering(165,2) = 355;
          numbering(165,3) = 353;
          numbering(165,4) = 215;
          numbering(165,5) = 45;
          numbering(165,6) = 361;
          numbering(165,7) = 359;
          numbering(166,0) = 354;
          numbering(166,1) = 353;
          numbering(166,2) = 356;
          numbering(166,3) = 357;
          numbering(166,4) = 360;
          numbering(166,5) = 359;
          numbering(166,6) = 362;
          numbering(166,7) = 71;
          numbering(167,0) = 353;
          numbering(167,1) = 355;
          numbering(167,2) = 358;
          numbering(167,3) = 356;
          numbering(167,4) = 359;
          numbering(167,5) = 361;
          numbering(167,6) = 70;
          numbering(167,7) = 362;
          numbering(168,0) = 38;
          numbering(168,1) = 181;
          numbering(168,2) = 326;
          numbering(168,3) = 317;
          numbering(168,4) = 211;
          numbering(168,5) = 218;
          numbering(168,6) = 363;
          numbering(168,7) = 355;
          numbering(169,0) = 181;
          numbering(169,1) = 40;
          numbering(169,2) = 327;
          numbering(169,3) = 326;
          numbering(169,4) = 218;
          numbering(169,5) = 219;
          numbering(169,6) = 364;
          numbering(169,7) = 363;
          numbering(170,0) = 317;
          numbering(170,1) = 326;
          numbering(170,2) = 328;
          numbering(170,3) = 62;
          numbering(170,4) = 355;
          numbering(170,5) = 363;
          numbering(170,6) = 365;
          numbering(170,7) = 358;
          numbering(171,0) = 326;
          numbering(171,1) = 327;
          numbering(171,2) = 65;
          numbering(171,3) = 328;
          numbering(171,4) = 363;
          numbering(171,5) = 364;
          numbering(171,6) = 366;
          numbering(171,7) = 365;
          numbering(172,0) = 211;
          numbering(172,1) = 218;
          numbering(172,2) = 363;
          numbering(172,3) = 355;
          numbering(172,4) = 45;
          numbering(172,5) = 222;
          numbering(172,6) = 367;
          numbering(172,7) = 361;
          numbering(173,0) = 218;
          numbering(173,1) = 219;
          numbering(173,2) = 364;
          numbering(173,3) = 363;
          numbering(173,4) = 222;
          numbering(173,5) = 13;
          numbering(173,6) = 368;
          numbering(173,7) = 367;
          numbering(174,0) = 355;
          numbering(174,1) = 363;
          numbering(174,2) = 365;
          numbering(174,3) = 358;
          numbering(174,4) = 361;
          numbering(174,5) = 367;
          numbering(174,6) = 369;
          numbering(174,7) = 70;
          numbering(175,0) = 363;
          numbering(175,1) = 364;
          numbering(175,2) = 366;
          numbering(175,3) = 365;
          numbering(175,4) = 367;
          numbering(175,5) = 368;
          numbering(175,6) = 72;
          numbering(175,7) = 369;
          numbering(176,0) = 63;
          numbering(176,1) = 318;
          numbering(176,2) = 339;
          numbering(176,3) = 340;
          numbering(176,4) = 357;
          numbering(176,5) = 356;
          numbering(176,6) = 370;
          numbering(176,7) = 371;
          numbering(177,0) = 318;
          numbering(177,1) = 62;
          numbering(177,2) = 341;
          numbering(177,3) = 339;
          numbering(177,4) = 356;
          numbering(177,5) = 358;
          numbering(177,6) = 372;
          numbering(177,7) = 370;
          numbering(178,0) = 340;
          numbering(178,1) = 339;
          numbering(178,2) = 342;
          numbering(178,3) = 68;
          numbering(178,4) = 371;
          numbering(178,5) = 370;
          numbering(178,6) = 373;
          numbering(178,7) = 374;
          numbering(179,0) = 339;
          numbering(179,1) = 341;
          numbering(179,2) = 67;
          numbering(179,3) = 342;
          numbering(179,4) = 370;
          numbering(179,5) = 372;
          numbering(179,6) = 375;
          numbering(179,7) = 373;
          numbering(180,0) = 357;
          numbering(180,1) = 356;
          numbering(180,2) = 370;
          numbering(180,3) = 371;
          numbering(180,4) = 71;
          numbering(180,5) = 362;
          numbering(180,6) = 376;
          numbering(180,7) = 377;
          numbering(181,0) = 356;
          numbering(181,1) = 358;
          numbering(181,2) = 372;
          numbering(181,3) = 370;
          numbering(181,4) = 362;
          numbering(181,5) = 70;
          numbering(181,6) = 378;
          numbering(181,7) = 376;
          numbering(182,0) = 371;
          numbering(182,1) = 370;
          numbering(182,2) = 373;
          numbering(182,3) = 374;
          numbering(182,4) = 377;
          numbering(182,5) = 376;
          numbering(182,6) = 379;
          numbering(182,7) = 20;
          numbering(183,0) = 370;
          numbering(183,1) = 372;
          numbering(183,2) = 375;
          numbering(183,3) = 373;
          numbering(183,4) = 376;
          numbering(183,5) = 378;
          numbering(183,6) = 73;
          numbering(183,7) = 379;
          numbering(184,0) = 62;
          numbering(184,1) = 328;
          numbering(184,2) = 350;
          numbering(184,3) = 341;
          numbering(184,4) = 358;
          numbering(184,5) = 365;
          numbering(184,6) = 380;
          numbering(184,7) = 372;
          numbering(185,0) = 328;
          numbering(185,1) = 65;
          numbering(185,2) = 351;
          numbering(185,3) = 350;
          numbering(185,4) = 365;
          numbering(185,5) = 366;
          numbering(185,6) = 381;
          numbering(185,7) = 380;
          numbering(186,0) = 341;
          numbering(186,1) = 350;
          numbering(186,2) = 352;
          numbering(186,3) = 67;
          numbering(186,4) = 372;
          numbering(186,5) = 380;
          numbering(186,6) = 382;
          numbering(186,7) = 375;
          numbering(187,0) = 350;
          numbering(187,1) = 351;
          numbering(187,2) = 69;
          numbering(187,3) = 352;
          numbering(187,4) = 380;
          numbering(187,5) = 381;
          numbering(187,6) = 383;
          numbering(187,7) = 382;
          numbering(188,0) = 358;
          numbering(188,1) = 365;
          numbering(188,2) = 380;
          numbering(188,3) = 372;
          numbering(188,4) = 70;
          numbering(188,5) = 369;
          numbering(188,6) = 384;
          numbering(188,7) = 378;
          numbering(189,0) = 365;
          numbering(189,1) = 366;
          numbering(189,2) = 381;
          numbering(189,3) = 380;
          numbering(189,4) = 369;
          numbering(189,5) = 72;
          numbering(189,6) = 385;
          numbering(189,7) = 384;
          numbering(190,0) = 372;
          numbering(190,1) = 380;
          numbering(190,2) = 382;
          numbering(190,3) = 375;
          numbering(190,4) = 378;
          numbering(190,5) = 384;
          numbering(190,6) = 386;
          numbering(190,7) = 73;
          numbering(191,0) = 380;
          numbering(191,1) = 381;
          numbering(191,2) = 383;
          numbering(191,3) = 382;
          numbering(191,4) = 384;
          numbering(191,5) = 385;
          numbering(191,6) = 19;
          numbering(191,7) = 386;
          numbering(192,0) = 9;
          numbering(192,1) = 255;
          numbering(192,2) = 387;
          numbering(192,3) = 321;
          numbering(192,4) = 178;
          numbering(192,5) = 256;
          numbering(192,6) = 388;
          numbering(192,7) = 322;
          numbering(193,0) = 255;
          numbering(193,1) = 53;
          numbering(193,2) = 389;
          numbering(193,3) = 387;
          numbering(193,4) = 256;
          numbering(193,5) = 257;
          numbering(193,6) = 390;
          numbering(193,7) = 388;
          numbering(194,0) = 321;
          numbering(194,1) = 387;
          numbering(194,2) = 391;
          numbering(194,3) = 64;
          numbering(194,4) = 322;
          numbering(194,5) = 388;
          numbering(194,6) = 392;
          numbering(194,7) = 325;
          numbering(195,0) = 387;
          numbering(195,1) = 389;
          numbering(195,2) = 74;
          numbering(195,3) = 391;
          numbering(195,4) = 388;
          numbering(195,5) = 390;
          numbering(195,6) = 393;
          numbering(195,7) = 392;
          numbering(196,0) = 178;
          numbering(196,1) = 256;
          numbering(196,2) = 388;
          numbering(196,3) = 322;
          numbering(196,4) = 40;
          numbering(196,5) = 260;
          numbering(196,6) = 394;
          numbering(196,7) = 327;
          numbering(197,0) = 256;
          numbering(197,1) = 257;
          numbering(197,2) = 390;
          numbering(197,3) = 388;
          numbering(197,4) = 260;
          numbering(197,5) = 54;
          numbering(197,6) = 395;
          numbering(197,7) = 394;
          numbering(198,0) = 322;
          numbering(198,1) = 388;
          numbering(198,2) = 392;
          numbering(198,3) = 325;
          numbering(198,4) = 327;
          numbering(198,5) = 394;
          numbering(198,6) = 396;
          numbering(198,7) = 65;
          numbering(199,0) = 388;
          numbering(199,1) = 390;
          numbering(199,2) = 393;
          numbering(199,3) = 392;
          numbering(199,4) = 394;
          numbering(199,5) = 395;
          numbering(199,6) = 75;
          numbering(199,7) = 396;
          numbering(200,0) = 53;
          numbering(200,1) = 265;
          numbering(200,2) = 397;
          numbering(200,3) = 389;
          numbering(200,4) = 257;
          numbering(200,5) = 266;
          numbering(200,6) = 398;
          numbering(200,7) = 390;
          numbering(201,0) = 265;
          numbering(201,1) = 15;
          numbering(201,2) = 399;
          numbering(201,3) = 397;
          numbering(201,4) = 266;
          numbering(201,5) = 267;
          numbering(201,6) = 400;
          numbering(201,7) = 398;
          numbering(202,0) = 389;
          numbering(202,1) = 397;
          numbering(202,2) = 401;
          numbering(202,3) = 74;
          numbering(202,4) = 390;
          numbering(202,5) = 398;
          numbering(202,6) = 402;
          numbering(202,7) = 393;
          numbering(203,0) = 397;
          numbering(203,1) = 399;
          numbering(203,2) = 76;
          numbering(203,3) = 401;
          numbering(203,4) = 398;
          numbering(203,5) = 400;
          numbering(203,6) = 403;
          numbering(203,7) = 402;
          numbering(204,0) = 257;
          numbering(204,1) = 266;
          numbering(204,2) = 398;
          numbering(204,3) = 390;
          numbering(204,4) = 54;
          numbering(204,5) = 270;
          numbering(204,6) = 404;
          numbering(204,7) = 395;
          numbering(205,0) = 266;
          numbering(205,1) = 267;
          numbering(205,2) = 400;
          numbering(205,3) = 398;
          numbering(205,4) = 270;
          numbering(205,5) = 55;
          numbering(205,6) = 405;
          numbering(205,7) = 404;
          numbering(206,0) = 390;
          numbering(206,1) = 398;
          numbering(206,2) = 402;
          numbering(206,3) = 393;
          numbering(206,4) = 395;
          numbering(206,5) = 404;
          numbering(206,6) = 406;
          numbering(206,7) = 75;
          numbering(207,0) = 398;
          numbering(207,1) = 400;
          numbering(207,2) = 403;
          numbering(207,3) = 402;
          numbering(207,4) = 404;
          numbering(207,5) = 405;
          numbering(207,6) = 77;
          numbering(207,7) = 406;
          numbering(208,0) = 64;
          numbering(208,1) = 391;
          numbering(208,2) = 407;
          numbering(208,3) = 345;
          numbering(208,4) = 325;
          numbering(208,5) = 392;
          numbering(208,6) = 408;
          numbering(208,7) = 346;
          numbering(209,0) = 391;
          numbering(209,1) = 74;
          numbering(209,2) = 409;
          numbering(209,3) = 407;
          numbering(209,4) = 392;
          numbering(209,5) = 393;
          numbering(209,6) = 410;
          numbering(209,7) = 408;
          numbering(210,0) = 345;
          numbering(210,1) = 407;
          numbering(210,2) = 411;
          numbering(210,3) = 18;
          numbering(210,4) = 346;
          numbering(210,5) = 408;
          numbering(210,6) = 412;
          numbering(210,7) = 349;
          numbering(211,0) = 407;
          numbering(211,1) = 409;
          numbering(211,2) = 78;
          numbering(211,3) = 411;
          numbering(211,4) = 408;
          numbering(211,5) = 410;
          numbering(211,6) = 413;
          numbering(211,7) = 412;
          numbering(212,0) = 325;
          numbering(212,1) = 392;
          numbering(212,2) = 408;
          numbering(212,3) = 346;
          numbering(212,4) = 65;
          numbering(212,5) = 396;
          numbering(212,6) = 414;
          numbering(212,7) = 351;
          numbering(213,0) = 392;
          numbering(213,1) = 393;
          numbering(213,2) = 410;
          numbering(213,3) = 408;
          numbering(213,4) = 396;
          numbering(213,5) = 75;
          numbering(213,6) = 415;
          numbering(213,7) = 414;
          numbering(214,0) = 346;
          numbering(214,1) = 408;
          numbering(214,2) = 412;
          numbering(214,3) = 349;
          numbering(214,4) = 351;
          numbering(214,5) = 414;
          numbering(214,6) = 416;
          numbering(214,7) = 69;
          numbering(215,0) = 408;
          numbering(215,1) = 410;
          numbering(215,2) = 413;
          numbering(215,3) = 412;
          numbering(215,4) = 414;
          numbering(215,5) = 415;
          numbering(215,6) = 79;
          numbering(215,7) = 416;
          numbering(216,0) = 74;
          numbering(216,1) = 401;
          numbering(216,2) = 417;
          numbering(216,3) = 409;
          numbering(216,4) = 393;
          numbering(216,5) = 402;
          numbering(216,6) = 418;
          numbering(216,7) = 410;
          numbering(217,0) = 401;
          numbering(217,1) = 76;
          numbering(217,2) = 419;
          numbering(217,3) = 417;
          numbering(217,4) = 402;
          numbering(217,5) = 403;
          numbering(217,6) = 420;
          numbering(217,7) = 418;
          numbering(218,0) = 409;
          numbering(218,1) = 417;
          numbering(218,2) = 421;
          numbering(218,3) = 78;
          numbering(218,4) = 410;
          numbering(218,5) = 418;
          numbering(218,6) = 422;
          numbering(218,7) = 413;
          numbering(219,0) = 417;
          numbering(219,1) = 419;
          numbering(219,2) = 2;
          numbering(219,3) = 421;
          numbering(219,4) = 418;
          numbering(219,5) = 420;
          numbering(219,6) = 423;
          numbering(219,7) = 422;
          numbering(220,0) = 393;
          numbering(220,1) = 402;
          numbering(220,2) = 418;
          numbering(220,3) = 410;
          numbering(220,4) = 75;
          numbering(220,5) = 406;
          numbering(220,6) = 424;
          numbering(220,7) = 415;
          numbering(221,0) = 402;
          numbering(221,1) = 403;
          numbering(221,2) = 420;
          numbering(221,3) = 418;
          numbering(221,4) = 406;
          numbering(221,5) = 77;
          numbering(221,6) = 425;
          numbering(221,7) = 424;
          numbering(222,0) = 410;
          numbering(222,1) = 418;
          numbering(222,2) = 422;
          numbering(222,3) = 413;
          numbering(222,4) = 415;
          numbering(222,5) = 424;
          numbering(222,6) = 426;
          numbering(222,7) = 79;
          numbering(223,0) = 418;
          numbering(223,1) = 420;
          numbering(223,2) = 423;
          numbering(223,3) = 422;
          numbering(223,4) = 424;
          numbering(223,5) = 425;
          numbering(223,6) = 80;
          numbering(223,7) = 426;
          numbering(224,0) = 40;
          numbering(224,1) = 260;
          numbering(224,2) = 394;
          numbering(224,3) = 327;
          numbering(224,4) = 219;
          numbering(224,5) = 293;
          numbering(224,6) = 427;
          numbering(224,7) = 364;
          numbering(225,0) = 260;
          numbering(225,1) = 54;
          numbering(225,2) = 395;
          numbering(225,3) = 394;
          numbering(225,4) = 293;
          numbering(225,5) = 294;
          numbering(225,6) = 428;
          numbering(225,7) = 427;
          numbering(226,0) = 327;
          numbering(226,1) = 394;
          numbering(226,2) = 396;
          numbering(226,3) = 65;
          numbering(226,4) = 364;
          numbering(226,5) = 427;
          numbering(226,6) = 429;
          numbering(226,7) = 366;
          numbering(227,0) = 394;
          numbering(227,1) = 395;
          numbering(227,2) = 75;
          numbering(227,3) = 396;
          numbering(227,4) = 427;
          numbering(227,5) = 428;
          numbering(227,6) = 430;
          numbering(227,7) = 429;
          numbering(228,0) = 219;
          numbering(228,1) = 293;
          numbering(228,2) = 427;
          numbering(228,3) = 364;
          numbering(228,4) = 13;
          numbering(228,5) = 297;
          numbering(228,6) = 431;
          numbering(228,7) = 368;
          numbering(229,0) = 293;
          numbering(229,1) = 294;
          numbering(229,2) = 428;
          numbering(229,3) = 427;
          numbering(229,4) = 297;
          numbering(229,5) = 59;
          numbering(229,6) = 432;
          numbering(229,7) = 431;
          numbering(230,0) = 364;
          numbering(230,1) = 427;
          numbering(230,2) = 429;
          numbering(230,3) = 366;
          numbering(230,4) = 368;
          numbering(230,5) = 431;
          numbering(230,6) = 433;
          numbering(230,7) = 72;
          numbering(231,0) = 427;
          numbering(231,1) = 428;
          numbering(231,2) = 430;
          numbering(231,3) = 429;
          numbering(231,4) = 431;
          numbering(231,5) = 432;
          numbering(231,6) = 81;
          numbering(231,7) = 433;
          numbering(232,0) = 54;
          numbering(232,1) = 270;
          numbering(232,2) = 404;
          numbering(232,3) = 395;
          numbering(232,4) = 294;
          numbering(232,5) = 300;
          numbering(232,6) = 434;
          numbering(232,7) = 428;
          numbering(233,0) = 270;
          numbering(233,1) = 55;
          numbering(233,2) = 405;
          numbering(233,3) = 404;
          numbering(233,4) = 300;
          numbering(233,5) = 301;
          numbering(233,6) = 435;
          numbering(233,7) = 434;
          numbering(234,0) = 395;
          numbering(234,1) = 404;
          numbering(234,2) = 406;
          numbering(234,3) = 75;
          numbering(234,4) = 428;
          numbering(234,5) = 434;
          numbering(234,6) = 436;
          numbering(234,7) = 430;
          numbering(235,0) = 404;
          numbering(235,1) = 405;
          numbering(235,2) = 77;
          numbering(235,3) = 406;
          numbering(235,4) = 434;
          numbering(235,5) = 435;
          numbering(235,6) = 437;
          numbering(235,7) = 436;
          numbering(236,0) = 294;
          numbering(236,1) = 300;
          numbering(236,2) = 434;
          numbering(236,3) = 428;
          numbering(236,4) = 59;
          numbering(236,5) = 304;
          numbering(236,6) = 438;
          numbering(236,7) = 432;
          numbering(237,0) = 300;
          numbering(237,1) = 301;
          numbering(237,2) = 435;
          numbering(237,3) = 434;
          numbering(237,4) = 304;
          numbering(237,5) = 17;
          numbering(237,6) = 439;
          numbering(237,7) = 438;
          numbering(238,0) = 428;
          numbering(238,1) = 434;
          numbering(238,2) = 436;
          numbering(238,3) = 430;
          numbering(238,4) = 432;
          numbering(238,5) = 438;
          numbering(238,6) = 440;
          numbering(238,7) = 81;
          numbering(239,0) = 434;
          numbering(239,1) = 435;
          numbering(239,2) = 437;
          numbering(239,3) = 436;
          numbering(239,4) = 438;
          numbering(239,5) = 439;
          numbering(239,6) = 82;
          numbering(239,7) = 440;
          numbering(240,0) = 65;
          numbering(240,1) = 396;
          numbering(240,2) = 414;
          numbering(240,3) = 351;
          numbering(240,4) = 366;
          numbering(240,5) = 429;
          numbering(240,6) = 441;
          numbering(240,7) = 381;
          numbering(241,0) = 396;
          numbering(241,1) = 75;
          numbering(241,2) = 415;
          numbering(241,3) = 414;
          numbering(241,4) = 429;
          numbering(241,5) = 430;
          numbering(241,6) = 442;
          numbering(241,7) = 441;
          numbering(242,0) = 351;
          numbering(242,1) = 414;
          numbering(242,2) = 416;
          numbering(242,3) = 69;
          numbering(242,4) = 381;
          numbering(242,5) = 441;
          numbering(242,6) = 443;
          numbering(242,7) = 383;
          numbering(243,0) = 414;
          numbering(243,1) = 415;
          numbering(243,2) = 79;
          numbering(243,3) = 416;
          numbering(243,4) = 441;
          numbering(243,5) = 442;
          numbering(243,6) = 444;
          numbering(243,7) = 443;
          numbering(244,0) = 366;
          numbering(244,1) = 429;
          numbering(244,2) = 441;
          numbering(244,3) = 381;
          numbering(244,4) = 72;
          numbering(244,5) = 433;
          numbering(244,6) = 445;
          numbering(244,7) = 385;
          numbering(245,0) = 429;
          numbering(245,1) = 430;
          numbering(245,2) = 442;
          numbering(245,3) = 441;
          numbering(245,4) = 433;
          numbering(245,5) = 81;
          numbering(245,6) = 446;
          numbering(245,7) = 445;
          numbering(246,0) = 381;
          numbering(246,1) = 441;
          numbering(246,2) = 443;
          numbering(246,3) = 383;
          numbering(246,4) = 385;
          numbering(246,5) = 445;
          numbering(246,6) = 447;
          numbering(246,7) = 19;
          numbering(247,0) = 441;
          numbering(247,1) = 442;
          numbering(247,2) = 444;
          numbering(247,3) = 443;
          numbering(247,4) = 445;
          numbering(247,5) = 446;
          numbering(247,6) = 83;
          numbering(247,7) = 447;
          numbering(248,0) = 75;
          numbering(248,1) = 406;
          numbering(248,2) = 424;
          numbering(248,3) = 415;
          numbering(248,4) = 430;
          numbering(248,5) = 436;
          numbering(248,6) = 448;
          numbering(248,7) = 442;
          numbering(249,0) = 406;
          numbering(249,1) = 77;
          numbering(249,2) = 425;
          numbering(249,3) = 424;
          numbering(249,4) = 436;
          numbering(249,5) = 437;
          numbering(249,6) = 449;
          numbering(249,7) = 448;
          numbering(250,0) = 415;
          numbering(250,1) = 424;
          numbering(250,2) = 426;
          numbering(250,3) = 79;
          numbering(250,4) = 442;
          numbering(250,5) = 448;
          numbering(250,6) = 450;
          numbering(250,7) = 444;
          numbering(251,0) = 424;
          numbering(251,1) = 425;
          numbering(251,2) = 80;
          numbering(251,3) = 426;
          numbering(251,4) = 448;
          numbering(251,5) = 449;
          numbering(251,6) = 451;
          numbering(251,7) = 450;
          numbering(252,0) = 430;
          numbering(252,1) = 436;
          numbering(252,2) = 448;
          numbering(252,3) = 442;
          numbering(252,4) = 81;
          numbering(252,5) = 440;
          numbering(252,6) = 452;
          numbering(252,7) = 446;
          numbering(253,0) = 436;
          numbering(253,1) = 437;
          numbering(253,2) = 449;
          numbering(253,3) = 448;
          numbering(253,4) = 440;
          numbering(253,5) = 82;
          numbering(253,6) = 453;
          numbering(253,7) = 452;
          numbering(254,0) = 442;
          numbering(254,1) = 448;
          numbering(254,2) = 450;
          numbering(254,3) = 444;
          numbering(254,4) = 446;
          numbering(254,5) = 452;
          numbering(254,6) = 454;
          numbering(254,7) = 83;
          numbering(255,0) = 448;
          numbering(255,1) = 449;
          numbering(255,2) = 451;
          numbering(255,3) = 450;
          numbering(255,4) = 452;
          numbering(255,5) = 453;
          numbering(255,6) = 21;
          numbering(255,7) = 454;
          numbering(256,0) = 11;
          numbering(256,1) = 191;
          numbering(256,2) = 192;
          numbering(256,3) = 193;
          numbering(256,4) = 455;
          numbering(256,5) = 456;
          numbering(256,6) = 457;
          numbering(256,7) = 458;
          numbering(257,0) = 191;
          numbering(257,1) = 41;
          numbering(257,2) = 194;
          numbering(257,3) = 192;
          numbering(257,4) = 456;
          numbering(257,5) = 459;
          numbering(257,6) = 460;
          numbering(257,7) = 457;
          numbering(258,0) = 193;
          numbering(258,1) = 192;
          numbering(258,2) = 195;
          numbering(258,3) = 43;
          numbering(258,4) = 458;
          numbering(258,5) = 457;
          numbering(258,6) = 461;
          numbering(258,7) = 462;
          numbering(259,0) = 192;
          numbering(259,1) = 194;
          numbering(259,2) = 42;
          numbering(259,3) = 195;
          numbering(259,4) = 457;
          numbering(259,5) = 460;
          numbering(259,6) = 463;
          numbering(259,7) = 461;
          numbering(260,0) = 455;
          numbering(260,1) = 456;
          numbering(260,2) = 457;
          numbering(260,3) = 458;
          numbering(260,4) = 84;
          numbering(260,5) = 464;
          numbering(260,6) = 465;
          numbering(260,7) = 466;
          numbering(261,0) = 456;
          numbering(261,1) = 459;
          numbering(261,2) = 460;
          numbering(261,3) = 457;
          numbering(261,4) = 464;
          numbering(261,5) = 85;
          numbering(261,6) = 467;
          numbering(261,7) = 465;
          numbering(262,0) = 458;
          numbering(262,1) = 457;
          numbering(262,2) = 461;
          numbering(262,3) = 462;
          numbering(262,4) = 466;
          numbering(262,5) = 465;
          numbering(262,6) = 468;
          numbering(262,7) = 87;
          numbering(263,0) = 457;
          numbering(263,1) = 460;
          numbering(263,2) = 463;
          numbering(263,3) = 461;
          numbering(263,4) = 465;
          numbering(263,5) = 467;
          numbering(263,6) = 86;
          numbering(263,7) = 468;
          numbering(264,0) = 41;
          numbering(264,1) = 202;
          numbering(264,2) = 203;
          numbering(264,3) = 194;
          numbering(264,4) = 459;
          numbering(264,5) = 469;
          numbering(264,6) = 470;
          numbering(264,7) = 460;
          numbering(265,0) = 202;
          numbering(265,1) = 12;
          numbering(265,2) = 204;
          numbering(265,3) = 203;
          numbering(265,4) = 469;
          numbering(265,5) = 471;
          numbering(265,6) = 472;
          numbering(265,7) = 470;
          numbering(266,0) = 194;
          numbering(266,1) = 203;
          numbering(266,2) = 205;
          numbering(266,3) = 42;
          numbering(266,4) = 460;
          numbering(266,5) = 470;
          numbering(266,6) = 473;
          numbering(266,7) = 463;
          numbering(267,0) = 203;
          numbering(267,1) = 204;
          numbering(267,2) = 44;
          numbering(267,3) = 205;
          numbering(267,4) = 470;
          numbering(267,5) = 472;
          numbering(267,6) = 474;
          numbering(267,7) = 473;
          numbering(268,0) = 459;
          numbering(268,1) = 469;
          numbering(268,2) = 470;
          numbering(268,3) = 460;
          numbering(268,4) = 85;
          numbering(268,5) = 475;
          numbering(268,6) = 476;
          numbering(268,7) = 467;
          numbering(269,0) = 469;
          numbering(269,1) = 471;
          numbering(269,2) = 472;
          numbering(269,3) = 470;
          numbering(269,4) = 475;
          numbering(269,5) = 88;
          numbering(269,6) = 477;
          numbering(269,7) = 476;
          numbering(270,0) = 460;
          numbering(270,1) = 470;
          numbering(270,2) = 473;
          numbering(270,3) = 463;
          numbering(270,4) = 467;
          numbering(270,5) = 476;
          numbering(270,6) = 478;
          numbering(270,7) = 86;
          numbering(271,0) = 470;
          numbering(271,1) = 472;
          numbering(271,2) = 474;
          numbering(271,3) = 473;
          numbering(271,4) = 476;
          numbering(271,5) = 477;
          numbering(271,6) = 89;
          numbering(271,7) = 478;
          numbering(272,0) = 43;
          numbering(272,1) = 195;
          numbering(272,2) = 212;
          numbering(272,3) = 213;
          numbering(272,4) = 462;
          numbering(272,5) = 461;
          numbering(272,6) = 479;
          numbering(272,7) = 480;
          numbering(273,0) = 195;
          numbering(273,1) = 42;
          numbering(273,2) = 214;
          numbering(273,3) = 212;
          numbering(273,4) = 461;
          numbering(273,5) = 463;
          numbering(273,6) = 481;
          numbering(273,7) = 479;
          numbering(274,0) = 213;
          numbering(274,1) = 212;
          numbering(274,2) = 215;
          numbering(274,3) = 14;
          numbering(274,4) = 480;
          numbering(274,5) = 479;
          numbering(274,6) = 482;
          numbering(274,7) = 483;
          numbering(275,0) = 212;
          numbering(275,1) = 214;
          numbering(275,2) = 45;
          numbering(275,3) = 215;
          numbering(275,4) = 479;
          numbering(275,5) = 481;
          numbering(275,6) = 484;
          numbering(275,7) = 482;
          numbering(276,0) = 462;
          numbering(276,1) = 461;
          numbering(276,2) = 479;
          numbering(276,3) = 480;
          numbering(276,4) = 87;
          numbering(276,5) = 468;
          numbering(276,6) = 485;
          numbering(276,7) = 486;
          numbering(277,0) = 461;
          numbering(277,1) = 463;
          numbering(277,2) = 481;
          numbering(277,3) = 479;
          numbering(277,4) = 468;
          numbering(277,5) = 86;
          numbering(277,6) = 487;
          numbering(277,7) = 485;
          numbering(278,0) = 480;
          numbering(278,1) = 479;
          numbering(278,2) = 482;
          numbering(278,3) = 483;
          numbering(278,4) = 486;
          numbering(278,5) = 485;
          numbering(278,6) = 488;
          numbering(278,7) = 91;
          numbering(279,0) = 479;
          numbering(279,1) = 481;
          numbering(279,2) = 484;
          numbering(279,3) = 482;
          numbering(279,4) = 485;
          numbering(279,5) = 487;
          numbering(279,6) = 90;
          numbering(279,7) = 488;
          numbering(280,0) = 42;
          numbering(280,1) = 205;
          numbering(280,2) = 220;
          numbering(280,3) = 214;
          numbering(280,4) = 463;
          numbering(280,5) = 473;
          numbering(280,6) = 489;
          numbering(280,7) = 481;
          numbering(281,0) = 205;
          numbering(281,1) = 44;
          numbering(281,2) = 221;
          numbering(281,3) = 220;
          numbering(281,4) = 473;
          numbering(281,5) = 474;
          numbering(281,6) = 490;
          numbering(281,7) = 489;
          numbering(282,0) = 214;
          numbering(282,1) = 220;
          numbering(282,2) = 222;
          numbering(282,3) = 45;
          numbering(282,4) = 481;
          numbering(282,5) = 489;
          numbering(282,6) = 491;
          numbering(282,7) = 484;
          numbering(283,0) = 220;
          numbering(283,1) = 221;
          numbering(283,2) = 13;
          numbering(283,3) = 222;
          numbering(283,4) = 489;
          numbering(283,5) = 490;
          numbering(283,6) = 492;
          numbering(283,7) = 491;
          numbering(284,0) = 463;
          numbering(284,1) = 473;
          numbering(284,2) = 489;
          numbering(284,3) = 481;
          numbering(284,4) = 86;
          numbering(284,5) = 478;
          numbering(284,6) = 493;
          numbering(284,7) = 487;
          numbering(285,0) = 473;
          numbering(285,1) = 474;
          numbering(285,2) = 490;
          numbering(285,3) = 489;
          numbering(285,4) = 478;
          numbering(285,5) = 89;
          numbering(285,6) = 494;
          numbering(285,7) = 493;
          numbering(286,0) = 481;
          numbering(286,1) = 489;
          numbering(286,2) = 491;
          numbering(286,3) = 484;
          numbering(286,4) = 487;
          numbering(286,5) = 493;
          numbering(286,6) = 495;
          numbering(286,7) = 90;
          numbering(287,0) = 489;
          numbering(287,1) = 490;
          numbering(287,2) = 492;
          numbering(287,3) = 491;
          numbering(287,4) = 493;
          numbering(287,5) = 494;
          numbering(287,6) = 92;
          numbering(287,7) = 495;
          numbering(288,0) = 84;
          numbering(288,1) = 464;
          numbering(288,2) = 465;
          numbering(288,3) = 466;
          numbering(288,4) = 496;
          numbering(288,5) = 497;
          numbering(288,6) = 498;
          numbering(288,7) = 499;
          numbering(289,0) = 464;
          numbering(289,1) = 85;
          numbering(289,2) = 467;
          numbering(289,3) = 465;
          numbering(289,4) = 497;
          numbering(289,5) = 500;
          numbering(289,6) = 501;
          numbering(289,7) = 498;
          numbering(290,0) = 466;
          numbering(290,1) = 465;
          numbering(290,2) = 468;
          numbering(290,3) = 87;
          numbering(290,4) = 499;
          numbering(290,5) = 498;
          numbering(290,6) = 502;
          numbering(290,7) = 503;
          numbering(291,0) = 465;
          numbering(291,1) = 467;
          numbering(291,2) = 86;
          numbering(291,3) = 468;
          numbering(291,4) = 498;
          numbering(291,5) = 501;
          numbering(291,6) = 504;
          numbering(291,7) = 502;
          numbering(292,0) = 496;
          numbering(292,1) = 497;
          numbering(292,2) = 498;
          numbering(292,3) = 499;
          numbering(292,4) = 4;
          numbering(292,5) = 505;
          numbering(292,6) = 506;
          numbering(292,7) = 507;
          numbering(293,0) = 497;
          numbering(293,1) = 500;
          numbering(293,2) = 501;
          numbering(293,3) = 498;
          numbering(293,4) = 505;
          numbering(293,5) = 93;
          numbering(293,6) = 508;
          numbering(293,7) = 506;
          numbering(294,0) = 499;
          numbering(294,1) = 498;
          numbering(294,2) = 502;
          numbering(294,3) = 503;
          numbering(294,4) = 507;
          numbering(294,5) = 506;
          numbering(294,6) = 509;
          numbering(294,7) = 95;
          numbering(295,0) = 498;
          numbering(295,1) = 501;
          numbering(295,2) = 504;
          numbering(295,3) = 502;
          numbering(295,4) = 506;
          numbering(295,5) = 508;
          numbering(295,6) = 94;
          numbering(295,7) = 509;
          numbering(296,0) = 85;
          numbering(296,1) = 475;
          numbering(296,2) = 476;
          numbering(296,3) = 467;
          numbering(296,4) = 500;
          numbering(296,5) = 510;
          numbering(296,6) = 511;
          numbering(296,7) = 501;
          numbering(297,0) = 475;
          numbering(297,1) = 88;
          numbering(297,2) = 477;
          numbering(297,3) = 476;
          numbering(297,4) = 510;
          numbering(297,5) = 512;
          numbering(297,6) = 513;
          numbering(297,7) = 511;
          numbering(298,0) = 467;
          numbering(298,1) = 476;
          numbering(298,2) = 478;
          numbering(298,3) = 86;
          numbering(298,4) = 501;
          numbering(298,5) = 511;
          numbering(298,6) = 514;
          numbering(298,7) = 504;
          numbering(299,0) = 476;
          numbering(299,1) = 477;
          numbering(299,2) = 89;
          numbering(299,3) = 478;
          numbering(299,4) = 511;
          numbering(299,5) = 513;
          numbering(299,6) = 515;
          numbering(299,7) = 514;
          numbering(300,0) = 500;
          numbering(300,1) = 510;
          numbering(300,2) = 511;
          numbering(300,3) = 501;
          numbering(300,4) = 93;
          numbering(300,5) = 516;
          numbering(300,6) = 517;
          numbering(300,7) = 508;
          numbering(301,0) = 510;
          numbering(301,1) = 512;
          numbering(301,2) = 513;
          numbering(301,3) = 511;
          numbering(301,4) = 516;
          numbering(301,5) = 22;
          numbering(301,6) = 518;
          numbering(301,7) = 517;
          numbering(302,0) = 501;
          numbering(302,1) = 511;
          numbering(302,2) = 514;
          numbering(302,3) = 504;
          numbering(302,4) = 508;
          numbering(302,5) = 517;
          numbering(302,6) = 519;
          numbering(302,7) = 94;
          numbering(303,0) = 511;
          numbering(303,1) = 513;
          numbering(303,2) = 515;
          numbering(303,3) = 514;
          numbering(303,4) = 517;
          numbering(303,5) = 518;
          numbering(303,6) = 96;
          numbering(303,7) = 519;
          numbering(304,0) = 87;
          numbering(304,1) = 468;
          numbering(304,2) = 485;
          numbering(304,3) = 486;
          numbering(304,4) = 503;
          numbering(304,5) = 502;
          numbering(304,6) = 520;
          numbering(304,7) = 521;
          numbering(305,0) = 468;
          numbering(305,1) = 86;
          numbering(305,2) = 487;
          numbering(305,3) = 485;
          numbering(305,4) = 502;
          numbering(305,5) = 504;
          numbering(305,6) = 522;
          numbering(305,7) = 520;
          numbering(306,0) = 486;
          numbering(306,1) = 485;
          numbering(306,2) = 488;
          numbering(306,3) = 91;
          numbering(306,4) = 521;
          numbering(306,5) = 520;
          numbering(306,6) = 523;
          numbering(306,7) = 524;
          numbering(307,0) = 485;
          numbering(307,1) = 487;
          numbering(307,2) = 90;
          numbering(307,3) = 488;
          numbering(307,4) = 520;
          numbering(307,5) = 522;
          numbering(307,6) = 525;
          numbering(307,7) = 523;
          numbering(308,0) = 503;
          numbering(308,1) = 502;
          numbering(308,2) = 520;
          numbering(308,3) = 521;
          numbering(308,4) = 95;
          numbering(308,5) = 509;
          numbering(308,6) = 526;
          numbering(308,7) = 527;
          numbering(309,0) = 502;
          numbering(309,1) = 504;
          numbering(309,2) = 522;
          numbering(309,3) = 520;
          numbering(309,4) = 509;
          numbering(309,5) = 94;
          numbering(309,6) = 528;
          numbering(309,7) = 526;
          numbering(310,0) = 521;
          numbering(310,1) = 520;
          numbering(310,2) = 523;
          numbering(310,3) = 524;
          numbering(310,4) = 527;
          numbering(310,5) = 526;
          numbering(310,6) = 529;
          numbering(310,7) = 24;
          numbering(311,0) = 520;
          numbering(311,1) = 522;
          numbering(311,2) = 525;
          numbering(311,3) = 523;
          numbering(311,4) = 526;
          numbering(311,5) = 528;
          numbering(311,6) = 97;
          numbering(311,7) = 529;
          numbering(312,0) = 86;
          numbering(312,1) = 478;
          numbering(312,2) = 493;
          numbering(312,3) = 487;
          numbering(312,4) = 504;
          numbering(312,5) = 514;
          numbering(312,6) = 530;
          numbering(312,7) = 522;
          numbering(313,0) = 478;
          numbering(313,1) = 89;
          numbering(313,2) = 494;
          numbering(313,3) = 493;
          numbering(313,4) = 514;
          numbering(313,5) = 515;
          numbering(313,6) = 531;
          numbering(313,7) = 530;
          numbering(314,0) = 487;
          numbering(314,1) = 493;
          numbering(314,2) = 495;
          numbering(314,3) = 90;
          numbering(314,4) = 522;
          numbering(314,5) = 530;
          numbering(314,6) = 532;
          numbering(314,7) = 525;
          numbering(315,0) = 493;
          numbering(315,1) = 494;
          numbering(315,2) = 92;
          numbering(315,3) = 495;
          numbering(315,4) = 530;
          numbering(315,5) = 531;
          numbering(315,6) = 533;
          numbering(315,7) = 532;
          numbering(316,0) = 504;
          numbering(316,1) = 514;
          numbering(316,2) = 530;
          numbering(316,3) = 522;
          numbering(316,4) = 94;
          numbering(316,5) = 519;
          numbering(316,6) = 534;
          numbering(316,7) = 528;
          numbering(317,0) = 514;
          numbering(317,1) = 515;
          numbering(317,2) = 531;
          numbering(317,3) = 530;
          numbering(317,4) = 519;
          numbering(317,5) = 96;
          numbering(317,6) = 535;
          numbering(317,7) = 534;
          numbering(318,0) = 522;
          numbering(318,1) = 530;
          numbering(318,2) = 532;
          numbering(318,3) = 525;
          numbering(318,4) = 528;
          numbering(318,5) = 534;
          numbering(318,6) = 536;
          numbering(318,7) = 97;
          numbering(319,0) = 530;
          numbering(319,1) = 531;
          numbering(319,2) = 533;
          numbering(319,3) = 532;
          numbering(319,4) = 534;
          numbering(319,5) = 535;
          numbering(319,6) = 23;
          numbering(319,7) = 536;
          numbering(320,0) = 12;
          numbering(320,1) = 277;
          numbering(320,2) = 278;
          numbering(320,3) = 204;
          numbering(320,4) = 471;
          numbering(320,5) = 537;
          numbering(320,6) = 538;
          numbering(320,7) = 472;
          numbering(321,0) = 277;
          numbering(321,1) = 56;
          numbering(321,2) = 279;
          numbering(321,3) = 278;
          numbering(321,4) = 537;
          numbering(321,5) = 539;
          numbering(321,6) = 540;
          numbering(321,7) = 538;
          numbering(322,0) = 204;
          numbering(322,1) = 278;
          numbering(322,2) = 280;
          numbering(322,3) = 44;
          numbering(322,4) = 472;
          numbering(322,5) = 538;
          numbering(322,6) = 541;
          numbering(322,7) = 474;
          numbering(323,0) = 278;
          numbering(323,1) = 279;
          numbering(323,2) = 57;
          numbering(323,3) = 280;
          numbering(323,4) = 538;
          numbering(323,5) = 540;
          numbering(323,6) = 542;
          numbering(323,7) = 541;
          numbering(324,0) = 471;
          numbering(324,1) = 537;
          numbering(324,2) = 538;
          numbering(324,3) = 472;
          numbering(324,4) = 88;
          numbering(324,5) = 543;
          numbering(324,6) = 544;
          numbering(324,7) = 477;
          numbering(325,0) = 537;
          numbering(325,1) = 539;
          numbering(325,2) = 540;
          numbering(325,3) = 538;
          numbering(325,4) = 543;
          numbering(325,5) = 98;
          numbering(325,6) = 545;
          numbering(325,7) = 544;
          numbering(326,0) = 472;
          numbering(326,1) = 538;
          numbering(326,2) = 541;
          numbering(326,3) = 474;
          numbering(326,4) = 477;
          numbering(326,5) = 544;
          numbering(326,6) = 546;
          numbering(326,7) = 89;
          numbering(327,0) = 538;
          numbering(327,1) = 540;
          numbering(327,2) = 542;
          numbering(327,3) = 541;
          numbering(327,4) = 544;
          numbering(327,5) = 545;
          numbering(327,6) = 99;
          numbering(327,7) = 546;
          numbering(328,0) = 56;
          numbering(328,1) = 287;
          numbering(328,2) = 288;
          numbering(328,3) = 279;
          numbering(328,4) = 539;
          numbering(328,5) = 547;
          numbering(328,6) = 548;
          numbering(328,7) = 540;
          numbering(329,0) = 287;
          numbering(329,1) = 16;
          numbering(329,2) = 289;
          numbering(329,3) = 288;
          numbering(329,4) = 547;
          numbering(329,5) = 549;
          numbering(329,6) = 550;
          numbering(329,7) = 548;
          numbering(330,0) = 279;
          numbering(330,1) = 288;
          numbering(330,2) = 290;
          numbering(330,3) = 57;
          numbering(330,4) = 540;
          numbering(330,5) = 548;
          numbering(330,6) = 551;
          numbering(330,7) = 542;
          numbering(331,0) = 288;
          numbering(331,1) = 289;
          numbering(331,2) = 58;
          numbering(331,3) = 290;
          numbering(331,4) = 548;
          numbering(331,5) = 550;
          numbering(331,6) = 552;
          numbering(331,7) = 551;
          numbering(332,0) = 539;
          numbering(332,1) = 547;
          numbering(332,2) = 548;
          numbering(332,3) = 540;
          numbering(332,4) = 98;
          numbering(332,5) = 553;
          numbering(332,6) = 554;
          numbering(332,7) = 545;
          numbering(333,0) = 547;
          numbering(333,1) = 549;
          numbering(333,2) = 550;
          numbering(333,3) = 548;
          numbering(333,4) = 553;
          numbering(333,5) = 100;
          numbering(333,6) = 555;
          numbering(333,7) = 554;
          numbering(334,0) = 540;
          numbering(334,1) = 548;
          numbering(334,2) = 551;
          numbering(334,3) = 542;
          numbering(334,4) = 545;
          numbering(334,5) = 554;
          numbering(334,6) = 556;
          numbering(334,7) = 99;
          numbering(335,0) = 548;
          numbering(335,1) = 550;
          numbering(335,2) = 552;
          numbering(335,3) = 551;
          numbering(335,4) = 554;
          numbering(335,5) = 555;
          numbering(335,6) = 101;
          numbering(335,7) = 556;
          numbering(336,0) = 44;
          numbering(336,1) = 280;
          numbering(336,2) = 295;
          numbering(336,3) = 221;
          numbering(336,4) = 474;
          numbering(336,5) = 541;
          numbering(336,6) = 557;
          numbering(336,7) = 490;
          numbering(337,0) = 280;
          numbering(337,1) = 57;
          numbering(337,2) = 296;
          numbering(337,3) = 295;
          numbering(337,4) = 541;
          numbering(337,5) = 542;
          numbering(337,6) = 558;
          numbering(337,7) = 557;
          numbering(338,0) = 221;
          numbering(338,1) = 295;
          numbering(338,2) = 297;
          numbering(338,3) = 13;
          numbering(338,4) = 490;
          numbering(338,5) = 557;
          numbering(338,6) = 559;
          numbering(338,7) = 492;
          numbering(339,0) = 295;
          numbering(339,1) = 296;
          numbering(339,2) = 59;
          numbering(339,3) = 297;
          numbering(339,4) = 557;
          numbering(339,5) = 558;
          numbering(339,6) = 560;
          numbering(339,7) = 559;
          numbering(340,0) = 474;
          numbering(340,1) = 541;
          numbering(340,2) = 557;
          numbering(340,3) = 490;
          numbering(340,4) = 89;
          numbering(340,5) = 546;
          numbering(340,6) = 561;
          numbering(340,7) = 494;
          numbering(341,0) = 541;
          numbering(341,1) = 542;
          numbering(341,2) = 558;
          numbering(341,3) = 557;
          numbering(341,4) = 546;
          numbering(341,5) = 99;
          numbering(341,6) = 562;
          numbering(341,7) = 561;
          numbering(342,0) = 490;
          numbering(342,1) = 557;
          numbering(342,2) = 559;
          numbering(342,3) = 492;
          numbering(342,4) = 494;
          numbering(342,5) = 561;
          numbering(342,6) = 563;
          numbering(342,7) = 92;
          numbering(343,0) = 557;
          numbering(343,1) = 558;
          numbering(343,2) = 560;
          numbering(343,3) = 559;
          numbering(343,4) = 561;
          numbering(343,5) = 562;
          numbering(343,6) = 102;
          numbering(343,7) = 563;
          numbering(344,0) = 57;
          numbering(344,1) = 290;
          numbering(344,2) = 302;
          numbering(344,3) = 296;
          numbering(344,4) = 542;
          numbering(344,5) = 551;
          numbering(344,6) = 564;
          numbering(344,7) = 558;
          numbering(345,0) = 290;
          numbering(345,1) = 58;
          numbering(345,2) = 303;
          numbering(345,3) = 302;
          numbering(345,4) = 551;
          numbering(345,5) = 552;
          numbering(345,6) = 565;
          numbering(345,7) = 564;
          numbering(346,0) = 296;
          numbering(346,1) = 302;
          numbering(346,2) = 304;
          numbering(346,3) = 59;
          numbering(346,4) = 558;
          numbering(346,5) = 564;
          numbering(346,6) = 566;
          numbering(346,7) = 560;
          numbering(347,0) = 302;
          numbering(347,1) = 303;
          numbering(347,2) = 17;
          numbering(347,3) = 304;
          numbering(347,4) = 564;
          numbering(347,5) = 565;
          numbering(347,6) = 567;
          numbering(347,7) = 566;
          numbering(348,0) = 542;
          numbering(348,1) = 551;
          numbering(348,2) = 564;
          numbering(348,3) = 558;
          numbering(348,4) = 99;
          numbering(348,5) = 556;
          numbering(348,6) = 568;
          numbering(348,7) = 562;
          numbering(349,0) = 551;
          numbering(349,1) = 552;
          numbering(349,2) = 565;
          numbering(349,3) = 564;
          numbering(349,4) = 556;
          numbering(349,5) = 101;
          numbering(349,6) = 569;
          numbering(349,7) = 568;
          numbering(350,0) = 558;
          numbering(350,1) = 564;
          numbering(350,2) = 566;
          numbering(350,3) = 560;
          numbering(350,4) = 562;
          numbering(350,5) = 568;
          numbering(350,6) = 570;
          numbering(350,7) = 102;
          numbering(351,0) = 564;
          numbering(351,1) = 565;
          numbering(351,2) = 567;
          numbering(351,3) = 566;
          numbering(351,4) = 568;
          numbering(351,5) = 569;
          numbering(351,6) = 103;
          numbering(351,7) = 570;
          numbering(352,0) = 88;
          numbering(352,1) = 543;
          numbering(352,2) = 544;
          numbering(352,3) = 477;
          numbering(352,4) = 512;
          numbering(352,5) = 571;
          numbering(352,6) = 572;
          numbering(352,7) = 513;
          numbering(353,0) = 543;
          numbering(353,1) = 98;
          numbering(353,2) = 545;
          numbering(353,3) = 544;
          numbering(353,4) = 571;
          numbering(353,5) = 573;
          numbering(353,6) = 574;
          numbering(353,7) = 572;
          numbering(354,0) = 477;
          numbering(354,1) = 544;
          numbering(354,2) = 546;
          numbering(354,3) = 89;
          numbering(354,4) = 513;
          numbering(354,5) = 572;
          numbering(354,6) = 575;
          numbering(354,7) = 515;
          numbering(355,0) = 544;
          numbering(355,1) = 545;
          numbering(355,2) = 99;
          numbering(355,3) = 546;
          numbering(355,4) = 572;
          numbering(355,5) = 574;
          numbering(355,6) = 576;
          numbering(355,7) = 575;
          numbering(356,0) = 512;
          numbering(356,1) = 571;
          numbering(356,2) = 572;
          numbering(356,3) = 513;
          numbering(356,4) = 22;
          numbering(356,5) = 577;
          numbering(356,6) = 578;
          numbering(356,7) = 518;
          numbering(357,0) = 571;
          numbering(357,1) = 573;
          numbering(357,2) = 574;
          numbering(357,3) = 572;
          numbering(357,4) = 577;
          numbering(357,5) = 104;
          numbering(357,6) = 579;
          numbering(357,7) = 578;
          numbering(358,0) = 513;
          numbering(358,1) = 572;
          numbering(358,2) = 575;
          numbering(358,3) = 515;
          numbering(358,4) = 518;
          numbering(358,5) = 578;
          numbering(358,6) = 580;
          numbering(358,7) = 96;
          numbering(359,0) = 572;
          numbering(359,1) = 574;
          numbering(359,2) = 576;
          numbering(359,3) = 575;
          numbering(359,4) = 578;
          numbering(359,5) = 579;
          numbering(359,6) = 105;
          numbering(359,7) = 580;
          numbering(360,0) = 98;
          numbering(360,1) = 553;
          numbering(360,2) = 554;
          numbering(360,3) = 545;
          numbering(360,4) = 573;
          numbering(360,5) = 581;
          numbering(360,6) = 582;
          numbering(360,7) = 574;
          numbering(361,0) = 553;
          numbering(361,1) = 100;
          numbering(361,2) = 555;
          numbering(361,3) = 554;
          numbering(361,4) = 581;
          numbering(361,5) = 583;
          numbering(361,6) = 584;
          numbering(361,7) = 582;
          numbering(362,0) = 545;
          numbering(362,1) = 554;
          numbering(362,2) = 556;
          numbering(362,3) = 99;
          numbering(362,4) = 574;
          numbering(362,5) = 582;
          numbering(362,6) = 585;
          numbering(362,7) = 576;
          numbering(363,0) = 554;
          numbering(363,1) = 555;
          numbering(363,2) = 101;
          numbering(363,3) = 556;
          numbering(363,4) = 582;
          numbering(363,5) = 584;
          numbering(363,6) = 586;
          numbering(363,7) = 585;
          numbering(364,0) = 573;
          numbering(364,1) = 581;
          numbering(364,2) = 582;
          numbering(364,3) = 574;
          numbering(364,4) = 104;
          numbering(364,5) = 587;
          numbering(364,6) = 588;
          numbering(364,7) = 579;
          numbering(365,0) = 581;
          numbering(365,1) = 583;
          numbering(365,2) = 584;
          numbering(365,3) = 582;
          numbering(365,4) = 587;
          numbering(365,5) = 5;
          numbering(365,6) = 589;
          numbering(365,7) = 588;
          numbering(366,0) = 574;
          numbering(366,1) = 582;
          numbering(366,2) = 585;
          numbering(366,3) = 576;
          numbering(366,4) = 579;
          numbering(366,5) = 588;
          numbering(366,6) = 590;
          numbering(366,7) = 105;
          numbering(367,0) = 582;
          numbering(367,1) = 584;
          numbering(367,2) = 586;
          numbering(367,3) = 585;
          numbering(367,4) = 588;
          numbering(367,5) = 589;
          numbering(367,6) = 106;
          numbering(367,7) = 590;
          numbering(368,0) = 89;
          numbering(368,1) = 546;
          numbering(368,2) = 561;
          numbering(368,3) = 494;
          numbering(368,4) = 515;
          numbering(368,5) = 575;
          numbering(368,6) = 591;
          numbering(368,7) = 531;
          numbering(369,0) = 546;
          numbering(369,1) = 99;
          numbering(369,2) = 562;
          numbering(369,3) = 561;
          numbering(369,4) = 575;
          numbering(369,5) = 576;
          numbering(369,6) = 592;
          numbering(369,7) = 591;
          numbering(370,0) = 494;
          numbering(370,1) = 561;
          numbering(370,2) = 563;
          numbering(370,3) = 92;
          numbering(370,4) = 531;
          numbering(370,5) = 591;
          numbering(370,6) = 593;
          numbering(370,7) = 533;
          numbering(371,0) = 561;
          numbering(371,1) = 562;
          numbering(371,2) = 102;
          numbering(371,3) = 563;
          numbering(371,4) = 591;
          numbering(371,5) = 592;
          numbering(371,6) = 594;
          numbering(371,7) = 593;
          numbering(372,0) = 515;
          numbering(372,1) = 575;
          numbering(372,2) = 591;
          numbering(372,3) = 531;
          numbering(372,4) = 96;
          numbering(372,5) = 580;
          numbering(372,6) = 595;
          numbering(372,7) = 535;
          numbering(373,0) = 575;
          numbering(373,1) = 576;
          numbering(373,2) = 592;
          numbering(373,3) = 591;
          numbering(373,4) = 580;
          numbering(373,5) = 105;
          numbering(373,6) = 596;
          numbering(373,7) = 595;
          numbering(374,0) = 531;
          numbering(374,1) = 591;
          numbering(374,2) = 593;
          numbering(374,3) = 533;
          numbering(374,4) = 535;
          numbering(374,5) = 595;
          numbering(374,6) = 597;
          numbering(374,7) = 23;
          numbering(375,0) = 591;
          numbering(375,1) = 592;
          numbering(375,2) = 594;
          numbering(375,3) = 593;
          numbering(375,4) = 595;
          numbering(375,5) = 596;
          numbering(375,6) = 107;
          numbering(375,7) = 597;
          numbering(376,0) = 99;
          numbering(376,1) = 556;
          numbering(376,2) = 568;
          numbering(376,3) = 562;
          numbering(376,4) = 576;
          numbering(376,5) = 585;
          numbering(376,6) = 598;
          numbering(376,7) = 592;
          numbering(377,0) = 556;
          numbering(377,1) = 101;
          numbering(377,2) = 569;
          numbering(377,3) = 568;
          numbering(377,4) = 585;
          numbering(377,5) = 586;
          numbering(377,6) = 599;
          numbering(377,7) = 598;
          numbering(378,0) = 562;
          numbering(378,1) = 568;
          numbering(378,2) = 570;
          numbering(378,3) = 102;
          numbering(378,4) = 592;
          numbering(378,5) = 598;
          numbering(378,6) = 600;
          numbering(378,7) = 594;
          numbering(379,0) = 568;
          numbering(379,1) = 569;
          numbering(379,2) = 103;
          numbering(379,3) = 570;
          numbering(379,4) = 598;
          numbering(379,5) = 599;
          numbering(379,6) = 601;
          numbering(379,7) = 600;
          numbering(380,0) = 576;
          numbering(380,1) = 585;
          numbering(380,2) = 598;
          numbering(380,3) = 592;
          numbering(380,4) = 105;
          numbering(380,5) = 590;
          numbering(380,6) = 602;
          numbering(380,7) = 596;
          numbering(381,0) = 585;
          numbering(381,1) = 586;
          numbering(381,2) = 599;
          numbering(381,3) = 598;
          numbering(381,4) = 590;
          numbering(381,5) = 106;
          numbering(381,6) = 603;
          numbering(381,7) = 602;
          numbering(382,0) = 592;
          numbering(382,1) = 598;
          numbering(382,2) = 600;
          numbering(382,3) = 594;
          numbering(382,4) = 596;
          numbering(382,5) = 602;
          numbering(382,6) = 604;
          numbering(382,7) = 107;
          numbering(383,0) = 598;
          numbering(383,1) = 599;
          numbering(383,2) = 601;
          numbering(383,3) = 600;
          numbering(383,4) = 602;
          numbering(383,5) = 603;
          numbering(383,6) = 25;
          numbering(383,7) = 604;
          numbering(384,0) = 14;
          numbering(384,1) = 215;
          numbering(384,2) = 359;
          numbering(384,3) = 360;
          numbering(384,4) = 483;
          numbering(384,5) = 482;
          numbering(384,6) = 605;
          numbering(384,7) = 606;
          numbering(385,0) = 215;
          numbering(385,1) = 45;
          numbering(385,2) = 361;
          numbering(385,3) = 359;
          numbering(385,4) = 482;
          numbering(385,5) = 484;
          numbering(385,6) = 607;
          numbering(385,7) = 605;
          numbering(386,0) = 360;
          numbering(386,1) = 359;
          numbering(386,2) = 362;
          numbering(386,3) = 71;
          numbering(386,4) = 606;
          numbering(386,5) = 605;
          numbering(386,6) = 608;
          numbering(386,7) = 609;
          numbering(387,0) = 359;
          numbering(387,1) = 361;
          numbering(387,2) = 70;
          numbering(387,3) = 362;
          numbering(387,4) = 605;
          numbering(387,5) = 607;
          numbering(387,6) = 610;
          numbering(387,7) = 608;
          numbering(388,0) = 483;
          numbering(388,1) = 482;
          numbering(388,2) = 605;
          numbering(388,3) = 606;
          numbering(388,4) = 91;
          numbering(388,5) = 488;
          numbering(388,6) = 611;
          numbering(388,7) = 612;
          numbering(389,0) = 482;
          numbering(389,1) = 484;
          numbering(389,2) = 607;
          numbering(389,3) = 605;
          numbering(389,4) = 488;
          numbering(389,5) = 90;
          numbering(389,6) = 613;
          numbering(389,7) = 611;
          numbering(390,0) = 606;
          numbering(390,1) = 605;
          numbering(390,2) = 608;
          numbering(390,3) = 609;
          numbering(390,4) = 612;
          numbering(390,5) = 611;
          numbering(390,6) = 614;
          numbering(390,7) = 109;
          numbering(391,0) = 605;
          numbering(391,1) = 607;
          numbering(391,2) = 610;
          numbering(391,3) = 608;
          numbering(391,4) = 611;
          numbering(391,5) = 613;
          numbering(391,6) = 108;
          numbering(391,7) = 614;
          numbering(392,0) = 45;
          numbering(392,1) = 222;
          numbering(392,2) = 367;
          numbering(392,3) = 361;
          numbering(392,4) = 484;
          numbering(392,5) = 491;
          numbering(392,6) = 615;
          numbering(392,7) = 607;
          numbering(393,0) = 222;
          numbering(393,1) = 13;
          numbering(393,2) = 368;
          numbering(393,3) = 367;
          numbering(393,4) = 491;
          numbering(393,5) = 492;
          numbering(393,6) = 616;
          numbering(393,7) = 615;
          numbering(394,0) = 361;
          numbering(394,1) = 367;
          numbering(394,2) = 369;
          numbering(394,3) = 70;
          numbering(394,4) = 607;
          numbering(394,5) = 615;
          numbering(394,6) = 617;
          numbering(394,7) = 610;
          numbering(395,0) = 367;
          numbering(395,1) = 368;
          numbering(395,2) = 72;
          numbering(395,3) = 369;
          numbering(395,4) = 615;
          numbering(395,5) = 616;
          numbering(395,6) = 618;
          numbering(395,7) = 617;
          numbering(396,0) = 484;
          numbering(396,1) = 491;
          numbering(396,2) = 615;
          numbering(396,3) = 607;
          numbering(396,4) = 90;
          numbering(396,5) = 495;
          numbering(396,6) = 619;
          numbering(396,7) = 613;
          numbering(397,0) = 491;
          numbering(397,1) = 492;
          numbering(397,2) = 616;
          numbering(397,3) = 615;
          numbering(397,4) = 495;
          numbering(397,5) = 92;
          numbering(397,6) = 620;
          numbering(397,7) = 619;
          numbering(398,0) = 607;
          numbering(398,1) = 615;
          numbering(398,2) = 617;
          numbering(398,3) = 610;
          numbering(398,4) = 613;
          numbering(398,5) = 619;
          numbering(398,6) = 621;
          numbering(398,7) = 108;
          numbering(399,0) = 615;
          numbering(399,1) = 616;
          numbering(399,2) = 618;
          numbering(399,3) = 617;
          numbering(399,4) = 619;
          numbering(399,5) = 620;
          numbering(399,6) = 110;
          numbering(399,7) = 621;
          numbering(400,0) = 71;
          numbering(400,1) = 362;
          numbering(400,2) = 376;
          numbering(400,3) = 377;
          numbering(400,4) = 609;
          numbering(400,5) = 608;
          numbering(400,6) = 622;
          numbering(400,7) = 623;
          numbering(401,0) = 362;
          numbering(401,1) = 70;
          numbering(401,2) = 378;
          numbering(401,3) = 376;
          numbering(401,4) = 608;
          numbering(401,5) = 610;
          numbering(401,6) = 624;
          numbering(401,7) = 622;
          numbering(402,0) = 377;
          numbering(402,1) = 376;
          numbering(402,2) = 379;
          numbering(402,3) = 20;
          numbering(402,4) = 623;
          numbering(402,5) = 622;
          numbering(402,6) = 625;
          numbering(402,7) = 626;
          numbering(403,0) = 376;
          numbering(403,1) = 378;
          numbering(403,2) = 73;
          numbering(403,3) = 379;
          numbering(403,4) = 622;
          numbering(403,5) = 624;
          numbering(403,6) = 627;
          numbering(403,7) = 625;
          numbering(404,0) = 609;
          numbering(404,1) = 608;
          numbering(404,2) = 622;
          numbering(404,3) = 623;
          numbering(404,4) = 109;
          numbering(404,5) = 614;
          numbering(404,6) = 628;
          numbering(404,7) = 629;
          numbering(405,0) = 608;
          numbering(405,1) = 610;
          numbering(405,2) = 624;
          numbering(405,3) = 622;
          numbering(405,4) = 614;
          numbering(405,5) = 108;
          numbering(405,6) = 630;
          numbering(405,7) = 628;
          numbering(406,0) = 623;
          numbering(406,1) = 622;
          numbering(406,2) = 625;
          numbering(406,3) = 626;
          numbering(406,4) = 629;
          numbering(406,5) = 628;
          numbering(406,6) = 631;
          numbering(406,7) = 112;
          numbering(407,0) = 622;
          numbering(407,1) = 624;
          numbering(407,2) = 627;
          numbering(407,3) = 625;
          numbering(407,4) = 628;
          numbering(407,5) = 630;
          numbering(407,6) = 111;
          numbering(407,7) = 631;
          numbering(408,0) = 70;
          numbering(408,1) = 369;
          numbering(408,2) = 384;
          numbering(408,3) = 378;
          numbering(408,4) = 610;
          numbering(408,5) = 617;
          numbering(408,6) = 632;
          numbering(408,7) = 624;
          numbering(409,0) = 369;
          numbering(409,1) = 72;
          numbering(409,2) = 385;
          numbering(409,3) = 384;
          numbering(409,4) = 617;
          numbering(409,5) = 618;
          numbering(409,6) = 633;
          numbering(409,7) = 632;
          numbering(410,0) = 378;
          numbering(410,1) = 384;
          numbering(410,2) = 386;
          numbering(410,3) = 73;
          numbering(410,4) = 624;
          numbering(410,5) = 632;
          numbering(410,6) = 634;
          numbering(410,7) = 627;
          numbering(411,0) = 384;
          numbering(411,1) = 385;
          numbering(411,2) = 19;
          numbering(411,3) = 386;
          numbering(411,4) = 632;
          numbering(411,5) = 633;
          numbering(411,6) = 635;
          numbering(411,7) = 634;
          numbering(412,0) = 610;
          numbering(412,1) = 617;
          numbering(412,2) = 632;
          numbering(412,3) = 624;
          numbering(412,4) = 108;
          numbering(412,5) = 621;
          numbering(412,6) = 636;
          numbering(412,7) = 630;
          numbering(413,0) = 617;
          numbering(413,1) = 618;
          numbering(413,2) = 633;
          numbering(413,3) = 632;
          numbering(413,4) = 621;
          numbering(413,5) = 110;
          numbering(413,6) = 637;
          numbering(413,7) = 636;
          numbering(414,0) = 624;
          numbering(414,1) = 632;
          numbering(414,2) = 634;
          numbering(414,3) = 627;
          numbering(414,4) = 630;
          numbering(414,5) = 636;
          numbering(414,6) = 638;
          numbering(414,7) = 111;
          numbering(415,0) = 632;
          numbering(415,1) = 633;
          numbering(415,2) = 635;
          numbering(415,3) = 634;
          numbering(415,4) = 636;
          numbering(415,5) = 637;
          numbering(415,6) = 113;
          numbering(415,7) = 638;
          numbering(416,0) = 91;
          numbering(416,1) = 488;
          numbering(416,2) = 611;
          numbering(416,3) = 612;
          numbering(416,4) = 524;
          numbering(416,5) = 523;
          numbering(416,6) = 639;
          numbering(416,7) = 640;
          numbering(417,0) = 488;
          numbering(417,1) = 90;
          numbering(417,2) = 613;
          numbering(417,3) = 611;
          numbering(417,4) = 523;
          numbering(417,5) = 525;
          numbering(417,6) = 641;
          numbering(417,7) = 639;
          numbering(418,0) = 612;
          numbering(418,1) = 611;
          numbering(418,2) = 614;
          numbering(418,3) = 109;
          numbering(418,4) = 640;
          numbering(418,5) = 639;
          numbering(418,6) = 642;
          numbering(418,7) = 643;
          numbering(419,0) = 611;
          numbering(419,1) = 613;
          numbering(419,2) = 108;
          numbering(419,3) = 614;
          numbering(419,4) = 639;
          numbering(419,5) = 641;
          numbering(419,6) = 644;
          numbering(419,7) = 642;
          numbering(420,0) = 524;
          numbering(420,1) = 523;
          numbering(420,2) = 639;
          numbering(420,3) = 640;
          numbering(420,4) = 24;
          numbering(420,5) = 529;
          numbering(420,6) = 645;
          numbering(420,7) = 646;
          numbering(421,0) = 523;
          numbering(421,1) = 525;
          numbering(421,2) = 641;
          numbering(421,3) = 639;
          numbering(421,4) = 529;
          numbering(421,5) = 97;
          numbering(421,6) = 647;
          numbering(421,7) = 645;
          numbering(422,0) = 640;
          numbering(422,1) = 639;
          numbering(422,2) = 642;
          numbering(422,3) = 643;
          numbering(422,4) = 646;
          numbering(422,5) = 645;
          numbering(422,6) = 648;
          numbering(422,7) = 115;
          numbering(423,0) = 639;
          numbering(423,1) = 641;
          numbering(423,2) = 644;
          numbering(423,3) = 642;
          numbering(423,4) = 645;
          numbering(423,5) = 647;
          numbering(423,6) = 114;
          numbering(423,7) = 648;
          numbering(424,0) = 90;
          numbering(424,1) = 495;
          numbering(424,2) = 619;
          numbering(424,3) = 613;
          numbering(424,4) = 525;
          numbering(424,5) = 532;
          numbering(424,6) = 649;
          numbering(424,7) = 641;
          numbering(425,0) = 495;
          numbering(425,1) = 92;
          numbering(425,2) = 620;
          numbering(425,3) = 619;
          numbering(425,4) = 532;
          numbering(425,5) = 533;
          numbering(425,6) = 650;
          numbering(425,7) = 649;
          numbering(426,0) = 613;
          numbering(426,1) = 619;
          numbering(426,2) = 621;
          numbering(426,3) = 108;
          numbering(426,4) = 641;
          numbering(426,5) = 649;
          numbering(426,6) = 651;
          numbering(426,7) = 644;
          numbering(427,0) = 619;
          numbering(427,1) = 620;
          numbering(427,2) = 110;
          numbering(427,3) = 621;
          numbering(427,4) = 649;
          numbering(427,5) = 650;
          numbering(427,6) = 652;
          numbering(427,7) = 651;
          numbering(428,0) = 525;
          numbering(428,1) = 532;
          numbering(428,2) = 649;
          numbering(428,3) = 641;
          numbering(428,4) = 97;
          numbering(428,5) = 536;
          numbering(428,6) = 653;
          numbering(428,7) = 647;
          numbering(429,0) = 532;
          numbering(429,1) = 533;
          numbering(429,2) = 650;
          numbering(429,3) = 649;
          numbering(429,4) = 536;
          numbering(429,5) = 23;
          numbering(429,6) = 654;
          numbering(429,7) = 653;
          numbering(430,0) = 641;
          numbering(430,1) = 649;
          numbering(430,2) = 651;
          numbering(430,3) = 644;
          numbering(430,4) = 647;
          numbering(430,5) = 653;
          numbering(430,6) = 655;
          numbering(430,7) = 114;
          numbering(431,0) = 649;
          numbering(431,1) = 650;
          numbering(431,2) = 652;
          numbering(431,3) = 651;
          numbering(431,4) = 653;
          numbering(431,5) = 654;
          numbering(431,6) = 116;
          numbering(431,7) = 655;
          numbering(432,0) = 109;
          numbering(432,1) = 614;
          numbering(432,2) = 628;
          numbering(432,3) = 629;
          numbering(432,4) = 643;
          numbering(432,5) = 642;
          numbering(432,6) = 656;
          numbering(432,7) = 657;
          numbering(433,0) = 614;
          numbering(433,1) = 108;
          numbering(433,2) = 630;
          numbering(433,3) = 628;
          numbering(433,4) = 642;
          numbering(433,5) = 644;
          numbering(433,6) = 658;
          numbering(433,7) = 656;
          numbering(434,0) = 629;
          numbering(434,1) = 628;
          numbering(434,2) = 631;
          numbering(434,3) = 112;
          numbering(434,4) = 657;
          numbering(434,5) = 656;
          numbering(434,6) = 659;
          numbering(434,7) = 660;
          numbering(435,0) = 628;
          numbering(435,1) = 630;
          numbering(435,2) = 111;
          numbering(435,3) = 631;
          numbering(435,4) = 656;
          numbering(435,5) = 658;
          numbering(435,6) = 661;
          numbering(435,7) = 659;
          numbering(436,0) = 643;
          numbering(436,1) = 642;
          numbering(436,2) = 656;
          numbering(436,3) = 657;
          numbering(436,4) = 115;
          numbering(436,5) = 648;
          numbering(436,6) = 662;
          numbering(436,7) = 663;
          numbering(437,0) = 642;
          numbering(437,1) = 644;
          numbering(437,2) = 658;
          numbering(437,3) = 656;
          numbering(437,4) = 648;
          numbering(437,5) = 114;
          numbering(437,6) = 664;
          numbering(437,7) = 662;
          numbering(438,0) = 657;
          numbering(438,1) = 656;
          numbering(438,2) = 659;
          numbering(438,3) = 660;
          numbering(438,4) = 663;
          numbering(438,5) = 662;
          numbering(438,6) = 665;
          numbering(438,7) = 7;
          numbering(439,0) = 656;
          numbering(439,1) = 658;
          numbering(439,2) = 661;
          numbering(439,3) = 659;
          numbering(439,4) = 662;
          numbering(439,5) = 664;
          numbering(439,6) = 117;
          numbering(439,7) = 665;
          numbering(440,0) = 108;
          numbering(440,1) = 621;
          numbering(440,2) = 636;
          numbering(440,3) = 630;
          numbering(440,4) = 644;
          numbering(440,5) = 651;
          numbering(440,6) = 666;
          numbering(440,7) = 658;
          numbering(441,0) = 621;
          numbering(441,1) = 110;
          numbering(441,2) = 637;
          numbering(441,3) = 636;
          numbering(441,4) = 651;
          numbering(441,5) = 652;
          numbering(441,6) = 667;
          numbering(441,7) = 666;
          numbering(442,0) = 630;
          numbering(442,1) = 636;
          numbering(442,2) = 638;
          numbering(442,3) = 111;
          numbering(442,4) = 658;
          numbering(442,5) = 666;
          numbering(442,6) = 668;
          numbering(442,7) = 661;
          numbering(443,0) = 636;
          numbering(443,1) = 637;
          numbering(443,2) = 113;
          numbering(443,3) = 638;
          numbering(443,4) = 666;
          numbering(443,5) = 667;
          numbering(443,6) = 669;
          numbering(443,7) = 668;
          numbering(444,0) = 644;
          numbering(444,1) = 651;
          numbering(444,2) = 666;
          numbering(444,3) = 658;
          numbering(444,4) = 114;
          numbering(444,5) = 655;
          numbering(444,6) = 670;
          numbering(444,7) = 664;
          numbering(445,0) = 651;
          numbering(445,1) = 652;
          numbering(445,2) = 667;
          numbering(445,3) = 666;
          numbering(445,4) = 655;
          numbering(445,5) = 116;
          numbering(445,6) = 671;
          numbering(445,7) = 670;
          numbering(446,0) = 658;
          numbering(446,1) = 666;
          numbering(446,2) = 668;
          numbering(446,3) = 661;
          numbering(446,4) = 664;
          numbering(446,5) = 670;
          numbering(446,6) = 672;
          numbering(446,7) = 117;
          numbering(447,0) = 666;
          numbering(447,1) = 667;
          numbering(447,2) = 669;
          numbering(447,3) = 668;
          numbering(447,4) = 670;
          numbering(447,5) = 671;
          numbering(447,6) = 26;
          numbering(447,7) = 672;
          numbering(448,0) = 13;
          numbering(448,1) = 297;
          numbering(448,2) = 431;
          numbering(448,3) = 368;
          numbering(448,4) = 492;
          numbering(448,5) = 559;
          numbering(448,6) = 673;
          numbering(448,7) = 616;
          numbering(449,0) = 297;
          numbering(449,1) = 59;
          numbering(449,2) = 432;
          numbering(449,3) = 431;
          numbering(449,4) = 559;
          numbering(449,5) = 560;
          numbering(449,6) = 674;
          numbering(449,7) = 673;
          numbering(450,0) = 368;
          numbering(450,1) = 431;
          numbering(450,2) = 433;
          numbering(450,3) = 72;
          numbering(450,4) = 616;
          numbering(450,5) = 673;
          numbering(450,6) = 675;
          numbering(450,7) = 618;
          numbering(451,0) = 431;
          numbering(451,1) = 432;
          numbering(451,2) = 81;
          numbering(451,3) = 433;
          numbering(451,4) = 673;
          numbering(451,5) = 674;
          numbering(451,6) = 676;
          numbering(451,7) = 675;
          numbering(452,0) = 492;
          numbering(452,1) = 559;
          numbering(452,2) = 673;
          numbering(452,3) = 616;
          numbering(452,4) = 92;
          numbering(452,5) = 563;
          numbering(452,6) = 677;
          numbering(452,7) = 620;
          numbering(453,0) = 559;
          numbering(453,1) = 560;
          numbering(453,2) = 674;
          numbering(453,3) = 673;
          numbering(453,4) = 563;
          numbering(453,5) = 102;
          numbering(453,6) = 678;
          numbering(453,7) = 677;
          numbering(454,0) = 616;
          numbering(454,1) = 673;
          numbering(454,2) = 675;
          numbering(454,3) = 618;
          numbering(454,4) = 620;
          numbering(454,5) = 677;
          numbering(454,6) = 679;
          numbering(454,7) = 110;
          numbering(455,0) = 673;
          numbering(455,1) = 674;
          numbering(455,2) = 676;
          numbering(455,3) = 675;
          numbering(455,4) = 677;
          numbering(455,5) = 678;
          numbering(455,6) = 118;
          numbering(455,7) = 679;
          numbering(456,0) = 59;
          numbering(456,1) = 304;
          numbering(456,2) = 438;
          numbering(456,3) = 432;
          numbering(456,4) = 560;
          numbering(456,5) = 566;
          numbering(456,6) = 680;
          numbering(456,7) = 674;
          numbering(457,0) = 304;
          numbering(457,1) = 17;
          numbering(457,2) = 439;
          numbering(457,3) = 438;
          numbering(457,4) = 566;
          numbering(457,5) = 567;
          numbering(457,6) = 681;
          numbering(457,7) = 680;
          numbering(458,0) = 432;
          numbering(458,1) = 438;
          numbering(458,2) = 440;
          numbering(458,3) = 81;
          numbering(458,4) = 674;
          numbering(458,5) = 680;
          numbering(458,6) = 682;
          numbering(458,7) = 676;
          numbering(459,0) = 438;
          numbering(459,1) = 439;
          numbering(459,2) = 82;
          numbering(459,3) = 440;
          numbering(459,4) = 680;
          numbering(459,5) = 681;
          numbering(459,6) = 683;
          numbering(459,7) = 682;
          numbering(460,0) = 560;
          numbering(460,1) = 566;
          numbering(460,2) = 680;
          numbering(460,3) = 674;
          numbering(460,4) = 102;
          numbering(460,5) = 570;
          numbering(460,6) = 684;
          numbering(460,7) = 678;
          numbering(461,0) = 566;
          numbering(461,1) = 567;
          numbering(461,2) = 681;
          numbering(461,3) = 680;
          numbering(461,4) = 570;
          numbering(461,5) = 103;
          numbering(461,6) = 685;
          numbering(461,7) = 684;
          numbering(462,0) = 674;
          numbering(462,1) = 680;
          numbering(462,2) = 682;
          numbering(462,3) = 676;
          numbering(462,4) = 678;
          numbering(462,5) = 684;
          numbering(462,6) = 686;
          numbering(462,7) = 118;
          numbering(463,0) = 680;
          numbering(463,1) = 681;
          numbering(463,2) = 683;
          numbering(463,3) = 682;
          numbering(463,4) = 684;
          numbering(463,5) = 685;
          numbering(463,6) = 119;
          numbering(463,7) = 686;
          numbering(464,0) = 72;
          numbering(464,1) = 433;
          numbering(464,2) = 445;
          numbering(464,3) = 385;
          numbering(464,4) = 618;
          numbering(464,5) = 675;
          numbering(464,6) = 687;
          numbering(464,7) = 633;
          numbering(465,0) = 433;
          numbering(465,1) = 81;
          numbering(465,2) = 446;
          numbering(465,3) = 445;
          numbering(465,4) = 675;
          numbering(465,5) = 676;
          numbering(465,6) = 688;
          numbering(465,7) = 687;
          numbering(466,0) = 385;
          numbering(466,1) = 445;
          numbering(466,2) = 447;
          numbering(466,3) = 19;
          numbering(466,4) = 633;
          numbering(466,5) = 687;
          numbering(466,6) = 689;
          numbering(466,7) = 635;
          numbering(467,0) = 445;
          numbering(467,1) = 446;
          numbering(467,2) = 83;
          numbering(467,3) = 447;
          numbering(467,4) = 687;
          numbering(467,5) = 688;
          numbering(467,6) = 690;
          numbering(467,7) = 689;
          numbering(468,0) = 618;
          numbering(468,1) = 675;
          numbering(468,2) = 687;
          numbering(468,3) = 633;
          numbering(468,4) = 110;
          numbering(468,5) = 679;
          numbering(468,6) = 691;
          numbering(468,7) = 637;
          numbering(469,0) = 675;
          numbering(469,1) = 676;
          numbering(469,2) = 688;
          numbering(469,3) = 687;
          numbering(469,4) = 679;
          numbering(469,5) = 118;
          numbering(469,6) = 692;
          numbering(469,7) = 691;
          numbering(470,0) = 633;
          numbering(470,1) = 687;
          numbering(470,2) = 689;
          numbering(470,3) = 635;
          numbering(470,4) = 637;
          numbering(470,5) = 691;
          numbering(470,6) = 693;
          numbering(470,7) = 113;
          numbering(471,0) = 687;
          numbering(471,1) = 688;
          numbering(471,2) = 690;
          numbering(471,3) = 689;
          numbering(471,4) = 691;
          numbering(471,5) = 692;
          numbering(471,6) = 120;
          numbering(471,7) = 693;
          numbering(472,0) = 81;
          numbering(472,1) = 440;
          numbering(472,2) = 452;
          numbering(472,3) = 446;
          numbering(472,4) = 676;
          numbering(472,5) = 682;
          numbering(472,6) = 694;
          numbering(472,7) = 688;
          numbering(473,0) = 440;
          numbering(473,1) = 82;
          numbering(473,2) = 453;
          numbering(473,3) = 452;
          numbering(473,4) = 682;
          numbering(473,5) = 683;
          numbering(473,6) = 695;
          numbering(473,7) = 694;
          numbering(474,0) = 446;
          numbering(474,1) = 452;
          numbering(474,2) = 454;
          numbering(474,3) = 83;
          numbering(474,4) = 688;
          numbering(474,5) = 694;
          numbering(474,6) = 696;
          numbering(474,7) = 690;
          numbering(475,0) = 452;
          numbering(475,1) = 453;
          numbering(475,2) = 21;
          numbering(475,3) = 454;
          numbering(475,4) = 694;
          numbering(475,5) = 695;
          numbering(475,6) = 697;
          numbering(475,7) = 696;
          numbering(476,0) = 676;
          numbering(476,1) = 682;
          numbering(476,2) = 694;
          numbering(476,3) = 688;
          numbering(476,4) = 118;
          numbering(476,5) = 686;
          numbering(476,6) = 698;
          numbering(476,7) = 692;
          numbering(477,0) = 682;
          numbering(477,1) = 683;
          numbering(477,2) = 695;
          numbering(477,3) = 694;
          numbering(477,4) = 686;
          numbering(477,5) = 119;
          numbering(477,6) = 699;
          numbering(477,7) = 698;
          numbering(478,0) = 688;
          numbering(478,1) = 694;
          numbering(478,2) = 696;
          numbering(478,3) = 690;
          numbering(478,4) = 692;
          numbering(478,5) = 698;
          numbering(478,6) = 700;
          numbering(478,7) = 120;
          numbering(479,0) = 694;
          numbering(479,1) = 695;
          numbering(479,2) = 697;
          numbering(479,3) = 696;
          numbering(479,4) = 698;
          numbering(479,5) = 699;
          numbering(479,6) = 121;
          numbering(479,7) = 700;
          numbering(480,0) = 92;
          numbering(480,1) = 563;
          numbering(480,2) = 677;
          numbering(480,3) = 620;
          numbering(480,4) = 533;
          numbering(480,5) = 593;
          numbering(480,6) = 701;
          numbering(480,7) = 650;
          numbering(481,0) = 563;
          numbering(481,1) = 102;
          numbering(481,2) = 678;
          numbering(481,3) = 677;
          numbering(481,4) = 593;
          numbering(481,5) = 594;
          numbering(481,6) = 702;
          numbering(481,7) = 701;
          numbering(482,0) = 620;
          numbering(482,1) = 677;
          numbering(482,2) = 679;
          numbering(482,3) = 110;
          numbering(482,4) = 650;
          numbering(482,5) = 701;
          numbering(482,6) = 703;
          numbering(482,7) = 652;
          numbering(483,0) = 677;
          numbering(483,1) = 678;
          numbering(483,2) = 118;
          numbering(483,3) = 679;
          numbering(483,4) = 701;
          numbering(483,5) = 702;
          numbering(483,6) = 704;
          numbering(483,7) = 703;
          numbering(484,0) = 533;
          numbering(484,1) = 593;
          numbering(484,2) = 701;
          numbering(484,3) = 650;
          numbering(484,4) = 23;
          numbering(484,5) = 597;
          numbering(484,6) = 705;
          numbering(484,7) = 654;
          numbering(485,0) = 593;
          numbering(485,1) = 594;
          numbering(485,2) = 702;
          numbering(485,3) = 701;
          numbering(485,4) = 597;
          numbering(485,5) = 107;
          numbering(485,6) = 706;
          numbering(485,7) = 705;
          numbering(486,0) = 650;
          numbering(486,1) = 701;
          numbering(486,2) = 703;
          numbering(486,3) = 652;
          numbering(486,4) = 654;
          numbering(486,5) = 705;
          numbering(486,6) = 707;
          numbering(486,7) = 116;
          numbering(487,0) = 701;
          numbering(487,1) = 702;
          numbering(487,2) = 704;
          numbering(487,3) = 703;
          numbering(487,4) = 705;
          numbering(487,5) = 706;
          numbering(487,6) = 122;
          numbering(487,7) = 707;
          numbering(488,0) = 102;
          numbering(488,1) = 570;
          numbering(488,2) = 684;
          numbering(488,3) = 678;
          numbering(488,4) = 594;
          numbering(488,5) = 600;
          numbering(488,6) = 708;
          numbering(488,7) = 702;
          numbering(489,0) = 570;
          numbering(489,1) = 103;
          numbering(489,2) = 685;
          numbering(489,3) = 684;
          numbering(489,4) = 600;
          numbering(489,5) = 601;
          numbering(489,6) = 709;
          numbering(489,7) = 708;
          numbering(490,0) = 678;
          numbering(490,1) = 684;
          numbering(490,2) = 686;
          numbering(490,3) = 118;
          numbering(490,4) = 702;
          numbering(490,5) = 708;
          numbering(490,6) = 710;
          numbering(490,7) = 704;
          numbering(491,0) = 684;
          numbering(491,1) = 685;
          numbering(491,2) = 119;
          numbering(491,3) = 686;
          numbering(491,4) = 708;
          numbering(491,5) = 709;
          numbering(491,6) = 711;
          numbering(491,7) = 710;
          numbering(492,0) = 594;
          numbering(492,1) = 600;
          numbering(492,2) = 708;
          numbering(492,3) = 702;
          numbering(492,4) = 107;
          numbering(492,5) = 604;
          numbering(492,6) = 712;
          numbering(492,7) = 706;
          numbering(493,0) = 600;
          numbering(493,1) = 601;
          numbering(493,2) = 709;
          numbering(493,3) = 708;
          numbering(493,4) = 604;
          numbering(493,5) = 25;
          numbering(493,6) = 713;
          numbering(493,7) = 712;
          numbering(494,0) = 702;
          numbering(494,1) = 708;
          numbering(494,2) = 710;
          numbering(494,3) = 704;
          numbering(494,4) = 706;
          numbering(494,5) = 712;
          numbering(494,6) = 714;
          numbering(494,7) = 122;
          numbering(495,0) = 708;
          numbering(495,1) = 709;
          numbering(495,2) = 711;
          numbering(495,3) = 710;
          numbering(495,4) = 712;
          numbering(495,5) = 713;
          numbering(495,6) = 123;
          numbering(495,7) = 714;
          numbering(496,0) = 110;
          numbering(496,1) = 679;
          numbering(496,2) = 691;
          numbering(496,3) = 637;
          numbering(496,4) = 652;
          numbering(496,5) = 703;
          numbering(496,6) = 715;
          numbering(496,7) = 667;
          numbering(497,0) = 679;
          numbering(497,1) = 118;
          numbering(497,2) = 692;
          numbering(497,3) = 691;
          numbering(497,4) = 703;
          numbering(497,5) = 704;
          numbering(497,6) = 716;
          numbering(497,7) = 715;
          numbering(498,0) = 637;
          numbering(498,1) = 691;
          numbering(498,2) = 693;
          numbering(498,3) = 113;
          numbering(498,4) = 667;
          numbering(498,5) = 715;
          numbering(498,6) = 717;
          numbering(498,7) = 669;
          numbering(499,0) = 691;
          numbering(499,1) = 692;
          numbering(499,2) = 120;
          numbering(499,3) = 693;
          numbering(499,4) = 715;
          numbering(499,5) = 716;
          numbering(499,6) = 718;
          numbering(499,7) = 717;
          numbering(500,0) = 652;
          numbering(500,1) = 703;
          numbering(500,2) = 715;
          numbering(500,3) = 667;
          numbering(500,4) = 116;
          numbering(500,5) = 707;
          numbering(500,6) = 719;
          numbering(500,7) = 671;
          numbering(501,0) = 703;
          numbering(501,1) = 704;
          numbering(501,2) = 716;
          numbering(501,3) = 715;
          numbering(501,4) = 707;
          numbering(501,5) = 122;
          numbering(501,6) = 720;
          numbering(501,7) = 719;
          numbering(502,0) = 667;
          numbering(502,1) = 715;
          numbering(502,2) = 717;
          numbering(502,3) = 669;
          numbering(502,4) = 671;
          numbering(502,5) = 719;
          numbering(502,6) = 721;
          numbering(502,7) = 26;
          numbering(503,0) = 715;
          numbering(503,1) = 716;
          numbering(503,2) = 718;
          numbering(503,3) = 717;
          numbering(503,4) = 719;
          numbering(503,5) = 720;
          numbering(503,6) = 124;
          numbering(503,7) = 721;
          numbering(504,0) = 118;
          numbering(504,1) = 686;
          numbering(504,2) = 698;
          numbering(504,3) = 692;
          numbering(504,4) = 704;
          numbering(504,5) = 710;
          numbering(504,6) = 722;
          numbering(504,7) = 716;
          numbering(505,0) = 686;
          numbering(505,1) = 119;
          numbering(505,2) = 699;
          numbering(505,3) = 698;
          numbering(505,4) = 710;
          numbering(505,5) = 711;
          numbering(505,6) = 723;
          numbering(505,7) = 722;
          numbering(506,0) = 692;
          numbering(506,1) = 698;
          numbering(506,2) = 700;
          numbering(506,3) = 120;
          numbering(506,4) = 716;
          numbering(506,5) = 722;
          numbering(506,6) = 724;
          numbering(506,7) = 718;
          numbering(507,0) = 698;
          numbering(507,1) = 699;
          numbering(507,2) = 121;
          numbering(507,3) = 700;
          numbering(507,4) = 722;
          numbering(507,5) = 723;
          numbering(507,6) = 725;
          numbering(507,7) = 724;
          numbering(508,0) = 704;
          numbering(508,1) = 710;
          numbering(508,2) = 722;
          numbering(508,3) = 716;
          numbering(508,4) = 122;
          numbering(508,5) = 714;
          numbering(508,6) = 726;
          numbering(508,7) = 720;
          numbering(509,0) = 710;
          numbering(509,1) = 711;
          numbering(509,2) = 723;
          numbering(509,3) = 722;
          numbering(509,4) = 714;
          numbering(509,5) = 123;
          numbering(509,6) = 727;
          numbering(509,7) = 726;
          numbering(510,0) = 716;
          numbering(510,1) = 722;
          numbering(510,2) = 724;
          numbering(510,3) = 718;
          numbering(510,4) = 720;
          numbering(510,5) = 726;
          numbering(510,6) = 728;
          numbering(510,7) = 124;
          numbering(511,0) = 722;
          numbering(511,1) = 723;
          numbering(511,2) = 725;
          numbering(511,3) = 724;
          numbering(511,4) = 726;
          numbering(511,5) = 727;
          numbering(511,6) = 6;
          numbering(511,7) = 728;
          return;
        }
      }
    }
    default:
    {
      std::cerr<<"Error in VTKHelperSpaMesh: itemtype "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt"); 
      return;
    }
  }
}

unsigned int VTKHelperSpaMesh::vtkItemType(const ElemTypesMesh& itemType)
{
  unsigned int celltype = 0; 
  switch (itemType)
  {
  case SpaMesh::line:				
    celltype = 3; //VTK_LINE
    break;
  case SpaMesh::tri:       
    celltype = 5;  //VTK_TRIANGLE
    break;
  case SpaMesh::quad:      
    celltype = 9;  //VTK_QUAD
    break;
  case SpaMesh::tet:      
    celltype = 10;  //VTK_TETRA
    break;
  case SpaMesh::hex:    
    celltype = 12;  //VTK_HEXAHEDRON
    break;
  case SpaMesh::prism:     
    celltype = 13;   //VTK_WEDGE
    break;
  case SpaMesh::pyram:
    celltype = 10;   //VTK_TETRA
    break;
  default:
    {
      std::cerr<<"element type "<<itemType<<" not implemented"<<std::endl;
      std::logic_error("halt");
    }
  }
  return celltype;
}
